@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 2/2/94
 *
-* $Revision: 6.2 $
+* $Revision: 6.4 $
 *
 * File Description:  Sequence editing utilities
 *
@@ -39,6 +39,12 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: edutil.h,v $
+* Revision 6.4  2003/02/10 22:57:45  kans
+* added BioseqCopyEx, which takes a BioseqPtr instead of a SeqIdPtr for the source
+*
+* Revision 6.3  2002/07/02 13:23:43  kans
+* added SeqLocDeleteEx
+*
 * Revision 6.2  1997/11/10 19:50:23  kans
 * Fixed incorrect comment for ISAGappedSeqLoc() function (SB).
 *
@@ -155,10 +161,16 @@ NLM_EXTERN Boolean LIBCALL BioseqDelete (SeqIdPtr target, Int4 from, Int4 to, Bo
 *      If do_feat == TRUE copies appropriate region of feature table from
 *        sourceid to new copy using SeqFeatsCopy().
 *
+*
+*   BioseqCopyEx(newid, oldbsp, from, to, strand, do_feat)
+*      Internal function called by BioseqCopy that takes BioseqPtr
+*
 *****************************************************************************/
 NLM_EXTERN BioseqPtr LIBCALL BioseqCopy (SeqIdPtr newid, SeqIdPtr sourceid, Int4 from, Int4 to,
                                Uint1 strand, Boolean do_feat);
 
+NLM_EXTERN BioseqPtr LIBCALL BioseqCopyEx (SeqIdPtr newid, BioseqPtr oldbsp, Int4 from, Int4 to,
+                               Uint1 strand, Boolean do_feat);
 
 /*****************************************************************************
 *
@@ -355,6 +367,8 @@ NLM_EXTERN SeqLocPtr LIBCALL SegLocToParts (BioseqPtr seg, SeqLocPtr slp);
 *****************************************************************************/
 NLM_EXTERN SeqLocPtr LIBCALL SeqLocDelete (SeqLocPtr head, SeqIdPtr target, Int4 from, Int4 to, Boolean merge, BoolPtr changed);
 
+NLM_EXTERN SeqLocPtr LIBCALL SeqLocDeleteEx (SeqLocPtr head, SeqIdPtr target, Int4 from, Int4 to, Boolean merge, BoolPtr changed, BoolPtr partial5, BoolPtr partial3);
+
 NLM_EXTERN SeqLocPtr LIBCALL SeqLocSubtract (SeqLocPtr head, SeqLocPtr piece);
 
 
