@@ -21,7 +21,7 @@
  *  Please cite the author in any work or product based on this material.
  *
  * ===========================================================================
- * $Id: salpacc.c,v 6.27 2000/11/16 22:13:44 shavirin Exp $
+ * $Id: salpacc.c,v 6.29 2004/05/20 20:21:24 bollin Exp $
  Collection of SeqAlign Accession utilities.
  Maintainer: Hugues Sicotte
  Authors of the original routines: Hugues Sicotte, Colombe Chappey, Tom Madden, Jinghui Zhang
@@ -132,7 +132,7 @@ NLM_EXTERN SeqIdPtr LIBCALL SeqAlignId (SeqAlignPtr salp, Int2 index)
      sip = SeqIdPtrFromSeqAlign(salp);
      if (sip != NULL)
      {
-        for (j=0; j<index; j++)
+        for (j=0; j<index && sip != NULL; j++)
            sip = sip->next;
      }
   }
@@ -567,24 +567,30 @@ NLM_EXTERN Uint1 LIBCALL SeqAlignMolType (SeqAlignPtr salp)
   Int2         dim;
   Uint1        moltype = 0;
   Boolean      molb;
+  SeqAlignPtr  tmpsalp;
  
   if (salp==NULL)
      return FALSE;
-  if (salp->segtype == 1) {
+  if (salp->segtype == SAS_DENDIAG) {
      ddp = (DenseDiagPtr) salp->segs;
      sip = ddp->id;
      dim = ddp->dim;
   }
-  else if (salp->segtype == 2) {
+  else if (salp->segtype == SAS_DENSEG) {
      dsp = (DenseSegPtr) salp->segs;
      sip = dsp->ids;
      dim = dsp->dim;
   }
-  else if (salp->segtype == 3) {
+  else if (salp->segtype == SAS_STD) {
      ssp = (StdSegPtr) salp->segs;
      sip = ssp->ids;
      dim = ssp->dim; 
   }
+  else if (salp->segtype == SAS_DISC) 
+  {
+  	 tmpsalp = (SeqAlignPtr) salp->segs;
+  	 return SeqAlignMolType (tmpsalp);
+  }
   if (sip!=NULL) {
    for (k = 0; k < dim && sip!=NULL; k++, sip = sip->next)
    {
