@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 9/94
 *
-* $Revision: 6.41 $
+* $Revision: 6.53 $
 *
 * File Description:  Manager for Bioseqs and BioseqSets
 *
@@ -40,6 +40,42 @@
 *
 *
 * $Log: seqmgr.h,v $
+* Revision 6.53  2004/02/24 20:57:45  kans
+* added genesByLocusTag field, SeqMgrGetNextGeneByLocusTag function, now that locus_tag should be a unique identifier
+*
+* Revision 6.52  2003/10/23 17:40:01  kans
+* added SeqMgrGetOverlappingOperon and bspextra operonsByPos and numoperons fields
+*
+* Revision 6.51  2003/09/22 16:13:20  kans
+* LockFarComponentsEx takes new SeqLocPtr parameter
+*
+* Revision 6.50  2003/02/12 14:20:47  kans
+* added IsNonGappedLiteral, used to allow compressed deltas as (previously always raw) parts of segsets
+*
+* Revision 6.49  2003/01/21 20:15:01  kans
+* support for SeqMgrSetSeqIdSetFunc and GetSeqIdSetForGI needed for validator
+*
+* Revision 6.48  2002/06/10 13:50:43  kans
+* added SeqMgrSetLenFunc, SeqMgrSetAccnVerFunc
+*
+* Revision 6.47  2002/05/08 18:58:10  kans
+* itemID is Uint4
+*
+* Revision 6.46  2002/04/12 20:42:39  kans
+* LookupFarSeqIDs has separate parameter for alignments and history
+*
+* Revision 6.45  2002/03/14 16:40:03  kans
+* SeqMgrIndexFeaturesEx takes dorevfeats, SeqMgrExploreFeaturesRev and featsByRev added for asn2gb
+*
+* Revision 6.44  2002/03/07 16:12:30  kans
+* TrimLocInSegment for asn2gb master style to constrain feature locations within the part sequence range used by the contig
+*
+* Revision 6.43  2001/12/28 13:19:12  kans
+* added seq_id_precache_func, SeqMgrSetPreCache, LookupFarSeqIDs
+*
+* Revision 6.42  2001/12/26 15:29:23  kans
+* FetchFromSeqIdGiCache and RecordInSeqIdGiCache are extern
+*
 * Revision 6.41  2001/11/15 18:15:48  kans
 * set bsp->omdp at creation, SeqMgrDeleteIndexesInRecord sets omdp->bulkIndexFree
 *
@@ -262,6 +298,11 @@ typedef BioseqPtr (LIBCALLBACK * BSFetchTop)
 
 typedef BioseqPtr (LIBCALLBACK * BSFetch) PROTO((SeqIdPtr sip, Pointer data));
 
+typedef Int4 (LIBCALLBACK * SIDPreCacheFunc) (SeqEntryPtr sep, Boolean components, Boolean locations, Boolean products, Boolean alignments, Boolean history);
+typedef Int4 (LIBCALLBACK * SeqLenLookupFunc) (Int4 gi);
+typedef CharPtr (LIBCALLBACK * AccnVerLookupFunc) (Int4 gi);
+typedef SeqIdPtr (LIBCALLBACK * SeqIdSetLookupFunc) (Int4 gi);
+
 typedef struct seqidindexelement {
 	CharPtr str;               /* PRINTID_FASTA_SHORT string */
 	ObjMgrDataPtr omdp;             /* the omdp containing the Bioseq */
@@ -293,6 +334,10 @@ typedef struct seqmng {        /* functions for sequence data management */
 	SeqIdIndexBlockPtr BioseqIndexData;    /* what BioseqIndex points to */
 	Boolean is_write_locked;
 	Int4 hold_indexing;      /* set by SeqMgrHoldIndexing */
+	SIDPreCacheFunc seq_id_precache_func;
+	SeqLenLookupFunc seq_len_lookup_func;
+	AccnVerLookupFunc accn_ver_lookup_func;
+	SeqIdSetLookupFunc seq_id_set_lookup_func;
 } SeqMgr, PNTR SeqMgrPtr;
 
 /**** All replaced in Object Manager ************/
@@ -388,6 +433,16 @@ NLM_EXTERN SeqEntryPtr LIBCALL SeqMgrGetSeqEntryForEntityID PROTO((Int2 id));
 *****************************************************************************/
 NLM_EXTERN SeqIdPtr LIBCALL GetSeqIdForGI PROTO((Int4 gi));
 
+/*****************************************************************************
+*
+*   GetSeqIdSetForGI(Int4)
+*     returns the chain of all SeqIds for a GI
+*     returns NULL if can't find it
+*     The returned SeqId chain is allocated. Caller must free it with SeqIdSetFree.
+*
+*****************************************************************************/
+NLM_EXTERN SeqIdPtr LIBCALL GetSeqIdSetForGI PROTO((Int4 gi));
+
 
 /*****************************************************************************
 *
@@ -762,6 +817,16 @@ NLM_EXTERN Boolean LIBCALL SpreadGapsInDeltaSeq PROTO((BioseqPtr bsp));
 *****************************************************************************/
 NLM_EXTERN Boolean LIBCALL CountGapsInDeltaSeq PROTO((BioseqPtr bsp, Int4Ptr num_segs, Int4Ptr num_gaps, Int4Ptr known_residues, Int4Ptr num_gaps_faked, CharPtr buf, Int4 buflen));
 
+/*****************************************************************************
+*
+*   IsNonGappedLiteral(BioseqPtr bsp)
+*      Returns TRUE if bsp is a delta seq is composed only of Seq-lits with
+*      actual sequence data.  These are now made to allow optimal compression
+*      of otherwise raw sequences with runs of ambiguous bases.
+*
+*****************************************************************************/
+NLM_EXTERN Boolean IsNonGappedLiteral (BioseqPtr bsp);
+
 /*****************************************************************************
 *
 *   GetUniGeneIDForSeqId(SeqIdPtr)
@@ -777,6 +842,16 @@ NLM_EXTERN Boolean LIBCALL CountGapsInDeltaSeq PROTO((BioseqPtr bsp, Int4Ptr num
 
 NLM_EXTERN Int4 LIBCALL GetUniGeneIDForSeqId PROTO((SeqIdPtr sip));
 
+/*****************************************************************************
+*
+*   FetchFromSeqIdGiCache(gi, sipp)
+*   RecordInSeqIdGiCache(gi, sip)
+*     Internal functions to cache gi - SeqId associations
+*
+*****************************************************************************/
+NLM_EXTERN Boolean FetchFromSeqIdGiCache (Int4 gi, SeqIdPtr PNTR sipp);
+NLM_EXTERN void RecordInSeqIdGiCache (Int4 gi, SeqIdPtr sip);
+
 
 /*****************************************************************************
 *
@@ -803,9 +878,9 @@ typedef struct smfeatitem {
   Boolean      farloc;   /* location has an accession not packaged in entity */
   Uint1        strand;   /* strand (mapped to segmented bioseq if segmented) */
   Uint1        subtype;  /* featdef subtype */
-  Uint2        itemID;   /* storing itemID so no need to gather again */
+  Uint4        itemID;   /* storing itemID so no need to gather again */
   Boolean      ignore;   /* ignore this second copy of a feature spanning the origin */
-  Uint2        index;    /* position index needed for SeqMgrGetDesiredFeature */
+  Uint4        index;    /* position index needed for SeqMgrGetDesiredFeature */
   Int4         overlap;  /* for xxxByPos, index of leftmost candidate that overlaps this */
 } SMFeatItem, PNTR SMFeatItemPtr;
 
@@ -824,14 +899,14 @@ typedef struct segpartsmap {
   Int4                     from;
   Int4                     to;
   Uint1                    strand;
-  Uint2                    itemID;         /* OBJ_BIOSEQ_SEG itemID */
+  Uint4                    itemID;         /* OBJ_BIOSEQ_SEG itemID */
 } SMSeqIdx, PNTR SMSeqIdxPtr;
 
 typedef struct smdescitem {
   SeqDescrPtr  sdp;         /* freed when TL_CACHED, later will implement reassignment when reloaded */
   SeqEntryPtr  sep;         /* SeqEntry containing SeqDescr, same reap/reload criteria as above */
-  Uint2        itemID;      /* storing itemID so no need to gather again */
-  Uint2        index;       /* position index needed for SeqMgrGetDesiredDescriptor */
+  Uint4        itemID;      /* storing itemID so no need to gather again */
+  Uint4        index;       /* position index needed for SeqMgrGetDesiredDescriptor */
   Uint2        level;       /* packaging level - 0 is on Bioseq itself */
   Uint1        seqdesctype; /* seqdesc subtype */
 } SMDescItem, PNTR SMDescItemPtr;
@@ -857,6 +932,7 @@ typedef struct bioseqextra {
   SMFeatItemPtr PNTR  featsByID;      /* array of all features on bioseq in original itemID order */
   SMFeatItemPtr PNTR  featsBySfp;     /* array of all features on bioseq sorted by SeqFeatPtr */
   SMFeatItemPtr PNTR  featsByPos;     /* array of all features on bioseq sorted by location */
+  SMFeatItemPtr PNTR  featsByRev;     /* array of all features on bioseq sorted by reverse location */
   SMFeatItemPtr PNTR  featsByLabel;   /* array of all features on bioseq sorted by label */
 
   SMFeatItemPtr PNTR  genesByPos;     /* subset of featsByPos array containing only gene features */
@@ -864,6 +940,9 @@ typedef struct bioseqextra {
   SMFeatItemPtr PNTR  CDSsByPos;      /* subset of featsByPos array containing only CDS features */
   SMFeatItemPtr PNTR  pubsByPos;      /* subset of featsByPos array containing only publication features */
   SMFeatItemPtr PNTR  orgsByPos;      /* subset of featsByPos array containing only biosource features */
+  SMFeatItemPtr PNTR  operonsByPos;   /* subset of featsByPos array containing only operon features */
+
+  SMFeatItemPtr PNTR  genesByLocusTag;  /* array of gene features sorted by locus_tag */
 
   BioseqPtr           parentBioseq;   /* segmented parent of this raw part all packaged together */
   SMSeqIdxPtr         segparthead;    /* linked list to speed mapping from parts to segmented bioseq */
@@ -879,12 +958,13 @@ typedef struct bioseqextra {
   Int4                numCDSs;        /* number of elements in CDSsByPos array */
   Int4                numpubs;        /* number of elements in pubsByPos array */
   Int4                numorgs;        /* number of elements in orgsByPos array */
+  Int4                numoperons;     /* number of elements in operonsByPos array */
 
   Int4                numsegs;        /* number of segments in partslist array */
 
   Int4                min;            /* used for finding best protein feature */
-  Uint2               bspItemID;      /* for bioseq explore functions */
-  Uint2               bspIndex;       /* for bioseq explore functions */
+  Uint4               bspItemID;      /* for bioseq explore functions */
+  Uint4               bspIndex;       /* for bioseq explore functions */
   Int2                blocksize;      /* size of SMFeatBlock.data array to avoid wasting space */
                                       /* additional fields to map between genome record and parts,
                                          genomic DNA and mRNA, and mRNA and protein */
@@ -917,7 +997,7 @@ NLM_EXTERN Pointer LIBCALLBACK SeqMgrFreeBioseqExtraFunc PROTO((Pointer data));
 *****************************************************************************/
 
 NLM_EXTERN SMFeatItemPtr LIBCALL SeqMgrFindSMFeatItemPtr PROTO((SeqFeatPtr sfp));
-NLM_EXTERN SMFeatItemPtr LIBCALL SeqMgrFindSMFeatItemByID PROTO((Uint2 entityID, BioseqPtr bsp, Uint2 itemID));
+NLM_EXTERN SMFeatItemPtr LIBCALL SeqMgrFindSMFeatItemByID PROTO((Uint2 entityID, BioseqPtr bsp, Uint4 itemID));
 NLM_EXTERN ValNodePtr LIBCALL SeqMgrGetSfpProductList (BioseqPtr bsp);
 
 /*****************************************************************************
@@ -937,6 +1017,20 @@ NLM_EXTERN Int4 LIBCALL SeqMgrMapPartToSegmentedBioseq PROTO((BioseqPtr in, Int4
 
 NLM_EXTERN SMSeqIdxPtr GenomePartToSegmentMap (BioseqPtr in, BioseqPtr bsp, Int4 from, Int4 to);
 
+/*****************************************************************************
+*
+*   TrimLocInSegment takes a location on an indexed far segmented part and trims
+*     trims it to the region referred to by the parent segmented or delta bioseq.
+*
+*****************************************************************************/
+
+NLM_EXTERN SeqLocPtr TrimLocInSegment (
+  BioseqPtr master,
+  SeqLocPtr location,
+  BoolPtr p5ptr,
+  BoolPtr p3ptr
+);
+
 /*****************************************************************************
 *
 *   SeqMgrIndexAlignments called by SeqMgrIndexFeatures, can be called separately
@@ -967,10 +1061,42 @@ NLM_EXTERN SeqAlignPtr LIBCALL SeqMgrFindSeqAlignByID PROTO((Uint2 entityID, Uin
 
 NLM_EXTERN ValNodePtr LockFarComponents (SeqEntryPtr sep);
 
-NLM_EXTERN ValNodePtr LockFarComponentsEx (SeqEntryPtr sep, Boolean components, Boolean locations, Boolean products);
+NLM_EXTERN ValNodePtr LockFarComponentsEx (SeqEntryPtr sep, Boolean components, Boolean locations, Boolean products, SeqLocPtr loc);
 
 NLM_EXTERN ValNodePtr UnlockFarComponents (ValNodePtr bsplist);
 
+/*****************************************************************************
+*
+*   SeqMgrSetPreCache registers the GiToSeqID precache function
+*   LookupFarSeqIDs calls any registered function to preload the cache
+*
+*****************************************************************************/
+
+NLM_EXTERN void LIBCALL SeqMgrSetPreCache (SIDPreCacheFunc func);
+
+NLM_EXTERN Int4 LookupFarSeqIDs (
+  SeqEntryPtr sep,
+  Boolean components,
+  Boolean locations,
+  Boolean products,
+  Boolean alignments,
+  Boolean history
+);
+
+/*****************************************************************************
+*
+*   SeqMgrSetLenFunc registers the GiToSeqLen lookup function
+*   SeqMgrSetAccnVerFunc registers the GiToAccnVer lookup function
+*   SeqMgrSetSeqIdSetFunc registers the GiToSeqIdSet lookup function
+*
+*****************************************************************************/
+
+NLM_EXTERN void LIBCALL SeqMgrSetLenFunc (SeqLenLookupFunc func);
+
+NLM_EXTERN void LIBCALL SeqMgrSetAccnVerFunc (AccnVerLookupFunc func);
+
+NLM_EXTERN void LIBCALL SeqMgrSetSeqIdSetFunc (SeqIdSetLookupFunc func);
+
 /*****************************************************************************
 *
 *   SeqEntryAsnOut (SeqEntryPtr sep, SeqIdPtr sip, Int2 retcode, AsnIoPtr aipout)
