@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   9/2/97
 *
-* $Revision: 6.79 $
+* $Revision: 6.98 $
 *
 * File Description: 
 *
@@ -47,6 +47,8 @@
 
 #include <ncbi.h>
 #include <sequtil.h>
+#include <objpubme.h>
+#include <util/creaders/alnread.h>
 
 #undef NLM_EXTERN
 #ifdef NLM_IMPORT
@@ -92,6 +94,8 @@ NLM_EXTERN SeqLocPtr   CreateWholeInterval (SeqEntryPtr sep);
 NLM_EXTERN SeqFeatPtr  CreateNewFeature (SeqEntryPtr sep, SeqEntryPtr placeHere, Uint1 choice, SeqFeatPtr useThis);
 NLM_EXTERN ValNodePtr  CreateNewDescriptor (SeqEntryPtr sep, Uint1 choice);
 
+NLM_EXTERN Boolean IsPopPhyEtcSet (Uint1 _class);
+
 /* Variants that call SeqMgrGetSeqEntryForData. The feature version allows a location
 to be specified, overriding the default full-length seq-int location.  (If location is
 not NULL, it copies it after deleting the existing sfp->location.)  For both functions
@@ -160,6 +164,11 @@ NLM_EXTERN SeqLocPtr SeqLocMergeEx (BioseqPtr target, SeqLocPtr to, SeqLocPtr fr
                                     Boolean single_interval, Boolean fuse_joints,
                                     Boolean merge_overlaps, Boolean add_null);
 
+NLM_EXTERN SeqLocPtr SeqLocMergeExEx (BioseqPtr target, SeqLocPtr to, SeqLocPtr from,
+                                      Boolean single_interval, Boolean fuse_joints,
+                                      Boolean merge_overlaps, Boolean add_null,
+                                      Boolean ignore_mixed);
+
 NLM_EXTERN Boolean CheckSeqLocForPartial (SeqLocPtr location, BoolPtr p5ptr, BoolPtr p3ptr);
 NLM_EXTERN void SetSeqLocPartial (SeqLocPtr location, Boolean partial5, Boolean partial3);
 NLM_EXTERN void FreeAllFuzz (SeqLocPtr location);
@@ -193,6 +202,16 @@ NLM_EXTERN void ResynchCodingRegionPartials (SeqEntryPtr sep);
 
 NLM_EXTERN void ResynchMessengerRNAPartials (SeqEntryPtr sep);
 
+/* resynchronizes protein feature with product peptide bioseq */
+
+NLM_EXTERN void ResynchProteinPartials (SeqEntryPtr sep);
+
+/* individual feature callbacks for above functions */
+
+NLM_EXTERN void ResynchMRNAPartials (SeqFeatPtr sfp, Pointer userdata);
+NLM_EXTERN void ResynchCDSPartials (SeqFeatPtr sfp, Pointer userdata);
+NLM_EXTERN void ResynchPeptidePartials (SeqFeatPtr sfp, Pointer userdata);
+
 /* functions to parse [org=Drosophila melanogaster] and [gene=lacZ] from titles */
 /* for example, passing "gene" to SqnTagFind returns "lacZ" */
 
@@ -203,6 +222,7 @@ typedef struct sqntag {
   Int2     num_tags;
   CharPtr  tag [MAX_SQN_TAGS];
   CharPtr  val [MAX_SQN_TAGS];
+  Boolean  used [MAX_SQN_TAGS];
 } SqnTag, PNTR SqnTagPtr;
 
 NLM_EXTERN SqnTagPtr SqnTagParse (CharPtr ttl);
@@ -248,6 +268,16 @@ NLM_EXTERN SeqHistPtr ParseTitleIntoSeqHist (
   SeqHistPtr shp
 );
 
+NLM_EXTERN SeqHistPtr ParseStringIntoSeqHist (
+  SeqHistPtr shp,
+  CharPtr str
+);
+
+NLM_EXTERN UserObjectPtr ParseTitleIntoTpaAssembly (
+  SqnTagPtr stp,
+  UserObjectPtr uop
+);
+
 /* UseLocalAsnloadDataAndErrMsg transiently sets paths to asnload, data, and errmsg
   if they are packaged in the same directory as the executing program. */
 
@@ -260,15 +290,19 @@ NLM_EXTERN void GetRidOfLocusInSeqIds (Uint2 entityID, SeqEntryPtr sep);
 NLM_EXTERN SeqLocPtr StripLocusFromSeqLoc (SeqLocPtr location);
 NLM_EXTERN SeqIdPtr SeqIdStripLocus (SeqIdPtr sip);
 
+/* LeaveBestCDD removes all but best CDD region in an area of overlapping features */
+
+NLM_EXTERN void LeaveBestCDD (SeqEntryPtr sep);
+
 /* ConvertPubSrcComDescsToFeats is useful when merging records */
 
-NLM_EXTERN Boolean ConvertPubSrcComDescsToFeats (SeqEntryPtr sep, Boolean pub, Boolean src, Boolean com, Boolean toProts);
+NLM_EXTERN Boolean ConvertPubSrcComDescsToFeats (SeqEntryPtr sep, Boolean pub, Boolean src, Boolean com, Boolean toProts, Boolean PNTR asked_about_prop, Boolean PNTR propagate_descriptions, CharPtr findstring);
 
 NLM_EXTERN void DeleteMultipleTitles (SeqEntryPtr sep, Pointer mydata, Int4 index, Int2 indent);
 
 NLM_EXTERN Uint1 FindTrnaAA (CharPtr str);
 NLM_EXTERN Uint1 FindTrnaAA3 (CharPtr str);
-NLM_EXTERN Uint1 ParseTRnaString (CharPtr strx, BoolPtr justTrnaText, Uint1Ptr codon);
+NLM_EXTERN Uint1 ParseTRnaString (CharPtr strx, BoolPtr justTrnaText, Uint1Ptr codon, Boolean noSingleLetter);
 NLM_EXTERN CharPtr FindTrnaAAIndex (CharPtr str);
 NLM_EXTERN ValNodePtr TokenizeTRnaString (CharPtr strx);
 NLM_EXTERN Boolean ParseDegenerateCodon (tRNAPtr trp, Uint1Ptr codon);
@@ -279,6 +313,10 @@ NLM_EXTERN Boolean SerialNumberInString (CharPtr str);
 NLM_EXTERN int LIBCALLBACK SortVnpByString (VoidPtr ptr1, VoidPtr ptr2);
 NLM_EXTERN ValNodePtr UniqueValNode (ValNodePtr list);
 
+/* for sorting valnode list by data.intvalue */
+
+NLM_EXTERN int LIBCALLBACK SortByIntvalue (VoidPtr ptr1, VoidPtr ptr2);
+
 /* keytag sorts/uniques and then owns valnode character list */
 
 typedef struct keytag {
@@ -316,6 +354,10 @@ of Bioseqs containing SeqGraphs, with individual reads removed and only contigs
 
 NLM_EXTERN SeqEntryPtr ReadPhrapFile (FILE *fp);
 
+/* Internal function to read quality scores, made available to parse separate DNA and quality score files */
+
+NLM_EXTERN SeqGraphPtr ReadPhrapQuality (FILE *fp, BioseqPtr bsp);
+
 /* SetPhrapContigOrder takes the results of ReadPhrapFile and a string indicating the order
 of contigs, and returns a SeqEntryList in the desired order, with all other contigs removed */
 
@@ -355,7 +397,8 @@ feature table with ReadAsnFastaOrFlatFile) to stand-alone gene features or prote
 and protein bioseqs.  It processes ALL features in the list - you give it the FIRST sfp. */
 
 NLM_EXTERN void PromoteXrefs (SeqFeatPtr sfp, BioseqPtr bsp, Uint2 entityID);
-NLM_EXTERN void PromoteXrefsEx (SeqFeatPtr sfp, BioseqPtr bsp, Uint2 entityID, Boolean include_stop, Boolean remove_trailingX);
+NLM_EXTERN void PromoteXrefsEx (SeqFeatPtr sfp, BioseqPtr bsp, Uint2 entityID, Boolean include_stop,
+                                Boolean remove_trailingX, Boolean gen_prod_set);
 
 /* SetEmptyGeneticCodes imposes genetic code on all coding regions within a feature table */
 
@@ -389,6 +432,10 @@ which does change the itemID structure, and is intended to be safe for a retrofi
 
 NLM_EXTERN void CautiousSeqEntryCleanup (SeqEntryPtr sep, SeqEntryFunc taxfun, SeqEntryFunc taxmerge);
 
+/* Convert a segmented or delta Bioseq to a raw Bioseq */
+
+NLM_EXTERN void SegOrDeltaBioseqToRaw (BioseqPtr bsp);
+
 /* general purpose text finite state machine */
 /* based on Practical Algorithms for Programmers by Binstock and Rex */
 
@@ -468,11 +515,16 @@ NLM_EXTERN Int4 VisitSetsInSet (BioseqSetPtr bssp, Pointer userdata, VisitSetsFu
 typedef void (*VisitElementsFunc) (SeqEntryPtr sep, Pointer userdata);
 NLM_EXTERN Int4 VisitElementsInSep (SeqEntryPtr sep, Pointer userdata, VisitElementsFunc callback);
 
-/* visits all SeqIds within a SeqLoc */
+/* visits all SeqIds within a SeqLoc, or within features, alignments, graphs, or annots */
 
 typedef void (*VisitSeqIdFunc) (SeqIdPtr sip, Pointer userdata);
 NLM_EXTERN Int4 VisitSeqIdsInSeqLoc (SeqLocPtr slp, Pointer userdata, VisitSeqIdFunc callback);
 
+NLM_EXTERN Int4 VisitSeqIdsInSeqFeat (SeqFeatPtr sfp, Pointer userdata, VisitSeqIdFunc callback);
+NLM_EXTERN Int4 VisitSeqIdsInSeqAlign (SeqAlignPtr sap, Pointer userdata, VisitSeqIdFunc callback);
+NLM_EXTERN Int4 VisitSeqIdsInSeqGraph (SeqGraphPtr sgp, Pointer userdata, VisitSeqIdFunc callback);
+NLM_EXTERN Int4 VisitSeqIdsInSeqAnnot (SeqAnnotPtr annot, Pointer userdata, VisitSeqIdFunc callback);
+
 /* visits all sub UserFields - if the data type is 11, VisitUserFieldsInUfp recurses */
 
 typedef void (*VisitUserFieldsFunc) (UserFieldPtr ufp, Pointer userdata);
@@ -513,6 +565,37 @@ NLM_EXTERN Int4 VisitBioSourcesInSep (SeqEntryPtr sep, Pointer userdata, VisitBi
 typedef void (*ScanBioseqSetFunc) (SeqEntryPtr sep, Pointer userdata);
 NLM_EXTERN Int4 ScanBioseqSetRelease (CharPtr inputFile, Boolean binary, Boolean compressed, Pointer userdata, ScanBioseqSetFunc callback);
 
+/* PubMed registered fetch functionality */
+
+NLM_EXTERN PubmedEntryPtr LIBCALL GetPubMedForUid (Int4 uid);
+
+/* internal support type, registration function */
+
+typedef PubmedEntryPtr (LIBCALLBACK * PubMedFetchFunc) (Int4 uid);
+
+NLM_EXTERN void LIBCALL PubMedSetFetchFunc (PubMedFetchFunc func);
+
+extern CharPtr MyFGetLine (FILE *fp, ValNodePtr PNTR current_data);
+
+#if defined (WIN32)
+extern char * __stdcall AbstractReadFunction (Pointer userdata);
+extern void __stdcall AbstractReportError (TErrorInfoPtr err_ptr, Pointer userdata);
+#else
+extern char * AbstractReadFunction (Pointer userdata);
+extern void AbstractReportError (TErrorInfoPtr err_ptr, Pointer userdata);
+#endif
+
+typedef struct readbuffer {
+  FILE *fp;
+  ValNodePtr current_data;
+} ReadBufferData, PNTR ReadBufferPtr;
+
+extern void FreeBufferedReadList (ValNodePtr vnp);
+
+extern SeqEntryPtr MakeSequinDataFromAlignment (TAlignmentFilePtr afp, Uint1 moltype);
+extern SeqEntryPtr make_seqentry_for_seqentry (SeqEntryPtr sep);
+
+
 
 #ifdef __cplusplus
 }
