@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 7/12/91
 *
-* $Revision: 6.16 $
+* $Revision: 6.23 $
 *
 * File Description:  various sequence objects to fasta output
 *
@@ -39,6 +39,24 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: tofasta.h,v $
+* Revision 6.23  2004/04/21 14:42:36  kans
+* added blocklen and grouplen parameters to BioseqFastaStream and SeqEntryFastaStream, former also has do_defline parameter
+*
+* Revision 6.22  2004/03/15 19:56:09  kans
+* BioseqFastaStream takes StreamFlgType flags parameter, added SeqEntryFastaStream for exploration
+*
+* Revision 6.21  2003/11/18 19:04:29  kans
+* added BioseqFastaStream, rapid generator based on SeqPortStream
+*
+* Revision 6.20  2002/07/18 19:28:35  kans
+*  working implementation of BioseqRawToFastaExtraEx, always zero out all of MyFsa struct, no need for fake seqloc
+*
+* Revision 6.18  2002/07/14 21:04:32  camacho
+* Added option to print a subsequence of a raw bioseq to fasta format
+*
+* Revision 6.17  2002/01/16 17:02:04  camacho
+* Changed buflen and seqlen in MyFsa struct. to use Uint4, as well as some function prototypes
+*
 * Revision 6.16  2001/08/31 15:29:54  kans
 * added CreateDefLineEx to ignore existing title descriptor, force generation of protein title computationally
 *
@@ -168,8 +186,8 @@ typedef struct descrinfo {   /* struct used for defline */
 
 typedef struct myfsa {   /* struct used for fasta searches */
     CharPtr buf;         /* buffer for strings (suggest 255 minimum) */
-    Int2 buflen,         /* length of buf */
-        seqlen;          /* length of sequence lines (must be buflen-1 or less) */
+    Uint4 buflen,         /* length of buf */
+          seqlen;          /* length of sequence lines (must be buflen-1 or less) */
     Pointer mydata;      /* pointer to your own data */
     FastaWriteFunc myfunc;  /* callback to process parts of fasta file */
     BioseqPtr bsp;       /* Bioseq being processed */
@@ -183,6 +201,7 @@ typedef struct myfsa {   /* struct used for fasta searches */
     Uint1	code;	/* coding of sequence */
     Boolean	formatdb; /* TRUE, if is used in formatdb */
     Boolean printid_general; /* show gi and gnl for SeqId */
+    SeqLocPtr seqloc;  /* sub-sequence of interest */
 } MyFsa, PNTR MyFsaPtr;
 
 typedef struct tofasta {
@@ -196,6 +215,7 @@ typedef struct tofasta {
     
 
 NLM_EXTERN Boolean BioseqRawToFastaExtra PROTO((BioseqPtr bsp, FILE *fp, Int2 line_length));
+NLM_EXTERN Boolean BioseqRawToFastaExtraEx PROTO((BioseqPtr bsp, FILE *fp, Int2 line_length, SeqLocPtr slp));
 NLM_EXTERN Boolean BioseqRawToFasta PROTO((BioseqPtr bsp, FILE * fp, Boolean is_na));
 NLM_EXTERN Boolean SeqEntryToFasta PROTO((SeqEntryPtr sep, FILE * fp, Boolean is_na));
 NLM_EXTERN Boolean SeqEntryToFastaEx PROTO((SeqEntryPtr sep, FILE * fp, Boolean is_na, Boolean printid_general));
@@ -226,6 +246,37 @@ NLM_EXTERN Boolean SeqEntrysToDefline PROTO((SeqEntryPtr sep,
 NLM_EXTERN Boolean BioseqRawToFastaX PROTO((BioseqPtr bsp, MyFsaPtr mfp, Boolean is_na));
 NLM_EXTERN Boolean BioseqToFastaX PROTO((BioseqPtr bsp, MyFsaPtr mfp, Boolean is_na));
 
+/*****************************************************************************
+*
+*   BioseqFastaStream(bsp, fp, flags, linelen, blocklen, grouplen, do_defline)
+*   SeqEntryFastaStream (sep, fp, flags, linelen, blocklen, grouplen,
+*                        do_na, do_aa, master_style)
+*
+*   	Rapid FASTA generators using SeqPortStream
+*
+*****************************************************************************/
+NLM_EXTERN void BioseqFastaStream (
+  BioseqPtr bsp,
+  FILE *fp,
+  StreamFlgType flags,
+  Int2 linelen,
+  Int2 blocklen,
+  Int2 grouplen,
+  Boolean do_defline
+);
+
+NLM_EXTERN void SeqEntryFastaStream (
+  SeqEntryPtr sep,
+  FILE *fp,
+  StreamFlgType flags,
+  Int2 linelen,
+  Int2 blocklen,
+  Int2 grouplen,
+  Boolean do_na,
+  Boolean do_aa,
+  Boolean master_style
+);
+
 /*****************************************************************************
 *
 *   FastaFileFunc(key, buf, data)
@@ -350,7 +401,7 @@ Boolean FastaReadSequenceMem
 *      buf should be at least 40 bytes
 *
 *****************************************************************************/
-NLM_EXTERN Boolean FastaId PROTO((BioseqPtr bsp, CharPtr buf, Int2 buflen));
+NLM_EXTERN Boolean FastaId PROTO((BioseqPtr bsp, CharPtr buf, Uint4 buflen));
 
 /*****************************************************************************
 *
@@ -365,10 +416,10 @@ NLM_EXTERN Boolean FastaId PROTO((BioseqPtr bsp, CharPtr buf, Int2 buflen));
 *       if (tech == MI_TECH_phase1 or phase2, adds order comment to defline)
 *
 *****************************************************************************/
-NLM_EXTERN Boolean FastaDefLine PROTO((BioseqPtr bsp, CharPtr buf, Int2 buflen, CharPtr accession, CharPtr organism, Uint1 tech));
+NLM_EXTERN Boolean FastaDefLine PROTO((BioseqPtr bsp, CharPtr buf, Uint4 buflen, CharPtr accession, CharPtr organism, Uint1 tech));
 
-NLM_EXTERN Boolean CreateDefLine PROTO((ItemInfoPtr dip, BioseqPtr bsp, CharPtr buf, Int2 buflen, Uint1 tech, CharPtr accession, CharPtr organism));
-NLM_EXTERN Boolean CreateDefLineEx (ItemInfoPtr iip, BioseqPtr bsp, CharPtr buf, Int2 buflen, Uint1 tech, CharPtr accession, CharPtr organism, Boolean ignoreTitle);
+NLM_EXTERN Boolean CreateDefLine PROTO((ItemInfoPtr dip, BioseqPtr bsp, CharPtr buf, Uint4 buflen, Uint1 tech, CharPtr accession, CharPtr organism));
+NLM_EXTERN Boolean CreateDefLineEx (ItemInfoPtr iip, BioseqPtr bsp, CharPtr buf, Uint4 buflen, Uint1 tech, CharPtr accession, CharPtr organism, Boolean ignoreTitle);
 /*****************************************************************************
 *
 *   FastaSeqPort(bsp, is_na, do_virtual)
@@ -377,6 +428,8 @@ NLM_EXTERN Boolean CreateDefLineEx (ItemInfoPtr iip, BioseqPtr bsp, CharPtr buf,
 *****************************************************************************/
 NLM_EXTERN SeqPortPtr FastaSeqPort PROTO((BioseqPtr bsp, Boolean is_na,
                                           Boolean do_virtual, Uint1 code));
+NLM_EXTERN SeqPortPtr FastaSeqPortEx PROTO((BioseqPtr bsp, Boolean is_na, 
+        Boolean do_virtual, Uint1 code, SeqLocPtr slp));
 
 /*****************************************************************************
 *
