@@ -29,7 +29,7 @@
 *
 * Version Creation Date: 3/4/91
 *
-* $Revision: 6.1 $
+* $Revision: 6.5 $
 *
 * File Description:
 *   Routines for parsing ASN.1 value nototation (text) messages
@@ -44,6 +44,18 @@
 * 04-20-93 Schuler     LIBCALL calling convention
 *
 * $Log: asnlex.c,v $
+* Revision 6.5  2000/12/12 15:56:12  ostell
+* added support BigInt
+*
+* Revision 6.4  2000/03/10 18:02:05  kans
+* increased size of tbuf in AsnLexReadOctets to handle long lines in hand-edited (?) ASN.1 records being submitted
+*
+* Revision 6.3  1999/12/23 17:25:44  kans
+* AsnTxtReadVal checks for NULL atp - same as recent check in AsnBinReadVal
+*
+* Revision 6.2  1998/06/12 19:27:51  kans
+* fixed unix compiler warnings
+*
 * Revision 6.1  1997/10/29 02:41:31  vakatov
 * Type castings to pass through the C++ compiler
 *
@@ -309,6 +321,10 @@ NLM_EXTERN Int2 LIBCALL  AsnTxtReadVal (AsnIoPtr aip, AsnTypePtr atp, DataValPtr
 	retval = 1;  /* assume success */
     currpsp = & aip->typestack[aip->type_indent];
 	curr_type = currpsp->type;
+
+	if (atp == NULL)
+		return 0;
+
 	base_type = AsnFindBaseType(atp);     
     if (base_type == NULL)     /* not found */
     {
@@ -403,6 +419,9 @@ NLM_EXTERN Int2 LIBCALL  AsnTxtReadVal (AsnIoPtr aip, AsnTypePtr atp, DataValPtr
 		case ENUM_TYPE:
 			valueptr->intvalue = AsnLexReadInteger(aip, atp);
 			break;
+		case BIGINT_TYPE:
+			valueptr->bigintvalue = AsnLexReadBigInt(aip, atp);
+			break;
 		case REAL_TYPE:
 			valueptr->realvalue = AsnLexReadReal(aip, atp);
 			break;
@@ -555,6 +574,42 @@ NLM_EXTERN Int4 AsnLexReadInteger (AsnIoPtr aip, AsnTypePtr atp)
 	return 0;
 }
 
+/*****************************************************************************
+*
+*   Int8 AsnLexReadBigInt(aip, atp)
+*   	expects an INTEGER next
+*   	assumes none of it has already been read
+*   	does not advance to next token
+*   	atp points to the definition of this integer for named values
+*
+*****************************************************************************/
+NLM_EXTERN Int8 AsnLexReadBigInt (AsnIoPtr aip, AsnTypePtr atp)
+{
+	Int2 token;
+
+	token = AsnLexWord(aip);      /* read the integer */
+
+	if (token == NUMBER)    /* just a number */
+	{
+		return AsnLexBigInt(aip);
+	}
+
+	if (token != IDENT)    /* not number or named value */
+	{
+		AsnIoErrorMsg(aip, 39, AsnErrGetTypeName(atp->name), aip->linenumber);
+		return 0;
+	}
+
+			/******************** read a named integer value *********/
+			/**** not supported for bigint *****/
+
+		   /******************* could it be a previously defined value? ***/
+
+	AsnIoErrorMsg(aip, 41, AsnErrGetTypeName(atp->name), aip->linenumber);
+
+	return (Int8)0;
+}
+
 /*****************************************************************************
 *
 *   Boolean AsnLexReadBoolean(aip, atp)
@@ -680,7 +735,7 @@ NLM_EXTERN ByteStorePtr AsnLexReadOctets (AsnIoPtr aip, AsnTypePtr atp)
 {
 	Int2 token, len;
 	ByteStorePtr ssp = NULL;
-	Byte tbuf[101];
+	Byte tbuf[256]; /* was 101 - changed to handle occasional hand-edited ASN.1? */
 	Int4 bytes, left, added;
 
 	token = AsnLexWord(aip);      /* read the start */
@@ -948,6 +1003,23 @@ NLM_EXTERN Int4 AsnLexInteger (AsnIoPtr aip)
 	return value;
 }
 
+/*****************************************************************************
+*
+*   Int8 AsnLexBigInt(aip)
+*
+*****************************************************************************/
+NLM_EXTERN Int8 AsnLexBigInt (AsnIoPtr aip)
+{
+	char tbuf[40];
+	Int8 value;
+	const char * result;
+
+	MemCopy(tbuf, aip->word, (size_t)aip->wordlen);
+	tbuf[aip->wordlen] = '\0';
+	value = Nlm_StringToInt8 (tbuf, &result);
+	return value;
+}
+
 /*****************************************************************************
 *
 *   Int2 AsnLexWord(aip)
@@ -1227,7 +1299,7 @@ NLM_EXTERN Boolean StrMatch (CharPtr a, CharPtr b, Int2 len)
 static Int2 AsnLexScan (AsnIoPtr aip, CharPtr name)
 {
 	register CharPtr pos;
-	Int2 token;
+	Int2 token = 0;
 	register int linepos, len;
 	int done;
 	Boolean started = FALSE, matched_ref = FALSE;
