@@ -1,4 +1,4 @@
-/* $Id: mmdbsrv.c,v 6.8 1998/04/16 17:51:38 madej Exp $
+/* $Id: mmdbsrv.c,v 6.28 2001/12/12 20:43:38 beloslyu Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -29,15 +29,81 @@
 *
 * Version Creation Date: 6 January 1997
 *
-* $Revision: 6.8 $
+* $Revision: 6.28 $
 *
 * File Description:
 *        MMDB WWW-server 
 *
 * $Log: mmdbsrv.c,v $
-* Revision 6.8  1998/04/16 17:51:38  madej
-* Tweak to print out author names in new/old format.
+* Revision 6.28  2001/12/12 20:43:38  beloslyu
+* change the name of ftp site to ftp.ncbi.nih.gov
 *
+* Revision 6.27  2001/01/25 23:02:56  lewisg
+* get rid of content-disp, add mdlLvl and maxModels args
+*
+* Revision 6.26  2000/08/21 16:12:42  lewisg
+* add save=asntext option
+*
+* Revision 6.25  2000/06/26 18:05:26  lewisg
+* fix another hardcoded url
+*
+* Revision 6.24  2000/06/23 22:35:58  lewisg
+* get rid of hardcoded urls
+*
+* Revision 6.23  2000/01/31 19:59:12  lewisg
+* fix output of date in pdb header
+*
+* Revision 6.22  1999/10/13 20:13:18  zimmerma
+* DZ: Removed use of temporary files - html ouput redirected to stdout
+*
+* Revision 6.21  1999/06/17 14:44:09  zimmerma
+* Added static char variable for cvsId in order to use unix "what"
+*
+* Revision 6.20  1999/06/15 18:15:34  addess
+* Initialized this_uid to 0 on line 1286
+*
+* Revision 6.19  1999/05/13 15:51:33  kimelman
+* bugfix: memory deallocation fixed
+*
+* Revision 6.19  1999/05/13 15:51:33  kimelman
+* bugfix: memory deallocation fixed
+*
+* Revision 6.18  1999/05/11 23:36:34  kimelman
+* make StrToInt4 static
+*
+* Revision 6.17  1999/05/11 17:50:23  kimelman
+* style
+*
+* Revision 6.16  1999/04/28 21:08:30  lewisg
+* fixed double freeing of Biostruc
+*
+* Revision 6.15  1999/04/28 18:26:15  kimelman
+* exit codes changed to 0
+*
+* Revision 6.14  1999/04/22 01:56:44  kimelman
+* 1. pdb reference read completely from configurstion files
+* 2. indentaiton
+* 3. NcbiMimeAsn1Free(mime) commented out <-- lead to core
+*    dump AFTER sending MIME (quick & dirty fix)
+*
+* Revision 6.13  1999/04/19 16:52:57  kimelman
+* spelling fix
+*
+* Revision 6.12  1999/03/05 19:18:47  addess
+* changed URL to link to RCSB structure explorer
+*
+* Revision 6.11  1998/11/06 16:25:31  addess
+* added features for layer-1 and PDB TITLE records
+*
+ * Revision 6.10  1998/07/17  18:45:53  madej
+ * Got rid of some unused URLs.
+ *
+ * Revision 6.9  1998/06/11  19:12:53  madej
+ * Changes in a message type for Cn3D by Yanli.
+ *
+ * Revision 6.8  1998/04/16  17:51:38  madej
+ * Tweak to print out author names in new/old format.
+ *
  * Revision 6.7  1998/04/15  14:54:05  kimelman
  * 1. Make retrieval unifirm from open server and sql server.
  * 2. mmdbsrv retrival performance tuning:
@@ -130,7 +196,7 @@ This describes what you need to run MMDBSRV:
 ...mmdb/vastdata/
 
 These contain the MMDB data on your local filesystem,
-as obtained from ftp://ncbi.nlm.nih.gov/mmdb/mmdbdata  
+as obtained from ftp://ftp.ncbi.nih.gov/mmdb/mmdbdata  
 and updated regularly by MIRROR.  You can 
 put them in the CGI-BIN or HTTP (HTML) directories
 if you prefer. If you wish to implement a local
@@ -144,7 +210,7 @@ http://123.456.789.10/cgi-bin/mmdb/
 
 The first contains the compiled mmdbsrv program and the
 config file, obtained from 
-ftp://ncbi.nlm.nih.gov/mmdb/www/mmdbsrv
+ftp://ftp.ncbi.nih.gov/mmdb/www/mmdbsrv
 The second will contain the log files written out.
 The third needs sshead.txt, sscode.txt, sstail.txt
 NOTE - you must edit "sstail.txt"
@@ -160,12 +226,12 @@ The first contains the html files to launch mmdbsrv,
 arrow.gif, strucsum.gif and any other gif files, and the
 associated MMDB help files struchelp.html, cn3d.html,
 strucinstall.html, vast.html, etc.  Master copies are kept
-at ftp://ncbi.nlm.nih.gov/mmdb/www/http
+at ftp://ftp.ncbi.nih.gov/mmdb/www/http
 You must hand edit or use scripts or SED to change links from 
 NCBI's to your own in all these HTML files.
 
 The second contains the GIF library as obtained from
-the NCBI ftp site ftp://ncbi.nlm.nih.gov/mmdb/mmdbgifs
+the NCBI ftp site ftp://ftp.ncbi.nih.gov/mmdb/mmdbgifs
 
 6] Your WWW-Server - make it aware of the following 
 new MIME-types:
@@ -240,14 +306,19 @@ TAXurl = http://www.ncbi.nlm.nih.gov/htbin-post/Taxonomy/wgetorg
  
 #include <ncbi.h>
 #include <ncbiwww.h>
+#include <accentr.h>
 #include <ncbistr.h>
 #include <miscutils.h>
 #include <sys/resource.h>
 #include <mmdbapi.h>
 #include <mmdbdata.h>
-#include <objmime.h>
-#include <accentr.h>
 #include <taxinc.h>
+#include <sys/resource.h>
+
+#include <lsqfetch.h>
+#include <netentr.h>
+#include <asnmime.h>
+#include <objmime.h>
 
 #ifdef OS_UNIX
 #include <signal.h>
@@ -301,9 +372,6 @@ static Char CGIName[256];
 static Char DATApath[PATH_MAX];
 static Char LOGpath[PATH_MAX];
 
-static Char GIFpath[PATH_MAX];
-static Char GIFurl[PATH_MAX];
-
 static Char ENTREZurl[PATH_MAX];
 static Char VASTurl[PATH_MAX];
 static Char MMDBurl[PATH_MAX];
@@ -314,88 +382,7 @@ static Char MAILto[256];
 static Char MAILTO[PATH_MAX];
 static Char ARROW[PATH_MAX];
 
-
-
-static FILE    *GlobalDisplayFile = NULL;
-static Char    GlobalDisplayName[PATH_MAX];
-static CharPtr SendFileName;
-static Int4    SendFileBytesOutput;
-
-
-
-/* Several of these routines are from Brandon Brylawski's WWW code */
-
-static void PrintFileTimedOut(int sig)
-/* This is called if the alarm in WWWSendFile goes off, indicating that 
-   part of a file took way too long to display. */
-{
-  ErrLogPrintf("#WWWSendFile timed out : wrote %ld of %ld bytes.\n",
-  	    SendFileBytesOutput, FileLength(SendFileName)); 
-  FileRemove(GlobalDisplayName);
-  exit(1);
-}
-
-
-
-/* This routine displays the given file to standard output.  It buffers and
- * safeguards it appropriately.
- *
- * The semantics have been changed from an earlier version.  Previously,
- * WWWSendFile had a single parameter, the file name 'Name', and opened the
- * file for reading.  It turned out that the safeguarding was insufficient.
- * E.g. if the user interrupted the data transfer during a "See File" or
- * "Save File" then the temporary file was left sitting in /var/tmp.  To
- * guard against this, the calling routine for WWWSendFile must now itself
- * open the file for reading, unlink it, and then call WWWSendFile with
- * the name and FILE pointer.  When WWWSendFile returns the caller should
- * close and remove the file.
- */
-
-static Int4 WWWSendFile(CharPtr Name, FILE *File)
-{
-#define ALARM_SECS 120
-#define PF_BUFSIZE 1500
-
-  char FullName[200], Buffer[PF_BUFSIZE];
-  Int4 NumChars;
-
-  if (!Name)
-    return 0;
-
-  SendFileName = Name;
-  SendFileBytesOutput = 0;
-
-#ifdef OS_UNIX
-  signal(SIGALRM, PrintFileTimedOut);
-#endif
-
-  if (!File)
-    return 0;
-
-  /* It is assumed the file is already open for reading! */
-  do
-    {
-      NumChars = FileRead(Buffer,sizeof(char),PF_BUFSIZE,File);
-      alarm(ALARM_SECS); 
-      if (FileWrite(Buffer,sizeof(char),NumChars,stdout) != NumChars)
-	{
-	  ErrLogPrintf("#WWWSendFile PrintFile write error!\n");
-	  alarm(0);
-	  break;
-	}
-      fflush(stdout);
-      SendFileBytesOutput += NumChars;
-      alarm(0);
-    }
-  while(NumChars == PF_BUFSIZE);
-
-  alarm(ALARM_SECS); 
-  fflush(stdout);
-  alarm(0);
-  return SendFileBytesOutput;
-}
-
-
+static char* cvsId_ = "@(#)$Id: mmdbsrv.c,v 6.28 2001/12/12 20:43:38 beloslyu Exp $";
 
 /*****************************************************
  * WWWPrintFileData looks in the current CGI-BIN directory 
@@ -442,7 +429,7 @@ static void WWWPrintFileData(CharPtr FName,  FILE *pFile)
 }
 
 
-Boolean
+static Boolean
 StrToInt4 (CharPtr str, Int4Ptr longval)
 {
   Nlm_Int2     i;
@@ -577,6 +564,42 @@ static Int4 GetOptionValue(CharPtr Value, Int4 NumElements, CharPtr *ElementText
 
 #define EntrezLink	"<A HREF=\"%s?uid=%ld&form=6&db=%c&dopt=%c\">%s</A>"
 
+/************ Added by Ken for Layer One PDB releases ************/
+
+static void My_StringNCpy(CharPtr str1, CharPtr str2, Int4 len)
+{
+  StringNCpy(str1,str2,len);
+  str1[len] = '\0';
+} 
+
+static Boolean IsLayerOne(ValNodePtr pvn)
+{
+  Boolean Layer = FALSE;
+  CharPtr comment_txt;
+  CharPtr LayerOne = {"This is Layer 1 Release"};
+  CharPtr temp_string;
+  Int4 len;
+  
+  while (pvn)
+  {
+     if (pvn->choice == BiostrucDescr_pdb_comment)
+     {
+        comment_txt = (CharPtr)pvn->data.ptrvalue;
+        temp_string = StringSave(LayerOne);
+        len = StringLen(LayerOne);
+        My_StringNCpy(temp_string, &comment_txt[18], len);
+        if (!StringICmp(temp_string, LayerOne))
+        {
+          Layer = TRUE;
+          break;
+        }
+     }
+     pvn = pvn->next;
+   }
+   
+   return Layer;
+}
+/***** End of functions added by Ken ********************/
 static void
 PrintMolecularContents(PDNMS ModelStruc, FILE *gdf)
 {
@@ -678,10 +701,12 @@ PrintStructureInfo(PDNMS ModelStruc,  FILE *File, CharPtr tax_save)
   *****/
   fprintf(File, "<font size=+2>");
   fprintf(File, "MMDB Id: <A HREF=\"%s%s?uid=%ld&form=6&db=t&Dopt=s\">%ld</A>&nbsp ",
-	URLcgi, "mmdbsrv", (long) pmsdThis->iMMDBid, (long) pmsdThis->iMMDBid);
-  fprintf(File, "PDB Id: <A HREF=\"%s?oPDBid=%s\">%s</a></font><BR>\n", TDBurl,
-	pmsdThis->pcPDBName, pmsdThis->pcPDBName);
-  fprintf(File, "<BR>\n");
+	URLcgi, CGIName, (long) pmsdThis->iMMDBid, (long) pmsdThis->iMMDBid);
+  fprintf(File, "PDB Id: <A HREF=\"%s=%s\">%s</a>\n", TDBurl,pmsdThis->pcPDBName, pmsdThis->pcPDBName);
+  if (IsLayerOne(pmsdThis->pbsBS->descr))
+    fprintf(File, " - Warning: <A HREF=\"http://www.pdb.bnl.gov/pdb-docs/what_is_LR.html\">Not validated!</A>\n");
+  fprintf(File, "</font>");  
+  fprintf(File, "<BR>\n<BR>\n");
   fprintf(File, "<TABLE>\n");
   PrintMolecularContents(ModelStruc, File);
   fprintf(File, "<TR>\n<TD VALIGN=TOP NOWRAP>");
@@ -710,8 +735,8 @@ PrintStructureInfo(PDNMS ModelStruc,  FILE *File, CharPtr tax_save)
    	depyear = pbssThis->database_entry_date->data[1];
    	depday = pbssThis->database_entry_date->data[3];
   	fprintf(File, "<TR>\n<TD VALIGN=TOP NOWRAP>");
-	fprintf(File, "<strong>PDB Deposition</strong>:</TD>\n<TD>%2d-%3s-%2d</TD>\n</TR>\n",
-		(int) depday, NCBI_months[pbssThis->database_entry_date->data[2] - 1], (int) depyear);
+	fprintf(File, "<strong>PDB Deposition</strong>:</TD>\n<TD>%2d-%3s-%02d</TD>\n</TR>\n",
+		(int) depday, NCBI_months[pbssThis->database_entry_date->data[2] - 1], (int) depyear%100);
   }
 
   if (pmsdThis->pcPdbClass) {
@@ -720,8 +745,11 @@ PrintStructureInfo(PDNMS ModelStruc,  FILE *File, CharPtr tax_save)
   }
   
   fprintf(File, "<TR>\n<TD VALIGN=TOP NOWRAP>");
-  fprintf(File, "<strong>PDB Compound</strong>:</TD>\n<TD>%s</TD>\n</TR>\n", pmsdThis->pcChemName);
-
+  pvnThis = ValNodeFindNext(pmsdThis->pGraphDescr,NULL,BiomolDescr_pdb_comment);
+  if (pvnThis)
+    fprintf(File, "<strong>PDB Title</strong>:</TD>\n<TD>%s</TD>\n</TR>\n", pmsdThis->pcChemName);
+  else
+    fprintf(File, "<strong>PDB Compound</strong>:</TD>\n<TD>%s</TD>\n</TR>\n", pmsdThis->pcChemName);
   /* insert a blank row in the table, for spacing */
   fprintf(File, "<TR>\n<TD VALIGN=TOP NOWRAP> </TD>\n<TD><BR></TD>\n</TR>\n");
 
@@ -736,7 +764,9 @@ static void
 PrintStructureView(PDNMS ModelStruc, FILE *File)
 {
 	PMSD pmsd;
- 
+      
+        if (ModelStruc == NULL) return;         
+
 	pmsd = (PMSD) ModelStruc->data.ptrvalue;
 	fprintf(File, "<FORM METHOD=POST ACTION=\"%s%s\">\n", URLcgi, CGIName);
 	fprintf(File, "<INPUT TYPE=HIDDEN NAME=uid VALUE=%ld>\n", (long) pmsd->iMMDBid);
@@ -1111,7 +1141,9 @@ SaveTaxonomyInfo(BiostrucPtr bsp, CharPtr save, Int2 n, CharPtr pname)
 
 
 
-static void DumpMime(AsnIoPtr aip, CharPtr title, Int2 style, Int2 choice, VoidPtr datum)
+/* Old version of DumpMime, preserved for compatibility with Cn3D v1.0. */
+
+static void DumpMime_v1(AsnIoPtr aip, CharPtr title, Int2 style, Int2 choice, VoidPtr datum)
 {
   NcbiMimeAsn1Ptr mime;
   EntrezGeneralPtr egp;
@@ -1131,6 +1163,26 @@ static void DumpMime(AsnIoPtr aip, CharPtr title, Int2 style, Int2 choice, VoidP
 
   egp->Data_data->data.ptrvalue = NULL; /* for clean free */
   NcbiMimeAsn1Free(mime);
+
+} /* end DumpMime_v1 */
+
+
+
+                                   /* yanli */
+static void DumpMime(AsnIoPtr aip, CharPtr title, VoidPtr datum)
+{
+  NcbiMimeAsn1Ptr mime;
+  EntrezGeneralPtr egp;
+
+  mime = (NcbiMimeAsn1Ptr) ValNodeNew(NULL);
+  mime->choice = NcbiMimeAsn1_strucseq;
+  mime->data.ptrvalue= datum;
+
+  NcbiMimeAsn1AsnWrite(mime, aip, NULL);
+  AsnIoFlush(aip);
+
+  mime->data.ptrvalue=NULL ; /* for clean free */
+  NcbiMimeAsn1Free(mime);
 }
 
 
@@ -1138,142 +1190,165 @@ static void DumpMime(AsnIoPtr aip, CharPtr title, Int2 style, Int2 choice, VoidP
 #define LAUNCH_VIEWER	0
 #define SEE_FILE	1
 #define SAVE_FILE	2
-
+#define SEND_FILE   4
 
 /* Handle the various View/See/Save file options. */
 
 static Int4
 SendStructureMIME(Char Filetype, Int4 uid, Int4 Mime, Int4 Complexity,
-			Int4 Models, Int4 Color, Int4 Render)
+		  Int4 Models, Int4 Color, Int4 Render)
 {
-	BiostrucPtr bsp;
-	PDNMS ModelStruc;
-	Int4 count, i4Sent = 0, error_flag = 0;
-	AsnIoPtr aip;
+  BiostrucPtr bsp;
+  PDNMS ModelStruc;
+  Int4 count, i4Sent = 0, error_flag = 0;
+  AsnIoPtr aip;
+        
+  /* yanli start */
+  NcbiMimeAsn1Ptr mime;
+  BiostrucSeqPtr bssp;
+  BiostrucGraphPtr bsgp;
+  MoleculeGraphPtr mgp;
+  SeqIdPtr sip;
+  SeqEntryPtr sep;
+  Int2 retcode = 3;
+  Int4 This_uid = 0;
 
+  if ((bsp = MMDBBiostrucGet((DocUid) uid, Complexity, Models)) == NULL) {
+    printf("Content-type: text/html\r\n\r\n");
+    printf("<h2>Error</h2>\n");
+    printf("Structure with UID = %ld failed to fetch on Server<p>\n", (long) uid);
+    printf("No such structure available on this MMDB server.\n");
+    return 0;
+  }
 
+  /*yanli*/
+  bsgp = bsp->chemical_graph;
+  mgp = bsgp->molecule_graphs;
+  sip = mgp->seq_id;
+  /*  This_uid =  EntrezFindSeqId(sip);    */
+  while(sip != NULL){
+    if(sip->choice == 12) This_uid = sip->data.intvalue;
+    sip = sip->next;
+  }
+  sep = EntrezSeqEntryGet(This_uid, retcode);
+     
+  bssp = BiostrucSeqNew();
+  bssp->structure = bsp;
+  ValNodeLink(&(bssp->sequences), sep);     
+        
+  mime = (NcbiMimeAsn1Ptr) ValNodeNew(NULL);    /* yanli */
+  mime->choice = NcbiMimeAsn1_strucseq;
+  mime->data.ptrvalue = bssp;
+  /*---yanli*/
+        
+  /* the following headers are format-independent */
+  switch (Mime) {
+  case SAVE_FILE:
+    printf("Content-type: application/octet-stream\r\n\r\n");
+    break;
+  case SEE_FILE:
+    printf ("Content-type: text/html\r\n\r\n");
+    printf ("<HTML><PRE>\r\n");
+    break;
+  case SEND_FILE:
+    printf ("Content-type: text/html\r\n\r\n");
+    break;
+  }
 
-	GlobalDisplayName[0] = '\0';
-	StringCat(GlobalDisplayName, TmpNam(NULL)); 
+  if (Filetype == 'j') {
+    /* Cn3D v2.0 asn.1 format */
+    switch (Mime) {
+    case LAUNCH_VIEWER:
+      printf ("Content-type: chemical/ncbi-asn1-binary\r\n\r\n");
+      aip = AsnIoNew(ASNIO_BIN_OUT, stdout, NULL, NULL, NULL);
+      DumpMime(aip, "MIME", bssp);    
+      break;
+    case SAVE_FILE:
+      aip = AsnIoNew(ASNIO_BIN_OUT, stdout, NULL, NULL, NULL);   
+      NcbiMimeAsn1AsnWrite(mime, aip, NULL);
+      break;
+    case SEE_FILE:
+      aip = AsnIoNew(ASNIO_TEXT_OUT, stdout, NULL, NULL, NULL);
+      NcbiMimeAsn1AsnWrite(mime, aip, NULL);
+      break;
+    case SEND_FILE:
+      aip = AsnIoNew(ASNIO_TEXT_OUT, stdout, NULL, NULL, NULL);
+      NcbiMimeAsn1AsnWrite(mime, aip, NULL);
+      break;
+    }
 
-	if ((GlobalDisplayFile = FileOpen(GlobalDisplayName, WRITE)) == NULL) {
-		printf("Content-type: text/html\r\n\r\n");
-		printf("<h2>Error</h2>\n");
-		printf("%s: Temp File Named [%s] Failed to open on Server<p>\n", CGIName,
-			GlobalDisplayName);
-		printf("Contact %s\n", MAILTO);
-		return 0;
-	}
+    AsnIoReset(aip);
+    AsnIoClose(aip);
+  }
+  else if (Filetype == 'i') {
+    /* Cn3D v1.0 asn.1 format.  Included for backwards compatibility, in case users
+     * want to stick with their old Cn3D for some reason.
+     */
+    switch (Mime) {
+    case LAUNCH_VIEWER:
+      printf ("Content-type: chemical/ncbi-asn1-binary\r\n\r\n");
+      /* printf ("Content-disposition: filename=\"%d.val\"\r\n\r\n", uid);*/
+      aip = AsnIoNew(ASNIO_BIN_OUT, stdout, NULL, NULL, NULL);
+      DumpMime_v1(aip, "MIME", Entrez_style_report, Data_data_structure, bsp);
+      break;
+    case SAVE_FILE:
+      aip = AsnIoNew(ASNIO_BIN_OUT, stdout, NULL, NULL, NULL);
+      BiostrucAsnWrite(bsp, aip, NULL);
+      break;
+    case SEE_FILE:
+      aip = AsnIoNew(ASNIO_TEXT_OUT, stdout, NULL, NULL, NULL);
+      BiostrucAsnWrite(bsp, aip, NULL);
+      break;
+    case SEND_FILE:
+      aip = AsnIoNew(ASNIO_TEXT_OUT, stdout, NULL, NULL, NULL);
+      BiostrucAsnWrite(bsp, aip, NULL);
+      break;
+    }
     
-	if ((bsp = MMDBBiostrucGet((DocUid) uid, Complexity, Models)) == NULL) {
-		fflush(GlobalDisplayFile);
-		FileClose(GlobalDisplayFile);
-		FileRemove(GlobalDisplayName);
-		printf("Content-type: text/html\r\n\r\n");
-		printf("<h2>Error</h2>\n");
-		printf("Structure with UID = %ld failed to fetch on Server<p>\n", (long) uid);
-		printf("No such structure available on this MMDB server.\n");
-		return 0;
-	}
-
-	/* the following headers are format-independent */
-	switch (Mime) {
-	case SAVE_FILE:
-		fprintf(GlobalDisplayFile, "Content-type: application/octet-stream\r\n\r\n");
-		break;
-	case SEE_FILE:
-		fprintf(GlobalDisplayFile, "Content-type: text/html\r\n\r\n");
-		fprintf(GlobalDisplayFile, "<HTML><PRE>\r\n");
-		break;
-	}
-
-	if (Filetype == 'i') {
-		/* Cn3D asn.1 format */
-		switch (Mime) {
-		case LAUNCH_VIEWER:
-			fprintf(GlobalDisplayFile, "Content-type: chemical/ncbi-asn1-binary\r\n\r\n");
-			aip = AsnIoNew(ASNIO_BIN_OUT, GlobalDisplayFile, NULL, NULL, NULL);
-			DumpMime(aip, "MIME", Entrez_style_report, Data_data_structure, bsp);
-			break;
-		case SAVE_FILE:
-			aip = AsnIoNew(ASNIO_BIN_OUT, GlobalDisplayFile, NULL, NULL, NULL);
-			BiostrucAsnWrite(bsp, aip, NULL);
-			break;
-		case SEE_FILE:
-			aip = AsnIoNew(ASNIO_TEXT_OUT, GlobalDisplayFile, NULL, NULL, NULL);
-			BiostrucAsnWrite(bsp, aip, NULL);
-			break;
-		}
-
-		AsnIoReset(aip);
-		AsnIoClose(aip);
-	}
-	else if (Filetype == 'r') {
-		/* RasMol format */
-		if (Mime == LAUNCH_VIEWER)
-			fprintf(GlobalDisplayFile, "Content-type: chemical/x-pdb\r\n\r\n");
-
-		ModelStruc = MakeAModelstruc(bsp);
-		WritePDBAllModel(ModelStruc, GlobalDisplayFile);
-		ClearStructures();
-	}
-	else if (Filetype == 'k') {
-		/* Mage format */
-		if (Mime == LAUNCH_VIEWER)
-			fprintf(GlobalDisplayFile, "Content-type: chemical/x-kinemage\r\n\r\n");
-
-		ModelStruc = MakeAModelstruc(bsp);
-		WriteKinAllModel(ModelStruc, GlobalDisplayFile, Color, Render);
-		ClearStructures();
-	}
-	else {
-		error_flag = 1;
-		printf("Content-type: text/html\r\n\r\n");
-		printf("<h2>Error</h2>\n");
-		printf("Unknown file type on structure fetch.<p>\n");
-		printf("Contact %s\n", MAILTO);
-	}
-
-	if ((Mime == SEE_FILE) && (!error_flag))
-		fprintf(GlobalDisplayFile, "</PRE></HTML>\r\n");
-
-	fflush(GlobalDisplayFile);
-	FileClose(GlobalDisplayFile);
-
-	if (!error_flag) {
-		FILE *sendfile;
-
-		sendfile = FileOpen(GlobalDisplayName, READ);
-
-		/* This unlink ensures that in case the process is killed during the
-		 * WWWSendFile, that the file will be removed (e.g. if a data transfer
-		 * interrupt occurs).
-		 */
-		if (unlink(GlobalDisplayName) != 0) {
-			printf("Content-type: text/html\r\n\r\n");
-			printf("<h2>Error in SendStructureMIME</h2>\n");
-			printf("Unlink of temp file %s failed!\n", GlobalDisplayName);
-			printf("Contact %s\n", MAILTO);
-			return 0;
-		}
-
-		i4Sent = WWWSendFile(GlobalDisplayName, sendfile);
-		FileClose(sendfile);
-	}
-	else
-		i4Sent = 0;
-
-	FileRemove(GlobalDisplayName);
-	BiostrucFree(bsp);
-	return i4Sent;
-
+    AsnIoReset(aip);
+    AsnIoClose(aip);
+  }
+  else if (Filetype == 'r') {
+    /* RasMol format */
+    if (Mime == LAUNCH_VIEWER)
+      printf ("Content-type: chemical/x-pdb\r\n\r\n");
+    /*      printf ("Content-disposition: filename=\"%d.pdb\"\r\n\r\n", uid);*/
+    
+    ModelStruc = MakeAModelstruc(bsp);
+    bssp->structure = NULL;  /* already linked into modelstruc */
+    WritePDBAllModel(ModelStruc, stdout);
+    ClearStructures();
+  }
+  else if (Filetype == 'k') {
+    /* Mage format */
+    if (Mime == LAUNCH_VIEWER)
+      printf ("Content-type: chemical/x-kinemage\r\n\r\n");
+    /* printf ("Content-disposition: filename=\"%d.kin\"\r\n\r\n", uid);*/
+    
+    ModelStruc = MakeAModelstruc(bsp);
+    bssp->structure = NULL;  /* already linked into modelstruc */
+    WriteKinAllModel(ModelStruc, stdout, Color, Render);
+    ClearStructures();
+  }
+  else {
+    error_flag = 1;
+    printf("Content-type: text/html\r\n\r\n");
+    printf("<h2>Error</h2>\n");
+    printf("Unknown file type on structure fetch.<p>\n");
+    printf("Contact %s\n", MAILTO);
+  }
+  
+  if ((Mime == SEE_FILE) && (!error_flag))
+    printf ("</PRE></HTML>\r\n");
+  
+  NcbiMimeAsn1Free(mime);           /* yanli */
+  return 1;
+  
 } /* end SendStructureMIME */
 
 
-
 static Int4  SendSummaryPage(Int4Ptr Uids, Int4 NumToDisplay, Int4 Save)
 {
-
   BiostrucPtr bsp;
   ValNodePtr descr;
   PDNMS ModelStruc;
@@ -1282,93 +1357,62 @@ static Int4  SendSummaryPage(Int4Ptr Uids, Int4 NumToDisplay, Int4 Save)
   Char tax_save[MAX_TBUFF];
   FILE *sendfile;
    
-     GlobalDisplayName[0] = '\0';
-     StringCat(GlobalDisplayName, TmpNam(NULL)); 
-     if  (!(GlobalDisplayFile = FileOpen(GlobalDisplayName,WRITE)))
-	   {
-	     printf("Content-type: text/html\n\n");
-	     printf("<h2>Error</h2>\n");
-	     printf("%s: Temp File Named [%s] Failed to open on Server<p>\n", CGIName, GlobalDisplayName);
-	     printf("Contact %s\n", MAILTO);
-             return 0;
-	   }   
-	   
-     if (Save == 2)
-      fprintf(GlobalDisplayFile, "Content-type: application/octet-stream\n\n");  
-     else
-      fprintf(GlobalDisplayFile, "Content-type: text/html\n\n");  
+  if (Save == 2)
+    printf ("Content-type: application/octet-stream\n\n");  
+  else
+    printf ("Content-type: text/html\n\n");  
      
-     if (!GlobalNonHtmlOutput)
-      {
-	if (GlobalTitles) 
-	    WWWPrintFileData(HEADFILE,  GlobalDisplayFile);
-	else
-	    fprintf(GlobalDisplayFile, "<html>\n<title>MMDB Structure Summary</title><body>"); 
-      }
+  if (!GlobalNonHtmlOutput) {
+    if (GlobalTitles) 
+      WWWPrintFileData(HEADFILE,  stdout);
+    else
+      printf ( "<html>\n<title>MMDB Structure Summary</title><body>"); 
+  }
   for (count = 0; count < NumToDisplay; count++)
-     {
-	  uid = Uids[count];
-	  bsp = MMDBBiostrucGet((DocUid) uid,3,0);
-	  if (bsp == NULL)
-	    {
-	      fprintf(GlobalDisplayFile,
-		      "%sNo structure found for UID %ld.%s\n",
-		      GlobalNonHtmlOutput ? "\n" : "<P><h4>",uid,
-		      GlobalNonHtmlOutput ? "\n" : "</h4><P>");
-              if (!GlobalNonHtmlOutput) 
-	        fprintf(GlobalDisplayFile, "<HR SIZE=5 NOSHADE><BR>\n");
-	      continue;
-	    }
+    {
+      uid = Uids[count];
+      bsp = MMDBBiostrucGet((DocUid) uid,3,0);
+      if (bsp == NULL)
+	{
+	  printf ("%sNo structure found for UID %ld.%s\n",
+		  GlobalNonHtmlOutput ? "\n" : "<P><h4>",uid,
+		  GlobalNonHtmlOutput ? "\n" : "</h4><P>");
+	  if (!GlobalNonHtmlOutput) 
+	    printf ("<HR SIZE=5 NOSHADE><BR>\n");
+	  continue;
+	}
 
-	  if (!GlobalNonHtmlOutput) {
-		CharPtr name;
+      if (!GlobalNonHtmlOutput) {
+	CharPtr name;
 
-		for (descr = bsp->descr, name = NULL; descr != NULL; descr = descr->next) {
-			if (descr->choice == BiomolDescr_name) {
-				name = descr->data.ptrvalue;
-				break;
-			}
-		}
-
-		taxret = SaveTaxonomyInfo(bsp, tax_save, MAX_TBUFF, name);
+	for (descr = bsp->descr, name = NULL; descr != NULL; descr = descr->next) {
+	  if (descr->choice == BiomolDescr_name) {
+	    name = descr->data.ptrvalue;
+	    break;
 	  }
-
-	  ModelStruc = MakeAModelstruc(bsp);  
-	  PrintStructureInfo(ModelStruc, GlobalDisplayFile, (taxret) ? tax_save : NULL);
-
-          if (!GlobalNonHtmlOutput) /* Rest of View/Save stuff */
-	    {
-	       PrintStructureLinks(ModelStruc, GlobalDisplayFile);
-	       PrintStructureView(ModelStruc,  GlobalDisplayFile);
-	       fprintf(GlobalDisplayFile, "<HR SIZE=5 NOSHADE><BR>\n");
-   	    }
-	 
-	  ClearStructures();
+	}
+	taxret = SaveTaxonomyInfo(bsp, tax_save, MAX_TBUFF, name);
       }
+
+      ModelStruc = MakeAModelstruc(bsp);  
+      PrintStructureInfo(ModelStruc, stdout, (taxret) ? tax_save : NULL);
+
+      if (!GlobalNonHtmlOutput) /* Rest of View/Save stuff */
+	{
+	  PrintStructureLinks(ModelStruc, stdout);
+	  PrintStructureView(ModelStruc,  stdout);
+	  printf ("<HR SIZE=5 NOSHADE><BR>\n");
+	}
+	 
+      ClearStructures();
+    }
   if (!GlobalNonHtmlOutput)
     if (GlobalTitles)
-      WWWPrintFileData(TAILFILE, GlobalDisplayFile );
+      WWWPrintFileData(TAILFILE, stdout);
     else
-      fprintf(GlobalDisplayFile, "</html>");
+      printf ("</html>");
   
-  
-  fflush(GlobalDisplayFile);
-  FileClose(GlobalDisplayFile);
-  sendfile = FileOpen(GlobalDisplayName, READ);
-
-  if (unlink(GlobalDisplayName) != 0) {
-    printf("Content-type: text/html\r\n\r\n");
-    printf("<h2>Error in SendSummaryPage</h2>\n");
-    printf("Unlink of temp file %s failed!\n", GlobalDisplayName);
-    printf("Contact %s\n", MAILTO);
-    return 0;
-  }
-
-  i4Sent = WWWSendFile(GlobalDisplayName, sendfile);
-  FileClose(sendfile);
-  FileRemove(GlobalDisplayName);
-  return i4Sent;
-    
+  return 1;
 } /* SendSummaryPage() */
 
 
@@ -1397,8 +1441,6 @@ static Boolean InitMMDBSrv(void)
   CGIName[0] = '\0';
   DATApath[0] = '\0';
   LOGpath[0] = '\0';
-  GIFpath[0] = '\0';
-  GIFurl[0] = '\0';
   ENTREZurl[0] = '\0';
   VASTurl[0] = '\0';
   MMDBurl[0] = '\0';
@@ -1461,18 +1503,6 @@ static Boolean InitMMDBSrv(void)
       	ErrPostEx(SEV_FATAL,0,0, "MMDB config file \nMMDBSRV section no LOGpath...\n");
 	return FALSE;
     }
-  FindPath("mmdb", "MMDBSRV", "GIFpath", GIFpath, PATH_MAX);
-  if (GIFpath[0] == '\0')
-    {
-      	ErrPostEx(SEV_FATAL,0,0, "MMDB config file \nMMDBSRV section no GIFpath...\n");
-	return FALSE;
-    }
-  GetAppParam("mmdb", "MMDBSRV", "GIFurl", "", GIFurl, PATH_MAX);
-  if (GIFurl[0] == '\0')
-    {
-      	ErrPostEx(SEV_FATAL,0,0, "MMDB config file \nMMDBSRV section no GIFurl...\n");
-	return FALSE;
-    } 
   GetAppParam("mmdb", "MMDBSRV", "ENTREZurl", "", ENTREZurl, PATH_MAX);
   if (ENTREZurl[0] == '\0')
     {
@@ -1607,6 +1637,26 @@ Int2 Main ()
        printf("Contact MMDBSRV administrator for this site: %s<p>\n",  MAILTO);   
 	exit(1);
     }
+                                                   /* yanli */
+   if (EntrezInit("mmdbsrv", NULL, FALSE) == FALSE)    
+     {
+       printf("Content-type: text/html\n\n");
+       printf("<h2>Error</h2>\n");
+       printf("MMDBSERV: EntrezInit Failed.<p>\n");
+       return;
+     }       
+                                                   /* yanli */
+/*if( !BioseqFetchInit(TRUE))                    
+    {
+      ErrLogPrintf("BioseqFetchInit failed! \n");
+      printf("Content-type: text/html\r\n\r\n");
+      printf("<H2>MMDBSRV Error</H2>\n - BioseqFetchInit failed! <p>"); 
+      printf("Contact MMDBSRV administrator for this site: %s<p>\n",  MAILTO);
+    exit(1);
+    }   */
+
+    SeqAsnLoad();
+    objmimeAsnLoad();
 
   IndexArgs = -1;
   if ((IndexArgs = WWWFindName(info,"db")) >= 0)
@@ -1649,7 +1699,7 @@ Int2 Main ()
  else
    {
      /* loop over entire set of tag=value entries and add all the UID's found together */
-     /* musht handle uid=1234&uid=1OMD&uid=1234,2345,2134&dopt=s&uid=3INS              */   
+     /* must handle uid=1234&uid=1OMD&uid=1234,2345,2134&dopt=s&uid=3INS              */   
       NumEntries = WWWGetNumEntries(info);
       for (IndexArgs = 0; IndexArgs < NumEntries; IndexArgs++)
         {
@@ -1705,6 +1755,10 @@ Int2 Main ()
 	 case 'I':
 	    DispOpt = 'i';
 	   break;
+	 case 'j':
+	 case 'J':
+		DispOpt = 'j';
+		break;
 	 case 'k':
 	 case 'K':
 	     DispOpt = 'k';
@@ -1719,7 +1773,7 @@ Int2 Main ()
 	    printf("Content-type: text/html\r\n\r\n");
 	    printf("<H2>MMDBSRV Error</H2>\n - Wrong Display Option Type: dopt=%s.<p>\n",  pcThis);
 	    printf("<pre>dopt supports:\n's' Structure summary\n");
-	    printf("'r' RasMol(PDB) format\n'k' Kinemage format\n'i' or 'a' ASN.1</pre><p>\n"); 
+	    printf("'r' RasMol(PDB) format\n'k' Kinemage format\n'i', 'j', or 'a' ASN.1</pre><p>\n"); 
 	    goto cleanout2;
 	    }
       } 
@@ -1772,7 +1826,19 @@ Int2 Main ()
 	    }    
     }  
 
+  IndexArgs = -1;
+  if ((IndexArgs = WWWFindName(info, "mdlLvl")) >= 0)
+    {
+      pcThis = WWWGetValueByIndex(info, IndexArgs);
+      Complex = atoi(pcThis);
+    }
 
+  IndexArgs = -1;
+  if ((IndexArgs = WWWFindName(info, "maxModels")) >= 0)
+    {
+      pcThis = WWWGetValueByIndex(info, IndexArgs);
+      MaxModels = atoi(pcThis);
+    }
   
   IndexArgs = -1;
   if ((IndexArgs = WWWFindName(info, "KinemageColor")) >= 0)
@@ -1806,6 +1872,10 @@ Int2 Main ()
         {
 	    Save = 3;  /* MacSave - not implemented */
 	}	
+      else if (StrICmp(pcThis, "asntext") == 0)
+        {
+	    Save = SEND_FILE;  /* writes out asn text without the html header */
+	}	
     }	 
   
   IndexArgs = -1;
@@ -1886,7 +1956,7 @@ cleanout2:
                 WWWGetAddress(info), WWWGetHost(info), WWWGetAgent(info), TimeNowStr, 
                 (long)iBytes); 
    fflush(stdout);
-   exit(1);
+   exit(0);
 
 cleanout:
   CloseMMDBAPI();
@@ -1899,7 +1969,7 @@ cleanout:
                 WWWGetAddress(info), WWWGetHost(info), WWWGetAgent(info), TimeNowStr, 
                 (long)iBytes); 
    fflush(stdout);
-   exit(1);
+   exit(0);
 
 earlyout:
    time_now = GetSecs();
@@ -1911,6 +1981,6 @@ earlyout:
   
   
   fflush(stdout);
-  exit(1);
+  exit(0);
 }
 
