@@ -1,4 +1,4 @@
-/*  $Id: ncbi_ansi_ext.c,v 6.8 2001/12/04 15:57:22 lavr Exp $
+/*  $Id: ncbi_ansi_ext.c,v 6.14 2002/10/29 22:18:29 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -26,10 +26,130 @@
  * Author:  Anton Lavrentiev
  *
  * File Description:
- *    Non-ANSI, yet widely used functions
+ *   Non-ANSI, yet widely used functions
  *
+ */
+
+#include "ncbi_ansi_ext.h"
+#include <ctype.h>
+#include <stdlib.h>
+
+
+#ifndef HAVE_STRDUP
+
+extern char* strdup(const char* str)
+{
+    size_t size = strlen(str) + 1;
+    char*  res  = (char*) malloc(size);
+    if (res)
+        memcpy(res, str, size);
+    return res;
+}
+
+#endif /*HAVE_STRDUP*/
+
+
+#ifndef HAVE_STRCASECMP
+
+/* We assume that we're using ASCII-based charsets */
+extern int strcasecmp(const char* s1, const char* s2)
+{
+    const unsigned char* p1 = (const unsigned char*) s1;
+    const unsigned char* p2 = (const unsigned char*) s2;
+    unsigned char c1, c2;
+
+    if (p1 == p2)
+        return 0;
+
+    do {
+        c1 = *p1++;
+        c2 = *p2++;
+        c1 = c1 >= ' ' && c1 <= 'Z' ? c1 :
+            (c1 >= 'a' && c1 <= 'z' ? c1 - ('a' - 'A') : toupper(c1));
+        c2 = c2 >= ' ' && c2 <= 'Z' ? c2 :
+            (c2 >= 'a' && c2 <= 'z' ? c2 - ('a' - 'A') : toupper(c2));
+    } while (c1  &&  c1 == c2);
+
+    return c1 - c2;
+}
+
+
+extern int strncasecmp(const char* s1, const char* s2, size_t n)
+{
+    const unsigned char* p1 = (const unsigned char*) s1;
+    const unsigned char* p2 = (const unsigned char*) s2;
+    unsigned char c1, c2;
+
+    if (p1 == p2  ||  n == 0)
+        return 0;
+
+    do {
+        c1 = *p1++;
+        c2 = *p2++;
+        c1 = c1 >= ' ' && c1 <= 'Z' ? c1 :
+            (c1 >= 'a' && c1 <= 'z' ? c1 - ('a' - 'A') : toupper(c1));
+        c2 = c2 >= ' ' && c2 <= 'Z' ? c2 :
+            (c2 >= 'a' && c2 <= 'z' ? c2 - ('a' - 'A') : toupper(c2));
+    } while (--n > 0  &&  c1  &&  c1 == c2);
+
+    return c1 - c2;
+}
+
+#endif /*HAVE_STRCASECMP*/
+
+
+extern char* strupr(char* s)
+{
+    unsigned char* t = (unsigned char*) s;
+
+    while ( *t ) {
+        *t = toupper(*t);
+        t++;
+    }
+    return s;
+}
+
+
+extern char* strlwr(char* s)
+{
+    unsigned char* t = (unsigned char*) s;
+
+    while ( *t ) {
+        *t = tolower(*t);
+        t++;
+    }
+    return s;
+}
+
+
+extern char* strncpy0(char* s1, const char* s2, size_t n)
+{
+    *s1 = '\0';
+    return strncat(s1, s2, n);
+}
+
+
+/*
  * --------------------------------------------------------------------------
  * $Log: ncbi_ansi_ext.c,v $
+ * Revision 6.14  2002/10/29 22:18:29  lavr
+ * Comply with man strdup(2) in the implementation of strdup() here
+ *
+ * Revision 6.13  2002/10/28 18:55:26  lavr
+ * Fix change log to remove duplicate log entry for R6.12
+ *
+ * Revision 6.12  2002/10/28 18:52:07  lavr
+ * Conditionalize definitions of strdup() and str[n]casecmp()
+ *
+ * Revision 6.11  2002/10/28 15:41:56  lavr
+ * Use "ncbi_ansi_ext.h" privately
+ *
+ * Revision 6.10  2002/09/24 15:05:45  lavr
+ * Log moved to end
+ *
+ * Revision 6.9  2002/03/19 22:12:28  lavr
+ * strcasecmp and strncasecmp are optimized (for ASCII range)
+ *
  * Revision 6.8  2001/12/04 15:57:22  lavr
  * Tiny style adjustement
  *
@@ -57,74 +177,3 @@
  *
  * ==========================================================================
  */
-
-#include <connect/ncbi_ansi_ext.h>
-#include <ctype.h>
-#include <stdlib.h>
-
-
-extern char* strdup(const char* str)
-{
-    size_t size = strlen(str) + 1;
-    return (char*) memcpy(malloc(size), str, size);
-}
-
-
-extern int strcasecmp(const char* s1, const char* s2)
-{
-    const unsigned char* p1 = (const unsigned char*) s1;
-    const unsigned char* p2 = (const unsigned char*) s2;
-    unsigned char c1, c2;
-
-    if (p1 == p2)
-        return 0;
-
-    do {
-        c1 = toupper(*p1++);
-        c2 = toupper(*p2++);
-    } while (c1  &&  c1 == c2);
-
-    return c1 - c2;
-}
-
-
-extern int strncasecmp(const char* s1, const char* s2, size_t n)
-{
-    const unsigned char* p1 = (const unsigned char*) s1;
-    const unsigned char* p2 = (const unsigned char*) s2;
-    unsigned char c1, c2;
-
-    if (p1 == p2  ||  n == 0)
-        return 0;
-
-    do {
-        c1 = toupper(*p1++);
-        c2 = toupper(*p2++);
-    } while (--n > 0  &&  c1  &&  c1 == c2);
-
-    return c1 - c2;
-}
-
-
-extern char* strupr(char* s)
-{
-    unsigned char* t = (unsigned char*) s;
-
-    while ( *t ) {
-        *t = toupper(*t);
-        t++;
-    }
-    return s;
-}
-
-
-extern char* strlwr(char* s)
-{
-    unsigned char* t = (unsigned char*) s;
-
-    while ( *t ) {
-        *t = tolower(*t);
-        t++;
-    }
-    return s;
-}
