@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   3/4/91
 *
-* $Revision: 6.23 $
+* $Revision: 6.35 $
 *
 * File Description: 
 *     portable file routines
@@ -43,6 +43,42 @@
 * 11-27-94 Ostell      moved includes to ncbiwin.h to avoid conflict MSC
 *
 * $Log: ncbifile.c,v $
+* Revision 6.35  2004/05/07 15:57:14  kans
+* added FileCache functions for buffered read, graceful handing of Unix, Mac, and DOS line endings
+*
+* Revision 6.34  2004/01/23 20:07:16  kans
+* fix to FileGets under Darwin, was losing last character if buffer was shorter than line being read
+*
+* Revision 6.33  2003/11/17 17:17:57  kans
+* changed C++ comments to C comments
+*
+* Revision 6.32  2003/05/05 11:53:37  rsmith
+* Codewarrior compiling for Win32 does not know about setmode or tempnam.
+*
+* Revision 6.31  2003/03/11 14:26:42  rsmith
+* previous change to Nlm_DirCatalog made for only OS_UNIX_DARWIN, not all OS_UNIX
+*
+* Revision 6.30  2003/03/10 15:55:49  rsmith
+* for OS_UNIX change implementation of DirCatalog from using popen with a ls command to using the opendir and readdir library calls.
+*
+* Revision 6.29  2003/02/25 15:31:03  rsmith
+* OS_UNIX_DARWIN (Mach on Macs) uses tempnam for temporary files, not tmpnam.
+*
+* Revision 6.28  2002/11/06 21:23:04  ucko
+* Don't assume MIPS is IRIX; allow Linux too.
+*
+* Revision 6.27  2002/10/03 17:22:29  kans
+* for OS_MAC or OS_UNIX_DARWIN, Nlm_FileGets does fgetc loop, stopping at the first \n or \r character, and leaving a \n as the last character before the null byte
+*
+* Revision 6.26  2002/10/02 14:34:17  kans
+* Nlm_GetOSType and Nlm_FileCreate have ifdef OS_MAC, not WIN_MAC
+*
+* Revision 6.25  2002/06/13 16:14:07  kans
+* fix includes for OS_UNIX_DARWIN with WIN_MAC (EN)
+*
+* Revision 6.24  2002/04/05 19:02:47  ivanov
+* Changed L_tmpnam to PATH_MAX in Nlm_TmpNam()
+*
 * Revision 6.23  2001/08/29 17:33:15  juran
 * Cleanup.
 *
@@ -161,8 +197,13 @@ Removed disabled CD routines.
 #define THIS_MODULE g_corelib
 #define THIS_FILE  _this_file
 
+#include <ncbilcl.h>
+
 #include "corepriv.h"
 #ifdef OS_MAC
+#ifdef OS_UNIX_DARWIN
+#include <Carbon.h>
+#endif
 #include <Navigation.h>
 #include <Script.h>
 #endif
@@ -172,7 +213,7 @@ Removed disabled CD routines.
 #define DEFAULT_RAW_CDROM "/dev/rsr0"
 #endif
 
-#ifdef PROC_MIPS
+#if defined(PROC_MIPS) && !defined(OS_UNIX_LINUX)
 #define DEFAULT_CDROM "/dev/scsi/sc0d4l0"
 #endif
 
@@ -204,12 +245,12 @@ static char * _this_file = __FILE__;
 
 #ifdef OS_MAC
 #define INLINE_MOREFILES
-//#include "FullPath.h"
-//#include "MoreFilesExtra.h"
+/* #include "FullPath.h" */
+/* #include "MoreFilesExtra.h" */
 #ifdef INLINE_MOREFILES
 
-// MoreFilesExtras.c
-// -----------------
+/* MoreFilesExtras.c */
+/* ----------------- */
 
 /*
 **	Apple Macintosh Developer Technical Support
@@ -409,14 +450,14 @@ static OSErr MacPathname2FSSpec(const char *inPathname, FSSpec *outFSS)
 		return paramErr;
 	}
 	
-	err = HGetVol(NULL, &vRefNum, &dirID);  // default volume and directory
+	err = HGetVol(NULL, &vRefNum, &dirID);  /* default volume and directory */
 	if (err != noErr) return err;
 	
 	len = strlen(inPathname);
 	
 	p = strchr(inPathname, ':');
 	if (p == NULL) {
-		// Partial pathname -- filename only
+		/* Partial pathname -- filename only */
 		Str31 filename;
 		assert(len <= 31);
 		c2pstrcpy(filename, inPathname);
@@ -425,21 +466,21 @@ static OSErr MacPathname2FSSpec(const char *inPathname, FSSpec *outFSS)
 		Str31 name;
 		int nameLen;
 		if (inPathname[0] == ':') {
-			// Relative pathname including directory path
+			/* Relative pathname including directory path */
 			
 		} else {
-			// Absolute pathname
-			//Str31 volName;  // We would use Str28 if it was defined -- 27, plus 1 for ':'.
+			/* Absolute pathname */
+			/* Str31 volName;  We would use Str28 if it was defined -- 27, plus 1 for ':'. */
 			nameLen = p - inPathname;
 			assert(nameLen <= 27);
 			name[0] = nameLen + 1;
-			memcpy(name + 1, inPathname, nameLen + 1);  // Copy the volume name and the colon.
+			memcpy(name + 1, inPathname, nameLen + 1);  /* Copy the volume name and the colon. */
 			err = DetermineVRefNum(name, 0, &vRefNum);
 			if (err != noErr) return err;
 			dirID = 2;
 		}
-		// vRefNum and dirID now specify the directory in which we should descend
-		// the path pointed to by p (pointing to the first colon).
+		/* vRefNum and dirID now specify the directory in which we should descend
+		   the path pointed to by p (pointing to the first colon). */
 		p++;
 		while (p != NULL && *p != '\0') {
 			char *q = strchr(p, ':');
@@ -461,7 +502,7 @@ static OSErr MacPathname2FSSpec(const char *inPathname, FSSpec *outFSS)
 					p = q + 1;
 				}
 			} else {
-				q = strchr(p, '\0');  // go to end of string
+				q = strchr(p, '\0');  /* go to end of string */
 				nameLen = q - p;
 				assert(nameLen > 0);
 				assert(nameLen <= 31);
@@ -489,7 +530,7 @@ static OSErr MacFSSpec2FullPathname(const FSSpec *inFSS, char **outPathname)
 	err = FSpGetFullPath(inFSS, &fullPathLength, &h);
 	if (err != noErr) return err;
 	
-	assert(fullPathLength >= 2);  // An absolute pathname must be at least two chars long
+	assert(fullPathLength >= 2);  /* An absolute pathname must be at least two chars long */
 	fullPath = (char *)Nlm_Malloc(fullPathLength + 1);
 	if (fullPath == NULL) {
 		err = memFullErr;
@@ -554,7 +595,7 @@ NLM_EXTERN FILE * LIBCALL Nlm_FileOpen(const char *filename, const char *mode)
         else
           f = fopen(filename, mode);
 
-#ifdef WIN32
+#if defined(WIN32)  &&  ! defined(COMP_METRO) 
         if (strchr(mode, 'b')  &&
             (f == stdin  ||  f == stdout  ||  f == stderr))
           setmode(fileno(f), O_BINARY);
@@ -641,7 +682,7 @@ NLM_EXTERN void LIBCALL  Nlm_FileClose (FILE *stream)
 #ifdef WIN_DUMB    
   if (stream == stdin  ||  stream == stdout  ||  stream == stderr)
     {
-#ifdef WIN32
+#if defined(WIN32)  &&  ! defined(COMP_METRO) 
       setmode(fileno(stream), O_TEXT);
 #endif
       return;
@@ -711,9 +752,40 @@ NLM_EXTERN int LIBCALL  Nlm_FilePuts (const char *ptr, FILE *fp)
 *****************************************************************************/
 NLM_EXTERN char * LIBCALL  Nlm_FileGets (Nlm_CharPtr ptr, size_t size, FILE *fp)
 {
-	if ((ptr == NULL) || (size <= 0) || (fp == NULL))
+#if defined(OS_MAC) || defined (OS_UNIX_DARWIN)
+    int         ch;
+	int         count;
+	Nlm_CharPtr tmp;
+#endif
+
+	if ((ptr == NULL) || (size < 1) || (fp == NULL))
 		return NULL;
+#if defined(OS_MAC) || defined (OS_UNIX_DARWIN)
+	ch = fgetc (fp);
+	count = 0;
+	tmp = ptr;
+	while (ch != EOF && ch != '\0' && ch != '\n' && ch != '\r' && count < size - 2) {
+	  *tmp = ch;
+	  tmp++;
+	  count++;
+	  ch = fgetc (fp);
+	}
+	if (ch == '\n' || ch == '\r') {
+	  *tmp = '\n';
+	  tmp++;
+	  count++;
+	} else if (ch != EOF && ch != '\0') {
+	  *tmp = ch;
+	  tmp++;
+	  count++;
+	}
+	*tmp = '\0';
+	if (count < 1)
+		return NULL;
+	return ptr;
+#else
 	return fgets(ptr,size,fp);
+#endif
 }
 
 
@@ -928,7 +1000,7 @@ NLM_EXTERN Nlm_Boolean LIBCALL Nlm_FileRename (Nlm_CharPtr oldFileName, Nlm_Char
 *   FileCreate()
 *
 *****************************************************************************/
-#ifdef WIN_MAC
+#ifdef OS_MAC
 static OSType Nlm_GetOSType (Nlm_CharPtr str, OSType dfault)
 
 {
@@ -945,7 +1017,7 @@ static OSType Nlm_GetOSType (Nlm_CharPtr str, OSType dfault)
 NLM_EXTERN void LIBCALL Nlm_FileCreate (Nlm_CharPtr fileName, Nlm_CharPtr type, Nlm_CharPtr creator)
 
 {
-#ifdef WIN_MAC
+#ifdef OS_MAC
   Nlm_Int2  fError;
   Nlm_Char  temp [256];
   OSType    fType;
@@ -957,15 +1029,15 @@ NLM_EXTERN void LIBCALL Nlm_FileCreate (Nlm_CharPtr fileName, Nlm_CharPtr type,
 
   if (fileName != NULL && fileName [0] != '\0') {
 
-#ifdef WIN_MAC
-    // note: the following assumes either full pathname or that the current
-    // directory is the proper location to find/create the file
+#ifdef OS_MAC
+    /* note: the following assumes either full pathname or that the current
+       directory is the proper location to find/create the file */
 
     Nlm_StringNCpy_0(temp, fileName, sizeof(temp));
     Nlm_CtoPstr ( temp);
     fError = FSMakeFSSpec( 0, 0, (StringPtr)temp, &spec);
     
-    // file not found, so create it...
+    /* file not found, so create it... */
     if( fError == fnfErr){
         fType = Nlm_GetOSType (type, 'TEXT');
         fCreator = Nlm_GetOSType (creator, '    ');
@@ -1049,6 +1121,10 @@ NLM_EXTERN Nlm_Boolean LIBCALL  Nlm_CreateDir (Nlm_CharPtr pathname)
 *
 *****************************************************************************/
 
+#ifdef OS_UNIX_DARWIN
+#include <dirent.h>
+#endif
+
 NLM_EXTERN ValNodePtr LIBCALL Nlm_DirCatalog (Nlm_CharPtr pathname)
 
 {
@@ -1063,12 +1139,17 @@ NLM_EXTERN ValNodePtr LIBCALL Nlm_DirCatalog (Nlm_CharPtr pathname)
   short           vRefNum;
 #endif
 #ifdef OS_UNIX
+  Nlm_Uint1       choice;
+#ifdef OS_UNIX_DARWIN
+  DIR             *dirp;
+  struct dirent   *dep;
+#else
   Nlm_Char        buf [256];
   Nlm_Char        ch;
-  Nlm_Uint1       choice;
   Nlm_Char        cmmd [PATH_MAX + 20];
   FILE            *fp;
   Nlm_CharPtr     ptr;
+#endif
 #endif
   ValNodePtr      vnp = NULL;
 
@@ -1132,6 +1213,21 @@ NLM_EXTERN ValNodePtr LIBCALL Nlm_DirCatalog (Nlm_CharPtr pathname)
     }}
 #endif
 #ifdef OS_UNIX
+#ifdef OS_UNIX_DARWIN
+    dirp = opendir(pathname);
+    if (dirp == NULL) return NULL;
+    while ((dep = readdir(dirp)) != NULL) {
+      /* ignore 'invisible' files. */
+      if (dep->d_namlen < 1 || dep->d_name[0] == '.')
+        continue;
+      if (dep->d_type == DT_DIR) /* directory */
+        choice = 1;
+      else          /* all other file types. */
+        choice = 0;
+      ValNodeCopyStr (&vnp, choice, dep->d_name);
+    }
+    closedir(dirp);
+#else
     sprintf (cmmd, "ls -1p %s 2>/dev/null", pathname);
     fp = popen (cmmd, "r");
     if (fp == NULL) return NULL;
@@ -1153,6 +1249,7 @@ NLM_EXTERN ValNodePtr LIBCALL Nlm_DirCatalog (Nlm_CharPtr pathname)
     }
     pclose (fp);
 #endif
+#endif
 #ifdef OS_VMS
 #endif
   }
@@ -1169,14 +1266,16 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TmpNam (Nlm_CharPtr s)
 {
 #ifdef TEMPNAM_AVAIL
     char *filename;
-    static Nlm_Char save_filename[L_tmpnam+30];
+    static Nlm_Char save_filename[PATH_MAX];
 
     /* emulate tmpnam(), except get the benefits of tempnam()'s ability to */
     /* place the files in another directory specified by the environment   */
     /* variable TMPDIR                                                     */
-
+#ifdef OS_UNIX_DARWIN
+    filename = tempnam("/tmp", "ncbi.");
+#else
     filename = tempnam(NULL, NULL);
-
+#endif
     if (s == NULL)
     { /* return pointer to static string */
         if (filename != NULL) {
@@ -1266,3 +1365,237 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TmpNam (Nlm_CharPtr s)
 #endif
 }
 
+/* FileCache provides buffered text read for handling Unix, Mac, and DOS line endings gracefully */
+
+/* attach file pointer (text read mode expected) to cache object (usually on stack) */
+
+NLM_EXTERN Nlm_Boolean Nlm_FileCacheSetup (
+  Nlm_FileCache PNTR fcp,
+  FILE *fp
+)
+
+{
+  if (fp == NULL || fcp == NULL) return FALSE;
+
+  MemSet ((Nlm_VoidPtr) fcp, 0, sizeof (Nlm_FileCache));
+
+  fcp->fp = fp;
+  fcp->offset = ftell (fp);
+
+  return TRUE;
+}
+
+static void Nlm_FileCacheReadBlock (
+  Nlm_FileCache PNTR fcp
+)
+
+{
+  int  total;
+
+  if (fcp == NULL || fcp->fp == NULL) return;
+
+  if (fcp->ctr >= fcp->total) {
+    fcp->offset += (Nlm_Int4) fcp->total;
+    fcp->ctr = 0;
+    fcp->total = 0;
+
+    fcp->buf [0] = '\0';
+    total = (int) Nlm_FileRead ((Nlm_VoidPtr) fcp->buf, sizeof (Nlm_Char), (size_t) 512, fcp->fp);
+    if (total < 0 || total > 512) {
+      total = 512;
+    }
+
+    fcp->buf [total] = '\0';
+    fcp->total = Nlm_StringLen (fcp->buf);
+  }
+}
+
+/* equivalent of getc */
+
+static Nlm_Char Nlm_FileCacheGetChar (
+  Nlm_FileCache PNTR fcp
+)
+
+{
+  Nlm_Char  ch = '\0', nxt;
+
+  if (fcp == NULL || fcp->fp == NULL) return ch;
+
+  /* read a fresh block if buffer is empty */
+
+  if (fcp->ctr >= fcp->total) {
+    Nlm_FileCacheReadBlock (fcp);
+  }
+
+  /* get next Nlm_Character in buffer */
+
+  if (fcp->ctr < fcp->total) {
+    ch = fcp->buf [(int) fcp->ctr];
+    (fcp->ctr)++;
+  }
+
+  if (ch == '\n' || ch == '\r') {
+    if (fcp->ctr >= fcp->total) {
+      Nlm_FileCacheReadBlock (fcp);
+    }
+    if (fcp->ctr < fcp->total) {
+
+      /* look for carriage return / linefeed pair - DOS file read on Mac or Unix platform */
+
+      nxt = fcp->buf [(int) fcp->ctr];
+
+      /* advance past second Nlm_Character in cr/lf pair */
+
+      if (ch == '\n' && nxt == '\r') {
+        (fcp->ctr)++;
+      } else if (ch == '\r' && nxt == '\n') {
+        (fcp->ctr)++;
+      }
+    }
+
+    /* cr or lf returned as newline */
+
+    ch = '\n';
+  }
+
+  return ch;
+}
+
+/* equivalent of fgets, leaves /n at end of string */
+
+NLM_EXTERN Nlm_CharPtr Nlm_FileCacheGetString (
+  Nlm_FileCache PNTR fcp,
+  Nlm_CharPtr str,
+  size_t size
+)
+
+{
+  Nlm_Char     ch;
+  Nlm_Int2     count;
+  Nlm_CharPtr  ptr;
+
+  if (fcp == NULL || fcp->fp == NULL || str == NULL || size < 1) return NULL;
+
+  ch = Nlm_FileCacheGetChar (fcp);
+  count = 0;
+  ptr = str;
+
+  while (ch != '\0' && ch != '\n' && ch != '\r' && count < size - 2) {
+    *ptr = ch;
+    ptr++;
+    count++;
+    ch = Nlm_FileCacheGetChar (fcp);
+  }
+
+  if (ch == '\n' || ch == '\r') {
+    *ptr = '\n';
+    ptr++;
+    count++;
+  } else if (ch != '\0') {
+    *ptr = ch;
+    ptr++;
+    count++;
+  }
+  *ptr = '\0';
+
+  if (count < 1) return NULL;
+
+  return str;
+}
+
+/* smarter fgets removes newline from end of string */
+
+NLM_EXTERN Nlm_CharPtr Nlm_FileCacheReadLine (
+  Nlm_FileCache PNTR fcp,
+  Nlm_CharPtr str,
+  size_t size,
+  Nlm_BoolPtr nonewline
+)
+
+{
+  Nlm_Char     ch;
+  Nlm_CharPtr  ptr;
+  Nlm_CharPtr  tmp;
+
+  if (fcp == NULL || fcp->fp == NULL || str == NULL || size < 1) return NULL;
+  *str = '\0';
+  tmp = Nlm_FileCacheGetString (fcp, str, size);
+  if (tmp != NULL) {
+    ptr = str;
+    ch = *ptr;
+    while (ch != '\0' && ch != '\n' && ch != '\r') {
+      ptr++;
+      ch = *ptr;
+    }
+    *ptr = '\0';
+    if (nonewline != NULL) {
+      if (ch != '\n' && ch != '\r') {
+        *nonewline = TRUE;
+      } else {
+        *nonewline = FALSE;
+      }
+    }
+  }
+  return tmp;
+}
+
+NLM_EXTERN void Nlm_FileCacheSeek (
+  Nlm_FileCache PNTR fcp,
+  Nlm_Int4 pos
+)
+
+{
+  if (fcp == NULL || fcp->fp == NULL) return;
+
+  if (fcp->offset <= pos && fcp->offset + (Nlm_Int4) fcp->total >= pos) {
+    fcp->ctr = (Nlm_Int2) (pos - fcp->offset);
+    return;
+  }
+
+  fcp->ctr = 0;
+  fcp->total = 0;
+  fcp->offset = pos;
+
+  fseek (fcp->fp, pos, SEEK_SET);
+}
+
+NLM_EXTERN Nlm_Int4 Nlm_FileCacheTell (
+  Nlm_FileCache PNTR fcp
+)
+
+{
+  Nlm_Int4  bytes;
+  Nlm_Int4  offset;
+
+  if (fcp == NULL || fcp->fp == NULL) return 0L;
+
+  offset = ftell (fcp->fp);
+  bytes = (Nlm_Int4) (fcp->total - fcp->ctr);
+  offset -= bytes;
+
+  return offset;
+}
+
+NLM_EXTERN Nlm_Boolean Nlm_FileCacheFree (
+  Nlm_FileCache PNTR fcp,
+  Nlm_Boolean restoreFilePos
+)
+
+{
+  Nlm_Int4  pos;
+
+  if (fcp == NULL || fcp->fp == NULL) return FALSE;
+
+  if (restoreFilePos) {
+
+    /* correct position of file pointer */
+
+    pos = Nlm_FileCacheTell (fcp);
+    fseek (fcp->fp, pos, SEEK_SET); 
+  }
+
+  MemSet ((Nlm_VoidPtr) fcp, 0, sizeof (Nlm_FileCache));
+
+  return TRUE;
+}
+
