@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   6/4/91
 *
-* $Revision: 6.8 $
+* $Revision: 6.25 $
 *
 * File Description:
 *   	portable memory handlers for Mac, PC, Unix
@@ -37,6 +37,60 @@
 * Modifications:
 * --------------------------------------------------------------------------
 * $Log: ncbimem.c,v $
+* Revision 6.25  2003/08/11 19:43:27  rsmith
+* Memory Handle functions apply to Mac OS Darwin (OSX) not just to OS_MAC (OS9)
+*
+* Revision 6.24  2002/11/06 21:24:35  ucko
+* Make sure MADV_NORMAL is actually defined before trying to use madvise.
+*
+* Revision 6.23  2002/10/28 20:19:56  kans
+* removed C++ style comment
+*
+* Revision 6.22  2002/07/11 22:14:31  vakatov
+* [LINUX, MMAP_AVAIL]  #define __USE_BSD to get MADV_*** constants
+*
+* Revision 6.21  2002/07/11 19:13:41  ivanov
+* Nlm_MemMapAdvise() -- fix arguments for madvise() on Sun Solaris
+*
+* Revision 6.20  2002/07/10 20:24:44  ivanov
+* Added functions Nlm_MemMapAdvise(), Nlm_MemMapAdvisePtr()
+*
+* Revision 6.19  2002/06/17 15:01:30  ivanov
+* Disable USE_SETHEAPLIMIT on BeOS platform
+*
+* Revision 6.18  2002/02/07 14:41:19  ivanov
+* Added MemSearch()
+*
+* Revision 6.17  2000/03/08 17:55:49  vakatov
+* Use Int8 for the file size.
+* Also, get rid of the WIN16 code, do other cleanup.
+*
+* Revision 6.16  1999/11/05 21:48:48  vakatov
+* [MMAP_AVAIL]  cast to get rid of a warning on the pointer comparison
+*
+* Revision 6.15  1999/08/25 17:40:12  madden
+* ifdef for AIX fcntl.h
+*
+* Revision 6.14  1998/09/25 15:57:31  vakatov
+* + #include <sys/fcntl.h> to provide O_RDONLY
+*
+* Revision 6.13  1998/08/21 16:37:42  vakatov
+* [MMAP_AVAIL]  Nlm_MemMapInit():  use "open()/close()" instead of
+* "fopen()/fclose()" to avoid problems when more than FOPEN_MAX files
+* are opened at the same time
+*
+* Revision 6.12  1998/06/25 19:40:56  vakatov
+* MemMapInit():  "FileLengthEx()" to catch non-existent files
+*
+* Revision 6.11  1998/06/23 16:30:02  vakatov
+* MemMapInit(): return zero-filled structure(not a NULL) if file is empty
+*
+* Revision 6.10  1998/06/11 21:02:27  shavirin
+* Fixed few warnings
+*
+* Revision 6.9  1998/05/28 19:10:45  vakatov
+* Nlm_MemMapInit():  allow one to open a zero-length file
+*
 * Revision 6.8  1998/04/28 14:35:02  vakatov
 * #define MAP_FAILED -- if not defined yet(e.g. for IRIX5)
 *
@@ -103,8 +157,10 @@
 #include <ncbi.h>
 
 #ifdef OS_UNIX
+#ifndef OS_UNIX_BEOS
 #define USE_SETHEAPLIMIT
 #endif
+#endif
 
 #ifdef USE_SETHEAPLIMIT
 #include <ncbithr.h>
@@ -114,10 +170,19 @@
 
 /* Used for UNIX memory-mapping. */
 #ifdef MMAP_AVAIL
-#include <sys/mman.h>
-#ifndef MAP_FAILED
-#define MAP_FAILED ((void *) -1)
-#endif
+#  ifdef OS_UNIX_LINUX
+/*   MADV_*** constants are not defined on Linux otherwise */
+#    define __USE_BSD
+#  endif
+#  include <sys/mman.h>
+#  ifdef OS_UNIX_AIX
+#    include <fcntl.h>
+#  else
+#    include <sys/fcntl.h>
+#  endif
+#  ifndef MAP_FAILED
+#    define MAP_FAILED ((void *) -1)
+#  endif
 #endif
 
 #ifdef USE_SETHEAPLIMIT
@@ -126,31 +191,6 @@
 
 
 
-#ifdef WIN16
-
-struct heapnode
-{
-    struct heapnode *next;
-    HGLOBAL hSeg;
-    WORD    wSeg;
-    WORD    wFree;
-    int     nItems;
-    int     sn;
-};
-
-size_t	g_nMemHeap = ((size_t)4*KBYTE);
-#define HEAPSIZE	g_nMemHeap
-size_t	g_nMemThresh = ((size_t)128);
-#define THRESHOLD	g_nMemThresh
-
-struct heapnode  headFirst;
-struct heapnode *headPtr = &headFirst;
-
-static BOOL NEAR  HeapNew(struct heapnode *pNode);
-static VOID NEAR  HeapFree(struct heapnode *pNode);
-
-#endif  /* WIN16 */
-
 short	g_bBadPtr;
 
 extern char *g_corelib;
@@ -258,7 +298,7 @@ static void* s_MemAllocator(void *ptr, size_t size,
     }
     if ( !size )
       return Nlm_MemFree(ptr);
-      
+
     x_ptr = Nlm_Realloc(ptr, size);
     break;
   }
@@ -451,7 +491,34 @@ NLM_EXTERN void * LIBCALL  Nlm_MemFill (void *buf, int value, size_t bytes)
 }
 
 
-#if defined(OS_MAC) || defined(OS_MSWIN) || defined(MSC_VIRT)
+/*****************************************************************************
+*
+*   void Nlm_MemSearch(Pointer where, where_size, Pointer what, what_size)
+*   	search a position one block of data into another
+*
+*****************************************************************************/
+
+NLM_EXTERN size_t LIBCALL Nlm_MemSearch(const void* where, size_t where_size,
+                                        const void* what, size_t what_size)
+{
+	size_t i, rbound, pos;
+	
+	rbound = where_size - what_size;
+	pos = -1;
+    i = 0;
+	if (where_size  &&  what_size  &&  where_size >= what_size) {
+		while ((i <= rbound)  &&  (pos == -1)) {
+			if (memcmp((char*)where + i, what, what_size)==0)
+                pos = i;
+            else
+                i++;
+		}
+	}
+	return pos;
+}
+
+
+#if defined(OS_MAC) || defined(OS_UNIX_DARWIN) || defined(OS_MSWIN) || defined(MSC_VIRT)
 /***** Handle functions are for Macintosh and Windows only *****/
 /***** or Microsoft virtual memory manager ****/
 
@@ -477,7 +544,7 @@ NLM_EXTERN Nlm_Handle LIBCALL  Nlm_HandGet (size_t size, Nlm_Boolean clear_out)
 
     if (size == 0) return NULL;
 
-#ifdef OS_MAC
+#if defined(OS_MAC) || defined(OS_UNIX_DARWIN)
     hnd = (Nlm_Handle) NewHandle (size);
 #endif
 
@@ -545,7 +612,7 @@ NLM_EXTERN Nlm_Handle LIBCALL  Nlm_HandMore (Nlm_Handle hnd, size_t size)
 		return NULL;
     }
 
-#ifdef OS_MAC
+#if defined(OS_MAC) || defined(OS_UNIX_DARWIN)
     SetHandleSize ((Handle)hnd, (Size)size);
     hnd2 = hnd;
     if (MemError() != noErr)
@@ -581,7 +648,7 @@ NLM_EXTERN Nlm_Handle LIBCALL  Nlm_HandFree (Nlm_Handle hnd)
 
     if (hnd) {
 
-#ifdef OS_MAC
+#if defined(OS_MAC) || defined(OS_UNIX_DARWIN)
         DisposeHandle ((Handle) hnd);
 #endif
 
@@ -618,7 +685,7 @@ NLM_EXTERN Nlm_VoidPtr LIBCALL  Nlm_HandLock (Nlm_Handle hnd)
         return NULL;
     }
 
-#ifdef OS_MAC
+#if defined(OS_MAC) || defined(OS_UNIX_DARWIN)
     HLock ((Handle) hnd);
     ptr = *((Handle) hnd);
 #endif
@@ -649,7 +716,7 @@ NLM_EXTERN Nlm_VoidPtr LIBCALL  Nlm_HandUnlock (Nlm_Handle hnd)
     if (hnd == NULL)
     	ErrPostEx(SEV_WARNING,E_Programmer,0,"HandUnlock: %s", _msgNullHnd);
     else {
-#ifdef OS_MAC
+#if defined(OS_MAC) || defined(OS_UNIX_DARWIN)
         HUnlock ((Handle) hnd);
 #endif
 
@@ -672,7 +739,6 @@ NLM_EXTERN Nlm_VoidPtr LIBCALL  Nlm_HandUnlock (Nlm_Handle hnd)
 
 
 
-
 #ifdef WIN_MAC
 #ifdef USE_MAC_MEMORY
 /*****************************************************************************
@@ -720,366 +786,17 @@ void mac_Free (void *ptr)
 #endif
 
 
-
-#ifdef WIN16
-/*****************************************************************************
-*
-*   Windows-specific functions:
-*
-*   win16_Malloc     Windows version of malloc
-*   win16_Calloc     Windows version of calloc
-*   win16_Realloc    Windows version of realloc
-*   win16_Free       Windows version of free
-*
-*   The following fucntions are not exported.
-*
-*   HeapNew     allocates an auxilliary local heap
-*   HeapFree    frees an auxilliary local heap
-*
-*****************************************************************************/
-
-LPVOID LIBCALL  win16_Malloc (size_t size)
-{
-	struct heapnode *pNode;
-	WORD      wSeg, wSize, wFree, wOfs;
-	HGLOBAL   hSeg;
-	LPVOID    pMem;
-
-    if (size == 0)  return NULL;
-
-   /*
-    * Two allocation strategies are used depending on the reguested
-    * size of the new block.  Large objects (size >= THRESHOLD) are
-    * allocated as independent blocks from the global heap in the
-    * usual way and then locked.  Small objects (lSize < THRESHOLD)
-    * are allocated from auxilliary local heaps that are created as
-    * needed.  References to these heaps are maintained in a linked
-    * list structure (located in the default data segment).
-    */
-
-    if (size < THRESHOLD)
-    {
-        wSize = (WORD)size;
-
-        for (pNode=headPtr; pNode; pNode=pNode->next)
-        {
-
-           /*
-            * If the current node is empty, we allocate a new heap.
-            * On failure, break out of the loop to do an error return.
-            */
-
-            if ((wSeg = pNode->wSeg) ==0)
-            {
-                if (!HeapNew (pNode))  break;
-                wSeg = pNode->wSeg;
-            }
-
-            if (wSize <= pNode->wFree)
-            {
-
-               /*
-                * Attempt to allocate memory from this node's heap.  To do
-                * this, we need to switch the DS register to the heap's
-                * segment, then do a LocalAlloc, and of course restore
-                * the DS register afterwards (some assembly required).
-                * If this is successful, we construct a far pointer to the
-                * allocated block and return it as the function result.
-                * Otherwise, we continue around the cycle...
-                */
-
-                _asm {
-                	push  ds
-                    mov   ax, wSeg
-                    mov   ds, ax
-                }
-
-                wOfs = (WORD) LocalAlloc(LMEM_FIXED, wSize);
-                wFree = LocalCompact(0);
-
-                _asm {
-               		pop   ds
-                }
-
-                if (wOfs)
-                {
-                    pNode->nItems +=1;
-                    pNode->wFree = wFree;
-                    pMem = (LPVOID) MAKELONG(wOfs, wSeg);
-                    return  pMem;
-                }
-            }
-
-           /*
-            * If we are at the end of the chain, create a new node and
-            * add it to the chain.  The new node becomes the current
-            * node in the next loop iteration.  If the allocation fails
-            * we will drop out of the loop and do an error return.
-            */
-
-            if ( ! pNode->next )
-			{
-                pNode->next = (struct heapnode*) calloc(sizeof(struct heapnode),1);
-				pNode->next->sn = pNode->sn +1;
-			}
-        }
-
-        return  NULL;
-    }
-	else 
-	{
-		/* 
-	    	 *  Large memory objects are allocated from the global heap as
-		 *  independent moveable blocks.  They are then locked to generate 
-		 *  the far pointer and they remain locked until freed.
-		 */
-
-	    if ((hSeg = GlobalAlloc(GMEM_MOVEABLE,size)) != NULL) 
-    	{
-        	pMem = GlobalLock(hSeg);
-			ASSERT(OFFSETOF(pMem)==0);
-        	return  pMem;
-	    }
-	}
-    return NULL;
-}
-
-
-LPVOID LIBCALL  win16_Calloc (size_t items, size_t size)
-{
-    LPVOID ptr;
-    unsigned long bytes = (unsigned long) items * size;
-
-     if (bytes > (unsigned long) SIZE_MAX)  return NULL;
-
-    if ((ptr = win16_Malloc((size_t) bytes)) != NULL)
-        _fmemset(ptr,0,(size_t) bytes);
-
-    return ptr;
-}
-
-
-LPVOID LIBCALL  win16_Realloc (LPVOID pMem, size_t size)
-{
-    WORD wSeg1 = SELECTOROF(pMem);
-    HGLOBAL   hSeg1, hSeg2;
-
-    if (size ==0) 
-    {
-        win16_Free(pMem);
-        return NULL;
-    }
-
-	if (OFFSETOF(pMem) ==0) 
-	{
-       /*
-        *  The existing block was not allocated from one of the
-        *  auxilliary local heaps, so it must be an independent
-        *  global block.  Use the normal GlobalReAlloc strategy.
-        */
-
-		hSeg1 = (HGLOBAL) LOWORD(GlobalHandle(wSeg1));
-		GlobalUnlock (hSeg1);
-		if ((hSeg2 = GlobalReAlloc(hSeg1,size,GMEM_MOVEABLE)) != NULL)
-			return GlobalLock(hSeg2);
-			/*
-		else
-			pMem = NULL;
-		return  pMem;
-		*/
-	}
-	else
-	{
-	    struct heapnode *pNode;
-	    WORD      wSize1, wOfs1;
-	    WORD      wSize2, wOfs2, wFree;
-	    LPVOID    pMem2;
-        
-       /*
-        *  Scan the linked list to see if the segment of the pointer
-        *  to be freed matches any of the auxilliary local heaps.
-        *  If so, LocalFree from that heap.  Otherwise we assume that
-        *  it is a normal global memory object and GlobalFree.
-        */
-
-        for (pNode=headPtr; pNode; pNode=pNode->next) 
-        {
-            if (pNode->wSeg == wSeg1)
-			{
-         	   wSize2 = (WORD) size;
-            	wOfs1 = LOWORD((DWORD)pMem);
-
-	            _asm {
-    	        	push  ds
-        	        mov   ax, wSeg1
-            	    mov   ds, ax   
-	            }
-
-    	        wSize1 = LocalSize((HLOCAL)wSeg1);
-        	    wOfs2 = (WORD) LocalReAlloc((HLOCAL)wOfs1,wSize2,LMEM_MOVEABLE);
-            	wFree = LocalCompact(0);
-
-	            _asm { 
-    	        	pop   ds 
-        	    }
-
-            	if (wOfs2)     /* successfully realloc'ed within same heap */
-				{
-    	            pNode->wFree = wFree;
-        	        pMem = (LPVOID) MAKELONG(wOfs2,wSeg1);
-            	    return  pMem;
-	            }
-
-    	       /*
-        	    *  We couldn't realloc the object within the same heap, so
-            	*  we will now attempt to allocate a new pointer and copy
-	            *  contents of the existing block to the new one.  If
-    	        *  successful, the old block is freed.
-        	    */
-
-            	if ((pMem2 = win16_Malloc(size)) != NULL) 
-	            {
-    	            _fmemcpy(pMem2,pMem,MIN(wSize1,wSize2));
-        	        win16_Free(pMem);
-            	}
-	            return  pMem2;
-    	    }
-		}
-    }
-
-    return NULL;
-}
-
-
-VOID LIBCALL  win16_Free (LPVOID pMem)
-{
-	WORD wSeg = SELECTOROF(pMem);
-	WORD wOfs = OFFSETOF(pMem);
-
-	g_bBadPtr = FALSE;
-
-	if (wOfs ==0)
-	{
-		HGLOBAL hSeg = (HGLOBAL) LOWORD(GlobalHandle(wSeg));
-		if (GlobalSize(hSeg))
-		{
-			GlobalUnlock(hSeg);
-			GlobalFree(hSeg);
-			return;
-		}
-	}
-	else
-	{
-		struct heapnode *pNode;
-        
-        /*
-         *  Scan the linked list to see if the segment of the pointer
-         *  to be freed matches any of the auxilliary local heaps.
-         *  If so, LocalFree from that heap.  Otherwise we assume that
-         *  it is a normal global memory object and GlobalFree.
-         */
-
-        for (pNode=headPtr; pNode; pNode=pNode->next) 
-        {
-            if (pNode->wSeg == wSeg)
-			{
-		    	WORD wFree;
-
-	           	_asm {
-	           		push  ds
-    	       		mov   ax, wSeg
-        	        mov   ds, ax   
-           		}
-
-				if (LocalSize((HLOCAL)wOfs))
-				{
-		            wOfs = (WORD) LocalFree((HLOCAL)wOfs);
-    		        wFree = LocalCompact(0);
-				}
-
-        	   	_asm { 
-           			pop   ds 
-	           	}
-
-    	        if (wOfs == NULL)         /* NULL means success here */
-				{
-            	    pNode->nItems -= 1;
-                	pNode->wFree = wFree;
-	                if (pNode->nItems ==0)  
-					{
-        	            HeapFree(pNode);  /* free the heap, but keep the node */
-					}
-					return;
-	            }
-	            goto BAD_POINTER;
-			}
-        }
-	    goto BAD_POINTER;
-    }
-    
-    
-BAD_POINTER :
-	g_bBadPtr = TRUE;
-	TRACE("win16_Free(%04X:%04X)  *** invalid pointer ***\n",wSeg,wOfs);
-}
-
-
-static BOOL NEAR  HeapNew (struct heapnode *pNode)
-{
-	HGLOBAL hMem;
-
-	pNode->nItems = 0;
-	pNode->hSeg = NULL;
-	pNode->wSeg = NULL;
-    if ((hMem = GlobalAlloc(GMEM_MOVEABLE,HEAPSIZE)) != NULL) 
-    {
-	    LPSTR pMem = GlobalLock(hMem);
-        WORD  wSeg = SELECTOROF(pMem);
-        WORD  wSize = (WORD) GlobalSize(hMem) -16;
-
-        if (LocalInit(wSeg,0,wSize)) 
-        {
-            pNode->hSeg = hMem;
-            pNode->wSeg = wSeg;
-            pNode->wFree = wSize;
-	        GlobalUnlock(hMem);    /* remove LocalInit's lock */
-
-			return TRUE;
-        }
-		GlobalUnlock(hMem);
-		GlobalFree(hMem);	
-    }
-	TRACE("HeapNew: ** unable to allocate %ld bytes\n",HEAPSIZE);
-    return FALSE;
-}
-
-static VOID NEAR  HeapFree (struct heapnode *pNode)
-{   
-    HGLOBAL hMem;
-
-    if (pNode->nItems==0 && ((hMem=pNode->hSeg) != NULL)) 
-    {
-        GlobalUnlock(hMem);
-        GlobalFree(hMem);
-        pNode->hSeg = NULL;
-        pNode->wSeg = 0;
-    }
-}
-
-
-#endif /* WIN16 */
-
 
 #ifdef _WINDLL
 /*****************************************************************************
 *
 *   Windows DLL-specific functions (shared memory)
-*   
+*
 *   dll_Malloc
 *   dll_Calloc	(not yet)
 *   dll_Realloc	(not yet)
-*   dll_Free   
-*   
+*   dll_Free
+*
 *****************************************************************************/
 
 
@@ -1102,23 +819,15 @@ void * dll_Malloc (size_t bytes)
 
 void   dll_Free (void *pMem)
 {
-	HGLOBAL hMem;
-#ifdef WIN16
-	DWORD dwHandle = GlobalHandle(SELECTOROF(pMem));
-	hMem = (HGLOBAL)LOWORD(dwHandle);
-#else
-	hMem = GlobalHandle(pMem);
-#endif
-
+	HGLOBAL hMem = GlobalHandle(pMem);
 	GlobalUnlock(hMem);
 	GlobalFree(hMem);
 }
-
-#endif
+#endif /* _WINDLL */
 
 
 /*********************************************************************
-*	Function to test whether memory-mapping is available. 
+*	Function to test whether memory-mapping is available.
 *
 *	returns TRUE if it is supported by NCBI routines.
 *********************************************************************/
@@ -1135,14 +844,17 @@ NLM_EXTERN Nlm_Boolean Nlm_MemMapAvailable(void)
 
 NLM_EXTERN Nlm_MemMapPtr Nlm_MemMapInit(const Nlm_Char PNTR name)
 {
-  Nlm_MemMapPtr	mem_mapp;
+  Nlm_MemMapPtr mem_mapp;
   if (!Nlm_MemMapAvailable()  ||  !name  ||  !*name  ||
-      !(mem_mapp = (Nlm_MemMapPtr)Nlm_MemNew(sizeof(Nlm_MemMap))) )
+      (mem_mapp = (Nlm_MemMapPtr)Nlm_MemNew(sizeof(Nlm_MemMap))) == NULL)
     return NULL;
 
   for (;;) {{ /* (quasi-TRY block) */
-    if ((mem_mapp->file_size = Nlm_FileLength((Nlm_CharPtr)name)) <= 0)
+    if ((mem_mapp->file_size = Nlm_FileLengthEx(name)) < 0)
       break;
+
+    if (mem_mapp->file_size == 0) /* Special case */
+      return mem_mapp;
 	
 #ifdef WIN32
     {{
@@ -1153,7 +865,7 @@ NLM_EXTERN Nlm_MemMapPtr Nlm_MemMapInit(const Nlm_Char PNTR name)
           *str = '/';  /* name of a file-mapping object cannot contain '\' */
 
       if ( !(mem_mapp->hMap =
-             OpenFileMapping(FILE_MAP_READ, FALSE, x_name)) ) 
+             OpenFileMapping(FILE_MAP_READ, FALSE, x_name)) )
         { /* If failed to attach to an existing file-mapping object then
            * create a new one(based on the specified file) */
           HANDLE hFile= CreateFile(name, GENERIC_READ, FILE_SHARE_READ, NULL,
@@ -1178,14 +890,14 @@ NLM_EXTERN Nlm_MemMapPtr Nlm_MemMapInit(const Nlm_Char PNTR name)
 
 #elif defined(MMAP_AVAIL)
     {{  /* UNIX memory mapping. */
-      FILE *fp = Nlm_FileOpen(name, "rb");
-      if ( !fp )
+      int fd = open(name, O_RDONLY);
+      if (fd < 0)
         break;
 
       mem_mapp->mmp_begin = mmap(NULL, mem_mapp->file_size, PROT_READ,
-                                 MAP_SHARED, fileno(fp), 0);
-      Nlm_FileClose( fp );
-      if (mem_mapp->mmp_begin == MAP_FAILED)
+                                 MAP_SHARED, fd, 0);
+      close(fd);
+      if ((void*) mem_mapp->mmp_begin == (void*) MAP_FAILED)
         break;
     }}
 #endif
@@ -1195,7 +907,7 @@ NLM_EXTERN Nlm_MemMapPtr Nlm_MemMapInit(const Nlm_Char PNTR name)
   }}
 
   /* Error;  cleanup */
-  Nlm_MemFree( mem_mapp );
+  Nlm_MemFree(mem_mapp);
   return NULL;
 }
 
@@ -1215,3 +927,40 @@ NLM_EXTERN void Nlm_MemMapFini(Nlm_MemMapPtr mem_mapp)
 
   Nlm_MemFree( mem_mapp );
 }
+
+
+NLM_EXTERN Nlm_Boolean Nlm_MemMapAdvise(void* addr, size_t len, EMemMapAdvise advise)
+{
+#if defined(HAVE_MADVISE) && defined(MADV_NORMAL)
+  int adv;
+  if (!addr || !len) {
+    return FALSE;
+  }
+  switch (advise) {
+	case eMMA_Random:
+	  adv = MADV_RANDOM;     break;
+	case eMMA_Sequential:
+      adv = MADV_SEQUENTIAL; break;
+	case eMMA_WillNeed:
+	  adv = MADV_WILLNEED;   break;
+	case eMMA_DontNeed:
+	  adv = MADV_DONTNEED;   break;
+	default:
+	  adv = MADV_NORMAL;
+  }
+  /* Conversion type of "addr" to char* -- Sun Solaris fix */
+  return madvise((char*)addr, len, adv) == 0;
+#else
+  return TRUE;
+#endif
+}
+
+
+NLM_EXTERN Nlm_Boolean Nlm_MemMapAdvisePtr(Nlm_MemMapPtr ptr, EMemMapAdvise advise)
+{
+#if defined(HAVE_MADVISE) && defined(MADV_NORMAL)
+  return ptr ? Nlm_MemMapAdvise(ptr->mmp_begin, ptr->file_size, advise) : FALSE;
+#else
+  return TRUE;
+#endif
+}
