@@ -1,3 +1,5 @@
+static char const rcsid[] = "$Id: bl2seq.c,v 6.60 2004/02/26 16:25:06 camacho Exp $";
+
 /**************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -25,6 +27,67 @@
 ***************************************************************************
 *
 * $Log: bl2seq.c,v $
+* Revision 6.60  2004/02/26 16:25:06  camacho
+* Fix uninitialized variable errors that caused core dumps on empty input files
+*
+* Revision 6.59  2003/06/26 18:56:05  coulouri
+* remove unnecessary variables
+*
+* Revision 6.58  2003/05/30 17:31:09  coulouri
+* add rcsid
+*
+* Revision 6.57  2003/05/13 16:02:42  coulouri
+* make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+*
+* Revision 6.56  2003/05/06 18:57:46  dondosha
+* Do not set cutoff_s for megablast, it is not needed
+*
+* Revision 6.55  2003/04/22 19:38:00  dondosha
+* Fix for ungapped search with tabular output
+*
+* Revision 6.54  2003/01/16 19:46:28  kans
+* include accid1.h to fix Mac compiler error on missing symbols
+*
+* Revision 6.53  2003/01/10 20:55:36  dondosha
+* Enable the Entrez client option without a special definition
+*
+* Revision 6.52  2003/01/07 15:47:11  dondosha
+* Hide the -A argument if NCBI_ENTREZ_CLIENT is not defined
+*
+* Revision 6.51  2002/09/18 18:21:04  camacho
+* Fixed memory leak when using the -U option
+*
+* Revision 6.50  2002/09/13 18:53:26  dondosha
+* Corrected the way query and subject deflines are shown in all types of output
+*
+* Revision 6.49  2002/08/09 19:41:25  camacho
+* 1) Added blast version number to command-line options
+* 2) Added explanations for some default parameters
+*
+* Revision 6.48  2002/06/26 21:44:57  dondosha
+* Set default gap penalties before checking command line options
+*
+* Revision 6.47  2002/06/24 18:24:05  dondosha
+* Multiple memory freeing bug fixed
+*
+* Revision 6.46  2002/05/09 15:37:52  dondosha
+* Call BLASTOptionNewEx instead of BLASTOptionNew, so megablast defaults are set in a central place
+*
+* Revision 6.45  2002/05/02 22:18:25  dondosha
+* Copy bioseq if getting it from ID1, removing non-residue characters from sequence
+*
+* Revision 6.44  2002/05/01 16:43:53  dondosha
+* Call BLASTOptionSetGapParams instead of setting options->matrix
+*
+* Revision 6.43  2002/04/29 19:55:26  madden
+* Use ARG_FLOAT for db length
+*
+* Revision 6.42  2002/03/19 23:29:38  dondosha
+* Do not increment options->wordsize by 4 for megablast any more
+*
+* Revision 6.41  2002/03/14 16:11:41  camacho
+* Extended BlastTwoSequences to allow comparison between sequence and PSSM
+*
 * Revision 6.40  2001/07/19 22:05:47  dondosha
 * Made db_length option a string, to convert to Int8 value
 *
@@ -152,30 +215,31 @@
 #include <txalign.h>
 #include <sqnutils.h>
 #include <mblast.h>
+#include <accid1.h>
 
 /* Used by the callback function. */
 FILE *global_fp=NULL;
 
+#define LOCAL_BUFLEN 255
 static BioseqPtr
-BioseqFromAccession(CharPtr accver, Int2 id_num, Boolean is_na)
+BioseqFromAccession(CharPtr accver, Int2 id_num, Boolean is_na, 
+                    Boolean believe_defline)
 {
-   CharPtr accession, new_defline, accver_var, version_str;
+   CharPtr accession, new_defline, version_str;
    Int4 version=0, gi, number, title_length, id_length;
    SeqIdPtr sip = NULL;
    TextSeqIdPtr tsip;
    PDBSeqIdPtr  psip;
-   BioseqPtr bsp = NULL;
+   BioseqPtr bsp = NULL, bsp_tmp = NULL;
    ObjectIdPtr  oid;
    SeqPortPtr spp;
    Int2 retval, buf_length=512;
    Uint1 buf[512];
-   Char tmp[255];
+   Char tmp[LOCAL_BUFLEN];
 
-#ifdef NCBI_ENTREZ_CLIENT
    if (!ID1BioseqFetchEnable ("bl2seq", TRUE))
-      ErrPostEx(SEV_FATAL, 0, 0, 
+      ErrPostEx(SEV_FATAL, 1, 0, 
                 "Entrez access interface currently unavailable\n");
-   accver_var = StringSave(accver);
    if (!IS_DIGIT(*accver)) {
       accession = StringTokMT(accver, ".", &version_str);
       if (version_str)
@@ -243,31 +307,37 @@ BioseqFromAccession(CharPtr accver, Int2 id_num, Boolean is_na)
 
    if (gi > 0) {
       ValNodeAddInt(&sip, SEQID_GI, gi);
-      if ((bsp = BioseqLockById(sip)) == NULL) {
+      if ((bsp_tmp = BioseqLockById(sip)) == NULL) {
          ErrPostEx(SEV_WARNING, 0, 0, "Gi %ld not found", gi);
          return NULL;
       }
    } 
    
-   if (ISA_na(bsp->mol) != is_na) {
-      BioseqUnlock(bsp);
+   if (ISA_na(bsp_tmp->mol) != is_na) {
+      BioseqUnlock(bsp_tmp);
       if (is_na)
-         ErrPostEx(SEV_FATAL, 0, 0, 
+         ErrPostEx(SEV_FATAL, 1, 0, 
                    "%s is a protein sequence, program requires nucleotide", 
                    accver);
       else
-         ErrPostEx(SEV_FATAL, 0, 0, 
+         ErrPostEx(SEV_FATAL, 1, 0, 
                    "%s is a nucleotide sequence, program requires protein", 
                    accver);
       return NULL;
    }
-
+   
+   bsp = BioseqNew();
+   bsp->length = bsp_tmp->length;
+   bsp->mol = bsp_tmp->mol;
+   bsp->repr = Seq_repr_raw;
+   bsp->seq_data = BSNew(bsp->length);
+                   
    if (ISA_na(bsp->mol)) {
-      spp = SeqPortNew(bsp, 0, -1, Seq_strand_plus, 
+      spp = SeqPortNew(bsp_tmp, 0, -1, Seq_strand_plus, 
                        Seq_code_iupacna);
       bsp->seq_data_type = Seq_code_iupacna;
    } else {
-      spp = SeqPortNew(bsp, 0, -1, Seq_strand_unknown, 
+      spp = SeqPortNew(bsp_tmp, 0, -1, Seq_strand_unknown, 
                        Seq_code_ncbieaa);
       bsp->seq_data_type = Seq_code_ncbieaa;
    }
@@ -280,16 +350,45 @@ BioseqFromAccession(CharPtr accver, Int2 id_num, Boolean is_na)
    
    while (number < bsp->length) {
       retval = SeqPortRead(spp, buf, buf_length);
-      if (retval <= 0)
+      if (retval < 0)
+         continue;
+      if (retval == 0)
          break;
       BSWrite(bsp->seq_data, buf, retval);
       number += retval;
    }
    
    SeqPortFree(spp);
+   if (believe_defline) {
+      new_defline = StringSave(BioseqGetTitle(bsp_tmp));
+   } else {
+      title_length = StringLen(BioseqGetTitle(bsp_tmp));
+      SeqIdWrite(bsp_tmp->id, tmp, PRINTID_FASTA_LONG, LOCAL_BUFLEN);
+      id_length = StringLen(tmp);
+      title_length += id_length;
+      title_length +=3;
+      new_defline = (CharPtr) MemNew(title_length*sizeof(Char));
+      StringCpy(new_defline, tmp);
+      *(new_defline+id_length) = ' ';
+      StringCpy(new_defline+id_length+1, BioseqGetTitle(bsp_tmp));
+      *(new_defline+title_length-1) = NULLB;
+   }
+
+   bsp->descr = ValNodeAddStr(NULL, Seq_descr_title, new_defline);
+   bsp->id = ValNodeNew(NULL);
+   oid = ObjectIdNew();
+   oid->str = (char*) Malloc(LOCAL_BUFLEN+1);
+   if (believe_defline) {
+      SeqIdWrite(bsp_tmp->id, oid->str, PRINTID_FASTA_LONG, LOCAL_BUFLEN);
+   } else {
+      sprintf(oid->str, "%d", id_num);
+   }
+   bsp->id->choice = SEQID_LOCAL;
+   bsp->id->data.ptrvalue = (Pointer) oid;
+   SeqMgrDeleteFromBioseqIndex(bsp_tmp);
+   BioseqUnlock(bsp_tmp);
    BioseqPack(bsp);
    ID1BioseqFetchDisable();
-#endif   
    return bsp;
 }
 		
@@ -307,16 +406,18 @@ static Args myargs [] = {
   { "alignment output file",
 	"stdout", NULL, NULL, FALSE, 'o', ARG_FILE_OUT, 0.0, 0, NULL},/* 4 */
   { "theor. db size (zero is real size)", 
-	"0", NULL, NULL, FALSE, 'd', ARG_STRING, 0.0, 0, NULL},/* 5 */
+	"0", NULL, NULL, FALSE, 'd', ARG_FLOAT, 0.0, 0, NULL},/* 5 */
   { "SeqAnnot output file",
 	NULL, NULL, NULL, TRUE, 'a', ARG_FILE_OUT, 0.0, 0, NULL},/* 6 */
-  { "Cost to open a gap (zero invokes default behavior)",
-        "0", NULL, NULL, FALSE, 'G', ARG_INT, 0.0, 0, NULL},/* 7 */
-  { "Cost to extend a gap (zero invokes default behavior)",
-        "0", NULL, NULL, FALSE, 'E', ARG_INT, 0.0, 0, NULL},/* 8 */
-  { "X dropoff value for gapped alignment (in bits) (zero invokes default behavior)",
+  { "Cost to open a gap (-1 invokes default behavior; non-default values can result in unreliable statistics)",
+        "-1", NULL, NULL, FALSE, 'G', ARG_INT, 0.0, 0, NULL},/* 7 */
+  { "Cost to extend a gap (-1 invokes default behavior; non-default values can result in unreliable statistics)",
+        "-1", NULL, NULL, FALSE, 'E', ARG_INT, 0.0, 0, NULL},/* 8 */
+  { "X dropoff value for gapped alignment (in bits) (zero invokes default "
+    "behavior)\n      blastn 30, megablast 20, tblastx 0, all others 15",
         "0", NULL, NULL, FALSE, 'X', ARG_INT, 0.0, 0, NULL},/* 9 */
-  { "Wordsize (zero invokes default behavior)",
+  { "Word size, default if zero (blastn 11, megablast 28, "
+        "all others 3)",
         "0", NULL, NULL, FALSE, 'W', ARG_INT, 0.0, 0, NULL},/* 10 */
   { "Matrix",
         "BLOSUM62", NULL, NULL, FALSE, 'M', ARG_STRING, 0.0, 0, NULL},/* 11 */
@@ -336,18 +437,18 @@ static Args myargs [] = {
         "F", NULL, NULL, TRUE, 'm', ARG_BOOLEAN, 0.0, 0, NULL},/* 18 */
   { "Effective length of the search space (use zero for the real size)",
         "0", NULL, NULL, FALSE, 'Y', ARG_FLOAT, 0.0, 0, NULL}, /* 19 */
-  { "Input sequences in the form of accession.version",
-        "F", NULL, NULL, FALSE, 'A', ARG_BOOLEAN, 0.0, 0, NULL}, /* 20 */
   { "Length of the largest intron allowed in tblastn for linking HSPs",
-        "0", NULL, NULL, FALSE, 't', ARG_INT, 0.0, 0, NULL},/* 21 */
+        "0", NULL, NULL, FALSE, 't', ARG_INT, 0.0, 0, NULL},/* 20 */
   { "Location on first sequence",
-        NULL, NULL, NULL, TRUE, 'I', ARG_STRING, 0.0, 0, NULL},/* 22 */
+        NULL, NULL, NULL, TRUE, 'I', ARG_STRING, 0.0, 0, NULL},/* 21 */
   { "Location on second sequence",
-        NULL, NULL, NULL, TRUE, 'J', ARG_STRING, 0.0, 0, NULL},/* 23 */
-  { "Output format: 0 - traditional, 1 - tabulated", 
-        "0", NULL, NULL, FALSE, 'D', ARG_INT, 0.0, 0, NULL}, /* 24 */
+        NULL, NULL, NULL, TRUE, 'J', ARG_STRING, 0.0, 0, NULL},/* 22 */
+  { "Output format: 0 - traditional, 1 - tabular", 
+        "0", NULL, NULL, FALSE, 'D', ARG_INT, 0.0, 0, NULL}, /* 23 */
   { "Use lower case filtering for the query sequence",
-        "F", NULL, NULL, TRUE, 'U', ARG_BOOLEAN, 0.0, 0, NULL}/* 25 */
+        "F", NULL, NULL, TRUE, 'U', ARG_BOOLEAN, 0.0, 0, NULL},/* 24 */
+  { "Input sequences in the form of accession.version",
+        "F", NULL, NULL, FALSE, 'A', ARG_BOOLEAN, 0.0, 0, NULL} /* 25 */
 };
 
 Int2 Main (void)
@@ -355,19 +456,19 @@ Int2 Main (void)
 {
 	
 	AsnIoPtr aip;
-	BioseqPtr fake_bsp, fake_subject_bsp, query_bsp, subject_bsp;
+	BioseqPtr fake_bsp = NULL, fake_subject_bsp = NULL, query_bsp = NULL, 
+                  subject_bsp = NULL;
         BioseqPtr bsp1, bsp2;
 	BLAST_KarlinBlkPtr ka_params=NULL, ka_params_gap=NULL;
 	BLAST_OptionsBlkPtr options;
 	Boolean seq1_is_na, seq2_is_na;
-	CharPtr ret_buffer=NULL, params_buffer=NULL;
+	CharPtr params_buffer=NULL;
         DbtagPtr        dbtagptr;
 	Uint1 align_type;
 	Uint4 align_options;
 	SeqAlignPtr  seqalign;
         SeqAnnotPtr seqannot;
 	SeqEntryPtr sep = NULL, sep1 = NULL;
-	TxDfDbInfoPtr dbinfo=NULL, dbinfo_head;
 	CharPtr blast_inputfile, blast_inputfile1, program_name, blast_outputfile;
 	FILE *infp, *infp1, *outfp;
 	ValNodePtr  mask_loc, mask_loc_start, vnp, other_returns=NULL, error_returns=NULL;
@@ -376,12 +477,16 @@ Int2 Main (void)
         int (LIBCALLBACK *handle_results)PROTO((VoidPtr search)) = NULL;
         Boolean entrez_lookup = FALSE;
         CharPtr query_accver = NULL, subject_accver = NULL;
-        const char *dummystr;
+        Char buf[256] = { '\0' };
+        Boolean html, seqannot_output, believe_query;
+        Uint1 tabular_output;
+        Boolean gapped_calculation;
 
-        if (! GetArgs ("bl2seq", NUMARG, myargs))
-        {
-                return (1);
-        }
+    StringCpy(buf, "bl2seq ");
+    StringNCat(buf, BlastGetVersionNumber(), sizeof(buf)-StringLen(buf)-1);
+    if (! GetArgs (buf, NUMARG, myargs)) {
+        return (1);
+    }
 
 	UseLocalAsnloadDataAndErrMsg ();
 
@@ -389,13 +494,10 @@ Int2 Main (void)
                 return 1;
 
 	ErrSetMessageLevel(SEV_WARNING);
-        entrez_lookup = (Boolean) myargs[20].intvalue;
-#ifndef NCBI_ENTREZ_CLIENT
-        if (entrez_lookup) {
-           ErrPostEx(SEV_WARNING, 0, 0, "Entrez client interface unavailable, -A option cannot be used");
-           return 1;
-        }
-#endif
+        entrez_lookup = (Boolean) myargs[25].intvalue;
+        html = (Boolean) myargs[17].intvalue;
+        seqannot_output = (myargs[6].strvalue != NULL);
+
         if (entrez_lookup) {
            query_accver = myargs [0].strvalue;
            subject_accver = myargs [1].strvalue;
@@ -411,44 +513,46 @@ Int2 Main (void)
 	    StringCmp(program_name, "blastx") && 
 	    StringCmp(program_name, "tblastn") && 
 	    StringCmp(program_name, "tblastx")) {
-		ErrPostEx(SEV_FATAL, 0, 0, "Program name must be blastn, blastp, blastx, tblastn or tblastx\n");
+		ErrPostEx(SEV_FATAL, 1, 0, "Program name must be blastn, blastp, blastx, tblastn or tblastx\n");
 		return (1);
 	}
 	   
-
 	align_type = BlastGetTypes(program_name, &seq1_is_na, &seq2_is_na);
 
 	if (!entrez_lookup && (infp = FileOpen(blast_inputfile, "r")) == NULL)
 	{
-		ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open input file %s\n", blast_inputfile);
+		ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open input file %s\n", blast_inputfile);
 		return (1);
 	}
 
 	if (!entrez_lookup && (infp1 = FileOpen(blast_inputfile1, "r")) == NULL)
 	{
-		ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open input file %s\n", blast_inputfile1);
+		ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open input file %s\n", blast_inputfile1);
 		return (1);
 	}
 
 	if ((outfp = FileOpen(blast_outputfile, "w")) == NULL)
 	{
-		ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open output file %s\n", blast_outputfile);
+		ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", blast_outputfile);
 		return (1);
 	}
 
         global_fp = outfp;
-	options = BLASTOptionNew(program_name, (Boolean) myargs[3].intvalue);
-        options->is_megablast_search = (Boolean) myargs[18].intvalue;
+        gapped_calculation = (Boolean) myargs[3].intvalue;
 
+	options = BLASTOptionNewEx(program_name, gapped_calculation,
+                                   (Boolean) myargs[18].intvalue);
+        tabular_output = (Uint1) myargs[23].intvalue; 
+        believe_query = (seqannot_output || entrez_lookup); 
         if (entrez_lookup) {
-           query_bsp = BioseqFromAccession(query_accver, 1, seq1_is_na);
+           query_bsp = BioseqFromAccession(query_accver, 1, seq1_is_na, TRUE);
         } else {
-           if (myargs[25].intvalue)
-              sep = FastaToSeqEntryForDb(infp, seq1_is_na, NULL, FALSE, 
-                                         NULL, NULL, 
+           if (myargs[24].intvalue)
+              sep = FastaToSeqEntryForDb(infp, seq1_is_na, NULL, 
+                                         believe_query, NULL, NULL, 
                                          &options->query_lcase_mask);
            else
-              sep = FastaToSeqEntry(infp, seq1_is_na);
+              sep = FastaToSeqEntryEx(infp, seq1_is_na, NULL, believe_query);
 
            if (sep != NULL) {
               query_bsp = NULL;
@@ -460,21 +564,21 @@ Int2 Main (void)
            }
         }
         if (query_bsp == NULL) {
-           ErrPostEx(SEV_FATAL, 0, 0, "Unable to obtain bioseq\n");
+           ErrPostEx(SEV_FATAL, 1, 0, "Unable to obtain bioseq\n");
            return 2;
         }
 
-        if (myargs[24].intvalue == 0)
+        if (!believe_query)
            fake_bsp = BlastMakeFakeBioseq(query_bsp, NULL);
         if (entrez_lookup) {
-           subject_bsp = BioseqFromAccession(subject_accver, 2, seq2_is_na);
+           subject_bsp = 
+              BioseqFromAccession(subject_accver, 2, seq2_is_na, FALSE);
         } else {
-           if (myargs[6].strvalue != NULL || myargs[17].intvalue != 0
-               || myargs[24].intvalue != 0)
-              sep1 = FastaToSeqEntry(infp1, seq2_is_na);
-           else
-              sep1 = FastaToSeqEntryEx(infp1, seq2_is_na, NULL, FALSE);
-	
+           /*if (believe_query || html != 0)
+             sep1 = FastaToSeqEntry(infp1, seq2_is_na);
+             else*/
+           sep1 = FastaToSeqEntryEx(infp1, seq2_is_na, NULL, FALSE);
+           
            if (sep1 != NULL) {
               subject_bsp = NULL;
               if (seq2_is_na)
@@ -486,39 +590,25 @@ Int2 Main (void)
         }
         
         if (subject_bsp == NULL) {
-           ErrPostEx(SEV_FATAL, 0, 0, "Unable to obtain bioseq\n");
+           ErrPostEx(SEV_FATAL, 1, 0, "Unable to obtain bioseq\n");
            return 2;
         }
 
-        if (myargs[6].strvalue == NULL && myargs[17].intvalue == 0 &&
-            myargs[24].intvalue == 0) {
-           if (!entrez_lookup) {
-              fake_subject_bsp = BioseqNew();
-              fake_subject_bsp->descr = subject_bsp->descr;
-              fake_subject_bsp->repr = subject_bsp->repr;
-              fake_subject_bsp->mol = subject_bsp->mol;
-              fake_subject_bsp->length = subject_bsp->length;
-              fake_subject_bsp->seq_data = subject_bsp->seq_data;
-              fake_subject_bsp->seq_data_type = subject_bsp->seq_data_type;
-              dbtagptr = DbtagNew();
-              dbtagptr->db = StringSave("BL_ORD_ID");
-              dbtagptr->tag = ObjectIdNew();
-              dbtagptr->tag->id = 0;
-              ValNodeAddPointer(&fake_subject_bsp->id, SEQID_GENERAL, dbtagptr);
-           } else {
-              Char tmp[256];
-              SeqIdWrite(subject_bsp->id, tmp, PRINTID_FASTA_LONG, 255);
-              fake_subject_bsp = BlastMakeFakeBioseq(subject_bsp, tmp);
-           }
-        }
-        if (options->is_megablast_search) {
-           options->gap_open = options->gap_extend = 0;
-           options->wordsize = 28;
-           options->block_width = 0;
-	}
+        fake_subject_bsp = BioseqNew();
+        fake_subject_bsp->descr = subject_bsp->descr;
+        fake_subject_bsp->repr = subject_bsp->repr;
+        fake_subject_bsp->mol = subject_bsp->mol;
+        fake_subject_bsp->length = subject_bsp->length;
+        fake_subject_bsp->seq_data = subject_bsp->seq_data;
+        fake_subject_bsp->seq_data_type = subject_bsp->seq_data_type;
+        dbtagptr = DbtagNew();
+        dbtagptr->db = StringSave("BL_ORD_ID");
+        dbtagptr->tag = ObjectIdNew();
+        dbtagptr->tag->str = StringSave(BioseqGetTitle(subject_bsp));
+        ValNodeAddPointer(&fake_subject_bsp->id, SEQID_GENERAL, dbtagptr);
 
     	if (myargs[19].floatvalue)
-        	options->searchsp_eff = (Nlm_FloatHi) myargs[19].floatvalue;
+           options->searchsp_eff = (Nlm_FloatHi) myargs[19].floatvalue;
 
 
 	options->filter_string = StringSave(myargs[14].strvalue);
@@ -530,14 +620,10 @@ Int2 Main (void)
                 options->reward = myargs[13].intvalue;
         }
 
-	options->db_length = StringToInt8(myargs[5].strvalue, &dummystr);
+	options->db_length = (Int8) myargs[5].floatvalue;
 
 	options->discontinuous = FALSE;
 
-        if (myargs[7].intvalue != 0)
-              options->gap_open = myargs[7].intvalue;
-        if (myargs[8].intvalue != 0)
-               options->gap_extend = myargs[8].intvalue;
         if (myargs[9].intvalue != 0)
 	{
                options->gap_x_dropoff = myargs[9].intvalue;
@@ -546,46 +632,42 @@ Int2 Main (void)
                options->wordsize = (Int2) myargs[10].intvalue;
 
 	if (options->is_megablast_search) {
-	   options->cutoff_s2 = options->wordsize;
-           options->wordsize += 4;
-	   options->cutoff_s = options->wordsize;
+	   options->cutoff_s2 = options->wordsize*options->reward;
         }
-	MemFree(options->matrix);
-	options->matrix = myargs[11].strvalue;
+	options->matrix = MemFree(options->matrix);
+        BLASTOptionSetGapParams(options, myargs[11].strvalue, 0, 0); 
+
+        if (myargs[7].intvalue != -1)
+              options->gap_open = myargs[7].intvalue;
+        if (myargs[8].intvalue != -1)
+               options->gap_extend = myargs[8].intvalue;
 
 	options->strand_option = myargs[16].intvalue;
 
-        /* Input longest intron length is in nucleotide scale; in the lower level
-           code it will be used in protein scale */
-        if (myargs[21].intvalue > 0) 
-           options->longest_intron = MAX(myargs[21].intvalue, MAX_INTRON_LENGTH);
-        if (myargs[24].intvalue == 1) {
-           options->output = (VoidPtr) outfp;
-           if (options->is_megablast_search)
-              handle_results = MegaBlastPrintAlignInfo;
-           else
-              handle_results = BlastPrintAlignInfo;
-        } else 
-           handle_results = NULL;
+        /* Input longest intron length is in nucleotide scale; in the lower 
+           level code it will be used in protein scale */
+        if (myargs[20].intvalue > 0) 
+           options->longest_intron = 
+              MAX(myargs[20].intvalue, MAX_INTRON_LENGTH);
 
-        if (myargs[6].strvalue || myargs[17].intvalue || 
-            handle_results) {
+        if (believe_query) {
            bsp1 = query_bsp;
-           bsp2 = subject_bsp;
         } else {
            bsp1 = fake_bsp;
-           bsp2 = fake_subject_bsp;
         }
 
-        if (!myargs[22].strvalue && !myargs[23].strvalue) {
-              seqalign = BlastTwoSequencesWithCallback(bsp1, bsp2, program_name, options, &other_returns, &error_returns, handle_results);
+        bsp2 = fake_subject_bsp;
+
+        if (!myargs[21].strvalue && !myargs[22].strvalue) {
+           seqalign = BlastTwoSequencesWithCallback(bsp1, bsp2, program_name, 
+              options, &other_returns, &error_returns, handle_results);
         } else {
            /* Location(s) are provided */
            CharPtr delimiters = " ,;";
            CharPtr location;
            Int4 from, to;
            SeqLocPtr slp1 = NULL, slp2 = NULL;
-           location = myargs[22].strvalue;
+           location = myargs[21].strvalue;
            if (location) {
               from = atoi(StringTokMT(location, delimiters, &location)) - 1;
               to = atoi(location) - 1;
@@ -595,7 +677,7 @@ Int2 Main (void)
                  to = bsp1->length - 1;
               to = MIN(to, bsp1->length - 1);
               if (from >= bsp1->length) {
-                 ErrPostEx(SEV_FATAL, 0, 0, 
+                 ErrPostEx(SEV_FATAL, 1, 0, 
                            "Location outside of the first sequence range\n");
                  return 3;
               }
@@ -604,7 +686,7 @@ Int2 Main (void)
            } else
               ValNodeAddPointer(&slp1, SEQLOC_WHOLE, SeqIdDup(SeqIdFindBestAccession(bsp1->id)));
 
-           location = myargs[23].strvalue;
+           location = myargs[22].strvalue;
            if (location) {
               from = atoi(StringTokMT(location, delimiters, &location)) - 1;
               to = atoi(location) - 1;
@@ -614,7 +696,7 @@ Int2 Main (void)
                  to = bsp2->length - 1;
               to = MIN(to, bsp2->length - 1);
               if (from >= bsp2->length) {
-                 ErrPostEx(SEV_FATAL, 0, 0, 
+                 ErrPostEx(SEV_FATAL, 1, 0, 
                            "Location outside of the second sequence range\n");
                  return 3;
               }
@@ -622,7 +704,7 @@ Int2 Main (void)
            } else
               ValNodeAddPointer(&slp2, SEQLOC_WHOLE, SeqIdDup(SeqIdFindBestAccession(bsp2->id)));
            
-           seqalign = BlastTwoSequencesByLocWithCallback(slp1, slp2, program_name, options, &other_returns, &error_returns, handle_results);
+           seqalign = BlastTwoSequencesByLocWithCallback(slp1, slp2, program_name, options, &other_returns, &error_returns, handle_results, NULL);
            SeqLocFree(slp1);
            SeqLocFree(slp2);
         }
@@ -634,48 +716,43 @@ Int2 Main (void)
            }
            ValNodeFree(error_returns);
         }
-        
-        if (!handle_results) {
-           dbinfo = NULL;
-           ka_params = NULL;
-           ka_params_gap = NULL;
-           params_buffer = NULL;
-           mask_loc = NULL;
-           matrix = NULL;
-           txmatrix = NULL;
-           for (vnp=other_returns; vnp; vnp = vnp->next) {
-              switch (vnp->choice) {
-              case TXDBINFO:
-                 dbinfo = vnp->data.ptrvalue;
-                 break;
-              case TXKABLK_NOGAP:
-                 ka_params = vnp->data.ptrvalue;
-                 break;
-              case TXKABLK_GAP:
-                 ka_params_gap = vnp->data.ptrvalue;
-                 break;
-              case TXPARAMETERS:
-                 params_buffer = vnp->data.ptrvalue;
-                 break;
-              case TXMATRIX:
-                 matrix = vnp->data.ptrvalue;
-                 if (matrix)
-                    txmatrix = BlastMatrixToTxMatrix(matrix);
-                 break;
-              case SEQLOC_MASKING_NOTSET:
-              case SEQLOC_MASKING_PLUS1:
-              case SEQLOC_MASKING_PLUS2:
-              case SEQLOC_MASKING_PLUS3:
-              case SEQLOC_MASKING_MINUS1:
-              case SEQLOC_MASKING_MINUS2:
-              case SEQLOC_MASKING_MINUS3:
-                 ValNodeAddPointer(&mask_loc, vnp->choice, vnp->data.ptrvalue);
-                 break;
-              default:
-                 break;
-              }
-           }	
-
+       
+        ka_params = NULL;
+        ka_params_gap = NULL;
+        params_buffer = NULL;
+        mask_loc = NULL;
+        matrix = NULL;
+        txmatrix = NULL;
+        for (vnp=other_returns; vnp; vnp = vnp->next) {
+           switch (vnp->choice) {
+           case TXKABLK_NOGAP:
+              ka_params = vnp->data.ptrvalue;
+              break;
+           case TXKABLK_GAP:
+              ka_params_gap = vnp->data.ptrvalue;
+              break;
+           case TXPARAMETERS:
+              params_buffer = vnp->data.ptrvalue;
+              break;
+           case TXMATRIX:
+              matrix = vnp->data.ptrvalue;
+              if (matrix && !tabular_output)
+                 txmatrix = BlastMatrixToTxMatrix(matrix);
+              break;
+           case SEQLOC_MASKING_NOTSET:
+           case SEQLOC_MASKING_PLUS1:
+           case SEQLOC_MASKING_PLUS2:
+           case SEQLOC_MASKING_PLUS3:
+           case SEQLOC_MASKING_MINUS1:
+           case SEQLOC_MASKING_MINUS2:
+           case SEQLOC_MASKING_MINUS3:
+              ValNodeAddPointer(&mask_loc, vnp->choice, vnp->data.ptrvalue);
+              break;
+           default:
+              break;
+           }
+        }	
+        if (!tabular_output || seqannot_output) {
            align_options = 0;
            align_options += TXALIGN_MATRIX_VAL;
            align_options += TXALIGN_SHOW_QS;
@@ -685,22 +762,15 @@ Int2 Main (void)
               align_options += TXALIGN_BLASTX_SPECIAL;
            }
            
-           if (myargs[17].intvalue)
+           if (html)
               align_options += TXALIGN_HTML;
-           
-           if (entrez_lookup)
-              AcknowledgeBlastQuery(query_bsp, 70, outfp, TRUE, myargs[17].intvalue);
-           else
-              AcknowledgeBlastQuery(query_bsp, 70, outfp, FALSE, myargs[17].intvalue);
-           
+
            seqannot = SeqAnnotNew();
            seqannot->type = 2;
            AddAlignInfoToSeqAnnot(seqannot, align_type);
            seqannot->data = seqalign;
-           ShowTextAlignFromAnnot(seqannot, 60, outfp, NULL, NULL, align_options, txmatrix, mask_loc, FormatScoreFunc);
-           
            aip = NULL;
-           if (myargs[6].strvalue)
+           if (seqannot_output)
               aip = AsnIoOpen (myargs[6].strvalue,"w");
            
            if (aip && seqannot) {
@@ -708,77 +778,59 @@ Int2 Main (void)
               AsnIoReset(aip);
               aip = AsnIoClose(aip);
            }
-           seqannot = SeqAnnotFree(seqannot);
+        }
+        if (!tabular_output) {    
+           AcknowledgeBlastQuery(query_bsp, 70, outfp, believe_query, html);
+           ShowTextAlignFromAnnot(seqannot, 60, outfp, NULL, NULL, align_options, txmatrix, mask_loc, FormatScoreFunc);
            
-           matrix = BLAST_MatrixDestruct(matrix);
+           seqannot = SeqAnnotFree(seqannot);
            if (txmatrix)
               txmatrix = TxMatrixDestruct(txmatrix);
            init_buff_ex(85);
-           dbinfo_head = dbinfo;
-           while (dbinfo) {
-              PrintDbReport(dbinfo, 70, outfp);
-              dbinfo = dbinfo->next;
-           }
-           dbinfo_head = TxDfDbInfoDestruct(dbinfo_head);
-           
+        
            if (ka_params) {
               PrintKAParameters(ka_params->Lambda, ka_params->K, ka_params->H, 70, outfp, FALSE);
-              MemFree(ka_params);
            }
-           
+        
            if (ka_params_gap) {
               PrintKAParameters(ka_params_gap->Lambda, ka_params_gap->K, ka_params_gap->H, 70, outfp, TRUE);
-              MemFree(ka_params_gap);
            }
-           
+        
            PrintTildeSepLines(params_buffer, 70, outfp);
-           MemFree(params_buffer);
            free_buff();
-           mask_loc_start = mask_loc;
-           while (mask_loc) {
-              SeqLocSetFree(mask_loc->data.ptrvalue);
-              mask_loc = mask_loc->next;
-           }
-           ValNodeFree(mask_loc_start);
-           
-           fake_bsp = BlastDeleteFakeBioseq(fake_bsp);
-        } else { 
-           /* Just destruct all other_returns parts */
-           for (vnp=other_returns; vnp; vnp = vnp->next) {
-              switch (vnp->choice) {
-              case TXDBINFO:
-                 TxDfDbInfoDestruct(vnp->data.ptrvalue);
-                 break;
-              case TXKABLK_NOGAP:
-              case TXKABLK_GAP:
-              case TXPARAMETERS:
-                 MemFree(vnp->data.ptrvalue);
-                 break;
-              case TXMATRIX:
-                 BLAST_MatrixDestruct(vnp->data.ptrvalue);
-                 break;
-              case SEQLOC_MASKING_NOTSET:
-              case SEQLOC_MASKING_PLUS1:
-              case SEQLOC_MASKING_PLUS2:
-              case SEQLOC_MASKING_PLUS3:
-              case SEQLOC_MASKING_MINUS1:
-              case SEQLOC_MASKING_MINUS2:
-              case SEQLOC_MASKING_MINUS3:
-                 SeqLocSetFree(vnp->data.ptrvalue);
-                 break;
-              default:
-                 break;
-              }
-           }
+        } else {
+           PrintTabularOutputHeader(NULL, query_bsp, NULL, 
+              program_name, 0, believe_query, outfp);
+
+           BlastPrintTabulatedResults(seqalign, query_bsp, NULL, 
+              1, program_name, !gapped_calculation,
+              believe_query, 0, 0, outfp, FALSE);
+           SeqAlignSetFree(seqalign);
         }
+
+        matrix = BLAST_MatrixDestruct(matrix);
+        MemFree(ka_params);
+        MemFree(ka_params_gap);
+        MemFree(params_buffer);
+    
+        mask_loc_start = mask_loc;
+        while (mask_loc) {
+           SeqLocSetFree(mask_loc->data.ptrvalue);
+           mask_loc = mask_loc->next;
+        }
+        ValNodeFree(mask_loc_start);
+        
+        fake_bsp = BlastDeleteFakeBioseq(fake_bsp);
+
         other_returns = ValNodeFree(other_returns);
+    options->query_lcase_mask = SeqLocSetFree(options->query_lcase_mask);
 	options = BLASTOptionDelete(options);
 	MemFree(program_name);
 	FileClose(outfp);
 
         if (entrez_lookup) {
-           BioseqUnlock(query_bsp);
-           BioseqUnlock(subject_bsp);
+           BioseqFree(query_bsp);
+           BioseqFree(subject_bsp);
         } else {
            SeqEntryFree(sep);
            SeqEntryFree(sep1);
