@@ -1,4 +1,6 @@
-/* $Id: megablast.c,v 6.83 2001/12/28 20:42:16 dondosha Exp $
+static char const rcsid[] = "$Id: megablast.c,v 6.114 2004/05/27 17:37:30 dondosha Exp $";
+
+/* $Id: megablast.c,v 6.114 2004/05/27 17:37:30 dondosha Exp $
 **************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -26,6 +28,102 @@
 ************************************************************************** 
  * $Revision 6.13$ *  
  * $Log: megablast.c,v $
+ * Revision 6.114  2004/05/27 17:37:30  dondosha
+ * Do not call GapXEditBlockDelete in formatting callback - this is now done when HSPs are freed
+ *
+ * Revision 6.113  2004/04/29 19:56:00  dondosha
+ * Mask filtered locations in query sequence lines in XML output
+ *
+ * Revision 6.112  2004/03/24 22:55:54  dondosha
+ * Ignore -m option when on-the-fly tabular output is returned
+ *
+ * Revision 6.111  2004/01/27 20:47:18  dondosha
+ * Set no_traceback option to 2 for -D4, 1 for -D0, 0 for others
+ *
+ * Revision 6.110  2003/11/05 22:28:06  dondosha
+ * No need to shift subsequence coordinates in tabular output, since they are already shifted in the seqalign
+ *
+ * Revision 6.109  2003/10/30 17:30:49  dondosha
+ * Fixed error in previous commit - output file pointer has to be set for all cases
+ *
+ * Revision 6.108  2003/10/29 17:48:15  dondosha
+ * Added -D4 option for 2-stage greedy gapped extension with format identical to -D2
+ *
+ * Revision 6.107  2003/05/30 17:31:09  coulouri
+ * add rcsid
+ *
+ * Revision 6.106  2003/05/13 16:02:42  coulouri
+ * make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+ *
+ * Revision 6.105  2003/05/06 18:57:46  dondosha
+ * Do not set cutoff_s for megablast, it is not needed
+ *
+ * Revision 6.104  2003/05/06 15:18:22  dondosha
+ * Fix in callback function for the -D1 option
+ *
+ * Revision 6.103  2003/03/20 13:44:24  madden
+ * Fix -m 10/11 output to make them SeqAnnots
+ *
+ * Revision 6.102  2003/03/13 15:26:15  dondosha
+ * Close output file; call ReadDBBioseqFetchEnable only when non-empty seqalign is found
+ *
+ * Revision 6.101  2002/12/31 22:47:16  boemker
+ * Added support for printing output as ASN (text, with -m 10, or binary, with
+ * -m 11).
+ *
+ * Revision 6.100  2002/08/09 19:41:25  camacho
+ * 1) Added blast version number to command-line options
+ * 2) Added explanations for some default parameters
+ *
+ * Revision 6.99  2002/07/18 19:44:40  dondosha
+ * Clarification in the description of max. HSPs option
+ *
+ * Revision 6.98  2002/07/18 19:40:45  dondosha
+ * Added an option to restrict number of HSPs per database sequence
+ *
+ * Revision 6.97  2002/07/17 22:28:13  dondosha
+ * Added support for megablast XML output
+ *
+ * Revision 6.96  2002/06/19 22:50:17  dondosha
+ * Added all queries information for tabular output with multiple queries
+ *
+ * Revision 6.95  2002/05/14 22:21:36  dondosha
+ * Renamed maximal discontiguous template type into optimal
+ *
+ * Revision 6.94  2002/05/09 15:37:52  dondosha
+ * Call BLASTOptionNewEx instead of BLASTOptionNew, so megablast defaults are set in a central place
+ *
+ * Revision 6.93  2002/05/08 22:46:21  dondosha
+ * Allocate query bioseq array 1 longer to make sure last ellement is NULL
+ *
+ * Revision 6.92  2002/05/04 13:04:43  madden
+ * Unsuppress options
+ *
+ * Revision 6.91  2002/04/29 19:55:25  madden
+ * Use ARG_FLOAT for db length
+ *
+ * Revision 6.90  2002/04/25 21:57:46  madden
+ * Strip options for release
+ *
+ * Revision 6.89  2002/04/24 19:55:14  madden
+ * Rolled back last change
+ *
+ * Revision 6.88  2002/04/23 20:58:53  madden
+ * Suppress options for release
+ *
+ * Revision 6.87  2002/04/09 18:17:34  dondosha
+ * Added a discontiguous word type option
+ *
+ * Revision 6.86  2002/03/08 20:22:56  dondosha
+ * Bug fix: masking locations were freed incorrectly in some cases
+ *
+ * Revision 6.85  2002/03/06 18:34:32  dondosha
+ * Pass the filtered locations back from the megablast engine to use in formatting
+ *
+ * Revision 6.84  2002/02/15 23:30:00  dondosha
+ * 1. Fix for very long queries
+ * 2. Added -m8 and -m9 options - needed in conjunction with -Q option.
+ *
  * Revision 6.83  2001/12/28 20:42:16  dondosha
  * Added options related to discontiguous words
  *
@@ -319,7 +417,8 @@ enum {
    MBLAST_ENDPOINTS = 0,
    MBLAST_SEGMENTS,
    MBLAST_ALIGNMENTS,
-   MBLAST_ALIGN_INFO
+   MBLAST_ALIGN_INFO,
+   MBLAST_DELAYED_TRACEBACK
 };
 
 static int LIBCALLBACK
@@ -335,7 +434,7 @@ MegaBlastPrintEndpoints(VoidPtr ptr)
    Int4 hsp_index;
    Boolean numeric_sip_type = FALSE;
    BLAST_HSPPtr hsp; 
-   Int2 context, descr_len;
+   Int2 context;
    Char context_sign;
    Uint4 header_index = 0;
    Int4 subject_gi, score;
@@ -503,13 +602,12 @@ MegaBlastPrintSegments(VoidPtr ptr)
    Int2 context;
    CharPtr query_buffer, title;
    SeqIdPtr sip, query_id; 
-   Int4 descr_len;
-   Int4 index, hsp_index, score;
+   Int4 hsp_index, score;
    Uint1Ptr query_seq, subject_seq = NULL;
    FloatHi perc_ident;
    Char strand;
    GapXEditScriptPtr esp;
-   Int4 q_start, q_end, s_start, s_end, query_length, subj_length, numseg;
+   Int4 q_start, q_end, s_start, s_end, query_length, numseg;
    Int4 q_off, num_ident, align_length, total_ident, q_shift=0, s_shift=0;
    Int4Ptr length, start;
    Uint1Ptr strands;
@@ -527,8 +625,8 @@ MegaBlastPrintSegments(VoidPtr ptr)
    subject_seq = search->subject->sequence_start + 1;
 
 
-   if (search->rdfp)
-      readdb_get_descriptor(search->rdfp, search->subject_id, &sip, &subject_descr);
+   if (rdfp)
+      readdb_get_descriptor(rdfp, search->subject_id, &sip, &subject_descr);
    else 
       sip = SeqIdSetDup(search->subject_info->sip);
 
@@ -549,7 +647,7 @@ MegaBlastPrintSegments(VoidPtr ptr)
 
    /* Only for the two sequences case, get offset shift if subject 
       is a subsequence */
-   if (!search->rdfp && search->query_slp->next)
+   if (!rdfp && search->query_slp->next)
        s_shift = SeqLocStart(search->query_slp->next);
    /* Get offset shift if query is a subsequence */
    q_shift = SeqLocStart(search->query_slp);
@@ -558,7 +656,6 @@ MegaBlastPrintSegments(VoidPtr ptr)
       hsp = search->current_hitlist->hsp_array[hsp_index];
       if (hsp==NULL || (search->pbp->cutoff_e > 0 && 
                         hsp->evalue > search->pbp->cutoff_e)) {
-         GapXEditBlockDelete(hsp->gap_info); /* Don't need it anymore */
 	 continue;
       }
       context = hsp->context;
@@ -653,7 +750,6 @@ MegaBlastPrintSegments(VoidPtr ptr)
       GXECollectDataForSeqalign(hsp->gap_info, hsp->gap_info->esp, numseg,
 				&start, &length, &strands, 
 				&q_off, &hsp->subject.offset);
-      GapXEditBlockDelete(hsp->gap_info); /* Don't need it anymore */
 
       if (start[0] < 0) {
          length[0] += start[0];
@@ -690,12 +786,11 @@ MegaBlastPrintSegments(VoidPtr ptr)
 	    StringCat(buffer, tmp_buffer);
 	 }
       }
-      if (100*total_ident < align_length*search->pbp->mb_params->perc_identity) {
-         GapXEditBlockDelete(hsp->gap_info); /* Don't need it anymore */
-         continue;
+      if (100*total_ident >= 
+          align_length*search->pbp->mb_params->perc_identity) {
+        StringCat(buffer, "}");
+        fprintf(fp, "%s\n", buffer);
       }
-      StringCat(buffer, "}");
-      fprintf(fp, "%s\n", buffer);
       MemFree(start);
       MemFree(length);
       MemFree(strands);
@@ -710,16 +805,18 @@ MegaBlastPrintSegments(VoidPtr ptr)
    return 1;
 }
 
+#define DO_NOT_SUPPRESS_BLAST_OP
+
 #define NUMARG (sizeof(myargs)/sizeof(myargs[0]))
 
 static Args myargs [] = {
   { "Database", 
     "nr", NULL, NULL, FALSE, 'd', ARG_STRING, 0.0, 0, NULL},       /* 0 */
   { "Query File", 
-	"stdin", NULL, NULL, FALSE, 'i', ARG_FILE_IN, 0.0, 0, NULL},/* 1 */
+	NULL, NULL, NULL, FALSE, 'i', ARG_FILE_IN, 0.0, 0, NULL},/* 1 */
   { "Expectation value", 
 	"1000000.0", NULL, NULL, FALSE, 'e', ARG_FLOAT, 0.0, 0, NULL},/* 2 */
-  { "alignment view options:\n0 = pairwise,\n1 = query-anchored showing identities,\n2 = query-anchored no identities,\n3 = flat query-anchored, show identities,\n4 = flat query-anchored, no identities,\n5 = query-anchored no identities and blunt ends,\n6 = flat query-anchored, no identities and blunt ends,\n7 = XML Blast output", 
+  { "alignment view options:\n0 = pairwise,\n1 = query-anchored showing identities,\n2 = query-anchored no identities,\n3 = flat query-anchored, show identities,\n4 = flat query-anchored, no identities,\n5 = query-anchored no identities and blunt ends,\n6 = flat query-anchored, no identities and blunt ends,\n7 = XML Blast output,\n8 = tabular, \n9 tabular with comment lines,\n10 ASN, text\n11 ASN, binary", 
         "0", NULL, NULL, FALSE, 'm', ARG_INT, 0.0, 0, NULL},       /* 3 */
   { "BLAST report Output File", 
 	"stdout", NULL, NULL, TRUE, 'o', ARG_FILE_OUT, 0.0, 0, NULL},/* 4 */
@@ -741,7 +838,7 @@ static Args myargs [] = {
         "0", NULL, NULL, FALSE, 'D', ARG_INT, 0.0, 0, NULL},       /* 12 */
   { "Number of processors to use",
         "1", NULL, NULL, FALSE, 'a', ARG_INT, 0.0, 0, NULL},       /* 13 */
-  { "ASN.1 SeqAlign file", 
+  { "ASN.1 SeqAlign file; must be used in conjunction with -D2 option", 
 	NULL, NULL, NULL, TRUE, 'O', ARG_FILE_OUT, 0.0, 0, NULL},  /* 14 */
   { "Believe the query defline",
         NULL, NULL, NULL, TRUE, 'J', ARG_STRING, 0.0, 0, NULL},    /* 15 */
@@ -750,7 +847,7 @@ static Args myargs [] = {
   { "Word size (length of best perfect match)", 
         "28", NULL, NULL, FALSE, 'W', ARG_INT, 0.0, 0, NULL},      /* 17 */
   { "Effective length of the database (use zero for the real size)", 
-        "0", NULL, NULL, FALSE, 'z', ARG_STRING, 0.0, 0, NULL},     /* 18 */
+        "0", NULL, NULL, FALSE, 'z', ARG_FLOAT, 0.0, 0, NULL},     /* 18 */
   { "Maximal number of positions for a hash value (set to 0 to ignore)",
         "0", NULL, NULL, FALSE, 'P', ARG_INT, 0.0, 0, NULL},       /* 19 */
   { "Query strands to search against database: 3 is both, 1 is top, 2 is bottom",
@@ -777,12 +874,25 @@ static Args myargs [] = {
         "0", NULL, NULL, FALSE, 'p', ARG_FLOAT, 0.0, 0, NULL},     /* 30 */
   { "Location on query sequence",                             
         NULL, NULL, NULL, TRUE, 'L', ARG_STRING, 0.0, 0, NULL},    /* 31 */
-  { "Multiple Hits window size (zero for single hit algorithm)",   /* 32 */
+  { "Multiple Hits window size",   /* 32 */
         "0", NULL, NULL, FALSE, 'A', ARG_INT, 0.0, 0, NULL},
   { "X dropoff value for ungapped extension",
 	"10", NULL, NULL, FALSE, 'y', ARG_INT, 0.0, 0, NULL},      /* 33 */
   { "X dropoff value for dynamic programming gapped extension",
-	"50", NULL, NULL, FALSE, 'Z', ARG_INT, 0.0, 0, NULL}       /* 34 */
+	"50", NULL, NULL, FALSE, 'Z', ARG_INT, 0.0, 0, NULL},       /* 34 */
+#ifdef DO_NOT_SUPPRESS_BLAST_OP
+  { "Length of a discontiguous word template (contiguous word if 0)",
+	"0", NULL, NULL, FALSE, 't', ARG_INT, 0.0, 0, NULL},       /* 35 */
+  {"Generate words for every base of the database (default is every 4th base)",
+        "F", NULL, NULL, TRUE, 'g', ARG_BOOLEAN, 0.0, 0, NULL},    /* 36 */
+  {"Use non-greedy (dynamic programming) extension for affine gap scores",
+        "F", NULL, NULL, TRUE, 'n', ARG_BOOLEAN, 0.0, 0, NULL},    /* 37 */
+  { "Type of a discontiguous word template (0 - coding, 1 - optimal, 2 - two simultaneous",
+	"0", NULL, NULL, FALSE, 'N', ARG_INT, 0.0, 0, NULL},       /* 38 */
+  { "Maximal number of HSPs to save per database sequence (0 = unlimited)",
+	"0", NULL, NULL, FALSE, 'H', ARG_INT, 0.0, 0, NULL}        /* 39 */
+#endif
+
 };
 
 #define MAX_NUM_QUERIES 16383 /* == 1/2 INT2_MAX */
@@ -790,36 +900,43 @@ static Args myargs [] = {
 Int2 Main (void)
  
 {
-	AsnIoPtr aip, xml_aip;
-	BioseqPtr query_bsp, PNTR query_bsp_array;
-	BioSourcePtr source;
-	BLAST_MatrixPtr matrix;
-	BLAST_OptionsBlkPtr options;
-	BLAST_KarlinBlkPtr ka_params=NULL, ka_params_gap=NULL;
-	BlastPruneSapStructPtr prune;
-	Boolean db_is_na, query_is_na, show_gi, believe_query=FALSE;
-	Boolean html=FALSE;
-	CharPtr params_buffer=NULL;
-	Int4 number_of_descriptions, number_of_alignments;
-	SeqAlignPtr  seqalign, PNTR seqalign_array;
-        SeqAnnotPtr seqannot;
-	SeqEntryPtr PNTR sepp;
-	TxDfDbInfoPtr dbinfo=NULL, dbinfo_head;
-	Uint1 align_type, align_view;
-	Uint4 align_options, print_options;
-	ValNodePtr  mask_loc, mask_loc_start, vnp, other_returns, error_returns;
-
-	CharPtr blast_program, blast_database, blast_inputfile, blast_outputfile;
-	FILE *infp, *outfp;
-	Int4 index, num_bsps, total_length, total_processed = 0;
-	Int2 ctr = 1;
-	Char prefix[2];
-	SeqLocPtr last_mask, mask_slp;
-	Boolean done, first_seq = TRUE, hits_found;
-	CharPtr masked_query_file;
-        const char *dummystr;
-
-        if (! GetArgs ("megablast", NUMARG, myargs))
+   AsnIoPtr aip, xml_aip;
+   BioseqPtr query_bsp, PNTR query_bsp_array;
+   BioSourcePtr source;
+   BLAST_MatrixPtr matrix;
+   BLAST_OptionsBlkPtr options;
+   BLAST_KarlinBlkPtr ka_params=NULL, ka_params_gap=NULL;
+   BlastPruneSapStructPtr prune;
+   Boolean db_is_na, query_is_na, show_gi, believe_query=FALSE;
+   Boolean html=FALSE;
+   CharPtr params_buffer=NULL;
+   Int4 number_of_descriptions, number_of_alignments;
+   SeqAlignPtr  seqalign, PNTR seqalign_array;
+   SeqAnnotPtr seqannot;
+   SeqEntryPtr PNTR sepp;
+   TxDfDbInfoPtr dbinfo=NULL, dbinfo_head;
+   Uint1 align_type, align_view;
+   Uint4 align_options, print_options;
+   ValNodePtr mask_loc, mask_loc_start, next_mask_loc;
+   ValNodePtr vnp, other_returns, error_returns;
+   
+   CharPtr blast_program, blast_database, blast_inputfile, blast_outputfile;
+   FILE *infp, *outfp, *mqfp;
+   Int4 index, num_bsps, total_length, total_processed = 0;
+   Int2 ctr = 1;
+   Char prefix[2];
+   Char buf[256] = { '\0' };
+   SeqLocPtr last_mask, mask_slp;
+   Boolean done, first_seq = TRUE, hits_found;
+   CharPtr masked_query_file;
+   Boolean lcase_masking;
+   BlastOutputPtr boutp = NULL;
+   MBXmlPtr mbxp = NULL;
+   Boolean traditional_formatting;
+   
+    StringCpy(buf, "megablast ");
+    StringNCat(buf, BlastGetVersionNumber(), sizeof(buf)-StringLen(buf)-1);
+    if (! GetArgs (buf, NUMARG, myargs))
 	   return (1);
 
 	UseLocalAsnloadDataAndErrMsg ();
@@ -837,39 +954,43 @@ Int2 Main (void)
 		html = TRUE;
 
 	if ((infp = FileOpen(blast_inputfile, "r")) == NULL) {
-	   ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open input file %s\n", blast_inputfile);
+	   ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open input file %s\n", blast_inputfile);
 	   return (1);
 	}
 
+	align_view = (Int1) myargs[3].intvalue;
 	outfp = NULL;
-	if (blast_outputfile != NULL) {
+        traditional_formatting = 
+           (myargs[12].intvalue == MBLAST_ALIGNMENTS ||
+            myargs[12].intvalue == MBLAST_DELAYED_TRACEBACK);
+	if ((!traditional_formatting ||
+            (align_view != 7 && align_view != 10 && align_view != 11)) && 
+            blast_outputfile != NULL) {
 	   if ((outfp = FileOpen(blast_outputfile, "w")) == NULL) {
-	      ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open output file %s\n", blast_outputfile);
+	      ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", blast_outputfile);
 	      return (1);
 	   }
 	}
 
-	align_view = (Int1) myargs[3].intvalue;
-
 	align_type = BlastGetTypes(blast_program, &query_is_na, &db_is_na);
- 
+
         if (myargs[15].strvalue) {
            if (myargs[15].strvalue[0] == 'f' || myargs[15].strvalue[0] == 'F' ||
                myargs[15].strvalue[0] == '0')
               believe_query = FALSE;
            else
               believe_query = TRUE;
-        } else if (myargs[12].intvalue == MBLAST_ALIGNMENTS && 
+        } else if (traditional_formatting && 
                    !myargs[14].strvalue)
            believe_query = FALSE;
         else
            believe_query = TRUE;
         
         
-	if (believe_query == FALSE && myargs[14].strvalue) 
-	   ErrPostEx(SEV_FATAL, 0, 0, "-J option must be TRUE to produce a SeqAlign file");
+	if (believe_query == FALSE && (myargs[14].strvalue || align_view == 10 || align_view == 11)) 
+	   ErrPostEx(SEV_FATAL, 1, 0, "-J option must be TRUE to produce a SeqAlign file");
 
-	options = BLASTOptionNew(blast_program, TRUE);
+	options = BLASTOptionNewEx(blast_program, TRUE, TRUE);
 	if (options == NULL)
 		return 3;
 
@@ -913,9 +1034,7 @@ Int2 Main (void)
         else 
            options->cutoff_s2 = myargs[25].intvalue;
 
-	options->cutoff_s = (options->wordsize + 4)*options->reward;
-
-        options->db_length = StringToInt8(myargs[18].strvalue, &dummystr);
+        options->db_length = (Int8) myargs[18].floatvalue;
 
 	options->perform_culling = FALSE;
 	/* Kludge */
@@ -923,6 +1042,16 @@ Int2 Main (void)
 
 	options->strand_option = myargs[20].intvalue;
         options->window_size = myargs[32].intvalue;
+#ifdef DO_NOT_SUPPRESS_BLAST_OP        
+        options->mb_template_length = myargs[35].intvalue;
+        options->mb_one_base_step = (Boolean) myargs[36].intvalue;
+        options->mb_disc_type = myargs[38].intvalue;
+#endif
+        lcase_masking = (Boolean) myargs[28].intvalue;
+        /* Allow dynamic programming gapped extension only with affine 
+           gap scores */
+        if (options->gap_open != 0 || options->gap_extend != 0)
+           options->mb_use_dyn_prog = (Boolean) myargs[37].intvalue;
 
         print_options = 0;
         align_options = 0;
@@ -954,30 +1083,32 @@ Int2 Main (void)
 	if (myargs[22].strvalue)
 	   options->gifile = StringSave(myargs[22].strvalue);
    
-	options->is_megablast_search = TRUE;
 	if (myargs[12].intvalue == MBLAST_ENDPOINTS)
-	   options->no_traceback = TRUE;
+      options->no_traceback = 1;
+   else if (myargs[12].intvalue == MBLAST_DELAYED_TRACEBACK)
+	   options->no_traceback = 2;
 	else
-	   options->no_traceback = FALSE;
+	   options->no_traceback = 0;
 
 	options->megablast_full_deflines = (Boolean) myargs[27].intvalue;
         options->perc_identity = (FloatLo) myargs[30].floatvalue;
+        options->hsp_num_max = myargs[39].intvalue;
+
         if (!believe_query)
            options->megablast_full_deflines = TRUE;
         /*if (options->megablast_full_deflines)
           believe_query = FALSE;*/
 
-	query_bsp_array = (BioseqPtr PNTR) MemNew(MAX_NUM_QUERIES*sizeof(BioseqPtr));
+	query_bsp_array = (BioseqPtr PNTR) MemNew((MAX_NUM_QUERIES+1)*sizeof(BioseqPtr));
 	sepp = (SeqEntryPtr PNTR) MemNew(MAX_NUM_QUERIES*sizeof(SeqEntryPtr));
 
 	StrCpy(prefix, "");
 
 	global_fp = outfp;
-        if (myargs[12].intvalue != MBLAST_ALIGNMENTS)
-           options->output = outfp;
+        options->output = outfp;
 
-	if (myargs[12].intvalue==MBLAST_ALIGNMENTS) {
-	   if (align_view != 7) {
+	if (traditional_formatting) {
+	   if (align_view < 7) {
               if (html) {
                  fprintf(outfp, "<HTML>\n<TITLE>MEGABLAST Search Results</TITLE>\n");
                  fprintf(outfp, "<BODY BGCOLOR=\"#FFFFFF\" LINK=\"#0000FF\" "
@@ -1004,13 +1135,23 @@ Int2 Main (void)
         aip = NULL;
         if (myargs[14].strvalue != NULL) {
            if ((aip = AsnIoOpen (myargs[14].strvalue,"w")) == NULL) {
-              ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open output file %s\n", "blastngp.sat");
+              ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", myargs[14].strvalue);
               return 1;
            }
         }
+    	else if (align_view == 10 || align_view == 11)
+    	{
+        	const char* mode = (align_view == 10) ? "w" : "wb";
+        	if ((aip = AsnIoOpen (blast_outputfile, (char*) mode)) == NULL) {
+                	ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", blast_outputfile);
+                	return 1;
+        	}
+    	}
 
-        if (align_view == 7)
+
+        if (align_view == 7) {
            xml_aip = AsnIoOpen(blast_outputfile, "wx");
+        }
 
         if (myargs[31].strvalue) {       
             CharPtr delimiters = " ,;";
@@ -1032,7 +1173,7 @@ Int2 Main (void)
 	   while ((sepp[num_bsps]=FastaToSeqEntryForDb(infp, query_is_na, NULL,
 						       believe_query, prefix, &ctr, 
 						       &mask_slp)) != NULL) {
-              if (!myargs[28].intvalue) /* Lower case ignored */
+              if (!lcase_masking) /* Lower case ignored */
                  mask_slp = SeqLocFree(mask_slp);
 	      if (mask_slp) {
 		 if (!last_mask)
@@ -1047,7 +1188,7 @@ Int2 Main (void)
               SeqEntryExplore(sepp[num_bsps], &query_bsp, FindNuc);
 
 	      if (query_bsp == NULL) {
-		 ErrPostEx(SEV_FATAL, 0, 0, "Unable to obtain bioseq\n");
+		 ErrPostEx(SEV_FATAL, 1, 0, "Unable to obtain bioseq\n");
 		 return 2;
 	      }
 	      
@@ -1066,11 +1207,14 @@ Int2 Main (void)
 		 break;
 	      }
 	   }
+
+           if (num_bsps == 0)
+               break;
+
 	   SeqMgrHoldIndexing(FALSE);
 	   other_returns = NULL;
 	   error_returns = NULL;
 	   
-	   ReadDBBioseqFetchEnable ("megablast", blast_database, db_is_na, TRUE);
 #if 0
 	   fprintf(stderr, "Process %d queries with total length %ld\n", 
 		   num_bsps, total_length);
@@ -1100,7 +1244,7 @@ Int2 Main (void)
 	   } else /* if (myargs[12].intvalue==MBLAST_ALIGNMENTS) */
 	      seqalign_array = BioseqMegaBlastEngine(query_bsp_array, blast_program,
 				  blast_database, options, &other_returns, 
-                                  &error_returns, align_view != 7 ? tick_callback : NULL,
+                                  &error_returns, align_view < 7 ? tick_callback : NULL,
                                   NULL, NULL, 0, NULL);
 	   
 #ifdef OS_UNIX
@@ -1116,7 +1260,7 @@ Int2 Main (void)
            }
               
               
-	   if (myargs[12].intvalue==MBLAST_ALIGNMENTS) {
+	   if (traditional_formatting) {
 	      dbinfo = NULL;
 	      ka_params = NULL;
 	      ka_params_gap = NULL;
@@ -1147,7 +1291,7 @@ Int2 Main (void)
 		 case SEQLOC_MASKING_MINUS1:
 		 case SEQLOC_MASKING_MINUS2:
 		 case SEQLOC_MASKING_MINUS3:
-		    /*ValNodeAddPointer(&mask_loc, vnp->choice, vnp->data.ptrvalue);*/
+		    ValNodeAddPointer(&mask_loc, vnp->choice, vnp->data.ptrvalue);
 		    break;
 		 default:
 		    break;
@@ -1155,7 +1299,7 @@ Int2 Main (void)
 	      }	
 	      
 #ifdef OS_UNIX
-              if(align_view != 7) {
+              if(align_view < 7) {
                  fprintf(global_fp, "%s\n", " done");
               }
 #endif
@@ -1163,32 +1307,77 @@ Int2 Main (void)
               masked_query_file = myargs[26].strvalue;
               hits_found = FALSE;
 
+              mask_loc_start = next_mask_loc = mask_loc;
+              mask_loc = NULL;
+
+              if (align_view == 7) {
+                 mbxp = PSIXmlInit(xml_aip, "megablast", blast_database, 
+                                   options, query_bsp_array[0], 0);
+              }
+
               if (seqalign_array) {
+	         ReadDBBioseqFetchEnable ("megablast", blast_database, db_is_na, TRUE);
                  for (index=0; index<num_bsps; index++) {
-                    if (seqalign_array[index]==NULL) continue;
+                    seqalign = seqalign_array[index];
+                    if (next_mask_loc && 
+                        SeqIdComp(SeqLocId((SeqLocPtr)next_mask_loc->data.ptrvalue), 
+                                  query_bsp_array[index]->id) == SIC_YES) {
+                       mask_loc = (SeqLocPtr) 
+                          MemDup(next_mask_loc, sizeof(SeqLoc));
+                       next_mask_loc = next_mask_loc->next;
+                       mask_loc->next = NULL;
+                    }
+                    if (masked_query_file) {
+                       mask_slp = MaskSeqLocFromSeqAlign(seqalign);
+                       if (mask_loc) 
+                          mask_slp = blastMergeFilterLocs(mask_slp, 
+                              (SeqLocPtr)mask_loc->data.ptrvalue,
+                              FALSE, 0, 0);
+                       if (first_seq)
+                          mqfp = FileOpen(masked_query_file, "w");
+                       PrintMaskedSequence(query_bsp_array[index], mask_slp,
+                                           mqfp, 50, lcase_masking);
+                       first_seq = FALSE;
+                       SeqLocSetFree(mask_slp);
+                    }
+                    if (seqalign==NULL) {
+                       mask_loc = MemFree(mask_loc);
+                       continue;
+                    }
                     hits_found = TRUE;
-                    if (align_view != 7) {
+                    if (align_view < 7) {
                        init_buff_ex(70);
                        AcknowledgeBlastQuery(query_bsp_array[index], 70, outfp, 
                                              believe_query, html);
                        free_buff();
                     }
-                    seqalign = seqalign_array[index];
-                    if (masked_query_file) {
-                       SeqLocPtr query_slp = NULL;
-                       ValNodeAddPointer(&query_slp, SEQLOC_WHOLE, SeqIdDup(SeqIdFindBest(query_bsp_array[index]->id, SEQID_GI)));
-                       mask_slp = MaskSeqLocFromSeqAlign(seqalign);
-                       PrintMaskedSequence(query_bsp_array[index], mask_slp,
-                                           masked_query_file, first_seq);
-                       first_seq = FALSE;
-                       SeqLocSetFree(mask_slp);
-                    }
-                    if(align_view == 7) {
-                       BXMLPrintOutput(xml_aip, seqalign, 
-                                       options, "megablast", blast_database, 
-                                       query_bsp_array[index], other_returns, 0, NULL);
-                       AsnIoReset(xml_aip);
+                    if (align_view == 8 || align_view == 9) {
+                       if (align_view == 9)
+                          PrintTabularOutputHeader(blast_database, 
+                             query_bsp_array[index], NULL, blast_program, 0,
+                             believe_query, global_fp);
+
+                       BlastPrintTabulatedResults(seqalign, 
+                          query_bsp_array[index], NULL, number_of_alignments,
+                          blast_program, !options->gapped_calculation, 
+                          believe_query, 0, 0, 
+                          global_fp, (align_view == 9));
+
                        SeqAlignSetFree(seqalign);
+                       mask_loc = MemFree(mask_loc);
+                       continue;
+                    } else if(align_view == 7) {
+                       IterationPtr iterp;
+
+                       iterp = BXMLBuildOneQueryIteration(seqalign, 
+                                  NULL, FALSE, 
+                                  !options->gapped_calculation, index, 
+                                  NULL, query_bsp_array[index], mask_loc);
+                       IterationAsnWrite(iterp, mbxp->aip, mbxp->atp);
+                       AsnIoFlush(mbxp->aip);
+                       IterationFree(iterp);
+                       SeqAlignSetFree(seqalign);
+                       mask_loc = MemFree(mask_loc);
                        continue;
                     }
                     seqannot = SeqAnnotNew();
@@ -1221,10 +1410,19 @@ Int2 Main (void)
                        ObjMgrFreeCache(0);
                     }
                     seqannot = SeqAnnotFree(seqannot);
+                    mask_loc = MemFree(mask_loc);
                  } /* End loop on seqaligns for different queries */
+                 ReadDBBioseqFetchDisable();
               } 
 
-              if (!hits_found)
+              if (mbxp != NULL) {
+                 MBXmlClose(mbxp, other_returns, !options->gapped_calculation);
+              }
+
+              if (masked_query_file)
+                 FileClose(mqfp);
+
+              if (!hits_found && align_view < 7)
                  fprintf(outfp, "\n\n ***** No hits found ******\n\n");
 
               matrix = BLAST_MatrixDestruct(matrix);
@@ -1233,7 +1431,7 @@ Int2 Main (void)
                  fprintf(outfp, "<PRE>\n");
               init_buff_ex(85);
               dbinfo_head = dbinfo;
-              if(align_view != 7) {
+              if(align_view < 7) {
                  while (dbinfo) {
                     PrintDbReport(dbinfo, 70, outfp);
                     dbinfo = dbinfo->next;
@@ -1242,27 +1440,25 @@ Int2 Main (void)
               dbinfo_head = TxDfDbInfoDestruct(dbinfo_head);
               
               if (ka_params) {
-                 if(align_view != 7)
+                 if(align_view < 7)
                     PrintKAParameters(ka_params->Lambda, ka_params->K, ka_params->H, 70, outfp, FALSE);
                  MemFree(ka_params);
               }
               if (ka_params_gap) {
-                 if(align_view != 7)
+                 if(align_view < 7)
                     PrintKAParameters(ka_params_gap->Lambda, ka_params_gap->K, ka_params_gap->H, 70, outfp, TRUE);
                  MemFree(ka_params_gap);
               }
-              if(align_view != 7)
+              if(align_view < 7)
                  PrintTildeSepLines(params_buffer, 70, outfp);
               MemFree(params_buffer);
               free_buff();
-              mask_loc_start = mask_loc;
+              mask_loc = mask_loc_start;
               while (mask_loc) {
                  SeqLocSetFree(mask_loc->data.ptrvalue);
                  mask_loc = mask_loc->next;
               }
               ValNodeFree(mask_loc_start);
-              
-              ReadDBBioseqFetchDisable();
 	   } else {
 	      /* Just destruct all other_returns parts */
 	      for (vnp=other_returns; vnp; vnp = vnp->next) {
@@ -1278,6 +1474,15 @@ Int2 Main (void)
 		 case TXMATRIX:
 		    BLAST_MatrixDestruct(vnp->data.ptrvalue);
 		    break;
+		 case SEQLOC_MASKING_NOTSET:
+		 case SEQLOC_MASKING_PLUS1:
+		 case SEQLOC_MASKING_PLUS2:
+		 case SEQLOC_MASKING_PLUS3:
+		 case SEQLOC_MASKING_MINUS1:
+		 case SEQLOC_MASKING_MINUS2:
+		 case SEQLOC_MASKING_MINUS3:
+                    mask_loc = vnp->data.ptrvalue;
+                    SeqLocSetFree(mask_loc);
 		 default:
 		    break;
 		 }
@@ -1301,18 +1506,19 @@ Int2 Main (void)
         
         aip = AsnIoClose(aip);
 
-        if (align_view == 7)
-           xml_aip = AsnIoClose(xml_aip);
+        /*if (align_view == 7)
+          xml_aip = AsnIoClose(xml_aip);*/
 
-        if (align_view != 7 && html) 
+        if (align_view < 7 && html) 
            fprintf(outfp, "</PRE>\n</BODY>\n</HTML>\n");
-        if (align_view != 7 && myargs[29].intvalue)
+        if (align_view < 7 && myargs[29].intvalue)
            fprintf(outfp, "Mega BLAST run finished, processed %d queries\n",
                    total_processed);
 	MemFree(query_bsp_array);
 	MemFree(sepp);
 	options = BLASTOptionDelete(options);
 	FileClose(infp);
+        FileClose(outfp);
 	
 	return 0;
 }
