@@ -1,4 +1,6 @@
-/* $Id: profiles.c,v 6.32 2000/11/20 14:37:20 madden Exp $
+static char const rcsid[] = "$Id: profiles.c,v 6.36 2003/05/30 17:31:10 coulouri Exp $";
+
+/* $Id: profiles.c,v 6.36 2003/05/30 17:31:10 coulouri Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -34,9 +36,21 @@ Contents: main routines for impala program to search a database of
   PSI-BLAST-generated position-specific score matrices
 
 =======
- $Revision: 6.32 $
+ $Revision: 6.36 $
 
  $Log: profiles.c,v $
+ Revision 6.36  2003/05/30 17:31:10  coulouri
+ add rcsid
+
+ Revision 6.35  2003/05/13 16:02:42  coulouri
+ make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+
+ Revision 6.34  2002/10/01 20:46:34  madden
+ Comment out call to SeqEntryFree that generates warning from ObjMgr when -J is set to T
+
+ Revision 6.33  2001/12/28 18:02:33  dondosha
+ Keep score and scoreThisAlign for each local alignment, so as to allow tie-breaking by score
+
  Revision 6.32  2000/11/20 14:37:20  madden
  Changed FileOpen mode for byte-encoded checkpoint files from "r" to "rb" or from "w" to "wb" to solve a problem on Windows NT.
 
@@ -190,7 +204,7 @@ static Uint1 *  readSequence(FILE * sequenceFile, Int4 * sequenceLength, SeqIdPt
        query_bsp = NULL;
        SeqEntryExplore(sep, &query_bsp, FindProt);
        if (query_bsp == NULL) {
-	 ErrPostEx(SEV_FATAL, 0, 0, "Unable to obtain bioseq\n");
+	 ErrPostEx(SEV_FATAL, 1, 0, "Unable to obtain bioseq\n");
 	 return NULL;
        }
        fake_bsp = BioseqNew();
@@ -276,7 +290,7 @@ static void readkbp(FILE * thisMatrixFile, Int4 dbSequenceLength,
 
   fscanf(thisMatrixFile, "%d", &lengthInFile);
   if (dbSequenceLength != lengthInFile) {
-    ErrPostEx(SEV_FATAL, 0, 0, "profile: for file %s length in sequence file is %ld, length in matrix file is %ld, probably due to a blank in sequence file\n", sequenceFileName, (long) dbSequenceLength, (long) lengthInFile);
+    ErrPostEx(SEV_FATAL, 1, 0, "profile: for file %s length in sequence file is %ld, length in matrix file is %ld, probably due to a blank in sequence file\n", sequenceFileName, (long) dbSequenceLength, (long) lengthInFile);
     return;
   }
   sequence = (Char *) MemNew((dbSequenceLength+1) * sizeof(Char));
@@ -934,7 +948,7 @@ static SeqAlignPtr convertSWalignsToSeqAligns(SWResults * SWAligns,  Uint1Ptr qu
                      curSW->reverseAlignScript, curSW->queryStart, 
 		     curSW->seqStart);
       nextSeqAlign = GapXEditBlockToSeqAlign(nextEditBlock, curSW->subject_id, query_id);
-      nextSeqAlign->score = addScoresToSeqAlign(curSW->score, 
+      nextSeqAlign->score = addScoresToSeqAlign(curSW->scoreThisAlign, 
            curSW->eValueThisAlign, curSW->Lambda, curSW->logK);
       if (NULL == seqAlignList)
         seqAlignList = nextSeqAlign;
@@ -1004,6 +1018,7 @@ SeqAlignPtr findMatchingProfiles(FILE *matrixAuxiliaryFile,
    Int4 numAligns; /*number of matching alignments*/
    Nlm_FloatHi firstEvalue, lowerBoundEvalue, thisEvalue, bestEvalue; 
        /*e-values for one profile, first two are estimates*/
+   Int4 bestScore, newScore; /*best score for this match, score for 1 alignment*/
    Int4 queryEnd, dbEnd, finalQueryEnd, finalDbEnd; /*end positions of optimal local alignment*/
    Int4 score; /*score of optimal local alignment*/
    Int4 *alignScript, *reverseAlignScript; 
@@ -1043,11 +1058,11 @@ SeqAlignPtr findMatchingProfiles(FILE *matrixAuxiliaryFile,
    return_sfp = (BLAST_ScoreFreqPtr) MemNew(1 * sizeof(BLAST_ScoreFreq));
 
    if ((matricesFile = FileOpen(matrixListFileName, "r")) == NULL)  {
-     ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open file with all matrices %s\n", matrixListFileName);
+     ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open file with all matrices %s\n", matrixListFileName);
 	return (NULL);
    }
    if ((matrixFileDesc = FileOpen(bigMatrixFileName, "rb")) == NULL)  {
-     ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open file with all matrices %s\n", bigMatrixFileName);
+     ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open file with all matrices %s\n", bigMatrixFileName);
 	return (NULL);
    }
    FileClose(matrixFileDesc);
@@ -1072,12 +1087,12 @@ SeqAlignPtr findMatchingProfiles(FILE *matrixAuxiliaryFile,
    /* addressForMatrix = mmap((caddr_t) 0, totalLength * sizeof(ScoreRow), PROT_READ,
               MAP_PRIVATE, matrixFileDesc, 0);
    if (addressForMatrix == MAP_FAILED) {
-       ErrPostEx(SEV_FATAL, 0, 0, "profiles: mmap failed\n");
+       ErrPostEx(SEV_FATAL, 1, 0, "profiles: mmap failed\n");
 	return (NULL);
    } */
    mmapResult =  Nlm_MemMapInit(bigMatrixFileName);
    if (NULL == mmapResult) {
-       ErrPostEx(SEV_FATAL, 0, 0, "profiles: mmap failed\n");
+       ErrPostEx(SEV_FATAL, 1, 0, "profiles: mmap failed\n");
 	return (NULL);
    } 
    addressForMatrix = mmapResult -> mmp_begin;
@@ -1119,7 +1134,7 @@ SeqAlignPtr findMatchingProfiles(FILE *matrixAuxiliaryFile,
        oneSequenceFileName[c1] = '\0';
      }
      if ((thisSequenceFile = FileOpen(oneSequenceFileName, "r")) == NULL)  {
-       ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open sequence file %s\n", oneSequenceFileName);
+       ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open sequence file %s\n", oneSequenceFileName);
 	return (NULL);
      }
      if ('\0' == directoryPrefix[0])
@@ -1132,7 +1147,7 @@ SeqAlignPtr findMatchingProfiles(FILE *matrixAuxiliaryFile,
        oneMatrixFileName[c1] = '\0';
      }
      if ((thisMatrixFile = FileOpen(oneMatrixFileName, "r")) == NULL)  {
-       ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open matrix file %s\n", oneMatrixFileName);
+       ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open matrix file %s\n", oneMatrixFileName);
 	return (NULL);
      }
      dbSequence = readSequence(thisSequenceFile, &dbSequenceLength, 
@@ -1228,6 +1243,7 @@ SeqAlignPtr findMatchingProfiles(FILE *matrixAuxiliaryFile,
 		 if ((XdropAlignScore < score) && (doublingCount < 3)) 
 		   MemFree(reverseAlignScript);
 	       } while ((XdropAlignScore < score) && (doublingCount < 3));
+               newScore = Nlm_Nint(((Nlm_FloatHi) XdropAlignScore) / scalingFactor);
 	       thisSequenceFile = FileOpen(oneSequenceFileName, "r");
 	       dbSequence = readSequence(thisSequenceFile, &dbSequenceLength, 
 					 &subject_id, TRUE, dbSequenceLength+2, i+1);
@@ -1237,8 +1253,11 @@ SeqAlignPtr findMatchingProfiles(FILE *matrixAuxiliaryFile,
 		 /* complement alignment script to put query on top*/
 		 *revASptr = -(*revASptr); 
 	       }
-	       if (!foundMatchForThisMatrix) 
+	       if (!foundMatchForThisMatrix) {
 		 bestEvalue = thisEvalue;
+		 bestScore = newScore;
+	       }
+
 	       newSW = (SWResults *) MemNew(1 * sizeof(SWResults)); 
 	       newSW->seq = dbSequence;
 	       newSW->seqStart = dbStart;
@@ -1246,7 +1265,8 @@ SeqAlignPtr findMatchingProfiles(FILE *matrixAuxiliaryFile,
 	       newSW->queryStart = queryStart;
 	       newSW->queryEnd = queryStart + finalQueryEnd;
 	       newSW->reverseAlignScript = reverseAlignScript;
-	       newSW->score = Nlm_Nint(((Nlm_FloatHi) XdropAlignScore) / scalingFactor);
+	       newSW->score = bestScore;
+	       newSW->scoreThisAlign = newScore;
 	       newSW->eValue = bestEvalue;
 	       newSW->eValueThisAlign = thisEvalue;
 	       newSW->subject_index = i;
@@ -1478,7 +1498,7 @@ Int2  Main(void)
    blast_outputfile = myargs [ARG_OUTPUT_FILE].strvalue;
    if ((infp = FileOpen(blast_inputfile, "r")) == NULL)
      {
-       ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open input file %s\n", blast_inputfile);
+       ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open input file %s\n", blast_inputfile);
        return (1);
      }
 
@@ -1487,7 +1507,7 @@ Int2  Main(void)
      {
        if ((outfp = FileOpen(blast_outputfile, "w")) == NULL)
 	 {
-	    ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open output file %s\n", blast_outputfile);
+	    ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open output file %s\n", blast_outputfile);
 	    return (1);
 	 }
      }
@@ -1500,12 +1520,12 @@ Int2  Main(void)
 
    if ((auxiliaryfp = FileOpen(auxiliaryFileName, "r")) == NULL)
      {
-       ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open auxiliary file %s\n", auxiliaryFileName);
+       ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open auxiliary file %s\n", auxiliaryFileName);
        return (1);
      }
 
    if ((sequencesfp = FileOpen(seqFileName, "r")) == NULL)  {
-     ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open file with all sequences %s\n", seqFileName);
+     ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open file with all sequences %s\n", seqFileName);
 	return (1);
    }
 
@@ -1517,12 +1537,12 @@ Int2  Main(void)
      {
        if (believe_query == FALSE)
 	 {
-	   ErrPostEx(SEV_FATAL, 0, 0, "-J option must be TRUE to use this option");
+	   ErrPostEx(SEV_FATAL, 1, 0, "-J option must be TRUE to use this option");
 	   return (1);
 	 }
        if ((aip = AsnIoOpen (myargs[ARG_SEQALIGN_FILE].strvalue,"w")) == NULL)
 	 {
-	   ErrPostEx(SEV_FATAL, 0, 0, "profiles: Unable to open output file %s\n", myargs[ARG_SEQALIGN_FILE].strvalue);
+	   ErrPostEx(SEV_FATAL, 1, 0, "profiles: Unable to open output file %s\n", myargs[ARG_SEQALIGN_FILE].strvalue);
 	   return 1;
 	 }
      }
@@ -1534,7 +1554,7 @@ Int2  Main(void)
 	SeqEntryExplore(sep, &query_bsp, FindProt);
 	if (query_bsp == NULL)
           {
-	    ErrPostEx(SEV_FATAL, 0, 0, "Unable to obtain bioseq\n");
+	    ErrPostEx(SEV_FATAL, 1, 0, "Unable to obtain bioseq\n");
 	    return 2;
 	  }
 		
@@ -1661,7 +1681,7 @@ Int2  Main(void)
 			   myargs[ARG_DB_LENGTH].intvalue);
           targetUngappedLambda = IMPALAfindUngappedLambda(matrixName);
 	  if (0.0 == targetUngappedLambda) {
-	    ErrPostEx(SEV_FATAL, 0, 0, "Cannot identify matrix %s",matrixName);
+	    ErrPostEx(SEV_FATAL, 1, 0, "Cannot identify matrix %s",matrixName);
 	    return (1);
           }
 
@@ -1782,7 +1802,7 @@ Int2  Main(void)
    if (NULL != query)
      MemFree(query);
    options = BLASTOptionDelete(options);
-   sep = SeqEntryFree(sep);
+   /*sep = SeqEntryFree(sep);*/
    search = BlastSearchBlkDestruct(search);
    if (believe_query == FALSE)
      {
