@@ -1,4 +1,6 @@
-/* $Id: rpsblast.c,v 6.38 2001/08/28 17:45:01 madden Exp $
+static char const rcsid[] = "$Id: rpsblast.c,v 6.55 2004/04/30 15:33:05 dondosha Exp $";
+
+/* $Id: rpsblast.c,v 6.55 2004/04/30 15:33:05 dondosha Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -29,12 +31,67 @@
 *
 * Initial Version Creation Date: 12/14/1999
 *
-* $Revision: 6.38 $
+* $Revision: 6.55 $
 *
 * File Description:
 *         Main file for RPS BLAST program
 *
 * $Log: rpsblast.c,v $
+* Revision 6.55  2004/04/30 15:33:05  dondosha
+* Added argument in call to BXMLPrintOutput
+*
+* Revision 6.54  2004/02/09 23:14:44  camacho
+* Fix to prevent printing multiple Seq-annots in ASN.1 output
+*
+* Revision 6.53  2004/02/06 05:32:36  camacho
+* Properly close ASN.1 stream
+*
+* Revision 6.52  2003/05/30 17:31:10  coulouri
+* add rcsid
+*
+* Revision 6.51  2003/05/13 16:02:42  coulouri
+* make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+*
+* Revision 6.50  2003/03/20 14:47:16  madden
+* StringSave on asn1_mode
+*
+* Revision 6.49  2003/03/20 13:44:24  madden
+* Fix -m 10/11 output to make them SeqAnnots
+*
+* Revision 6.48  2002/12/31 22:47:16  boemker
+* Added support for printing output as ASN (text, with -m 10, or binary, with
+* -m 11).
+*
+* Revision 6.47  2002/11/29 20:13:13  camacho
+* Fix incorrect parameter to FastaToSeqEntryForDb
+*
+* Revision 6.46  2002/10/17 20:36:00  camacho
+* Disallow -L option for tblastn
+*
+* Revision 6.45  2002/10/13 22:43:51  camacho
+* Minor correction
+*
+* Revision 6.44  2002/10/10 14:49:43  camacho
+*
+* 1. Removed irrelevant options: -E, -G, -S, -H
+* 2. Added -L option to provide range restriction on query sequence
+*
+* Revision 6.43  2002/09/18 20:34:30  camacho
+* Restored -P option
+*
+* Revision 6.42  2002/08/20 15:17:42  camacho
+* Fixed small memory leak
+*
+* Revision 6.41  2002/08/09 19:41:25  camacho
+* 1) Added blast version number to command-line options
+* 2) Added explanations for some default parameters
+*
+* Revision 6.40  2002/06/19 22:50:17  dondosha
+* Added all queries information for tabular output with multiple queries
+*
+* Revision 6.39  2002/04/29 19:55:25  madden
+* Use ARG_FLOAT for db length
+*
 * Revision 6.38  2001/08/28 17:45:01  madden
 * Add -m 9 as tabular output with comments
 *
@@ -162,11 +219,9 @@
 #include "/am/purew/solaris2/new/../purify/purify-4.5-solaris2/purify.h"
 #endif
 
-AsnIoPtr aip_glb=NULL;
-
 #define NUMARG (sizeof(myargs)/sizeof(myargs[0]))
 
-static Args myargs [] = {
+static Args myargs[] = {
     {"Input query sequence (this parameter must be set)",  /* 0 */
      "stdin", NULL,NULL,FALSE,'i',ARG_FILE_IN, 0.0,0,NULL},
     {"RPS BLAST Database",            /* 1 */
@@ -181,48 +236,44 @@ static Args myargs [] = {
       "stdout", NULL, NULL, TRUE, 'o', ARG_FILE_OUT, 0.0, 0, NULL},
     { "Dropoff (X) for blast extensions in bits (default if zero)", /* 6 */
       "7.0", NULL, NULL, FALSE, 'y', ARG_FLOAT, 0.0, 0, NULL},
-    { "0 for multiple hits 1-pass, 1 for single hit 1-pass, 2 for 2-pass", /* 7 */
-      "0", NULL, NULL, FALSE, 'P', ARG_INT, 0.0, 0, NULL},
+    { "0 for multiple hit, 1 for single hit",/* 7 */
+       "0",  NULL, NULL, FALSE, 'P', ARG_INT, 0.0, 0, NULL},
     { "Filter query sequence with SEG", /* 8 */
       "F", NULL, NULL, FALSE, 'F', ARG_STRING, 0.0, 0, NULL},
-    { "Cost to open a gap",     /* 9 */
-      "11", NULL, NULL, FALSE, 'G', ARG_INT, 0.0, 0, NULL},
-    { "Cost to extend a gap",   /* 10 */
-      "1", NULL, NULL, FALSE, 'E', ARG_INT, 0.0, 0, NULL},
-    { "X dropoff value for gapped alignment (in bits)", /* 11 */
+    { "X dropoff value for gapped alignment (in bits)", /* 9 */
       "15", NULL, NULL, FALSE, 'X', ARG_INT, 0.0, 0, NULL},
-    { "Number of bits to trigger gapping", /* 12 */
+    { "Number of bits to trigger gapping", /* 10 */
       "22.0", NULL, NULL, FALSE, 'N', ARG_FLOAT, 0.0, 0, NULL},
-    { "Gapped",                 /* 13 */
+    { "Gapped",                 /* 11 */
       "T", NULL, NULL, FALSE, 'g', ARG_BOOLEAN, 0.0, 0, NULL},
-    { "Start of required region in query", /* 14 */
-      "1", NULL, NULL, FALSE, 'S', ARG_INT, 0.0, 0, NULL},
-    { "End of required region in query (-1 indicates end of query)", /* 15 */
-      "-1", NULL, NULL, FALSE, 'H', ARG_INT, 0.0, 0, NULL},
-    { "Number of processors to use", /* 16 */
+    { "Number of processors to use", /* 12 */
       "1", NULL, NULL, FALSE, 'a', ARG_INT, 0.0, 0, NULL},
-    { "Show GI's in deflines",  /* 17 */
+    { "Show GI's in deflines",  /* 13 */
       "F", NULL, NULL, FALSE, 'I', ARG_BOOLEAN, 0.0, 0, NULL},
-    { "Believe the query defline", /* 18 */
+    { "Believe the query defline", /* 14 */
       "F", NULL, NULL, FALSE, 'J', ARG_BOOLEAN, 0.0, 0, NULL},
-    { "X dropoff value for final gapped alignment (in bits)", /* 19 */
+    { "X dropoff value for final gapped alignment (in bits)", /* 15 */
       "25", NULL, NULL, FALSE, 'Z', ARG_INT, 0.0, 0, NULL},
-    { "SeqAlign file ('Believe the query defline' must be TRUE)", /* 20 */
+    { "SeqAlign file ('Believe the query defline' must be TRUE)", /* 16 */
       NULL, NULL, NULL, TRUE, 'O', ARG_FILE_OUT, 0.0, 0, NULL},
-    { "Number of database sequences to show one-line descriptions for (V)", /* 21 */
+    { "Number of database sequences to show one-line descriptions for (V)", /* 17 */
       "500", NULL, NULL, FALSE, 'v', ARG_INT, 0.0, 0, NULL},
-    { "Number of database sequence to show alignments for (B)", /* 22 */
+    { "Number of database sequence to show alignments for (B)", /* 18 */
       "250", NULL, NULL, FALSE, 'b', ARG_INT, 0.0, 0, NULL},
-    { "Effective length of the database (use zero for the real size)", /* 23 */
-      "0", NULL, NULL, FALSE, 'z', ARG_INT, 0.0, 0, NULL},
-    { "Effective length of the search space (use zero for the real size)", /* 24 */
+    { "Effective length of the database (use zero for the real size)", /* 19 */
+      "0", NULL, NULL, FALSE, 'z', ARG_FLOAT, 0.0, 0, NULL},
+    { "Effective length of the search space (use zero for the real size)", /* 20 */
       "0", NULL, NULL, FALSE, 'Y', ARG_FLOAT, 0.0, 0, NULL},
-    { "Produce HTML output",  /* 25 */
+    { "Produce HTML output",  /* 21 */
       "F", NULL, NULL, FALSE, 'T', ARG_BOOLEAN, 0.0, 0, NULL},
-    {"Logfile name ",  /* 26 */
+    {"Logfile name ",  /* 22 */
      "rpsblast.log", NULL,NULL,TRUE,'l',ARG_FILE_OUT, 0.0,0,NULL},
-    {"Use lower case filtering of FASTA sequence",    /* 27 */
+    {"Use lower case filtering of FASTA sequence",    /* 23 */
      "F", NULL,NULL,TRUE,'U',ARG_BOOLEAN, 0.0,0,NULL},
+    { "Range restriction on query sequence (Format: start,stop) blastp only\n"
+      "      0 in 'start' refers to the beginning of the sequence\n"
+      "      0 in 'stop' refers to the end of the sequence",
+      "0,0", NULL, NULL, TRUE, 'L', ARG_STRING, 0.0, 0, NULL}, /* 24 */
 };
 
 void PGPGetPrintOptions(Boolean gapped, Uint4Ptr align_options_out, 
@@ -238,12 +289,12 @@ void PGPGetPrintOptions(Boolean gapped, Uint4Ptr align_options_out,
     align_options += TXALIGN_COMPRESS;
     align_options += TXALIGN_END_NUM;
 
-    if (myargs[17].intvalue) {
+    if (myargs[13].intvalue) {
         align_options += TXALIGN_SHOW_GI;
         print_options += TXALIGN_SHOW_GI;
     } 
     
-    if (myargs[25].intvalue) {
+    if (myargs[21].intvalue) {
         align_options += TXALIGN_HTML;
         print_options += TXALIGN_HTML;
     }
@@ -274,10 +325,12 @@ void RPSBlastOptionsFree(RPSBlastOptionsPtr rpsbop)
 {
 
     FileClose(rpsbop->outfp);
+    AsnIoClose(rpsbop->aip);
     BLASTOptionDelete(rpsbop->options);
     readdb_destruct(rpsbop->rdfp);
 
     MemFree(rpsbop->rps_database);    
+    MemFree(rpsbop->out_filename);
     MemFree(rpsbop);
     
     return;
@@ -288,98 +341,120 @@ static RPSBlastOptionsPtr RPSReadBlastOptions(void)
     RPSBlastOptionsPtr rpsbop;
     BLAST_OptionsBlkPtr options;
     static Int4 count=0;
+    CharPtr location = NULL;
     
     rpsbop = MemNew(sizeof(RPSBlastOptions));
+    
 
-    if (myargs [5].strvalue != NULL) {
-	rpsbop->out_filename = StringSave(myargs [5].strvalue);
-        if ((rpsbop->outfp = FileOpen(myargs [5].strvalue, "a")) == NULL) {
-            ErrPostEx(SEV_FATAL, 0, 0, "rpsblast: Unable to open output "
-                      "file %s\n", myargs [5].strvalue);
-            return NULL;
-        }
-    }
+    if (myargs[5].strvalue != NULL) 
+	rpsbop->out_filename = StringSave(myargs[5].strvalue);
 
     /* Note: these 2 parameters are necessary to intialize RPS Blast */
-    rpsbop->rps_database = StringSave(myargs [1].strvalue);    
+    rpsbop->rps_database = StringSave(myargs[1].strvalue);    
     rpsbop->query_is_protein = myargs[2].intvalue;
-    rpsbop->num_threads = myargs[16].intvalue;
+    rpsbop->num_threads = myargs[12].intvalue;
 
-    if((rpsbop->rdfp = readdb_new(myargs [1].strvalue, TRUE)) == NULL)
+    if((rpsbop->rdfp = readdb_new(myargs[1].strvalue, TRUE)) == NULL)
         return NULL;
     
     /* rpsbop->rpsinfo = RPSInfoAttach(rpsinfo_main); */
 
-    if (myargs[18].intvalue != 0)
+    if (myargs[14].intvalue != 0)
         rpsbop->believe_query = TRUE;
     
     options = BLASTOptionNew(rpsbop->query_is_protein ? "blastp" : "tblastn", 
-                             (Boolean)myargs[13].intvalue);
+                             (Boolean)myargs[11].intvalue);
     rpsbop->options = options;
     
     /* rpsbop->options->query_lcase_mask = slp; External filtering */
     
-    if (myargs[23].intvalue)
-        options->db_length = (Int8) myargs[23].intvalue;
+    if (myargs[19].floatvalue)
+        options->db_length = (Int8) myargs[19].floatvalue;
     
-    if (myargs[24].floatvalue)
-        options->searchsp_eff = (Nlm_FloatHi) myargs[24].floatvalue;
+    if (myargs[20].floatvalue)
+        options->searchsp_eff = (Nlm_FloatHi) myargs[20].floatvalue;
     
     /* Necessary options for RPS Blast */
     options->do_sum_stats = FALSE;
     options->is_rps_blast = TRUE; 
     
-    rpsbop->number_of_descriptions = myargs[21].intvalue;
-    rpsbop->number_of_alignments = myargs[22].intvalue;
+    rpsbop->number_of_descriptions = myargs[17].intvalue;
+    rpsbop->number_of_alignments = myargs[18].intvalue;
     
     /* Set default gap params for matrix. */
     BLASTOptionSetGapParams(options, "BLOSUM62", 0, 0);
     
-    if(myargs[4].intvalue == 7)
+    if(myargs[4].intvalue == 7) {
        rpsbop->is_xml_output = TRUE;
+    }
     else if (myargs[4].intvalue == 8)
        rpsbop->is_tabular = TRUE;
-    else if (myargs[4].intvalue == 8 || myargs[4].intvalue == 9)
+    else if (myargs[4].intvalue == 9)
     {
        rpsbop->is_tabular = TRUE;
        rpsbop->is_tabular_comments = TRUE;
     }
+    else if (myargs[4].intvalue == 10 || myargs[4].intvalue == 11)
+    {
+       rpsbop->is_asn1_output = TRUE;
+	if (myargs[4].intvalue == 10)
+		rpsbop->asn1_mode = StringSave("w");
+	else
+		rpsbop->asn1_mode = StringSave("wb");
+    }
     else
        PGPGetPrintOptions(options->gapped_calculation, 
                           &rpsbop->align_options, &rpsbop->print_options);
     
-    /* decrement by one to agree with program values. */
-    options->required_start = myargs[14].intvalue - 1;
-    options->required_end = myargs[15].intvalue;
-    
-    if (options->required_end != -1) {
-        options->required_end--;
+
+    if (!rpsbop->is_xml_output && !rpsbop->is_asn1_output && (rpsbop->outfp = FileOpen(rpsbop->out_filename, "a")) == NULL) {
+            ErrPostEx(SEV_FATAL, 1, 0, "rpsblast: Unable to open output "
+                      "file %s\n", rpsbop->out_filename);
+            return NULL;
     }
 
-    options->dropoff_2nd_pass  = (Int4) myargs [6].floatvalue;
-    options->expect_value  = (Nlm_FloatHi) myargs [3].floatvalue;
+    if (myargs[16].strvalue != NULL) {
+        if (myargs[14].intvalue == 0) {
+            ErrPostEx(SEV_FATAL, 1, 0, 
+                      "-J option must be TRUE to use this option");
+            return NULL;
+        } else  {
+            if ((rpsbop->aip = AsnIoOpen (myargs[16].strvalue,"w")) == NULL) {
+                ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output "
+                          "file %s\n", myargs[16].strvalue);
+                return NULL;
+            }
+    	}
+    }
+    else if (rpsbop->is_asn1_output)
+    {
+            if ((rpsbop->aip = AsnIoOpen (rpsbop->out_filename, rpsbop->asn1_mode)) == NULL) {
+                ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output "
+                          "file %s\n", rpsbop->out_filename);
+                return NULL;
+            }
+    }
+
+    options->dropoff_2nd_pass  = (Int4) myargs[6].floatvalue;
+    options->expect_value  = (Nlm_FloatHi) myargs[3].floatvalue;
     options->hitlist_size = MAX(rpsbop->number_of_descriptions, 
                                 rpsbop->number_of_alignments);
     
-    if (myargs[13].intvalue != 0) {
-        if (myargs[7].intvalue == 0) {
-            options->two_pass_method  = FALSE;
-            options->multiple_hits_only  = TRUE;
-        } else if (myargs[7].intvalue == 1) {
+    if (myargs[11].intvalue != 0) {
+        if (myargs[7].intvalue == 1) {
             options->two_pass_method  = FALSE;
             options->multiple_hits_only  = FALSE;
         } else {
-            options->two_pass_method  = TRUE;
-            options->multiple_hits_only  = FALSE;
+            /* all other inputs, including the default 0 use 2-hit method */
+            options->two_pass_method  = FALSE;
+            options->multiple_hits_only  = TRUE;
         }
-        options->gap_open = myargs[9].intvalue;
-        options->gap_extend = myargs[10].intvalue;
 
         /*  options->decline_align = myargs[??].intvalue; */
 
-        options->gap_x_dropoff = myargs[11].intvalue;
-        options->gap_x_dropoff_final = myargs[19].intvalue;
-        options->gap_trigger = myargs[12].floatvalue;
+        options->gap_x_dropoff = myargs[9].intvalue;
+        options->gap_x_dropoff_final = myargs[15].intvalue;
+        options->gap_trigger = myargs[10].floatvalue;
     }
     
     if (StringICmp(myargs[8].strvalue, "T") == 0) {
@@ -389,16 +464,33 @@ static RPSBlastOptionsPtr RPSReadBlastOptions(void)
     }
 
     /* Only one CPU may be used at this time inside the core engine*/    
-    options->number_of_cpus = (Int2) myargs[16].intvalue;
+    options->number_of_cpus = (Int2) myargs[12].intvalue;
     
     options->isPatternSearch = FALSE;
 
     options = BLASTOptionValidate(options, rpsbop->query_is_protein ? 
                                   "blastp" : "tblastn");
     
-    if (options == NULL)
-        return NULL;
-    
+    /* Kludge: use required_{start,end} to store offsets that will restrict
+     * the query sequence. These values are later converted into a SeqLoc */
+    if ((location = myargs[24].strvalue)) {
+        options->required_start = atol(StringTokMT(location, " ,;", 
+                                        &location));
+        options->required_end = atol(location);
+        if (options->required_start < 0 || options->required_end < 0) {
+            ErrPostEx(SEV_ERROR,0,0, "Invalid range restriction: %ld to %ld",
+                    options->required_start, options->required_end);
+            return NULL;
+        }
+        /* Range restriction is only allowed for protein queries */
+        if (rpsbop->query_is_protein == FALSE && 
+            (options->required_start != 0 && options->required_end != 0)) {
+            ErrPostEx(SEV_ERROR, 0, 0, "Range restriction supported only for "
+                    "protein queries");
+            options->required_start = options->required_end = 0;
+        }
+    }
+
     if (options == NULL)
         return NULL;
     
@@ -412,9 +504,24 @@ static void RPSViewSeqAlign(BioseqPtr query_bsp,
     SeqAnnotPtr seqannot;
     BlastPruneSapStructPtr prune;
     Uint1 align_type;
-    CharPtr title;
     ValNodePtr vnp, mask;
 
+    seqannot = SeqAnnotNew();
+    seqannot->type = 2;
+    align_type = BlastGetProgramNumber(rpsbop->query_is_protein ?
+                                       "blastp" : "blastx");
+    AddAlignInfoToSeqAnnot(seqannot, align_type); /* blastp or tblastn */
+    
+    if (rpsbop->aip != NULL) {     
+        CharPtr title = readdb_get_title(rpsbop->rdfp); /* we don't own title */
+        BLASTAddBlastDBTitleToSeqAnnot(seqannot, title);
+    }
+
+    if (rpsbop->outfp == NULL) {
+        seqannot = SeqAnnotFree(seqannot);
+        return;
+    }
+
     free_buff();    
     init_buff_ex(128);
     
@@ -425,6 +532,7 @@ static void RPSViewSeqAlign(BioseqPtr query_bsp,
                           rpsbop->believe_query, rpsbop->html);
     if(seqalign == NULL) {
         fprintf(rpsbop->outfp, "\nNo hits found for the sequence...\n\n");
+        seqannot = SeqAnnotFree(seqannot);
         return;
     }
     
@@ -433,18 +541,6 @@ static void RPSViewSeqAlign(BioseqPtr query_bsp,
     PrintDefLinesFromSeqAlign(prune->sap, 80, rpsbop->outfp, 
                               rpsbop->print_options, FIRST_PASS, NULL);
 
-    seqannot = SeqAnnotNew();
-    seqannot->type = 2;
-    align_type = BlastGetProgramNumber(rpsbop->query_is_protein ?
-                                       "blastp" : "blastx");
-    
-    AddAlignInfoToSeqAnnot(seqannot, align_type); /* blastp or tblastn */
-    
-    /* Now adding database title from SeqAnnot */
-
-    title = readdb_get_title(rpsbop->rdfp);
-    
-    BLASTAddBlastDBTitleToSeqAnnot(seqannot, title);
     
     /* --------------------------------------- */
     
@@ -452,9 +548,10 @@ static void RPSViewSeqAlign(BioseqPtr query_bsp,
                                        prune);
     seqannot->data = prune->sap;
 
-    if(aip_glb != NULL) {     
-        SeqAnnotAsnWrite(seqannot, aip_glb, NULL);
-	AsnIoReset(aip_glb);
+    /* Write the ASN.1 Seq-annot */
+    if (rpsbop->aip != NULL) {     
+        SeqAnnotAsnWrite(seqannot, rpsbop->aip, NULL);
+        AsnIoReset(rpsbop->aip);
     }
 
     /* ------ Mask needed ----- */
@@ -490,6 +587,7 @@ static void RPSViewSeqAlign(BioseqPtr query_bsp,
 
     seqannot->data = NULL;
     seqannot = SeqAnnotFree(seqannot);
+    seqannot = SeqAnnotFree(seqannot);
 
     free_buff();
     return;
@@ -578,19 +676,19 @@ static SeqEntryPtr LIBCALLBACK RPSGetNextSeqEntry(SeqLocPtr PNTR slp, VoidPtr da
 
     /* Opening file with input sequences */
     if(infp == NULL) {
-        if ((infp = FileOpen(myargs [0].strvalue, "r")) == NULL) {
-            ErrPostEx(SEV_FATAL, 0, 0, 
+        if ((infp = FileOpen(myargs[0].strvalue, "r")) == NULL) {
+            ErrPostEx(SEV_FATAL, 1, 0, 
                       "rpsblast: Unable to open input file %s\n", 
-                      myargs [0].strvalue);
+                      myargs[0].strvalue);
             end_of_data = TRUE;
             NlmMutexUnlock(read_mutex);
         }
     }
     
-    if(myargs[27].intvalue) {
-        sep = FastaToSeqEntryForDb (infp, !myargs[2].intvalue, NULL, myargs[18].intvalue, NULL, NULL, slp);
+    if(myargs[23].intvalue) {
+        sep = FastaToSeqEntryForDb (infp, !myargs[2].intvalue, NULL, myargs[14].intvalue, NULL, NULL, slp);
     } else {
-        sep = FastaToSeqEntryEx (infp, !myargs[2].intvalue, NULL, myargs[18].intvalue);
+        sep = FastaToSeqEntryEx (infp, !myargs[2].intvalue, NULL, myargs[14].intvalue);
     }
     
     if(sep == NULL) {            /* Probably last FASTA entry */
@@ -616,7 +714,7 @@ static Boolean LIBCALLBACK RPSResultsCallback(BioseqPtr query_bsp,
        BXMLPrintOutput(aip, seqalign, rpsbop->options, 
                        rpsbop->query_is_protein ? 
                        "blastp" : "tblastn", rpsbop->rps_database, 
-                       query_bsp, other_returns, 0, NULL);
+                       query_bsp, other_returns, 0, NULL, NULL);
        AsnIoClose(aip);
     } else if (rpsbop->is_tabular) {
 	if (rpsbop->is_tabular_comments)
@@ -627,7 +725,8 @@ static Boolean LIBCALLBACK RPSResultsCallback(BioseqPtr query_bsp,
                                   rpsbop->number_of_alignments,
                                   rpsbop->query_is_protein ? 
                                   "blastp" : "tblastn", FALSE,
-                                  rpsbop->believe_query, 0, 0, rpsbop->outfp);
+                                  rpsbop->believe_query, 0, 0, rpsbop->outfp,
+                                  FALSE);
     } else {
         RPSViewSeqAlign(query_bsp, seqalign, rpsbop, other_returns);
         RPSFormatFooter(rpsbop, other_returns);
@@ -641,11 +740,14 @@ Int2 Main(void)
     FILE *fd;
     Int4 i;
     RPSBlastOptionsPtr rpsbop;
+    Char buf[256] = { '\0' };
 
-    if (!GetArgs("rpsblast", NUMARG, myargs))
-	return 1;
+    StringCpy(buf, "rpsblast ");
+    StringNCat(buf, BlastGetVersionNumber(), sizeof(buf)-StringLen(buf)-1);
+    if (!GetArgs(buf, NUMARG, myargs))
+        return 1;
     
-    if ( !ErrSetLog (myargs[26].strvalue) ) { /* Logfile */
+    if ( !ErrSetLog (myargs[22].strvalue) ) { /* Logfile */
         ErrShow();
         return 1;
     } else {
@@ -658,29 +760,15 @@ Int2 Main(void)
         return 1;
         
     /* Truncate output file */
-    if((fd = FileOpen(myargs [5].strvalue, "w")) != NULL)
+    if((fd = FileOpen(myargs[5].strvalue, "w")) != NULL)
         FileClose(fd);
     
-    if (myargs[20].strvalue != NULL) {
-        if (myargs[18].intvalue == 0) {
-            ErrPostEx(SEV_FATAL, 0, 0, 
-                      "-J option must be TRUE to use this option");
-            return 1;
-        } else  {
-            if ((aip_glb = AsnIoOpen (myargs[20].strvalue,"w")) == NULL) {
-                ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open output "
-                          "file %s\n", myargs[20].strvalue);
-                return 1;
-            }
-    	}
-    }
-        
     if((rpsbop = RPSReadBlastOptions()) == NULL) {
-        ErrPostEx(SEV_FATAL, 0, 0, "Unable to create RPS Blast options");
+        ErrPostEx(SEV_FATAL, 1, 0, "Unable to create RPS Blast options");
         return 1;
     }
 
-    if (!rpsbop->is_xml_output && !rpsbop->is_tabular)
+    if (!rpsbop->is_xml_output && !rpsbop->is_tabular && !rpsbop->is_asn1_output)
 	    BlastPrintVersionInfo("RPS-BLAST", rpsbop->html, rpsbop->outfp);
     
     /* VoidPtr bsp_data = NULL, print_data = NULL; */
