@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.32 $
+* $Revision: 6.49 $
 *
 * File Description: 
 *
@@ -65,13 +65,13 @@ ENUM_ALIST(orgmod_subtype_alist)
   {"Forma-specialis",  26},
   {"Group",            15},
   {"Isolate",          17},
-  {"Natural-host",     21},
   {"Old Lineage",     253},
   {"Old Name",        254},
   {"Pathovar",         11},
   {"Serogroup",         8},
   {"Serotype",          7},
   {"Serovar",           9},
+  {"Specific-host",    21},
   {"Specimen-voucher", 23},
   {"Strain",            2},
   {"Subgroup",         16},
@@ -100,21 +100,25 @@ ENUM_ALIST(subsource_subtype_alist)
   {"Genotype",               6},
   {"Germline",              14},
   {"Haplotype",              5},
+    /*
   {"Ins-seq-name",          21},
+    */
   {"Isolation-source",      28},
   {"Lab-host",              16},
   {"Map",                    2},
   {"Plasmid-name",          19},
   {"Plastid-name",          22},
   {"Pop-variant",           17},
+  {"Rearranged",            15},
   {"Segment",               24},
   {"Sex",                    7},
   {"Subclone",               4},
-  {"Rearranged",            15},
   {"Tissue-lib",            18},
   {"Tissue-type",           10},
   {"Transgenic",            26},
+    /*
   {"Transposon-name",       20},
+    */
 END_ENUM_ALIST
 
 static ENUM_ALIST(biosource_genome_alist)
@@ -128,8 +132,10 @@ static ENUM_ALIST(biosource_genome_alist)
   {"Macronuclear",         7},
   {"Extrachromosomal",     8},
   {"Plasmid",              9},
+    /*
   {"Transposon",          10},
   {"Insertion Sequence",  11},
+    */
   {"Cyanelle",            12},
   {"Proviral",            13},
   {"Virion",              14},
@@ -151,8 +157,10 @@ ENUM_ALIST(biosource_genome_simple_alist)
   {"Macronuclear",         7},
   {"Extrachromosomal",     8},
   {"Plasmid",              9},
+    /*
   {"Transposon",          10},
   {"Insertion Sequence",  11},
+    */
   {"Cyanelle",            12},
   {"Proviral",            13},
   {"Virion",              14},
@@ -212,6 +220,7 @@ typedef struct genbiopage {
   TexT            orgcomment;
   DialoG          subsource;
   TexT            subcomment;
+  ButtoN          add_type_strain_to_orgcomment;
   TexT            lineage;
   TexT            gbDiv;
   DialoG          db;
@@ -220,9 +229,14 @@ typedef struct genbiopage {
   GrouP           orgGrp [5];
   GrouP           modGrp [5];
   GrouP           miscGrp [5];
+  PrompT          gbacr;
+  PrompT          gbana;
+  PrompT          gbsyn;
   CharPtr         origTaxName;
   Boolean         stripOldName;
   EnumFieldAssoc  PNTR genomeAlist;
+  Uint1           orgname_choice;
+  Pointer         orgname_data;
 } GenBioPage, PNTR GenBioPagePtr;
 
 typedef struct genbioform {
@@ -728,6 +742,8 @@ static void AutoScrollTax (GenBioPagePtr gbp, TexT t, Boolean isSciName,
   }
 }
 
+static void FreeGenBioOrgNameData (GenBioPagePtr gbp);
+
 static void TaxNameText (TexT t)
 
 {
@@ -757,6 +773,7 @@ static void TaxNameText (TexT t)
             vnp->next = NULL;
             DbtagFree (dbt);
             ValNodeFree (vnp);
+            FreeGenBioOrgNameData (gbp);
           } else {
             prevvnp = (ValNodePtr PNTR) &(vnp->next);
           }
@@ -829,6 +846,7 @@ static void ReleaseTaxName (DoC d, PoinT pt)
         CopyStrFromTaxPtr (str, sizeof (str) - 2, row, 2);
         SafeSetTitle (gbp->commonName, str);
         Select (gbp->taxName);
+        TaxNameText (gbp->taxName);
         SetCodes (gbp, row, TRUE);
       }
     }
@@ -973,9 +991,100 @@ static CharPtr MergeValNodeStrings (ValNodePtr list)
   return ptr;
 }
 
+static void AddTypeStrainButtonProc (ButtoN b)
+{
+  GenBioPagePtr  gbp;
+  CharPtr        old_orgcomment;
+  Int4           old_orgcomment_len;
+  CharPtr        org_name;
+  Int4           org_name_len;
+  const CharPtr  ts = "type strain of ";
+  const CharPtr  sep = "; ";
+  CharPtr        new_orgcomment;
+
+  gbp = GetObjectExtra(b);
+  if (gbp == NULL) return;
+  old_orgcomment_len = TextLength (gbp->orgcomment) + 1;
+  old_orgcomment = MemNew (old_orgcomment_len + 1);
+  if (old_orgcomment == NULL) return;
+  org_name_len = TextLength (gbp->taxName) + 1;
+  org_name = MemNew (org_name_len + 1);
+  if (org_name == NULL) 
+  {
+    MemFree (old_orgcomment);
+    return;
+  }
+  new_orgcomment = MemNew (old_orgcomment_len
+			+ StringLen (sep)
+			+ StringLen (ts)
+			+ org_name_len
+			+ 1);
+  if (new_orgcomment == NULL)
+  {
+    MemFree (old_orgcomment);
+    MemFree (org_name);
+  }
+
+  GetTitle (gbp->orgcomment, old_orgcomment, old_orgcomment_len);
+  TrimSpacesAroundString (old_orgcomment);
+  GetTitle (gbp->taxName, org_name, org_name_len);
+  TrimSpacesAroundString (org_name);
+  if (old_orgcomment[0] != 0)
+  {
+    StringCpy(new_orgcomment, old_orgcomment);
+    StringCat(new_orgcomment, sep);
+  }
+  else
+  {
+    new_orgcomment[0] = 0;
+  }
+    
+  StringCat (new_orgcomment, ts);
+  StringCat (new_orgcomment, org_name);
+  SetTitle (gbp->orgcomment, new_orgcomment);
+  MemFree (org_name);
+  MemFree (old_orgcomment);
+  MemFree (new_orgcomment);
+  Disable (b);
+}
+
 static Char useGenomicText [] = "\
 (Use 'Genomic' for a sequence encoded by a nuclear gene.)\n";
 
+static Pointer MakeOrgNameDataCopy (OrgNamePtr onp)
+{
+  OrgNamePtr onp_copy;
+  Pointer    retval;
+
+  if (onp == NULL) return NULL;
+
+  onp_copy = (OrgNamePtr) AsnIoMemCopy (onp, (AsnReadFunc) OrgNameAsnRead, (AsnWriteFunc) OrgNameAsnWrite);
+  if (onp_copy == NULL) return NULL;
+  retval = onp_copy->data;
+  onp_copy->data = NULL;
+  onp_copy->choice = 0;
+  OrgNameFree (onp_copy);
+  return retval;
+}
+
+static void FreeGenBioOrgNameData (GenBioPagePtr gbp)
+{
+  OrgNamePtr     onp;
+
+  if (gbp != NULL) {
+    if (gbp->orgname_choice != 0) {
+      onp = OrgNameNew ();
+      if (onp != NULL) {
+        onp->choice = gbp->orgname_choice;
+        onp->data = gbp->orgname_data;
+        OrgNameFree (onp);
+        gbp->orgname_choice = 0;
+        gbp->orgname_data = NULL;
+      }
+    }
+  }
+}
+
 static void BioSourcePtrToGenBioPage (DialoG d, Pointer data)
 
 {
@@ -1013,6 +1122,9 @@ static void BioSourcePtrToGenBioPage (DialoG d, Pointer data)
     SafeSetValue (gbp->mgcode, 1);
     SafeSetValue (gbp->simplecode, 1);
     SafeSetTitle (gbp->orgcomment, "");
+    SafeSetTitle (gbp->gbacr, "");
+    SafeSetTitle (gbp->gbana, "");
+    SafeSetTitle (gbp->gbsyn, "");
     SafeSetTitle (gbp->subcomment, "");
     SafeSetTitle (gbp->lineage, "");
     PointerToDialog (gbp->db, NULL);
@@ -1035,28 +1147,48 @@ static void BioSourcePtrToGenBioPage (DialoG d, Pointer data)
         PointerToDialog (gbp->mod, orp->mod);
         onp = orp->orgname;
         if (onp != NULL) {
+          /* store orgname data for unaltered retrieval later */
+          gbp->orgname_choice = onp->choice;
+          gbp->orgname_data = MakeOrgNameDataCopy(onp);
+
           SafeSetTitle (gbp->lineage, onp->lineage);
           PointerToDialog (gbp->orgmod, onp->mod);
           mod = onp->mod;
           head = NULL;
           while (mod != NULL) {
-            if (mod->subtype == 255) {
-              ValNodeAddStr (&head, 0, mod->subname);
-              /*
-              SetTitle (gbp->orgcomment, mod->subname);
-              */
+            switch (mod->subtype) {
+              case 32 :
+                SetTitle (gbp->gbacr, mod->subname);
+                break;
+              case 33 :
+                SetTitle (gbp->gbana, mod->subname);
+                break;
+              case 34 :
+                SetTitle (gbp->gbsyn, mod->subname);
+                break;
+              case 255 :
+                ValNodeAddStr (&head, 0, mod->subname);
+                /*
+                SetTitle (gbp->orgcomment, mod->subname);
+                */
+                break;
+              default :
+                break;
             }
             mod = mod->next;
           }
           if (head != NULL) {
             str = MergeValNodeStrings (head);
             SetTitle (gbp->orgcomment, str);
+            if ( StringStr (str, "type strain of ")) {
+              Disable (gbp->add_type_strain_to_orgcomment);
+            }
             MemFree (str);
           }
           ValNodeFree (head);
         }
       }
-      PointerToDialog (gbp->subsource, biop->subtype);
+      PointerToDialog (gbp->subsource, biop->subtype); 
       ssp = biop->subtype;
       head = NULL;
       while (ssp != NULL) {
@@ -1123,10 +1255,12 @@ static void BioSourcePtrToGenBioPage (DialoG d, Pointer data)
   }
 }
 
+
 static Pointer GenBioPageToBioSourcePtr (DialoG d)
 
 {
   BioSourcePtr   biop;
+  Char           buf [256];
   Char           ch;
   CharPtr        chptr;
   FILE           *f;
@@ -1176,6 +1310,12 @@ static Pointer GenBioPageToBioSourcePtr (DialoG d)
         onp = OrgNameNew ();
         orp->orgname = onp;
         if (onp != NULL) {
+          /* retrieve unaltered orgname data */
+          onp->choice = gbp->orgname_choice;
+          onp->data = gbp->orgname_data;
+          gbp->orgname_choice = 0;
+          gbp->orgname_data = NULL;
+
           if (gbp->simplecode != NULL) {
             if (GetEnumPopup (gbp->genome, gbp->genomeAlist, &genome)) {
               if (genome == 4 || genome == 5) {
@@ -1207,8 +1347,8 @@ static Pointer GenBioPageToBioSourcePtr (DialoG d)
                 }
               }
               if (f != NULL) {
-                if (fgets (str, sizeof (str), f) != NULL) {
-                  goOn = (Boolean) (fgets (str, sizeof (str), f) != NULL);
+                if (FileGets (str, sizeof (str), f) != NULL) {
+                  goOn = (Boolean) (FileGets (str, sizeof (str), f) != NULL);
                   while (goOn) {
                     ptr = StringChr (str, '\t');
                     if (ptr != NULL) {
@@ -1226,7 +1366,7 @@ static Pointer GenBioPageToBioSourcePtr (DialoG d)
                         goOn = FALSE;
                       }
                     }
-                    goOn = (Boolean) (goOn && (fgets (str, sizeof (str), f) != NULL));
+                    goOn = (Boolean) (goOn && (FileGets (str, sizeof (str), f) != NULL));
                   }
                 }
                 FileClose (f);
@@ -1236,6 +1376,57 @@ static Pointer GenBioPageToBioSourcePtr (DialoG d)
             onp->lineage = SaveStringFromTextAndStripNewlines (gbp->lineage);
           }
           onp->mod = DialogToPointer (gbp->orgmod);
+          GetTitle (gbp->gbacr, buf, sizeof (buf) - 1);
+          if (! StringHasNoText (buf)) {
+            mod = OrgModNew ();
+            if (onp->mod == NULL) {
+              onp->mod = mod;
+            } else {
+              tmpmod = onp->mod;
+              while (tmpmod->next != NULL) {
+                tmpmod = tmpmod->next;
+              }
+              tmpmod->next = mod;
+            }
+            if (mod != NULL) {
+              mod->subtype = 32;
+              mod->subname = StringSave (buf);
+            }
+          }
+          GetTitle (gbp->gbana, buf, sizeof (buf) - 1);
+          if (! StringHasNoText (buf)) {
+            mod = OrgModNew ();
+            if (onp->mod == NULL) {
+              onp->mod = mod;
+            } else {
+              tmpmod = onp->mod;
+              while (tmpmod->next != NULL) {
+                tmpmod = tmpmod->next;
+              }
+              tmpmod->next = mod;
+            }
+            if (mod != NULL) {
+              mod->subtype = 33;
+              mod->subname = StringSave (buf);
+            }
+          }
+          GetTitle (gbp->gbsyn, buf, sizeof (buf) - 1);
+          if (! StringHasNoText (buf)) {
+            mod = OrgModNew ();
+            if (onp->mod == NULL) {
+              onp->mod = mod;
+            } else {
+              tmpmod = onp->mod;
+              while (tmpmod->next != NULL) {
+                tmpmod = tmpmod->next;
+              }
+              tmpmod->next = mod;
+            }
+            if (mod != NULL) {
+              mod->subtype = 34;
+              mod->subname = StringSave (buf);
+            }
+          }
           if (! TextHasNoText (gbp->orgcomment)) {
             mod = OrgModNew ();
             if (onp->mod == NULL) {
@@ -1290,6 +1481,17 @@ static Pointer GenBioPageToBioSourcePtr (DialoG d)
           ssp->name = SaveStringFromTextAndStripNewlines (gbp->subcomment);
         }
       }
+      /* look for plasmid-name - if we find one, set the location to plasmid */
+      tmpssp = biop->subtype;
+      while (tmpssp != NULL)
+      {
+        if (tmpssp->subtype == SUBSRC_plasmid_name) {
+          biop->genome = GENOME_plasmid;
+          break;
+        }
+        tmpssp = tmpssp->next;
+      }
+ 
       if (orp != NULL) {
         if (orp->taxname == NULL && orp->common == NULL &&
             orp->mod == NULL && orp->db == NULL && orp->syn == NULL &&
@@ -1355,6 +1557,7 @@ void ReplaceBioSourceGenomePopup (DialoG d, PopuP genome)
   }
 }
 
+
 extern DialoG CreateSimpleBioSourceDialog (GrouP h, CharPtr title)
 
 {
@@ -1416,7 +1619,7 @@ extern DialoG CreateSimpleBioSourceDialog (GrouP h, CharPtr title)
     SelectFont (programFont);
     height = LineHeight ();
     SelectFont (systemFont);
-    gbp->orglist = DocumentPanel (f, stdCharWidth * 25, height * 5);
+    gbp->orglist = DocumentPanel (f, stdCharWidth * 25, height * 6);
     SetObjectExtra (gbp->orglist, gbp, NULL);
     SetDocAutoAdjust (gbp->orglist, FALSE);
     orgListCol [0].pixWidth = screenRect.right - screenRect.left;
@@ -1485,7 +1688,8 @@ static void OrgModPtrToOrgmodDialog (DialoG d, Pointer data)
   if (tlp != NULL) {
     head = NULL;
     while (list != NULL) {
-      if (list->subname != NULL && list->subtype != 255) {
+      if (list->subname != NULL && list->subtype != 255 &&
+          list->subtype != 32 && list->subtype != 33 && list->subtype != 34) {
         vnp = ValNodeNew (head);
         if (head == NULL) {
           head = vnp;
@@ -1703,11 +1907,11 @@ EnumFieldAssocPtr subsource_alists [] = {
 };
 
 Uint2 orgmod_widths [] = {
-  0, 15
+  0, 25
 };
 
 Uint2 subsource_widths [] = {
-  0, 15
+  0, 25
 };
 
 Uint2 orgmod_types [] = {
@@ -1731,11 +1935,11 @@ static CharPtr orgTabs [] = {
 };
 
 static CharPtr modTabs [] = {
-  "Source", "Organism", NULL
+  "Source", "Organism", "GenBank", NULL
 };
 
 static CharPtr modTabsUns [] = {
-  "Source", "Organism", "Unstructured", NULL
+  "Source", "Organism", "GenBank", "Unstructured", NULL
 };
 
 static CharPtr miscTabs [] = {
@@ -1766,10 +1970,10 @@ static void ChangeModSubPage (VoidPtr data, Int2 newval, Int2 oldval)
 
   gbp = (GenBioPagePtr) data;
   if (gbp != NULL) {
-    if (oldval >= 0 && oldval <= 2) {
+    if (oldval >= 0 && oldval <= 3) {
       SafeHide (gbp->modGrp [oldval]);
     }
-    if (newval >= 0 && newval <= 2) {
+    if (newval >= 0 && newval <= 3) {
       SafeShow (gbp->modGrp [newval]);
     }
     Update ();
@@ -1816,6 +2020,7 @@ static void CleanupBioSourceDialog (GraphiC g, VoidPtr data)
   gbp = (GenBioPagePtr) data;
   if (gbp != NULL) {
     gbp->origTaxName = MemFree (gbp->origTaxName);
+    FreeGenBioOrgNameData (gbp);
   }
   StdCleanupExtraProc (g, data);
 }
@@ -1827,7 +2032,7 @@ static DialoG CreateBioSourceDialog (GrouP h, CharPtr title, GrouP PNTR pages,
 {
   ButtoN         b;
   GrouP          c;
-  GrouP          f, f1, f2, f3;
+  GrouP          f, f1, f2, f3, f4;
   GrouP          g;
   GenBioPagePtr  gbp;
   Int2           height;
@@ -1859,6 +2064,8 @@ static DialoG CreateBioSourceDialog (GrouP h, CharPtr title, GrouP PNTR pages,
     gbp->fromdialog = GenBioPageToBioSourcePtr;
     gbp->dialogmessage = BioSourceMessage;
     gbp->testdialog = TestGenBioDialog;
+    gbp->orgname_choice = 0;
+    gbp->orgname_data = NULL;
 
     if (title != NULL && title [0] != '\0') {
       s = NormalGroup (p, 0, -2, title, systemFont, NULL);
@@ -1901,7 +2108,7 @@ static DialoG CreateBioSourceDialog (GrouP h, CharPtr title, GrouP PNTR pages,
     SelectFont (programFont);
     height = LineHeight ();
     SelectFont (systemFont);
-    gbp->orglist = DocumentPanel (f, stdCharWidth * 25, height * 5);
+    gbp->orglist = DocumentPanel (f, stdCharWidth * 25, height * 6);
     SetObjectExtra (gbp->orglist, gbp, NULL);
     SetDocAutoAdjust (gbp->orglist, FALSE);
     orgListCol [0].pixWidth = screenRect.right - screenRect.left;
@@ -2040,25 +2247,44 @@ static DialoG CreateBioSourceDialog (GrouP h, CharPtr title, GrouP PNTR pages,
     x = MultiLinePrompt (f1, "Additional Organism Information", max, programFont);
     gbp->orgcomment = ScrollText (f1, 20, 3, programFont, TRUE, NULL);
     AlignObjects (ALIGN_MIDDLE, (HANDLE) x, (HANDLE) gbp->orgcomment, NULL);
-    AlignObjects (ALIGN_CENTER, (HANDLE) gbp->orgmod, (HANDLE) f1, NULL);
+    f4 = HiddenGroup (g, 1, 0, NULL);
+    gbp->add_type_strain_to_orgcomment = PushButton (f4, "Add type strain to comment", AddTypeStrainButtonProc);
+    SetObjectExtra (gbp->add_type_strain_to_orgcomment, gbp, NULL);
+    AlignObjects (ALIGN_CENTER, (HANDLE) gbp->orgmod, (HANDLE) f1, (HANDLE) f4, NULL);
 
     Hide (gbp->modGrp [1]);
 
     gbp->modGrp [2] = HiddenGroup (k, -1, 0, NULL);
     SetGroupSpacing (gbp->modGrp [2], 10, 10);
 
+    g = HiddenGroup (gbp->modGrp [2], 2, 0, NULL);
+    SetGroupSpacing (g, 3, 10);
+
+    StaticPrompt (g, "Assigned Acronym", 0, stdLineHeight, programFont, 'l');
+    gbp->gbacr = StaticPrompt (g, "", 15 * stdCharWidth, stdLineHeight, systemFont, 'l');
+    StaticPrompt (g, "Assigned Anamorph", 0, stdLineHeight, programFont, 'l');
+    gbp->gbana = StaticPrompt (g, "", 15 * stdCharWidth, stdLineHeight, systemFont, 'l');
+    StaticPrompt (g, "Assigned Synonym", 0, stdLineHeight, programFont, 'l');
+    gbp->gbsyn = StaticPrompt (g, "", 15 * stdCharWidth, stdLineHeight, systemFont, 'l');
+
+    Hide (gbp->modGrp [2]);
+
+    gbp->modGrp [3] = HiddenGroup (k, -1, 0, NULL);
+    SetGroupSpacing (gbp->modGrp [3], 10, 10);
+
     if (showUnstructMods) {
-      f3 = HiddenGroup (gbp->modGrp [2], 0, 2, NULL);
+      f3 = HiddenGroup (gbp->modGrp [3], 0, 2, NULL);
       StaticPrompt (f3, "Unstructured Modifiers", 0, 0, programFont, 'c');
       gbp->mod = CreateVisibleStringDialog (f3, 3, -1, 15);
     }
 
-    Hide (gbp->modGrp [2]);
+    Hide (gbp->modGrp [3]);
 
     AlignObjects (ALIGN_CENTER, (HANDLE) tbs,
                   (HANDLE) gbp->modGrp [0],
                   (HANDLE) gbp->modGrp [1],
-                  (HANDLE) gbp->modGrp [2], NULL);
+                  (HANDLE) gbp->modGrp [2],
+                  (HANDLE) gbp->modGrp [3], NULL);
 
     Hide (pages [1]);
 
