@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   4/30/95
 *
-* $Revision: 6.35 $
+* $Revision: 6.50 $
 *
 * File Description: 
 *
@@ -62,6 +62,15 @@
 extern "C" {
 #endif
 
+typedef struct seqPanelLines {
+  Int4    bioSeqLine;   /* this line refers to bioseq line           */
+  Int2    lineType;     /* what to draw on this line (see ELineType) */
+  Int4    idx;          /* feature index                             */
+  Int4    row;          /* index in alignment                        */
+  Boolean protProduct;
+} SeqPanLine, PNTR SeqPanLinePtr;
+
+
 /* bioseqviewdata pointer is passed to callbacks to display views */
 
 typedef struct bioseqviewdata {
@@ -85,6 +94,21 @@ typedef struct bioseqviewdata {
   GrouP           newGphControlGrp;
   GrouP           pnlParentGrp;
 
+  PaneL           seqView;
+  GrouP           seqViewParentGrp;
+  PopuP           newFeatControl;
+  PopuP           newNumControl;
+  PopuP           newGridControl;
+
+  /* for new sequence and alignment viewers */
+  Int4            LineHeight, LineSpace, CharHeight, CharWidth, BlocksAtLine, CharsAtLine, TotalLines, TargetRow;
+  Int2            SeqStartPosX;  /* Draw Sequence from this x pos */
+  SeqAlignPtr     salp;
+  Boolean         DrawGrid, seqAlignMode;
+  SeqPanLinePtr   PNTR SeqPanLines;
+  ValNodePtr      Selection;
+
+
   GrouP           udvParentGrp;
   /*
   GrouP           ddvParentGrp;
@@ -92,7 +116,6 @@ typedef struct bioseqviewdata {
   PrompT          clickMe;
 
   Boolean         useScrollText;
-  Boolean         showContigJoin;
   Boolean         launchEditors;
   Boolean         launchSubviewers;
   Boolean         sendSelectMessages;
@@ -115,11 +138,15 @@ typedef struct bioseqviewdata {
   PopuP           numControl;
 
   PopuP           ffModeCtrl;
+  PopuP           ffStyleCtrl;
+  ButtoN          ffCustomBtn;
 
   PopuP           newGphStyle;
   PopuP           newGphFilter;
   PopuP           newGphScale;
-  PopuP           newGphLayout;
+  PopuP           newGphAlnScore;
+  PopuP           newGphAlnCutoff;
+  ChoicE          newGphLayout;
 
   ValNodePtr      slp_list;
   ValNodePtr      g_list;
@@ -138,6 +165,8 @@ typedef struct bioseqviewdata {
   PoinT           pnt_stop;
   Boolean         old_rect_shown;
 
+  ValNodePtr      bsplist;
+
   FonT            displayFont;
   ValNodePtr      sentinelList;
   ValNodePtr      entityList;       /* for parts of genome record */
@@ -189,13 +218,19 @@ extern BioseqPageData gnbkPageData;
 extern BioseqPageData emblPageData;
 extern BioseqPageData ddbjPageData;
 extern BioseqPageData gnptPageData;
+extern BioseqPageData ftblPageData;
 
 extern BioseqPageData fstaPageData;
 extern BioseqPageData qualPageData;
 extern BioseqPageData asnPageData;
 extern BioseqPageData xmlPageData;
+extern BioseqPageData gbseqPageData;
 extern BioseqPageData dskPageData;
 
+extern BioseqPageData asn2gphGphPageData;
+extern BioseqPageData seqpnlPageData;
+extern BioseqPageData seqAlnPnlPageData;
+
 /*
 *  The SeqViewProcsPtr may be registered with a call to SetAppProperty
 *  e.g., SetAppProperty ("SeqDisplayForm", &viewprocs), where viewprocs
@@ -255,6 +290,42 @@ typedef struct seqviewprocs {
   Boolean          lockFarComponents;
 } SeqViewProcs, PNTR SeqViewProcsPtr;
 
+typedef struct bioseqviewform {
+  FORM_MESSAGE_BLOCK
+
+  BioseqPagePtr   bioseqNucPageList;
+  BioseqPagePtr   bioseqProtPageList;
+  BioseqPagePtr   currentBioseqPage;
+  ButtoN          pubseq;
+
+  Int2            currentNucPage;
+  Int2            currentProtPage;
+
+  Handle          nucViewControl;
+  Handle          protViewControl;
+  Handle          targetControl;
+  EnumFieldAssoc  PNTR targetAlist;
+  Boolean         usePopupForTarget;
+  Int4            numTargets;
+  Int4            targetScratchSpace;
+  GrouP           controls;
+  GrpActnProc     updateControls;
+  GrouP           retrieveAlignments;
+  SeqViewUpdateFetchCounts  updateCounts;
+  Boolean         hasaligns;
+
+  EnumFieldAssoc  PNTR workingAlist;
+  Int4            workingCount;
+  Int4            workingTargets;
+
+  BioseqViewData  bvd;
+
+  Boolean         cleanupObjectPtr;
+  WndActnProc     activateForm;
+
+  ForM            toolForm;
+} BioseqViewForm, PNTR BioseqViewFormPtr;
+
 #define REGISTER_NEW_SEQENTRY_VIEW ObjMgrProcLoad(OMPROC_VIEW,"View Bioseq Report","Bioseq Report",OBJ_BIOSEQ,0,OBJ_BIOSEQ,0,NULL,NewSeqEntryViewGenFunc,PROC_PRIORITY_DEFAULT)
 #define REGISTER_SMART_SEQENTRY_VIEW ObjMgrProcLoad(OMPROC_VIEW,"View Smart Bioseq Report","Bioseq Report",OBJ_BIOSEQ,0,OBJ_BIOSEQ,0,NULL,SmartSeqEntryViewGenFunc,PROC_PRIORITY_DEFAULT)
 
@@ -318,3 +389,4 @@ extern void EnableDisableLegendItem (BioseqViewPtr bvp, Boolean enable);
 #endif
 
 #endif /* ndef _SEQVIEW_ */
+
