@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.16 $
+* $Revision: 6.20 $
 *
 * File Description: 
 *
@@ -100,7 +100,9 @@ extern ValNodePtr AddStringToValNodeChain (ValNodePtr head, CharPtr str, Uint1 c
   GrouP           useGeneXref;      \
   GrouP           newGeneGrp;       \
   TexT            geneSymbol;       \
+  TexT            geneAllele;       \
   TexT            geneDesc;         \
+  TexT            locusTag;         \
   ButtoN          editGeneBtn;      \
   TexT            protXrefName;     \
   DialoG          location;         \
@@ -108,7 +110,8 @@ extern ValNodePtr AddStringToValNodeChain (ValNodePtr head, CharPtr str, Uint1 c
   DialoG          featcits;         \
   DialoG          dbxrefs;          \
   DialoG          gbquals;          \
-  DialoG          usrobjext;
+  DialoG          usrobjext;        \
+  UserObjectPtr   goTermUserObj;
 
 typedef struct descform {
   DESCRIPTOR_FORM_BLOCK
@@ -118,6 +121,7 @@ typedef struct featform {
   FEATURE_FORM_BLOCK
 } FeatureForm, PNTR FeatureFormPtr;
 
+extern Int2 LIBCALLBACK DescriptorPropagate (Pointer data);
 extern Boolean DescFormReplaceWithoutUpdateProc (ForM f);
 extern void StdDescFormActnProc (ForM f);
 
