@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   2/5/97
 *
-* $Revision: 6.39 $
+* $Revision: 6.75 $
 *
 * File Description: 
 *
@@ -43,9 +43,9 @@
 */
 
 #include <bspview.h>
-#include <asn2ff.h>
-#include <ffprint.h>
+#include <asn2gnbp.h>
 #include <tofasta.h>
+#include <explore.h>
 
 static ParData ffParFmt = {FALSE, FALSE, FALSE, FALSE, TRUE, 0, 0};
 static ColData ffColFmt = {0, 0, 80, 0, NULL, 'l', FALSE, FALSE, FALSE, FALSE, TRUE};
@@ -56,27 +56,27 @@ typedef struct docdescrstruct {
 } DocDescrStruct, PNTR DocDescrPtr;
 
 typedef struct flatstruct {
-  Asn2ffJobPtr      ajp;
-  FFPrintArrayPtr   pap;
-  SeqEntryPtr       sep;
-  Int4              pap_size;
-  Int4              numdescr;
-  DocDescrPtr       descr;
+  Asn2gbJobPtr       ajp;
+  BaseBlockPtr PNTR  paragraphs;
+  Int4               numParagraphs;
+  SeqEntryPtr        sep;
+  Int4               numdescr;
+  DocDescrPtr        descr;
 } FlatStruct, PNTR FlatStructPtr;
 
 static Boolean GetIDsFromDoc (DoC d, Int2 item, Uint2Ptr entityPtr,
                               Uint2Ptr itemPtr, Uint2Ptr typePtr)
 
 {
-  BioseqViewPtr    bvp;
-  Pointer          dataPtr;
-  DescrStructPtr   descr;
-  unsigned int     entityID;
-  FlatStructPtr    fsp;
-  unsigned int     itemID;
-  unsigned int     itemtype;
-  Boolean          okay;
-  FFPrintArrayPtr  pap;
+  BaseBlockPtr       bbp;
+  BioseqViewPtr      bvp;
+  Pointer            dataPtr;
+  unsigned int       entityID;
+  FlatStructPtr      fsp;
+  unsigned int       itemID;
+  unsigned int       itemtype;
+  Boolean            okay;
+  BaseBlockPtr PNTR  paragraphs;
 
   okay = FALSE;
   entityID = 0;
@@ -89,13 +89,13 @@ static Boolean GetIDsFromDoc (DoC d, Int2 item, Uint2Ptr entityPtr,
     GetItemParams (d, item, NULL, NULL, NULL, NULL, &dataPtr);
     if (dataPtr != NULL) {
       fsp = (FlatStructPtr) dataPtr;
-      pap = fsp->pap;
-      if (pap != NULL) {
-        descr = pap [item - 1].descr;
-        if (descr != NULL) {
-          entityID = descr->entityID;
-          itemID = descr->itemID;
-          itemtype = descr->itemtype;
+      paragraphs = fsp->paragraphs;
+      if (paragraphs != NULL) {
+        bbp = paragraphs [item - 1];
+        if (bbp != NULL) {
+          entityID = bbp->entityID;
+          itemID = bbp->itemID;
+          itemtype = bbp->itemtype;
           okay = (Boolean) (entityID > 0 && itemID > 0 && itemtype > 0);
         }
       }
@@ -469,8 +469,7 @@ static void DocFreeFlat (DoC d, VoidPtr data)
 
   if (data != NULL) {
     fsp = (FlatStructPtr) data;
-    asn2ff_cleanup (fsp->ajp);
-    MemFree (fsp->ajp);
+    fsp->ajp = asn2gnbk_cleanup (fsp->ajp);
     MemFree (fsp->descr);
     MemFree (fsp);
   }
@@ -479,17 +478,17 @@ static void DocFreeFlat (DoC d, VoidPtr data)
 static CharPtr FFPrintFunc (DoC d, Int2 index, Pointer data)
 
 {
-  BaseFormPtr      bfp;
-  BioseqPtr        bsp;
-  BioseqViewPtr    bvp;
-  DescrStructPtr   descr;
-  FlatStructPtr    fsp;
-  ErrSev           level;
-  SeqEntryPtr      oldsep = NULL;
-  OMUserDataPtr    omudp;
-  FFPrintArrayPtr  pap;
-  CharPtr          str = NULL;
-  SeqEntryPtr      topsep;
+  BaseBlockPtr       bbp;
+  BaseFormPtr        bfp;
+  BioseqPtr          bsp;
+  BioseqViewPtr      bvp;
+  FlatStructPtr      fsp;
+  ErrSev             level;
+  SeqEntryPtr        oldsep = NULL;
+  OMUserDataPtr      omudp;
+  BaseBlockPtr PNTR  paragraphs;
+  CharPtr            str = NULL;
+  SeqEntryPtr        topsep;
 
   bvp = (BioseqViewPtr) GetObjectExtra (d);
   fsp = (FlatStructPtr) data;
@@ -501,18 +500,18 @@ static CharPtr FFPrintFunc (DoC d, Int2 index, Pointer data)
     topsep = GetTopSeqEntryForEntityID (bfp->input_entityID);
     oldsep = SeqEntrySetScope (topsep);
   }
-  str = FFPrint (fsp->pap, (Int4) (index - 1), fsp->pap_size);
+  str = asn2gnbk_format (fsp->ajp, (Int4) (index - 1));
   if (oldsep != NULL) {
     SeqEntrySetScope (oldsep);
   }
   if (bfp != NULL) {
-    pap = fsp->pap;
-    if (pap != NULL) {
-      descr = pap [index - 1].descr;
-      if (descr != NULL && descr->entityID != 0 && descr->entityID != bfp->input_entityID) {
-        if (! InBioseqViewEntityList (descr->entityID, bvp)) {
-          ValNodeAddInt (&(bvp->entityList), 0, (Int4) descr->entityID);
-          omudp = ObjMgrAddUserData (descr->entityID, bfp->procid,
+    paragraphs = fsp->paragraphs;
+    if (paragraphs != NULL) {
+      bbp = paragraphs [index - 1];
+      if (bbp != NULL && bbp->entityID != 0 && bbp->entityID != bfp->input_entityID) {
+        if (! InBioseqViewEntityList (bbp->entityID, bvp)) {
+          ValNodeAddInt (&(bvp->entityList), 0, (Int4) bbp->entityID);
+          omudp = ObjMgrAddUserData (bbp->entityID, bfp->procid,
 	                                 OMPROC_VIEW, bfp->userkey);
           if (omudp != NULL) {
             omudp->userdata.ptrvalue = (Pointer) bfp;
@@ -564,34 +563,244 @@ static int LIBCALLBACK SortDescrProc (VoidPtr vp1, VoidPtr vp2)
   return 0;
 }
 
-static void LookForGEDseqID (BioseqPtr bsp, Pointer userdata)
+typedef struct lookforids {
+  Boolean isGED;
+  Boolean isNTorNW;
+  Boolean isNC;
+  Boolean isTPA;
+  Boolean isAEorCH;
+} LookForIDs, PNTR LookForIDsPtr;
+
+static void LookForSeqIDs (BioseqPtr bsp, Pointer userdata)
 
 {
-  BoolPtr   isGEDPtr;
-  SeqIdPtr  sip;
+  LookForIDsPtr  lfip;
+  SeqIdPtr       sip;
+  TextSeqIdPtr   tsip;
 
-  isGEDPtr = (BoolPtr) userdata;
+  lfip = (LookForIDsPtr) userdata;
   for (sip = bsp->id; sip != NULL; sip = sip->next) {
     switch (sip->choice) {
       case SEQID_GENBANK :
       case SEQID_EMBL :
       case SEQID_DDBJ :
-        *isGEDPtr = TRUE;
-        return;
+        lfip->isGED = TRUE;
+        tsip = (TextSeqIdPtr) sip->data.ptrvalue;
+        if (tsip != NULL) {
+          if (StringNCmp (tsip->accession, "AE", 2) == 0) {
+            lfip->isAEorCH = TRUE;
+          } else if (StringNCmp (tsip->accession, "CH", 2) == 0) {
+            lfip->isAEorCH = TRUE;
+          }
+        }
+        break;
+      case SEQID_TPG :
+      case SEQID_TPE :
+      case SEQID_TPD :
+        lfip->isTPA = TRUE;
+        break;
+      case SEQID_OTHER :
+        tsip = (TextSeqIdPtr) sip->data.ptrvalue;
+        if (tsip != NULL) {
+          if (StringNCmp (tsip->accession, "NC_", 3) == 0) {
+            lfip->isNC = TRUE;
+          } else if (StringNCmp (tsip->accession, "NT_", 3) == 0) {
+            lfip->isNTorNW = TRUE;
+          } else if (StringNCmp (tsip->accession, "NW_", 3) == 0) {
+            lfip->isNTorNW = TRUE;
+          }
+        }
+        break;
       default :
         break;
     }
   }
 }
 
-static Boolean PopulateFF (DoC d, SeqEntryPtr sep, BioseqPtr bsp, Uint1 format, Uint1 mode, Boolean show_gene, Boolean showContigJoin)
+static void LookForGEDetc (
+  SeqEntryPtr topsep,
+  BoolPtr isGED,
+  BoolPtr isNTorNW,
+  BoolPtr isNC,
+  BoolPtr isTPA,
+  BoolPtr isAEorCH
+)
 
 {
-  Asn2ffJobPtr    ajp;
+  LookForIDs  lfi;
+
+  MemSet ((Pointer) &lfi, 0, sizeof (LookForIDs));
+  VisitBioseqsInSep (topsep, (Pointer) &lfi, LookForSeqIDs);
+  *isGED = lfi.isGED;
+  *isNTorNW = lfi.isNTorNW;
+  *isNC = lfi.isNC;
+  *isTPA = lfi.isTPA;
+  *isAEorCH = lfi.isAEorCH;
+}
+
+static void LookForNonLocalID (BioseqPtr bsp, Pointer userdata)
+
+{
+  BoolPtr   isNonLocalPtr;
+  SeqIdPtr  sip;
+
+  isNonLocalPtr = (BoolPtr) userdata;
+  for (sip = bsp->id; sip != NULL; sip = sip->next) {
+    switch (sip->choice) {
+      case SEQID_LOCAL :
+        break;
+      default :
+        *isNonLocalPtr = TRUE;
+        break;
+    }
+  }
+}
+
+static void LookForPubs (BioseqPtr bsp, Pointer userdata)
+
+{
+  BoolPtr  hasPub;
+
+  hasPub = (BoolPtr) userdata;
+  *hasPub = TRUE;
+}
+
+static void CheckVersionWithGi (BioseqPtr bsp, Pointer userdata)
+
+{
+  Boolean       hasGi = FALSE;
+  BoolPtr       missingVersion;
+  SeqIdPtr      sip;
+  TextSeqIdPtr  tsip;
+  Boolean       zeroVersion = FALSE;
+
+  for (sip = bsp->id; sip != NULL; sip = sip->next) {
+    switch (sip->choice) {
+      case SEQID_TPG:
+      case SEQID_TPE:
+      case SEQID_TPD:
+      case SEQID_GENBANK:
+      case SEQID_EMBL:
+      case SEQID_DDBJ:
+        tsip = (TextSeqIdPtr) sip->data.ptrvalue;
+        if (tsip != NULL && tsip->version == 0) {
+          zeroVersion = TRUE;
+        }
+        break;
+      case SEQID_GI :
+        hasGi = TRUE;
+        break;
+      default :
+        break;
+    }
+  }
+  if (hasGi && zeroVersion) {
+    missingVersion = (BoolPtr) userdata;
+    *missingVersion = TRUE;
+  }
+}
+
+static CharPtr relmodemsg1 = "Record cannot only have a local Seq-id for release mode";
+static CharPtr relmodemsg2 = "Record must have a publication for release mode";
+static CharPtr relmodemsg3 = "Record with gi must have version number for release mode";
+static CharPtr relmodemsg4 = "Release mode failure";
+
+static CharPtr RelModeFailText (
+  BioseqPtr bsp,
+  SeqEntryPtr usethetop,
+  SeqEntryPtr topsep
+)
+
+{
+  SeqMgrDescContext  dcontext;
+  SeqMgrFeatContext  fcontext;
+  Boolean            missingVersion;
+  Boolean            nonLocalID;
+  Boolean            hasPubs;
+
+  nonLocalID = FALSE;
+  if (usethetop) {
+    VisitBioseqsInSep (usethetop, (Pointer) &nonLocalID, LookForNonLocalID);
+  } else {
+    LookForNonLocalID (bsp, (Pointer) &nonLocalID);
+  }
+  if (! nonLocalID) {
+    return StringSave (relmodemsg1);
+  }
+
+  hasPubs = FALSE;
+  if (usethetop) {
+    if (VisitPubdescsInSep (usethetop, NULL, NULL) == 0) {
+      return StringSave (relmodemsg2);
+    }
+  } else {
+    if (SeqMgrGetNextDescriptor (bsp, NULL, Seq_descr_pub, &dcontext) == NULL) {
+      if (SeqMgrGetNextFeature (bsp, NULL, SEQFEAT_PUB, 0, &fcontext) == NULL) {
+        return StringSave (relmodemsg2);
+      }
+    }
+  }
+
+  missingVersion = FALSE;
+  VisitBioseqsInSep (topsep, (Pointer) &missingVersion, CheckVersionWithGi);
+  if (missingVersion) {
+    return StringSave (relmodemsg3);
+  }
+
+  return StringSave (relmodemsg4);
+}
+
+static Int2 asn2gb_line_estimate [27] = {
+  0,
+  1,
+  1,
+  1,
+  1,
+  1,
+  1,
+  1,
+  1,
+  1,
+  1,
+  1,
+  2, /* organism */
+  6, /* reference */
+  4, /* primary */
+  4, /* comment */
+  1,
+  4, /* source */
+  6, /* feature */
+  1,
+  1,
+ 20, /* sequence */
+  4, /* contig */
+  1,
+  1,
+  1,
+  1
+};
+
+static Boolean PopulateFF (
+  DoC d,
+  SeqEntryPtr sep,
+  BioseqPtr bsp,
+  SeqEntryPtr usethetop,
+  FmtType format,
+  ModType mode,
+  StlType style,
+  FlgType flags,
+  CstType custom
+)
+
+{
+  Asn2gbJobPtr    ajp;
+  BaseBlockPtr    bbp;
+  BlockType       blocktype;
+  BioseqSetPtr    bssp;
   BioseqViewPtr   bvp;
-  DescrStructPtr  descr;
   DocDescrPtr     doscr;
   DocDescrPtr     doscrp;
+  Int2            estimate;
   FonT            fnt;
   FlatStructPtr   fsp;
   Int4            index;
@@ -610,42 +819,30 @@ static Boolean PopulateFF (DoC d, SeqEntryPtr sep, BioseqPtr bsp, Uint1 format,
       fsp->sep = sep;
       SetDocData (d, (Pointer) fsp, DocFreeFlat);
       level = ErrSetMessageLevel (SEV_MAX);
-      ajp = MemNew (sizeof (Asn2ffJob));
+      if (usethetop != NULL && IS_Bioseq_set (usethetop)) {
+        bssp = (BioseqSetPtr) usethetop->data.ptrvalue;
+        ajp = asn2gnbk_setup (NULL, bssp, NULL, format, mode, style, flags, 0, custom, NULL);
+      } else {
+        ajp = asn2gnbk_setup (bsp, NULL, NULL, format, mode, style, flags, 0, custom, NULL);
+      }
       if (ajp == NULL) return FALSE;
       fsp->ajp = ajp;
-      ajp->sep = sep;
-      ajp->mode = mode;
-      ajp->format = format;
-      ajp->show_version = TRUE;
-      ajp->gb_style = TRUE;
-      ajp->show_seq = TRUE;
-      ajp->show_gi = TRUE;
-      ajp->error_msgs = FALSE;
-      ajp->non_strict = TRUE;
-      ajp->Spop = spop;
-      ajp->show_gene = show_gene;
-      if (/* IsAGenomeRecord (sep) || */
-          IsSegmentedBioseqWithoutParts (sep)) {
-        ajp->only_one = TRUE;
-        ajp->genome_view = TRUE;
-        ajp->contig_view = (Boolean) (! showContigJoin);
-      } else if (IsADeltaBioseq (sep) && (! DeltaLitOnly (bsp))) {
-        ajp->only_one = TRUE;
-        ajp->genome_view = TRUE;
-        ajp->contig_view = (Boolean) (! showContigJoin);
-      }
-      if (GetAppProperty ("InternalNcbiSequin") != NULL) {
-        ajp->bankit = TRUE;
-      }
-      fsp->pap_size = asn2ff_setup (ajp, &(fsp->pap));
-      if (fsp->pap_size > 0) {
-        asn2ff_set_output (NULL, "\n");
+      fsp->numParagraphs = ajp->numParagraphs;
+      fsp->paragraphs = ajp->paragraphArray;
+      if (fsp->numParagraphs > 0 && fsp->paragraphs != NULL) {
         fsp->numdescr = 0;
-        for (index = 0; index < fsp->pap_size; index++) {
+        for (index = 0; index < fsp->numParagraphs; index++) {
+          estimate = 1;
+          bbp = fsp->paragraphs [index];
+          if (bbp != NULL) {
+            blocktype = bbp->blocktype;
+            if (blocktype >= HEAD_BLOCK && blocktype <= TAIL_BLOCK) {
+              estimate = asn2gb_line_estimate [(int) blocktype];
+            }
+          }
           AppendItem (d, FFPrintFunc, (Pointer) fsp, FALSE,
-                      fsp->pap [index].estimate, &ffParFmt,
-                      &ffColFmt, fnt);
-          if (fsp->pap [index].descr != NULL) {
+                      estimate, &ffParFmt, &ffColFmt, fnt);
+          if (bbp != NULL) {
             (fsp->numdescr)++;
           }
         }
@@ -653,13 +850,13 @@ static Boolean PopulateFF (DoC d, SeqEntryPtr sep, BioseqPtr bsp, Uint1 format,
         fsp->descr = doscrp;
         fsp->numdescr = 0;
         if (doscrp != NULL) {
-          for (index = 0; index < fsp->pap_size; index++) {
-            descr = fsp->pap [index].descr;
-            if (descr != NULL) {
+          for (index = 0; index < fsp->numParagraphs; index++) {
+            bbp = fsp->paragraphs [index];
+            if (bbp != NULL) {
               doscr = &(doscrp [fsp->numdescr]);
-              doscr->entityID = descr->entityID;
-              doscr->itemID = descr->itemID;
-              doscr->itemtype = descr->itemtype;
+              doscr->entityID = bbp->entityID;
+              doscr->itemID = bbp->itemID;
+              doscr->itemtype = bbp->itemtype;
               doscr->docitem = index + 1;
               (fsp->numdescr)++;
             }
@@ -674,37 +871,131 @@ static Boolean PopulateFF (DoC d, SeqEntryPtr sep, BioseqPtr bsp, Uint1 format,
   return rsult;
 }
 
-static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format, Boolean show_gene)
+static void LookForTpa (
+  SeqDescrPtr sdp,
+  Pointer userdata
+)
 
 {
-  Asn2ffJobPtr  ajp;
-  BioseqPtr     bsp;
-  DoC           doc;
-  Uint2         entityID;
-  FILE          *fp;
-  Int2          into;
-  Boolean       isGenBankEMBLorDDBJ;
-  Boolean       showContigJoin;
-  Int2          item;
-  ErrSev        level;
-  Uint1         mode;
-  SeqEntryPtr   oldsep;
-  Char          path [PATH_MAX];
-  BaR           sb = NULL;
-  SeqEntryPtr   sep;
-  Int4          startsAt;
-  SeqEntryPtr   topsep;
-  TexT          txt;
+  BoolPtr        hastpaP;
+  ObjectIdPtr    oip;
+  UserObjectPtr  uop;
+
+  if (sdp == NULL || sdp->choice != Seq_descr_user) return;
+  uop = (UserObjectPtr) sdp->data.ptrvalue;
+  if (uop == NULL) return;
+
+  oip = uop->type;
+  if (oip == NULL) return;
+  if (StringCmp (oip->str, "TpaAssembly") != 0) return;
+
+  hastpaP = (BoolPtr) userdata;
+  *hastpaP = TRUE;
+}
+
+static void PopulateFlatFile (BioseqViewPtr bvp, FmtType format, FlgType flags)
+
+{
+  BioseqPtr          bsp;
+  SeqMgrFeatContext  context;
+  CstType            custom = 0;
+  DoC                doc;
+  Boolean            doLockFarComponents = FALSE;
+  Uint2              entityID;
+  Int4               feats_with_product_count;
+  FonT               fnt;
+  FILE               *fp;
+  Boolean            hastpaaligns;
+  Int2               into;
+  Boolean            isAEorCH;
+  Boolean            isGED;
+  Boolean            isNTorNW;
+  Boolean            isNC;
+  Boolean            isTPA;
+  Int2               item;
+  ErrSev             level;
+  Boolean            lockFar = FALSE;
+  Boolean            lookupFar = FALSE;
+  ModType            mode = SEQUIN_MODE;
+  SeqEntryPtr        oldsep;
+  Char               path [PATH_MAX];
+  BaR                sb = NULL;
+  SeqEntryPtr        sep;
+  SeqFeatPtr         sfp;
+  Int4               startsAt;
+  CharPtr            str;
+  StlType            style = NORMAL_STYLE;
+  SeqViewProcsPtr    svpp;
+  SeqEntryPtr        topsep;
+  TexT               txt;
+  SeqEntryPtr        usethetop = NULL;
+  Int2               val;
 
   if (bvp == NULL) return;
-  if (bvp->hasTargetControl) {
-    if (bvp->ffModeCtrl != NULL && GetValue (bvp->ffModeCtrl) == 2) {
-      mode = RELEASE_MODE;
+  if (bvp->hasTargetControl && bvp->ffModeCtrl != NULL) {
+    val = GetValue (bvp->ffModeCtrl);
+    switch (val) {
+      case 1 :
+        mode = RELEASE_MODE;
+        break;
+      case 2 :
+        mode = ENTREZ_MODE;
+        break;
+      case 3 :
+        mode = SEQUIN_MODE;
+        break;
+      case 4 :
+        mode = DUMP_MODE;
+        break;
+      default :
+        break;
+    }
+  }
+  /* now using control instead of seqid type */
+  if (bvp->ffStyleCtrl != NULL) {
+    val = GetValue (bvp->ffStyleCtrl);
+    switch (val) {
+      case 1 :
+        style = NORMAL_STYLE;
+        lookupFar = TRUE;
+        break;
+      case 2 :
+        style = SEGMENT_STYLE;
+        lockFar = TRUE;
+        break;
+      case 3 :
+        style = MASTER_STYLE;
+        lockFar = TRUE;
+        break;
+      case 4 :
+        style = CONTIG_STYLE;
+        lookupFar = TRUE;
+        break;
+      default :
+        break;
+    }
+  }
+
+  bsp = bvp->bsp;
+  entityID = ObjMgrGetEntityIDForPointer (bsp);
+  topsep = GetTopSeqEntryForEntityID (entityID);
+  LookForGEDetc (topsep, &isGED, &isNTorNW, &isNC, &isTPA, &isAEorCH);
+
+  if ((flags & SHOW_CONTIG_FEATURES) != 0 || (flags & SHOW_CONTIG_SOURCES) != 0) {
+    if (isNTorNW || isTPA) {
+      lockFar = FALSE;
+      lookupFar = TRUE;
+      if (GetAppProperty ("InternalNcbiSequin") != NULL) {
+        doLockFarComponents = TRUE;
+      }
     } else {
-      mode = SEQUIN_MODE;
+      lockFar = TRUE;
+    }
+  }
+  if (bvp->hasTargetControl && bvp->ffCustomBtn != NULL) {
+    if (GetStatus (bvp->ffCustomBtn)) {
+      custom |= SHOW_TRANCRIPTION | SHOW_PEPTIDE;
     }
-  } else {
-    mode = RELEASE_MODE;
   }
   doc = NULL;
   txt = NULL;
@@ -734,11 +1025,61 @@ static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format, Boolean show_gene
       Message (MSG_ERROR, "StdPrintOptionsNew failed");
     }
   }
+
+  if (GetAppProperty ("OldFlatfileSource") != NULL) {
+    flags |= USE_OLD_SOURCE_ORG;
+  }
+
+  svpp = (SeqViewProcsPtr) GetAppProperty ("SeqDisplayForm");
+  if (svpp != NULL) {
+    if (svpp->lockFarComponents) {
+      doLockFarComponents = TRUE;
+    }
+  }
+
+  if (mode == ENTREZ_MODE) {
+    doLockFarComponents = FALSE;
+    lockFar = FALSE;
+    lookupFar = FALSE;
+    flags = flags ^ (SHOW_CONTIG_FEATURES | SHOW_CONTIG_SOURCES | SHOW_FAR_TRANSLATION);
+  }
+
+  if (doLockFarComponents) {
+    entityID = ObjMgrGetEntityIDForPointer (bsp);
+    sep = GetTopSeqEntryForEntityID (entityID);
+    if (bvp->bsplist == NULL && lockFar) {
+      bvp->bsplist = LockFarComponentsEx (sep, TRUE, FALSE, FALSE, NULL);
+    }
+    if (lookupFar) {
+      feats_with_product_count = 0;
+      sfp = SeqMgrGetNextFeature (bsp, NULL, 0, 0, &context);
+      while (sfp != NULL) {
+        if (sfp->product != NULL) {
+          feats_with_product_count++;
+        }
+        sfp = SeqMgrGetNextFeature (bsp, sfp, 0, 0, &context);
+      }
+      if (feats_with_product_count > 500) {
+        /* too many to lookup with current caching implementation */
+        lookupFar = FALSE;
+      }
+    }
+    if (lookupFar) {
+      hastpaaligns = FALSE;
+      VisitDescriptorsInSep (sep, (Pointer) &hastpaaligns, LookForTpa);
+      LookupFarSeqIDs (sep, TRUE, TRUE, TRUE, FALSE, hastpaaligns);
+    }
+  }
+
   sep = SeqMgrGetSeqEntryForData (bsp);
   entityID = ObjMgrGetEntityIDForChoice (sep);
   if (bvp->hasTargetControl) {
     if (bvp->viewWholeEntity) {
       sep = GetTopSeqEntryForEntityID (entityID);
+      usethetop = sep;
+      if (format == FTABLE_FMT) {
+        custom |= SHOW_PROT_FTABLE;
+      }
     } else if (ISA_na (bsp->mol) && bsp->repr == Seq_repr_seg) {
       sep = GetBestTopParentForData (entityID, bsp);
     } else if (ISA_aa (bsp->mol) && bsp->repr == Seq_repr_seg) {
@@ -757,67 +1098,66 @@ static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format, Boolean show_gene
   WatchCursor ();
   ffColFmt.pixWidth = screenRect.right - screenRect.left;
   ffColFmt.pixInset = 8;
-  isGenBankEMBLorDDBJ = FALSE;
-  VisitBioseqsInSep (topsep, (Pointer) &isGenBankEMBLorDDBJ, LookForGEDseqID);
-  /* now using control instead of seqid type */
-  showContigJoin = bvp->showContigJoin;
-  if (bvp->useScrollText) {
-    ajp = MemNew (sizeof (Asn2ffJob));
-    if (ajp != NULL) {
-      TmpNam (path);
-      fp = FileOpen (path, "w");
-      if (fp != NULL) {
-        level = ErrSetMessageLevel (SEV_MAX);
-        ajp->sep = sep;
-        ajp->mode = mode;
-        ajp->format = format;
-        ajp->show_version = TRUE;
-        ajp->gb_style = TRUE;
-        ajp->show_seq = TRUE;
-        ajp->show_gi = TRUE;
-        ajp->error_msgs = FALSE;
-        ajp->non_strict = TRUE;
-        ajp->Spop = spop;
-        ajp->show_gene = show_gene;
-        if (/* IsAGenomeRecord (sep) || */
-            IsSegmentedBioseqWithoutParts (sep)) {
-          ajp->only_one = TRUE;
-          ajp->genome_view = TRUE;
-          ajp->contig_view = (Boolean) (! showContigJoin);
-        } else if (IsADeltaBioseq (sep) && (! DeltaLitOnly (bsp))) {
-          ajp->only_one = TRUE;
-          ajp->genome_view = TRUE;
-          ajp->contig_view = (Boolean) (! showContigJoin);
-        }
-        if (GetAppProperty ("InternalNcbiSequin") != NULL) {
-          ajp->bankit = TRUE;
-        }
-        ajp->fp = fp;
-        if (SeqEntryToFlatAjp (ajp, sep, fp, format, mode)) {
-          FileClose (fp);
-          if (! FileToScrollText (txt, path)) {
-            SetTitle (txt, "(Text is too large to be displayed in this control.)");
-          }
-        } else {
-          FileClose (fp);
-        }
-        ErrSetMessageLevel (level);
-      }
-      FileRemove (path);
+  /* LookForGEDetc (topsep, &isGED, &isNTorNW, &isNC, &isTPA); */
+  if ((flags & SHOW_CONTIG_FEATURES) != 0 || (flags & SHOW_CONTIG_SOURCES) != 0) {
+    if (isNTorNW || isTPA) {
+      flags |= ONLY_NEAR_FEATURES;
+    } else if (isNC) {
+      flags |= NEAR_FEATURES_SUPPRESS;
+    } else if (isAEorCH) {
+      flags |= NEAR_FEATURES_SUPPRESS;
     }
-    MemFree (ajp);
+  }
+  if (bvp->useScrollText) {
+    TmpNam (path);
+    fp = FileOpen (path, "w");
+    if (fp != NULL) {
+      level = ErrSetMessageLevel (SEV_MAX);
+      if (SeqEntryToGnbk (sep, NULL, format, mode, style, flags, 0, custom, NULL, fp)) {
+        FileClose (fp);
+        if (! FileToScrollText (txt, path)) {
+          SetTitle (txt, "(Text is too large to be displayed in this control.)");
+        }
+      } else if (mode == RELEASE_MODE) {
+        str = RelModeFailText (bsp, usethetop, topsep);
+        if (str != NULL) {
+          fprintf (fp, "%s", str);
+        }
+        FileClose (fp);
+        if (str != NULL) {
+          FileToScrollText (txt, path);
+        }
+        MemFree (str);
+      } else {
+        FileClose (fp);
+      }
+      ErrSetMessageLevel (level);
+    }
+    FileRemove (path);
   } else {
-    PopulateFF (doc, sep, bsp, format, mode, show_gene, showContigJoin);
-    SetDocShade (doc, DrawIcon, NULL, NULL, NULL);
-    SetDocProcs (doc, ClickIcon, NULL, ReleaseIcon, NULL);
-    SetDocCache (doc, StdPutDocCache, StdGetDocCache, StdResetDocCache);
-    SetDocAutoAdjust (doc, FALSE);
-    ForceFormat (doc, item);
-    SetDocAutoAdjust (doc, TRUE);
-    AdjustDocScroll (doc);
-    GetItemParams4 (doc, item, &startsAt, NULL, NULL, NULL, NULL);
-    CorrectBarValue (sb, startsAt + into);
-    UpdateDocument (doc, 0, 0);
+    if (PopulateFF (doc, sep, bsp, usethetop, format, mode, style, flags, custom)) {
+      SetDocShade (doc, DrawIcon, NULL, NULL, NULL);
+      SetDocProcs (doc, ClickIcon, NULL, ReleaseIcon, NULL);
+      SetDocCache (doc, StdPutDocCache, StdGetDocCache, StdResetDocCache);
+      SetDocAutoAdjust (doc, FALSE);
+      ForceFormat (doc, item);
+      SetDocAutoAdjust (doc, TRUE);
+      AdjustDocScroll (doc);
+      GetItemParams4 (doc, item, &startsAt, NULL, NULL, NULL, NULL);
+      CorrectBarValue (sb, startsAt + into);
+      UpdateDocument (doc, 0, 0);
+    } else if (mode == RELEASE_MODE) {
+      str = RelModeFailText (bsp, usethetop, topsep);
+      if (str != NULL) {
+        fnt = programFont;
+        if (bvp != NULL && bvp->displayFont != NULL) {
+          fnt = bvp->displayFont;
+        }
+        AppendText (doc, str, &ffParFmt, &ffColFmt, fnt);
+      }
+      MemFree (str);
+      UpdateDocument (doc, 0, 0);
+    }
   }
 
   SeqEntrySetScope (oldsep);
@@ -829,7 +1169,7 @@ static void PopulateFlatFile (BioseqViewPtr bvp, Uint1 format, Boolean show_gene
 static void PopulateGenBank (BioseqViewPtr bvp)
 
 {
-  PopulateFlatFile (bvp, GENBANK_FMT, TRUE);
+  PopulateFlatFile (bvp, GENBANK_FMT, SHOW_CONTIG_FEATURES | SHOW_CONTIG_SOURCES | SHOW_FAR_TRANSLATION);
 }
 
 static void PopulateEMBL (BioseqViewPtr bvp)
@@ -837,43 +1177,50 @@ static void PopulateEMBL (BioseqViewPtr bvp)
 {
   if (bvp == NULL) return;
   if (bvp->hasTargetControl) {
-    PopulateFlatFile (bvp, PSEUDOEMBL_FMT, TRUE);
+    PopulateFlatFile (bvp, EMBL_FMT, 0);
   } else {
-    PopulateFlatFile (bvp, EMBL_FMT, TRUE);
+    PopulateFlatFile (bvp, EMBL_FMT, 0);
   }
 }
 
 static void PopulateDDBJ (BioseqViewPtr bvp)
 
 {
-  PopulateFlatFile (bvp, GENBANK_FMT, FALSE);
+  PopulateFlatFile (bvp, GENBANK_FMT, DDBJ_VARIANT_FORMAT);
 }
 
 static void PopulateGenPept (BioseqViewPtr bvp)
 
 {
-  PopulateFlatFile (bvp, GENPEPT_FMT, TRUE);
+  PopulateFlatFile (bvp, GENPEPT_FMT, 0);
+}
+
+static void PopulateFTable (BioseqViewPtr bvp)
+
+{
+  PopulateFlatFile (bvp, FTABLE_FMT, 0);
 }
 
 static void PopulateFasta (BioseqViewPtr bvp)
 
 {
-  BioseqPtr    bsp;
-  DoC          doc;
-  Uint2        entityID;
-  Boolean      fastaOK;
-  Boolean      fastaNucOK;
-  Boolean      fastaPrtOK;
-  FonT         fnt;
-  FILE         *fp;
-  Uint1        group_segs;
-  Int2         into;
-  Int2         item;
-  Char         path [PATH_MAX];
-  BaR          sb = NULL;
-  SeqEntryPtr  sep;
-  Int4         startsAt;
-  TexT         txt;
+  BioseqPtr        bsp;
+  DoC              doc;
+  Uint2            entityID;
+  Boolean          fastaOK;
+  Boolean          fastaNucOK;
+  Boolean          fastaPrtOK;
+  FonT             fnt;
+  FILE             *fp;
+  Uint1            group_segs;
+  Int2             into;
+  Int2             item;
+  Char             path [PATH_MAX];
+  BaR              sb = NULL;
+  SeqEntryPtr      sep;
+  Int4             startsAt;
+  SeqViewProcsPtr  svpp;
+  TexT             txt;
 
   if (bvp == NULL) return;
   doc = NULL;
@@ -895,6 +1242,16 @@ static void PopulateFasta (BioseqViewPtr bvp)
     SetDocAutoAdjust (doc, FALSE);
   }
   if (bsp == NULL) return;
+
+  svpp = (SeqViewProcsPtr) GetAppProperty ("SeqDisplayForm");
+  if (svpp != NULL && svpp->lockFarComponents) {
+    entityID = ObjMgrGetEntityIDForPointer (bsp);
+    sep = GetTopSeqEntryForEntityID (entityID);
+    if (bvp->bsplist == NULL) {
+      bvp->bsplist = LockFarComponentsEx (sep, TRUE, FALSE, FALSE, NULL);
+    }
+  }
+
   sep = SeqMgrGetSeqEntryForData (bsp);
   if (bvp->hasTargetControl) {
     if (bvp->viewWholeEntity) {
@@ -1099,21 +1456,36 @@ static void PopulateQuality (BioseqViewPtr bvp)
   Update ();
 }
 
-static void PopulateAsnOrXML (BioseqViewPtr bvp, CharPtr mode)
+NLM_EXTERN void AsnPrintNewLine PROTO((AsnIoPtr aip));
+
+static void PopulateAsnOrXML (BioseqViewPtr bvp, CharPtr outmode, Boolean doGbseq)
 
 {
-  AsnIoPtr     aipout;
+  AsnIoPtr     aipout = NULL;
+  AsnTypePtr   atp = NULL;
   BioseqPtr    bsp;
+  CstType      custom = 0;
   DoC          doc;
   Uint2        entityID;
+  XtraPtr      extra = NULL;
   FonT         fnt;
+  FmtType      format = GENBANK_FMT;
+  GBSeq        gbsq;
+  GBSet        gbst;
   Int2         into;
   Int2         item;
+  Boolean      lockFar = FALSE;
+  Boolean      lookupFar = FALSE;
+  ModType      mode = SEQUIN_MODE;
   Char         path [PATH_MAX];
   BaR          sb = NULL;
   SeqEntryPtr  sep;
   Int4         startsAt;
+  StlType      style = NORMAL_STYLE;
   TexT         txt;
+  Int2         val;
+  Char         xmlbuf [128];
+  XtraBlock    xtra;
 
   if (bvp == NULL) return;
   doc = NULL;
@@ -1153,12 +1525,104 @@ static void PopulateAsnOrXML (BioseqViewPtr bvp, CharPtr mode)
   ffColFmt.pixWidth = screenRect.right - screenRect.left;
   ffColFmt.pixInset = 8;
   TmpNam (path);
-  aipout = AsnIoOpen (path, mode);
-  if (aipout != NULL) {
-    fnt = programFont;
-    if (bvp != NULL && bvp->displayFont != NULL) {
-      fnt = bvp->displayFont;
+  aipout = AsnIoOpen (path, outmode);
+  fnt = programFont;
+  if (bvp != NULL && bvp->displayFont != NULL) {
+    fnt = bvp->displayFont;
+  }
+  if (bsp != NULL && ISA_aa (bsp->mol)) {
+    format = GENPEPT_FMT;
+  }
+  if (doGbseq && aipout != NULL) {
+    if (bvp->hasTargetControl && bvp->ffModeCtrl != NULL) {
+      val = GetValue (bvp->ffModeCtrl);
+      switch (val) {
+        case 1 :
+          mode = RELEASE_MODE;
+          break;
+        case 2 :
+          mode = ENTREZ_MODE;
+          break;
+        case 3 :
+          mode = SEQUIN_MODE;
+          break;
+        case 4 :
+          mode = DUMP_MODE;
+          break;
+        default :
+          break;
+      }
     }
+    /* now using control instead of seqid type */
+    if (bvp->ffStyleCtrl != NULL) {
+      val = GetValue (bvp->ffStyleCtrl);
+      switch (val) {
+        case 1 :
+          style = NORMAL_STYLE;
+          lookupFar = TRUE;
+          break;
+        case 2 :
+          style = SEGMENT_STYLE;
+          lockFar = TRUE;
+          break;
+        case 3 :
+          style = MASTER_STYLE;
+          lockFar = TRUE;
+          break;
+        case 4 :
+          style = CONTIG_STYLE;
+          lookupFar = TRUE;
+          break;
+        default :
+          break;
+      }
+    }
+    if (bvp->hasTargetControl && bvp->ffCustomBtn != NULL) {
+      if (GetStatus (bvp->ffCustomBtn)) {
+        custom = SHOW_TRANCRIPTION | SHOW_PEPTIDE;
+      }
+    }
+    if (GetAppParam ("NCBI", "SETTINGS", "XMLPREFIX", NULL, xmlbuf, sizeof (xmlbuf))) {
+      AsnSetXMLmodulePrefix (StringSave (xmlbuf));
+    }
+    objgbseqAsnLoad ();
+    objinsdseqAsnLoad ();
+    MemSet ((Pointer) &xtra, 0, sizeof (XtraBlock));
+    MemSet ((Pointer) &gbsq, 0, sizeof (GBSeq));
+    xtra.gbseq = &gbsq;
+    xtra.aip = aipout;
+    /*
+    atp = AsnLinkType (NULL, AsnFind ("GBSet"));
+    xtra.atp = AsnLinkType (NULL, AsnFind ("GBSet.E"));
+    */
+    atp = AsnLinkType (NULL, AsnFind ("INSDSet"));
+    xtra.atp = AsnLinkType (NULL, AsnFind ("INSDSet.E"));
+    extra = &xtra;
+    MemSet ((Pointer) &gbst, 0, sizeof (GBSet));
+    AsnOpenStruct (aipout, atp, (Pointer) &gbst);
+    if (SeqEntryToGnbk (sep, NULL, format, mode, style, CREATE_XML_GBSEQ_FILE, 0, custom, extra, NULL)) {
+      AsnCloseStruct (aipout, atp, NULL);
+      AsnPrintNewLine (aipout);
+      AsnIoClose (aipout);
+      if (bvp->useScrollText) {
+        if (! FileToScrollText (txt, path)) {
+          SetTitle (txt, "(Text is too large to be displayed in this control.)");
+        }
+      } else {
+        DisplayFancy (doc, path, &ffParFmt, &ffColFmt, fnt, 4);
+        SetDocCache (doc, StdPutDocCache, StdGetDocCache, StdResetDocCache);
+        SetDocAutoAdjust (doc, FALSE);
+        ForceFormat (doc, item);
+        SetDocAutoAdjust (doc, TRUE);
+        AdjustDocScroll (doc);
+        GetItemParams4 (doc, item, &startsAt, NULL, NULL, NULL, NULL);
+        CorrectBarValue (sb, startsAt + into);
+        UpdateDocument (doc, 0, 0);
+      }
+    } else {
+      AsnIoClose (aipout);
+    }
+  } else if (aipout != NULL) {
     if (SeqEntryAsnWrite (sep, aipout, NULL)) {
       AsnIoClose (aipout);
       if (bvp->useScrollText) {
@@ -1188,13 +1652,19 @@ static void PopulateAsnOrXML (BioseqViewPtr bvp, CharPtr mode)
 static void PopulateXML (BioseqViewPtr bvp)
 
 {
-  PopulateAsnOrXML (bvp, "wx");
+  PopulateAsnOrXML (bvp, "wx", FALSE);
 }
 
 static void PopulateAsn (BioseqViewPtr bvp)
 
 {
-  PopulateAsnOrXML (bvp, "w");
+  PopulateAsnOrXML (bvp, "w", FALSE);
+}
+
+static void PopulateGBSeq (BioseqViewPtr bvp)
+
+{
+  PopulateAsnOrXML (bvp, "wx", TRUE);
 }
 
 static void ShowFlatFile (BioseqViewPtr bvp, Boolean show)
@@ -1208,8 +1678,8 @@ static void ShowFlatFile (BioseqViewPtr bvp, Boolean show)
       SafeShow (bvp->doc);
     }
     SafeShow (bvp->baseCtgControlGrp);
-    SafeShow (bvp->docTxtControlGrp);
     SafeShow (bvp->modeControlGrp);
+    SafeShow (bvp->docTxtControlGrp);
     SafeShow (bvp->clickMe);
   } else {
     SafeHide (bvp->text);
@@ -1242,8 +1712,42 @@ static void ShowFastaOrAsn (BioseqViewPtr bvp, Boolean show)
       SafeShow (bvp->doc);
     }
     SafeHide (bvp->modeControlGrp);
-    SafeShow (bvp->docTxtControlGrp);
     SafeHide (bvp->baseCtgControlGrp);
+    SafeShow (bvp->docTxtControlGrp);
+    SafeHide (bvp->clickMe);
+  } else {
+    SafeHide (bvp->text);
+    SafeHide (bvp->doc);
+    Reset (bvp->text);
+    Reset (bvp->doc);
+    SetDocShade (bvp->doc, NULL, NULL, NULL, NULL);
+    SetDocProcs (bvp->doc, NULL, NULL, NULL, NULL);
+    SetDocCache (bvp->doc, NULL, NULL, NULL);
+    SafeHide (bvp->styleControlGrp);
+    SafeHide (bvp->scaleControlGrp);
+    EnableDisableLegendItem (bvp, FALSE);
+    SafeHide (bvp->findGeneGrp);
+    SafeHide (bvp->docTxtControlGrp);
+    SafeHide (bvp->baseCtgControlGrp);
+    SafeHide (bvp->modeControlGrp);
+    SafeHide (bvp->newGphControlGrp);
+    SafeHide (bvp->clickMe);
+  }
+}
+
+static void ShowGBSeq (BioseqViewPtr bvp, Boolean show)
+
+{
+  if (bvp == NULL) return;
+  if (show) {
+    if (bvp->useScrollText) {
+      SafeShow (bvp->text);
+    } else {
+      SafeShow (bvp->doc);
+    }
+    SafeShow (bvp->modeControlGrp);
+    SafeShow (bvp->baseCtgControlGrp);
+    SafeShow (bvp->docTxtControlGrp);
     SafeHide (bvp->clickMe);
   } else {
     SafeHide (bvp->text);
@@ -1511,17 +2015,20 @@ static void PrintFlatFileFastaOrAsn (BioseqViewPtr bvp)
   }
 }
 
-static void ExportFlatFileFastaOrAsn (BioseqViewPtr bvp, CharPtr filename, CharPtr dfault)
+static void ExportFlatFileFastaOrAsnEx (BioseqViewPtr bvp, CharPtr filename, CharPtr dfault, Boolean specialSave)
 
 {
-  Char  ch;
-  Char  dfaultFile [32];
-  DoC   doc;
-  FILE  *f;
-  Int2  j;
-  Int2  k;
-  Char  path [PATH_MAX];
-  TexT  txt;
+  Char     ch;
+  Char     dfaultFile [32];
+  DoC      doc;
+  FILE     *f;
+  Int2     i;
+  Int2     j;
+  Int2     k;
+  Int2     numItems;
+  Char     path [PATH_MAX];
+  CharPtr  str;
+  TexT     txt;
 
   if (bvp == NULL) return;
   if (bvp->bsp == NULL) return;
@@ -1582,6 +2089,15 @@ static void ExportFlatFileFastaOrAsn (BioseqViewPtr bvp, CharPtr filename, CharP
       if (f != NULL) {
         if (bvp->useScrollText) {
           ScrollTextToFile (txt, path);
+        } else if (specialSave) {
+          GetDocParams (doc, &numItems, NULL);
+          for (i = 1; i <= numItems; i++) {
+            str = GetDocText (doc, i, 0, 0);
+            if (! StringHasNoText (str)) {
+              fprintf (f, "%s", str);
+            }
+            MemFree (str);
+          }
         } else {
           SaveDocument (doc, f);
         }
@@ -1592,6 +2108,18 @@ static void ExportFlatFileFastaOrAsn (BioseqViewPtr bvp, CharPtr filename, CharP
   }
 }
 
+static void ExportFlatFileFastaOrAsn (BioseqViewPtr bvp, CharPtr filename, CharPtr dfault)
+
+{
+  ExportFlatFileFastaOrAsnEx (bvp, filename, dfault, FALSE);
+}
+
+static void ExportFeatureTable (BioseqViewPtr bvp, CharPtr filename, CharPtr dfault)
+
+{
+  ExportFlatFileFastaOrAsnEx (bvp, filename, dfault, TRUE);
+}
+
 static CharPtr asnconfirmmsg =
 "'Export' saves only the targeted portion of the record.\n\
 Use 'Save' to save the entire record to a file, so that\n\
@@ -1655,8 +2183,15 @@ BioseqPageData gnptPageData = {
   ExportFlatFileFastaOrAsn, NULL, ResizeFlatFileFastaOrAsn, NULL
 };
 
+BioseqPageData ftblPageData = {
+  "Table", TRUE, TRUE, TRUE, FALSE, -1,
+  PopulateFTable, ShowFlatFile, SelectFlatFile,
+  CopyFlatFileFastaOrAsn, PrintFlatFileFastaOrAsn,
+  ExportFeatureTable, NULL, ResizeFlatFileFastaOrAsn, NULL
+};
+
 BioseqPageData fstaPageData = {
-  "FASTA", TRUE, TRUE, FALSE, FALSE, -1,
+  "FASTA", TRUE, TRUE, TRUE, FALSE, -1,
   PopulateFasta, ShowFastaOrAsn, NULL,
   CopyFlatFileFastaOrAsn, PrintFlatFileFastaOrAsn,
   ExportFlatFileFastaOrAsn, NULL, ResizeFlatFileFastaOrAsn, NULL
@@ -1683,3 +2218,10 @@ BioseqPageData xmlPageData = {
   ExportAsnAfterConfirming, NULL, ResizeFlatFileFastaOrAsn, NULL
 };
 
+BioseqPageData gbseqPageData = {
+  "INSDSeq", TRUE, TRUE, TRUE, FALSE, -1,
+  PopulateGBSeq, ShowGBSeq, NULL,
+  CopyFlatFileFastaOrAsn, PrintFlatFileFastaOrAsn,
+  ExportAsnAfterConfirming, NULL, ResizeFlatFileFastaOrAsn, NULL
+};
+
