@@ -28,7 +28,7 @@
 *   
 * Version Creation Date: 8/31/93
 *
-* $Revision: 6.0 $
+* $Revision: 6.14 $
 *
 * File Description:  Medline Utilities for MedArch
 *   Assumes user calls MedArchInit and Fini
@@ -44,6 +44,58 @@
 *
 * RCS Modification History:
 * $Log: medutil.c,v $
+* Revision 6.14  2004/03/19 18:32:03  bazhin
+* One more patch related to the previous one.
+*
+* Revision 6.13  2004/03/19 14:53:18  bazhin
+* Added function PropogateInPress(), which is called from FixPubEquiv().
+* It sets Imprint.prepub value to "in-press" if input Cit-art was flagged
+* as "in-press", but had Medline or/and Pubmed id(s) and successful lookup
+* returned Cit-art with no article references, such as volume and page
+* numbers.
+*
+* Revision 6.12  2004/03/17 16:59:35  bazhin
+* Slight modification in ten_authors() function. If the number of input
+* authors greater than 10 and number of MedArch ones less or equal 11,
+* or input has more than 25 and MedArch has less or equal 26, then
+* it will take input set instead of MedArch one.
+*
+* Revision 6.11  2004/03/16 18:53:37  bazhin
+* Now does not ignore in-press'ed Cit-arts for lookup.
+*
+* Revision 6.10  2003/10/01 13:08:33  bazhin
+* Modified ten_authors() function to handle consortiums properly.
+*
+* Revision 6.9  2003/09/10 18:47:28  bazhin
+* "print_pub()" function now is aware of consortiums.
+*
+* Revision 6.8  2003/03/25 19:14:59  bazhin
+* Function "ten_authors()" became public (from static).
+*
+* Revision 6.7  2003/03/12 20:35:02  bazhin
+* "ten_authors()" function now can handle consortiums.
+*
+* Revision 6.6  2001/03/08 12:47:44  ostell
+* made FixPub work if no medline uid returned, but pmid is returned.
+*
+* Revision 6.5  2000/08/18 17:01:02  kans
+* added FetchPubPmId, enhanced FixPubEquiv to handle records with pmid but no muid
+*
+* Revision 6.4  1999/11/17 18:16:15  bazhin
+* If failed to get pub from MedArch server, then does not return NULL,
+* but use input one.
+* Also removed unused variables.
+*
+* Revision 6.3  1999/11/08 19:32:44  kans
+* removed unnecessary for loop initialization picked up by IRIX C comnpiler
+*
+* Revision 6.2  1999/05/06 18:53:53  kans
+* FixPubEquiv now works on PMID as well as MUID
+*
+* Revision 6.1  1998/09/14 20:54:51  bazhin
+* Changes in "ten_authors()" : if successful medline lookup has returned
+* no authors, then will use input ones.
+*
 * Revision 6.0  1997/08/25 18:35:48  madden
 * Revision changed to 6.0
 *
@@ -144,14 +196,22 @@ void print_pub(ValNodePtr pub, Boolean found, Boolean auth, Int4 muid)
 	      
 	      if (art->authors->choice == 1) {
 		 	aup = v->data.ptrvalue;
-		 	if (aup != NULL) {
-		    	namestd = aup->name->data;
-		    	if (namestd->names[0]) {
-		    		last = namestd->names[0];
-		    	}
-		    	if (namestd->names[4]) {
-		    		first = namestd->names[4];
-		    	}
+		 	if (aup != NULL)
+			{
+				if(aup->name->choice == 2)
+				{
+			    		namestd = aup->name->data;
+				    	if (namestd->names[0])
+					{
+			    			last = namestd->names[0];
+				    	}
+			    		if (namestd->names[4])
+					{
+				    		first = namestd->names[4];
+				    	}
+				}
+				else if(aup->name->choice == 5)
+					last = aup->name->data;
 		 	}
 	      } else  {
 		 	first = "";
@@ -192,7 +252,7 @@ void print_pub(ValNodePtr pub, Boolean found, Boolean auth, Int4 muid)
 		vol = "no volume number";
 	}
 	if (auth) {
-		ErrPostEx(SEV_WARNING, ERR_REFERENCE_MedlineMatchIgnored,  
+		ErrPostEx(SEV_ERROR, ERR_REFERENCE_MedlineMatchIgnored,  
     	"Too many author name differences: %ld|%s %s|%s|(%d)|%s|%s", 
     				(long) muid, last, first, s_title, (int) year, vol, page);
     	return;
@@ -221,81 +281,189 @@ void print_pub(ValNodePtr pub, Boolean found, Boolean auth, Int4 muid)
     return;
 }
 
-/*--------------------------- ten_authors() ---------------------------*/
-
-/****************************************************************************
-*    ten_authors   
-* 											
-****************************************************************************/
-static Boolean ten_authors(CitArtPtr art, CitArtPtr art_tmp) 
+/**********************************************************/
+static Boolean ten_authors_compare(CitArtPtr capold, CitArtPtr capnew)
 {
-	Int2 num, numnew, i, match, n;
-	ValNodePtr v, pub;
-	CharPtr ptr, mu[10];
-	AuthorPtr	aup;
-    NameStdPtr  namestd;
-    Boolean		no_compare = TRUE, ret = TRUE;
-		
-		if (art == NULL || art_tmp == NULL) {
-			return TRUE;
-		}
-		numnew = 0;
-		num = 0;
-		match = 0;
-		if (art->authors != NULL)
-		{
-			for (v = art->authors->names; v != NULL;
-									v = v->next, num++);
-		}
-		if (art_tmp->authors != NULL)
-		{
-			for (v = art_tmp->authors->names; v != NULL && numnew < 10;
-								v = v->next, numnew++) {
-				aup = v->data.ptrvalue;
-				namestd	= aup->name->data;
-				mu[numnew] = namestd->names[0];	
-			}
-		}
+    ValNodePtr old;
+    ValNodePtr new;
+    ValNodePtr vnp;
+    CharPtr    namesnew[10];
+    Int4       numold;
+    Int4       numnew;
+    Int4       match;
+    Int4       i;
 
-		if (art->authors != NULL && art_tmp->authors != NULL) {
-			if (art->authors->choice == 1 && art_tmp->authors->choice == 1) {
-				no_compare = FALSE;
-				for (v = art->authors->names; v != NULL; v = v->next) {
-					aup = v->data.ptrvalue;
-					namestd	= aup->name->data;
-					ptr = namestd->names[0];
-					for (i = 0; i < numnew; i++) {
-						if (StringICmp(ptr, mu[i]) == 0) {
-							match++;
-							break;
-						}
-					}	
-				}
-			} 
-		}
-		if (no_compare) {
-			ret = TRUE;  /* replace */
-			if (num == 0 && numnew == 0) {
-					ret = TRUE;			/* replace */
-			} else if (num == 0) {   /* no original authors - replace*/
-				ret = TRUE;
-			} else if (numnew == 0) { /* no Medline authors -  don't replace*/
-				ret = FALSE;
-			}
-		} else {
-				n = num < numnew?num:numnew;
-				if (n > 3*match) { 
-					ret = FALSE;
-				}
-		}
-		if (ret && num > 10)     /* original article has > 10 authors */
-		{
-			AuthListFree(art_tmp->authors);  /* keep original authors */
-			art_tmp->authors = art->authors;
-			art->authors = NULL;
-		}
-		return ret;
+    if(capold == NULL || capold->authors == NULL ||
+       capold->authors->names == NULL)
+        return(TRUE);
+    if(capnew == NULL || capnew->authors == NULL ||
+       capnew->authors->names == NULL)
+        return(FALSE);
+    if(capold->authors->choice != capnew->authors->choice ||
+       capold->authors->choice == 1)
+        return(TRUE);
+
+    old = capold->authors->names;
+    new = capnew->authors->names;
+
+    for(numnew = 0, vnp = old; vnp != NULL; vnp = vnp->next)
+        if(vnp->data.ptrvalue != NULL)
+            numnew++;
+
+    for(numold = 0, vnp = new; vnp != NULL && numold < 10; vnp = vnp->next)
+        if(vnp->data.ptrvalue != NULL)
+            namesnew[numold++] = vnp->data.ptrvalue;
+
+    for(match = 0, vnp = old; vnp != NULL; vnp = vnp->next)
+    {
+        if(vnp->data.ptrvalue == NULL)
+            continue;
+
+        for(i = 0; i < numnew; i++)
+        {
+            if(StringICmp(vnp->data.ptrvalue, namesnew[i]) == 0)
+            {
+                match++;
+                break;
+            }
+        }
+    }
+
+    i = (numold < numnew) ? numold : numnew;
+    if(i > 3 * match)
+        return(FALSE);
+
+    if(numold > 10)
+    {
+        AuthListFree(capnew->authors);
+        capnew->authors = capold->authors;
+        capold->authors = NULL;
+    }
+    return(TRUE);
 }
+
+/**********************************************************/
+Boolean ten_authors(CitArtPtr art, CitArtPtr art_tmp)
+{
+    NameStdPtr namestd;
+    ValNodePtr v;
+    AuthorPtr  aup;
+    CharPtr    mu[10];
+    CharPtr    oldcon;
+    CharPtr    newcon;
+    Int2       num;
+    Int2       numnew;
+    Int2       numtmp;
+    Int2       i;
+    Int2       match;
+
+    if(art_tmp == NULL)
+        return(FALSE);
+    if(art_tmp->authors == NULL || art_tmp->authors->names == NULL)
+    {
+        if(art != NULL && art->authors != NULL)
+        {
+            if(art_tmp->authors != NULL)
+                AuthListFree(art_tmp->authors);
+            art_tmp->authors = art->authors;
+            art->authors = NULL;
+        }
+        return(TRUE);
+    }
+    if(art == NULL || art->authors == NULL || art->authors->names == NULL ||
+       art->authors->choice != art_tmp->authors->choice)
+        return(TRUE);
+
+    if(art->authors->choice != 1)
+        return(ten_authors_compare(art, art_tmp));
+
+    oldcon = NULL;
+    for(num = 0, v = art->authors->names; v != NULL; v = v->next)
+    {
+        aup = v->data.ptrvalue;
+        if(aup->name->choice == 2)
+            num++;
+        else if(aup->name->choice == 5)
+            oldcon = aup->name->data;
+    }
+
+    newcon = NULL;
+    for(numtmp = 0, v = art_tmp->authors->names; v != NULL; v = v->next)
+    {
+        aup = v->data.ptrvalue;
+        if(aup->name->choice == 2)
+            numtmp++;
+        else if(aup->name->choice == 5)
+            newcon = aup->name->data;
+    }
+
+    if(oldcon != NULL)
+    {
+        if(newcon == NULL)
+        {
+            ErrPostEx(SEV_WARNING, ERR_REFERENCE_NoConsortAuthors,
+                      "Publication as returned by MedArch lacks consortium authors of the original publication: \"%s\".",
+                      oldcon);
+            aup = AuthorNew();
+            aup->name = PersonIdNew();
+            aup->name->choice = 5;
+            aup->name->data = StringSave(oldcon);
+            v = ValNodeNew(NULL);
+            v->data.ptrvalue = aup;
+            v->next = art_tmp->authors->names;
+            art_tmp->authors->names = v;
+            newcon = oldcon;
+        }
+        else if(StringICmp(oldcon, newcon) != 0)
+        {
+            ErrPostEx(SEV_WARNING, ERR_REFERENCE_DiffConsortAuthors,
+                      "Consortium author names differ. Original is \"%s\". MedArch's is \"%s\".",
+                      oldcon, newcon);
+        }
+        if(num == 0)
+            return(TRUE);
+    }
+
+    numnew = 0;
+    for(v = art_tmp->authors->names; v != NULL && numnew < 10; v = v->next)
+    {
+        aup = v->data.ptrvalue;
+        if(aup->name->choice != 2)
+            continue;
+        namestd = aup->name->data;
+        mu[numnew++] = namestd->names[0];
+    }
+
+    for(match = 0, v = art->authors->names; v != NULL; v = v->next)
+    {
+        aup = v->data.ptrvalue;
+        if(aup->name->choice != 2)
+            continue;
+
+        namestd = aup->name->data;
+        for(i = 0; i < numnew; i++)
+        {
+            if(StringICmp(namestd->names[0], mu[i]) == 0)
+            {
+                match++;
+                break;
+            }
+        }
+    }
+
+    i = (num < numnew) ? num : numnew;
+    if(i > 3 * match)
+        return(FALSE);
+
+    if(numtmp == 0 || (num > 10 && numtmp < 12) || (num > 25 && numtmp < 27))
+    {
+        AuthListFree(art_tmp->authors);
+        art_tmp->authors = art->authors;
+        art->authors = NULL;
+    }
+    return(TRUE);
+}
+
 /*****************************************************************************
 *
 *   FindPub
@@ -329,7 +497,7 @@ void FindPub(SeqEntryPtr sep, Pointer data, Int4 index, Int2 indent)
 		sap = bssp->annot;
 	}
 
-	for (vnp; vnp != NULL; vnp = vnp->next)
+	for (; vnp != NULL; vnp = vnp->next)
 	{
 		if (vnp->choice == Seq_descr_pub)
 		{
@@ -339,7 +507,7 @@ void FindPub(SeqEntryPtr sep, Pointer data, Int4 index, Int2 indent)
 		}
 	}
 
-	for (sap; sap != NULL; sap = sap->next)
+	for (; sap != NULL; sap = sap->next)
 	{
 		if (sap->type == 1)   /* feature table */
 		{
@@ -384,9 +552,8 @@ void FindPub(SeqEntryPtr sep, Pointer data, Int4 index, Int2 indent)
 *****************************************************************************/
 ValNodePtr FixPub (ValNodePtr pub, FindPubOptionPtr fpop)
 {
-	ValNodePtr newpub, tmp, v;
-	Int4 		muid;
-	Int2 		num, numnew;
+	ValNodePtr newpub, tmp;
+	Int4 		muid, pmid;
 	CitArtPtr	cit;
 	
 
@@ -447,6 +614,48 @@ ValNodePtr FixPub (ValNodePtr pub, FindPubOptionPtr fpop)
 					newpub = pub;
 					MedlineToISO(pub);
 				}
+				break; /* got muid, done */
+			}
+
+			pmid = MedArchCitMatchPmId(pub);
+			if (pmid)              /* matched it */
+			{
+				print_pub(pub, TRUE, FALSE, muid);
+				fpop->lookups_succeeded++;
+				if (fpop->replace_cit)
+				{
+					fpop->fetches_attempted++;
+					tmp = FetchPubPmId(pmid);
+				
+					if (tmp != NULL)
+					{
+						if (ten_authors(pub->data.ptrvalue,
+									tmp->data.ptrvalue)) 
+						{
+
+							fpop->fetches_succeeded++;
+							PubFree(pub);
+							pub = ValNodeNew(tmp);
+							pub->choice = PUB_PMid;
+							pub->data.intvalue = pmid;
+							newpub = ValNodeNew(NULL);
+							newpub->choice = PUB_Equiv;
+							newpub->data.ptrvalue = tmp;
+						}
+						else 
+						{
+							print_pub(pub, FALSE, TRUE, muid);
+							newpub = pub;
+							MedlineToISO(pub);
+						}
+					}
+				}
+				else
+				{
+					print_pub(pub, FALSE, FALSE, pmid);
+					newpub = pub;
+					MedlineToISO(pub);
+				}
 			}
 			break;
 		case PUB_Equiv:
@@ -488,6 +697,58 @@ ValNodePtr SplitMedlineEntry(MedlineEntryPtr mep)
 	return uid;
 }
 
+/**********************************************************/
+static Boolean if_inpress_set(CitArtPtr cap)
+{
+    CitJourPtr jour;
+    CitBookPtr book;
+
+    if(cap == NULL || cap->fromptr == NULL)
+        return(FALSE);
+
+    if(cap->from == 1)
+    {
+        jour = cap->fromptr;
+        if(jour->imp != NULL && jour->imp->prepub == 2)
+            return(TRUE);
+    }
+    else if(cap->from == 2 || cap->from == 3)
+    {
+        book = cap->fromptr;
+        if(book->imp != NULL && book->imp->prepub == 2)
+            return(TRUE);
+    }
+    return(FALSE);
+}
+
+/**********************************************************/
+static void PropogateInPress(Boolean inpress, ValNodePtr vnp)
+{
+    CitJourPtr jour;
+    CitBookPtr book;
+    CitArtPtr  cap;
+
+    if(inpress == FALSE || vnp == NULL || vnp->data.ptrvalue == NULL)
+        return;
+
+    cap = vnp->data.ptrvalue;
+    if(in_press(cap) == FALSE || cap->fromptr == NULL)
+        return;
+
+    if(cap->from == 1)
+    {
+        jour = cap->fromptr;
+        if(jour->imp != NULL)
+            jour->imp->prepub = 2;
+    }
+    else if(cap->from == 2 || cap->from == 3)
+    {
+        book = cap->fromptr;
+        if(book->imp != NULL)
+            book->imp->prepub = 2;
+    }
+}
+
 /*****************************************************************************
 *
 *   FixPubEquiv()
@@ -495,16 +756,25 @@ ValNodePtr SplitMedlineEntry(MedlineEntryPtr mep)
 *****************************************************************************/
 ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 {
-	ValNodePtr tmp, newpub, muidptr=NULL, citartptr=NULL, mepptr = NULL,
+	ValNodePtr tmp, muidptr=NULL, pmidptr=NULL, citartptr=NULL, mepptr = NULL,
 		otherptr = NULL, tmp2, next, new;
-	Int2 muidctr = 0, citartctr = 0, mepctr = 0, otherctr = 0;
-	Int4 muid = 0, oldmuid=0;
-	Int2 num, numnew;
-	ValNodePtr v;
+	Int2 muidctr = 0, pmidctr = 0, citartctr = 0, mepctr = 0, otherctr = 0;
+	Int4 muid = 0, oldmuid=0, pmid = 0, oldpmid=0;
 	CitArtPtr	cit;
+	Boolean		got;
+	Boolean		inpress;
 
 	if (pube == NULL) return NULL;
 
+	for(got = FALSE, tmp = pube; tmp != NULL; tmp = tmp->next)
+	{
+		if(tmp->choice == PUB_Muid || tmp->choice == PUB_PMid)
+		{
+			got = TRUE;
+			break;
+		}
+	}
+
 	for (tmp = pube; tmp != NULL; tmp = next)
 	{
 		next = tmp->next;
@@ -522,9 +792,20 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 					tmp2->next = tmp;
 				}
 				break;
+			case PUB_PMid:
+				pmidctr++;
+				if (pmidptr == NULL)
+					pmidptr = tmp;
+				else
+				{
+					for (tmp2 = pmidptr; tmp2->next != NULL; tmp2 = tmp2->next)
+						continue;
+					tmp2->next = tmp;
+				}
+				break;
 			case PUB_Article:
 				cit = tmp->data.ptrvalue;
-				if (cit->from == 2 || in_press(cit)) {
+				if (cit->from == 2 || (in_press(cit) && got == FALSE)) {
 					otherctr++;
 					if (otherptr == NULL)
 						otherptr = tmp;
@@ -580,7 +861,7 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 			continue;
 	}
 	
-	if ((muidctr != 0) && (! fpop->always_look))   /* got an muid */
+	if ((muidctr != 0 || pmidctr != 0) && (! fpop->always_look))   /* got an muid */
 	{
 		if (mepptr != NULL)
 		{
@@ -595,6 +876,18 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 			while (tmp2->next != NULL)
 				tmp2 = tmp2->next;
 		}
+		if (pmidptr != NULL) {
+			if (tmp2 == NULL)
+			{
+				tmp2 = pmidptr;
+				pube = tmp2;
+			}
+			else
+				tmp2->next = pmidptr;
+			
+			while (tmp2->next != NULL)
+				tmp2 = tmp2->next;
+		}
 		if (muidptr != NULL)
 		{
 			if (tmp2 == NULL)
@@ -629,6 +922,7 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 			mepptr->next = PubEquivFree(mepptr->next);
 		}
 		PubEquivFree(muidptr);   /* ditch others */
+		PubEquivFree(pmidptr);
 		PubEquivFree(citartptr);
 		tmp = SplitMedlineEntry((MedlineEntryPtr)(mepptr->data.ptrvalue));
 		ValNodeFree(mepptr);
@@ -639,6 +933,23 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 		return pube;
 	}
 
+	if (pmidctr != 0)    /* have a pmid */
+	{
+		oldpmid = pmidptr->data.intvalue;
+		if (pmidctr > 1)     /* more than one, just take the first */
+		{
+			for (tmp = pmidptr->next; tmp != NULL; tmp = tmp->next)
+			{
+				if (tmp->data.intvalue != pmid)
+				{
+					ErrPostEx(SEV_WARNING, ERR_REFERENCE_Multiple_muid, "Two different pmids in Pub-equiv [%ld] [%ld]\n",
+						(long) oldpmid, (long) (tmp->data.intvalue));
+				}
+			}
+		 	pmidptr->next = PubEquivFree(pmidptr->next);
+		}
+	}
+
 	if (muidctr != 0)    /* have an muid */
 	{
 		oldmuid = muidptr->data.intvalue;
@@ -664,6 +975,7 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 			citartptr->next = PubEquivFree(citartptr->next);
 		}
 
+		inpress = if_inpress_set(citartptr->data.ptrvalue);
 		fpop->lookups_attempted++;
 		muid = MedArchCitMatch(citartptr);
 		if (muid != 0)   /* success */
@@ -705,6 +1017,7 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 						tmp->next = new;
 						
 						PubFree(citartptr);
+						citartptr = new;
 					} else {
 						print_pub(citartptr, FALSE, TRUE, muid);
 						PubFree(new);
@@ -717,6 +1030,23 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 							citartptr->next = muidptr;
 					}
 				}
+				else
+				{
+					ErrPostEx(SEV_ERROR, 0, 0, "Failed to get pub from MedArch server for muid = %ld. Input one is preserved.", muid);
+					if (muidptr != NULL)
+						tmp = muidptr;
+					else
+					{
+						tmp = ValNodeNew(tmp2);
+						tmp->choice = PUB_Muid;
+					}
+					if (tmp2 == NULL)
+						pube = tmp;
+					tmp->data.intvalue = muid;
+					tmp2 = tmp;
+					tmp->next = citartptr;
+					MedlineToISO(citartptr);
+				}
 			}
 			else
 			{
@@ -734,8 +1064,103 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 				tmp->next = citartptr;
 				MedlineToISO(citartptr);
 			}
+			PropogateInPress(inpress, citartptr);
 			return pube;
 		}
+
+		/* muid lookup failed */
+
+		pmid = MedArchCitMatchPmId(citartptr);
+		if (pmid != 0)   /* success */
+		{
+			print_pub(citartptr, TRUE, FALSE, pmid);
+			fpop->lookups_succeeded++;
+			if (oldpmid)   /* already had an muid */
+			{
+				if (oldpmid != pmid)
+				{
+					ErrPostEx(SEV_ERROR, ERR_REFERENCE_MuidMissmatch, "OldPMID=%ld doesn't match lookup (%ld). Keeping lookup.",
+						(long)oldpmid, (long)pmid);
+				}
+			}
+			if (fpop->replace_cit)
+			{
+				fpop->fetches_attempted++;
+				new = FetchPubPmId(pmid);
+				if (new != NULL) 
+				{
+					fpop->fetches_succeeded++;
+
+					if (ten_authors(citartptr->data.ptrvalue,
+											new->data.ptrvalue))
+					{
+						if (pmidptr != NULL)
+							tmp = pmidptr;
+						else
+						{
+							tmp = ValNodeNew(tmp2);
+							tmp->choice = PUB_PMid;
+						}
+						if (tmp2 == NULL) {
+							pube = tmp;
+						} else {
+							tmp2->next = tmp;
+						}
+						tmp->data.intvalue = pmid;
+						tmp->next = new;
+						
+						PubFree(citartptr);
+						citartptr = new;
+					} else {
+						print_pub(citartptr, FALSE, TRUE, pmid);
+						PubFree(new);
+						if (tmp2 == NULL) {
+							pube = citartptr;
+						} else {
+							tmp2->next = citartptr;
+						}
+						if (pmidptr != NULL)
+							citartptr->next = pmidptr;
+					}
+				}
+				else
+				{
+					ErrPostEx(SEV_ERROR, 0, 0, "Failed to get pub from MedArch server for pmid = %ld. Input one is preserved.", pmid);
+					if (pmidptr != NULL)
+						tmp = pmidptr;
+					else
+					{
+						tmp = ValNodeNew(tmp2);
+						tmp->choice = PUB_PMid;
+					}
+					if (tmp2 == NULL)
+						pube = tmp;
+					tmp->data.intvalue = pmid;
+					tmp2 = tmp;
+					tmp->next = citartptr;
+					MedlineToISO(citartptr);
+				}
+			}
+			else
+			{
+				if (pmidptr != NULL)
+					tmp = pmidptr;
+				else
+				{
+					tmp = ValNodeNew(tmp2);
+					tmp->choice = PUB_PMid;
+				}
+				if (tmp2 == NULL)
+					pube = tmp;
+				tmp->data.intvalue = pmid;
+				tmp2 = tmp;
+				tmp->next = citartptr;
+				MedlineToISO(citartptr);
+			}
+			PropogateInPress(inpress, citartptr);
+			return pube;
+		}
+
 		print_pub(citartptr, FALSE, FALSE, oldmuid);
 		if (tmp2 == NULL)
 			pube = citartptr;
@@ -743,10 +1168,40 @@ ValNodePtr FixPubEquiv(ValNodePtr pube, FindPubOptionPtr fpop)
 			tmp2->next = citartptr;
 		if (muidptr != NULL)   /* ditch the mismatched muid */
 			PubFree(muidptr);
+		PropogateInPress(inpress, citartptr);
 		return pube;
 
 	}
 		
+	if (oldpmid != 0)    /* have a pmid but no cit-art */
+	{
+		fpop->fetches_attempted++;
+		tmp = MedArchGetPubPmId(oldpmid);
+		if (tmp == NULL)
+		{
+			ErrPostEx(SEV_WARNING, ERR_REFERENCE_No_reference, "Cant find article for pmid [%ld]", (long) oldpmid);
+		}
+		else
+		{
+			fpop->fetches_succeeded++;
+			if (fpop->replace_cit)
+			{
+				tmp = MedlineToISO(tmp);
+				if (tmp2 == NULL)
+				 	pube = tmp;
+				else
+					tmp2->next = tmp;
+				tmp2 = tmp;
+			} else {
+				PubFree(tmp);
+			}
+		}
+		if (tmp2 == NULL)
+		 	pube = pmidptr;
+		else
+			tmp2->next = pmidptr;
+	}
+
 	if (oldmuid != 0)    /* have an muid but no cit-art */
 	{
 		fpop->fetches_attempted++;
@@ -797,6 +1252,16 @@ ValNodePtr FetchPub (Int4 muid)
 	return tmp;
 }
 
+ValNodePtr FetchPubPmId (Int4 pmid)
+{
+	ValNodePtr tmp;
+
+	tmp = MedArchGetPubPmId(pmid);
+	if (tmp == NULL) return NULL;
+	tmp = MedlineToISO(tmp);
+	return tmp;
+}
+
 /*****************************************************************************
 *
 *   MedlineToISO(tmp)
@@ -807,9 +1272,6 @@ ValNodePtr MedlineToISO (ValNodePtr tmp)
 {
 	ValNodePtr titlenode, oldnames, curr, tmp2, v;
 	AuthListPtr alp;
-	AuthorPtr ap;
-	PersonIdPtr pip;
-	NameStdPtr nsp;
 	CitArtPtr cap;
 	CitJourPtr cjp;
 	Int4 titlenum;
@@ -929,7 +1391,7 @@ void SplitMlAuthorName( CharPtr name, CharPtr last, CharPtr initials, CharPtr su
     	while((p > last) && (*p == ' '))
 		{
 			*p = '\0';
-			*p--;
+			p--;
 		}
 
 	    /* If the last token is not all upper case, and there are more than
@@ -999,7 +1461,7 @@ ValNodePtr get_std_auth(CharPtr token, Uint1 format)
 {
    static CharPtr      auth, eptr;
 	Char last[80], initials[20], suffix[20];
-   ValNodePtr  tmp;
+   ValNodePtr  tmp = NULL;
    NameStdPtr  namestd = NULL;
    AuthorPtr  aup;
    PersonIdPtr pid;
