@@ -1,11 +1,10 @@
 #include <utilpub.h>
 #include <taxutil.h>
 #include <taxuerr.h>
+#include <utilpars.h>
 
 static char *this_module ="taxutil";
 
-CharPtr TextSave();
-
 #ifdef THIS_MODULE
 #undef THIS_MODULE
 #endif
@@ -23,7 +22,7 @@ static TaxBlk cash[MAXIDLIST];
 
 static
 GeneticCodeListPtr
-GeneticCodeListNew()
+GeneticCodeListNew(void)
 {
 
 	return (GeneticCodeListPtr) MemNew(sizeof(GeneticCodeList));
@@ -118,7 +117,6 @@ Int4 taxname_match(CharPtr orgname, Boolean err)
 OrgRefPtr get_tax_org(CharPtr name)
 {
 	Int4	i;
-	OrgRefPtr orp;
 	
 	i = taxname_match(name, FALSE);
 	if (i != -1) {
@@ -211,11 +209,11 @@ static ValNodePtr TokenString(CharPtr str, Char delimiter)
    ValNodePtr  curtoken, token = NULL;
   
    /* skip first several delimiters if any existed */
-	for (ptr = str; *ptr == delimiter; ++ptr);
+	for (ptr = str; *ptr == delimiter; ++ptr) continue;
 
 	for (; *ptr != '\0';) {
 		for (bptr = ptr, len = 0; *ptr != delimiter && *ptr != '\0';
-                                                              ptr++, ++len);
+                                                              ptr++, ++len) continue;
 		curtoken = ValNodeNew(NULL);
 		curtoken->data.ptrvalue = TextSave(bptr, len);
 		token = tie_prev(token, curtoken);
@@ -544,11 +542,13 @@ static Boolean TaxCmpOrgById(BioSourcePtr b1, BioSourcePtr b2)
 		if (d1->tag->id == d2->tag->id) {
 			return TRUE;
 		} else {
+			/*
 			if (tax1_join(d1->tag->id, d2->tag->id) != 1) {
 				return TRUE;
 			} else {
 				return FALSE;
 			}
+			*/
 		}
 	} else if (StringICmp(b1->org->taxname, b2->org->taxname) == 0) {
 			return TRUE;
