@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.4 $
+* $Revision: 6.5 $
 *
 * File Description:  Object manager interface for type Pubdesc from
 *                    NCBI-Sequence.  This is separate to avoid typedef
@@ -45,6 +45,9 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: objpubd.h,v $
+* Revision 6.5  2002/02/20 20:47:18  kans
+* added MI_TECH_wgs
+*
 * Revision 6.4  2000/09/28 11:53:13  ostell
 * added Pubdesc.reftype = 3
 *
@@ -229,6 +232,7 @@ typedef struct molinfo {
 #define MI_TECH_fli_cdna 17  /* full-length insert cDNA sequence */
 #define MI_TECH_htgs_0 18    /* unordered single pass reads */
 #define MI_TECH_htc 19       /* High Throughput CDNA.. unfinished */
+#define MI_TECH_wgs 20       /* Whole Genome Shotgun */
 #define MI_TECH_other 255           /* doesnt' fit anything */
 
 NLM_EXTERN MolInfoPtr LIBCALL MolInfoNew PROTO((void));
