@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.71 $
+* $Revision: 6.138 $
 *
 * File Description: 
 *
@@ -104,10 +104,9 @@ extern "C" {
 #define NUM_SEQ_PKG           9
 
 #define SEQ_FMT_FASTA         1
-#define SEQ_FMT_CONTIGUOUS    2 
-#define SEQ_FMT_INTERLEAVE    3 
-#define NUM_SEQ_FMT           3 
-
+#define SEQ_FMT_ALIGNMENT     2 
+#define NUM_SEQ_FMT           2 
+  
 /*
 #define SEQ_FMT_FASTAGAP      2
 #define SEQ_FMT_PHYLIP        3
@@ -115,10 +114,14 @@ extern "C" {
 #define SEQ_FMT_PAUP          5
 */
 
+#define SEQ_ORIG_SUBMISSION   1
+#define SEQ_TPA_SUBMISSION    2
+
 typedef struct fmtblk {
   Int2         seqPackage;
   Int2         seqFormat;
   Int2         numSeqs;
+  Int2         submType;
 } FormatBlock, PNTR FormatBlockPtr;
 
 typedef struct sqnblk {
@@ -156,6 +159,16 @@ extern Boolean  docSumUp;
 extern Boolean  bioseqViewUp;
 #endif
 
+extern void SwapQualifiers (IteM i);
+extern void EditFeatureStrand (IteM i);
+extern void PrefixAuthorityWithOrganism (IteM i);
+extern void UpdateFastaSet (IteM i);
+extern void SeqLocAdjustByOffset (SeqLocPtr slp, Int4 offset);
+extern void SplitSegmentedFeatsMenuItem (IteM i);
+extern SeqLocPtr SeqLocWholeNew (BioseqPtr bsp);
+extern SeqFeatPtr SeqFeatCopy (SeqFeatPtr sfp);
+extern SeqLocPtr SeqLocReplaceLocalID (SeqLocPtr slp,
+				       SeqIdPtr  new_sip);
 extern SequinBlockPtr SequinBlockFree (SequinBlockPtr sbp);
 
 extern ForM CreateStartupForm (Int2 left, Int2 top, CharPtr title,
@@ -199,9 +212,27 @@ extern ForM CreateGenomeCenterForm (Int2 left, Int2 top, CharPtr title,
                                     Boolean buildContig,
                                     WndActnProc activateForm);
 
+extern Boolean HasZeroLengthSequence (ForM newForm);
 extern Boolean SequencesFormHasProteins (ForM f);
 extern Boolean SequencesFormHasTooManyNucleotides (ForM f);
 
+extern void AppendOrReplaceString (
+  CharPtr PNTR string_loc,
+  CharPtr new_value,
+  Boolean PNTR asked_question,
+  Boolean PNTR do_replace,
+  Boolean PNTR use_semicolon
+);
+
+extern void ConsolidateOrganismNotes (IteM i);
+extern void ConsolidateLikeModifiersWithSemicolons (IteM i);
+extern void ConsolidateLikeModifiersWithoutSemicolons (IteM i);
+
+extern void CountryLookup (IteM i);
+
+extern void ExtendPartialFeatures (IteM i);
+extern void TrimOrganismName (IteM i);
+
 extern void ConfirmSequencesFormParsing (ForM f, FormActnFunc putItAllTogether);
 
 extern ForM CreateHelpForm (Int2 left, Int2 top, CharPtr title,
@@ -269,6 +300,8 @@ extern void SetupSpecialMenu (MenU m, BaseFormPtr bfp);
 extern void SetupNewFeaturesMenu (MenU m, BaseFormPtr bfp);
 extern void SetupNewDescriptorsMenu (MenU m, BaseFormPtr bfp);
 extern void SetupNewPublicationsMenu (MenU m, BaseFormPtr bfp);
+extern void SetupBatchApplyMenu (MenU s, BaseFormPtr bfp);
+extern void SetupBatchEditMenu (MenU s, BaseFormPtr bfp);
 extern MenU CreateAnalysisMenu (WindoW w, BaseFormPtr bfp, Boolean bspviewOK, Boolean docsumOK);
 extern void SetupSequinFilters (void);
 extern void SetupBioseqPageList (void);
@@ -281,6 +314,7 @@ extern void SequinCheckSocketsProc (void);
 
 extern Int4 MySeqEntryToAsn3 (SeqEntryPtr sep, Boolean strip, Boolean correct, Boolean force);
 extern void ValSeqEntryForm (ForM f);
+extern void ValSeqEntryFormEx (ForM f, Boolean doAligns);
 
 extern void InitSequinExtras (void);
 extern void FiniSequinExtras (void);
@@ -293,6 +327,8 @@ extern Uint2 PackageFormResults (SequinBlockPtr sbp, SeqEntryPtr sep,
 extern void EnableFeaturesPerTarget (BaseFormPtr bfp);
 extern void EnableAnalysisItems (BaseFormPtr bfp, Boolean isDocSum);
 
+extern void ExtendSeqLocToEnd (SeqLocPtr slp, BioseqPtr bsp, Boolean end5);
+
 #define REGISTER_BIOSEQ_SEG_EDIT ObjMgrProcLoad(OMPROC_EDIT,"Edit Bioseq Seg","BioseqSegEditor",OBJ_BIOSEQ_SEG,0,OBJ_BIOSEQ_SEG,0,NULL,BioseqSegEditFunc,PROC_PRIORITY_DEFAULT)
 extern Int2 LIBCALLBACK BioseqSegEditFunc (Pointer data);
 
@@ -321,6 +357,8 @@ NLM_EXTERN SeqEntryPtr SequinFastaToSeqEntryEx
 
 /* Many miscellaneous extern functions within sequin source files */
 
+extern Boolean WriteSequinAppParam (CharPtr section, CharPtr type, CharPtr value);
+
 extern Boolean PropagateFromGenBankBioseqSet (SeqEntryPtr sep, Boolean ask);
 
 extern Uint2 SmartAttachSeqAnnotToSeqEntry (Uint2 entityID, SeqAnnotPtr sap);
@@ -331,13 +369,17 @@ extern CharPtr SearchForString (CharPtr str, CharPtr sub, Boolean case_counts, B
 extern void AddAboutAndHelpMenuItems (MenU m);
 extern void NetConfigureProc (IteM i);
 extern void EntrezQueryProc (IteM i);
+extern void Entrez2QueryProc (IteM i);
 extern void SetupEditSecondary (MenU m, BaseFormPtr bfp);
 extern void SimplePowerBlastProc (IteM i);
 extern void SimpleCDDBlastProc (IteM i);
+extern void SimpleCDDSearchFeatProc (IteM i);
+extern void SimpleCDDSearchAlignProc (IteM i);
 extern void VectorScreenProc (IteM i);
 extern void GenerateAutomaticDefLinesCommon (IteM i, Boolean addMods, Boolean smartMods, ButtoN b);
 extern void ForceTaxonFixupBtn (IteM i, ButtoN b);
 extern void CommonAddOrgOrModsToDefLines (IteM i, Int2 orgmod, Int2 subsource, ButtoN b);
+extern void PrefixDefLines (IteM i);
 extern void MRnaFromCdsProc (Uint2 entityID);
 extern void BioseqViewFormToolBar (GrouP h);
 extern Boolean DoBuildContig (void);
@@ -356,6 +398,8 @@ extern void EditSequenceHistory (IteM i);
 extern void FindGeneAndProtForCDS (Uint2 entityID, SeqFeatPtr cds,
                                    SeqFeatPtr PNTR gene, SeqFeatPtr PNTR prot);
 extern SeqFeatPtr FindBestProtein (Uint2 entityID, SeqLocPtr product);
+extern void ExportAlignmentInterleave (IteM i);
+extern void ExportAlignmentContiguous (IteM i);
 extern void NewDescriptorMenuFunc (ObjMgrProcPtr ompp, BaseFormPtr bfp, Uint2 descsubtype);
 extern Boolean PropagateFromGenBankBioseqSet (SeqEntryPtr sep, Boolean ask);
 extern CharPtr MergeValNodeStrings (ValNodePtr list, Boolean useReturn);
@@ -364,28 +408,39 @@ extern void PrepareToConvertToCDS (SeqEntryPtr sep, Uint2 entityID,
                                    Uint2 subtype, CharPtr findthis);
 extern void ConvertToLocalProc (IteM i);
 extern void PromoteToBestIDProc (IteM i);
-extern void RemoveIDsFromBioseqs (IteM i);
+extern void PromoteToWorstIDProc (IteM i);
+extern void RemoveGBIDsFromBioseqs (IteM i);
+extern void RemoveGBIDsFromProteins (IteM i);
+extern void RemoveGIsFromBioseqs (IteM i);
 extern void VectorScreenProc (IteM i);
 extern void SimplePowerBlastProc (IteM i);
 extern CharPtr MergeValNodeStrings (ValNodePtr list, Boolean useReturn);
 extern CharPtr JustSaveStringFromText (TexT t);
 
+extern void CommonApplyToAllProc (BaseFormPtr bfp, Int2 type);
 extern void ApplyTitle (IteM i);
 extern void ApplyCDS (IteM i);
 extern void ApplyRRNA (IteM i);
 extern void ApplyImpFeat (IteM i);
+extern void LoadTPAAccessionNumbersFromFile (IteM i);
+extern void LoadSecondaryAccessionNumbersFromFile (IteM i);
+extern void LoadHistoryAccessionNumbersFromFile (IteM i);
+extern void LoadOrganismModifierTable (IteM i);
+extern void LoadFeatureQualifierTable (IteM i);
 
 extern void RemoveRNA (IteM i);
 extern void ConvertRNA (IteM i);
 extern void AddRNA (IteM i);
 extern void EditRNA (IteM i);
 
+extern void RemoveRedundantProproteinMiscFeats (IteM i);
+extern void AddTypeStrainCommentsToAll (IteM i);
+extern void RemoveSequencesFromAlignment (IteM i);
+
 extern void ParseDefToSource (IteM i);
 extern void ParseLocalIDToSource (IteM i);
 extern void ParseFileToSource (IteM i);
-extern void AddStrainToOrg (IteM i);
-extern void AddCloneToOrg (IteM i);
-extern void AddSubspeciesToOrg (IteM i);
+extern void AddModToOrg (IteM i);
 
 extern void ParseInMoreProteins (IteM i);
 extern void ParseInNucUpdates (IteM i);
@@ -398,13 +453,28 @@ extern void AutoParseFeatureTableProc (IteM i);
 extern void RecomputeSuggest (IteM i);
 extern void RetranslateCdRegionsNoStop (IteM i);
 extern void RetranslateCdRegionsDoStop (IteM i);
+extern void RetranslateCdRegionsNoStopExceptEndCompleteCDS (IteM i);
+extern void AddGlobalCodeBreak (IteM i);
 extern void CorrectCDSGenCodes (IteM i);
 /* extern void CorrectCDSStartCodon (IteM i); */
 /* extern Boolean RetranslateOneCDS (SeqFeatPtr sfp, Uint2 entityID, Boolean include_stop); */
+extern void UpdateProteinsFromCDS (IteM i);
 
 extern void GenerateAutoDefLinesNoMods (IteM i);
 extern void GenerateAutoDefLinesWithMods (IteM i);
 extern void GenerateAutoDefLinesSmartMods (IteM i);
+extern void testAutoDef (IteM i);
+extern void testAutoDefWithOptions (IteM i);
+extern void AutoDefWithoutModifiers (IteM i);
+extern void AutoDefBaseFormCommon (BaseFormPtr bfp, Boolean use_form, Boolean use_modifiers);
+extern void AutoDefToolBtn (ButtoN b);
+extern void AutoDefOptionsToolBtn (ButtoN b);
+
+extern void RemoveDefLinesToolBtn (ButtoN b);
+extern void FindStringProcToolBtn (ButtoN b);
+extern void ParseLocalIDToSourceToolBtn (ButtoN b);
+extern void ResolveExistingLocalIDsToolBtn (ButtoN b);
+extern void GroupExplodeToolBtn (ButtoN b);
 
 extern void EditEvidenceFlag (IteM i);
 extern void EditExceptionFlag (IteM i);
@@ -417,6 +487,8 @@ extern void SetupEditSecondary (MenU m, BaseFormPtr bfp);
 extern void EditLocusProc (IteM i);
 extern void ConvertToLocalProc (IteM i);
 
+extern ValNodePtr BuildDescriptorValNodeList (void);
+
 extern void RemoveFeature (IteM i);
 extern void RemoveDescriptor (IteM i);
 
@@ -426,9 +498,20 @@ extern void SelectBioseq (IteM i);
 
 extern void FuseFeature (IteM i);
 
+extern void MakeExonsFromCDSIntervals (IteM i);
+extern void MakeExonsFromMRNAIntervals (IteM i);
+
+extern Int2 LIBCALLBACK CreateDeleteByTextWindow (Pointer data);
+extern Int2 LIBCALLBACK CreateSegregateByTextWindow (Pointer data);
+extern Int2 LIBCALLBACK CreateSegregateByFeatureWindow (Pointer data);
+extern Int2 LIBCALLBACK CreateSegregateByDescriptorWindow (Pointer data);
+extern Int2 LIBCALLBACK RemoveExtraneousSets (Pointer data);
+extern void RemoveOrphanProteins (Uint2 entityID, SeqEntryPtr sep);
 extern void ParseAsnOrFlatfileToAnywhere (IteM i);
+extern void ParseCommentToAnywhere (IteM i);
+extern void ParseLocalIDToAnywhere (IteM i);
 extern void RemoveTextInsideString (IteM i);
-extern void GeneralTextConversion (IteM i);
+extern void RemoveTextOutsideString (IteM i);
 
 extern void BioseqViewFormToolBar (GrouP h);
 
@@ -443,6 +526,8 @@ extern void FindGeneProc (IteM i);
 extern void FindProtProc (IteM i);
 extern void FindPosProc (IteM i);
 
+extern Boolean MeetsStringConstraint (SeqFeatPtr sfp, CharPtr str);
+
 extern Boolean SaveSeqSubmitProc (BaseFormPtr bfp, Boolean saveAs);
 
 extern void ExciseString (CharPtr str, CharPtr from, CharPtr to);
@@ -466,23 +551,132 @@ extern Boolean SequinEntrezInit (CharPtr appl_id, Boolean no_warnings, BoolPtr i
 extern void JustRegisterSeqEntry (BaseFormPtr bfp, Boolean freeit);
 extern void JustRegisterSeqEntryBtn (ButtoN b);
 extern void AddSubmitBlockToSeqEntry (ForM f);
-extern void SqnReadAlignView (BioseqPtr target_bsp, SeqEntryPtr source_sep);
+
+extern void SqnReadAlignView (BaseFormPtr bfp, BioseqPtr target_bsp, SeqEntryPtr source_sep, Boolean do_update);
 extern void DownloadAndUpdateProc (ButtoN b);
+extern void DownloadAndExtendProc (ButtoN b);
+extern void UpdateSeqAfterDownload (BaseFormPtr bfp, BioseqPtr oldbsp, BioseqPtr newbsp);
+extern void ExtendSeqAfterDownload (BaseFormPtr bfp, BioseqPtr oldbsp, BioseqPtr newbsp);
+extern void NewUpdateSequence (IteM i);
+extern void NewUpdateSequenceNewBlast (IteM i);
+extern void NewExtendSequence (IteM i);
 
 extern void FastaNucDirectToSeqEdProc (IteM i);
 
+extern void ParseCodonsFromtRNAComment (IteM i);
+extern void ParseAntiCodonsFromtRNAComment (IteM i);
+
 extern void RemoveAlignment (IteM i);
 extern void RemoveGraph (IteM i);
 extern void RemoveProteins (IteM i);
+extern void RemoveProteinsAndRenormalize (IteM i);
+
+extern void GlobalAddTranslExcept (IteM i);
+
+extern void ReadAlignment (IteM i);
+extern SeqEntryPtr SeqEntryFromAlignmentFile (FILE *fp, CharPtr missing, CharPtr match,
+                                              CharPtr beginning_gap, CharPtr middle_gap,
+                                              CharPtr end_gap, CharPtr alphabet, Uint1 moltype,
+                                              CharPtr no_org_err_msg);
 
 extern SeqAlignPtr Sqn_GlobalAlignTwoSeq (BioseqPtr bsp1, BioseqPtr bsp2, BoolPtr revcomp);
 extern SeqAlignPtr Sequin_GlobalAlignTwoSeq (BioseqPtr bsp1, BioseqPtr bsp2, Int4Ptr endsfixed);
 
+/* eventually this declaration should be moved to libncbitool */
+NLM_EXTERN SeqAlignPtr Sqn_LocalAlign2SeqEx (BioseqPtr bsp1, BioseqPtr bsp2, BoolPtr revcomp, Boolean use_new_blast);
+extern void SqnNewAlign (BioseqPtr bsp1, BioseqPtr bsp2, SeqAlignPtr PNTR salp);
+
+extern Boolean CreateUpdateCitSubFromBestTemplate (SeqEntryPtr top_sep, SeqEntryPtr upd_sep);
 
 #ifndef WIN_MAC
 extern void CreateSqnInitialFormMenus (WindoW w);
 #endif
 
+#define NUM_PAGES  8
+
+typedef struct sequencesform {
+  FORM_MESSAGE_BLOCK
+  GrouP           pages [NUM_PAGES];
+  Int2            currentPage;
+  Int2            tagFromPage [NUM_PAGES];
+  Int2            numPages;
+  DialoG          tbs;
+
+  DialoG          genbio;
+  Uint1           dnamolfrommolinfo;
+  PopuP           moltypePopup;
+  EnumFieldAssoc  PNTR moltypeAlist;
+  PopuP           topologyPopup;
+  ButtoN          partial5;
+  ButtoN          partial3;
+  GrouP           singleIdGrp;
+  TexT            singleSeqID;
+  ButtoN          makeAlign;
+  DialoG          dnaseq;
+
+  Int2            seqPackage;
+  Int2            seqFormat;
+  Int2            numSeqs;
+  Int2            submType;
+
+  ButtoN          protTechBoth;
+  ButtoN          partialN;
+  ButtoN          partialC;
+  Boolean         makeMRNA;
+  DialoG          protseq;
+
+  DialoG          mrnaseq;
+  ButtoN          partialmRNA5;
+  ButtoN          partialmRNA3;
+
+  GrouP           annotType;
+  GrouP           annotGrp;
+  ButtoN          partialLft;
+  ButtoN          partialRgt;
+  TexT            geneName;
+  PrompT          protOrRnaPpt;
+  TexT            protOrRnaName;
+  TexT            featcomment;
+  TexT            defline;
+  ButtoN          orgPrefix;
+  PopuP           genome;
+  PopuP           gencode;
+
+  ButtoN          nextBtn;
+  ButtoN          prevBtn;
+  BtnActnProc     goToNext;
+  BtnActnProc     goToPrev;
+
+  SeqEntryPtr     topSeqForConfirm;
+  SeqEntryPtr     currConfirmSeq;
+  FormActnFunc    putItAllTogether;
+  Int2            currConfirmCount;
+} SequencesForm, PNTR SequencesFormPtr;
+
+typedef int (LIBCALLBACK *CompareFunc) (Nlm_VoidPtr, Nlm_VoidPtr);
+
+extern int LIBCALLBACK CompareImpFeatEnumFieldAssoc (VoidPtr ptr1, VoidPtr ptr2);
+extern int LIBCALLBACK CompareFeatureValNodeStrings (VoidPtr ptr1, VoidPtr ptr2);
+
+extern ValNodePtr InsertMostUsedFeatureValNodes (ValNodePtr old_list);
+
+extern void SortEnumFieldAssocPtrArray (EnumFieldAssocPtr alist, CompareFunc compar);
+
+extern ValNodePtr FindExactStringInStrings ( ValNodePtr strings, CharPtr value);
+
+extern EnumFieldAssocPtr InsertMostUsedFeatureEnumFieldAssoc (
+  EnumFieldAssocPtr alist
+);
+
+extern ValNodePtr BuildFeatureValNodeList (
+  Boolean prefer_most_used,
+  CharPtr wild_card_name,
+  Int4 wild_card_value,
+  Boolean skip_unusual,
+  Boolean skip_import
+);
+
+extern void SetTaxNameAndRemoveTaxRef (OrgRefPtr orp, CharPtr taxname);
 
 #ifdef __cplusplus
 }
