@@ -32,8 +32,66 @@ Contents: prototypes for "public" BLAST functions (ones that other utilitiles
 
 ******************************************************************************/
 
-/* $Revision: 6.48 $ 
+/* $Revision: 6.67 $ 
 * $Log: blast.h,v $
+* Revision 6.67  2003/03/25 22:21:53  boemker
+* Clarified behavior of BLAST_Wizard.
+*
+* Revision 6.66  2003/03/25 19:58:18  boemker
+* Moved code to initialize search options from blastcgicmd.cpp to here, as
+* BLAST_Wizard et al.
+*
+* Revision 6.65  2003/03/24 19:42:14  madden
+* Changes to support query concatenation for blastn and tblastn
+*
+* Revision 6.64  2003/01/14 20:28:54  madden
+* New function BLASTAddBlastDBTitleToSeqAnnotEx
+*
+* Revision 6.63  2002/10/22 17:57:48  camacho
+* Changes to B2SPssmMultipleQueries
+*
+* Revision 6.62  2002/10/21 23:13:36  camacho
+* Added B2SPssmOnTheFly functions
+*
+* Revision 6.61  2002/09/18 20:23:20  camacho
+* Added BLASTCalculateSearchSpace
+*
+* Revision 6.60  2002/09/02 21:15:20  camacho
+* Changed comment for psi-blast2sequences
+*
+* Revision 6.59  2002/08/30 18:56:02  dondosha
+* Made BlastMakeTempProteinBioseq and HackSeqLocId public: needed for Cn3D
+*
+* Revision 6.58  2002/08/29 20:44:38  camacho
+* Added description of psi-blast2sequences
+*
+* Revision 6.57  2002/08/09 19:39:20  camacho
+* Added constants for some blast search parameters
+*
+* Revision 6.56  2002/08/01 20:47:24  dondosha
+* Prototypes changed for megablast functions related to traceback
+*
+* Revision 6.55  2002/07/02 17:08:00  dondosha
+* Reverse previous change - not needed
+*
+* Revision 6.54  2002/07/01 22:52:06  dondosha
+* Added CheckStartForGappedAlignmentEx with an extra window size parameter
+*
+* Revision 6.53  2002/05/28 22:00:12  camacho
+* *** empty log message ***
+*
+* Revision 6.52  2002/05/13 13:51:33  dondosha
+* Made two functions public
+*
+* Revision 6.51  2002/05/09 15:35:51  dondosha
+* Added BLASTOptionNewEx function with an extra argument for megablast
+*
+* Revision 6.50  2002/03/14 16:11:40  camacho
+* Extended BlastTwoSequences to allow comparison between sequence and PSSM
+*
+* Revision 6.49  2002/02/15 23:36:23  dondosha
+* Correction for megablast with non-greedy extensions
+*
 * Revision 6.48  2001/07/09 15:12:47  shavirin
 * Functions BLbasicSmithWatermanScoreOnly() and BLSmithWatermanFindStart()
 * used to calculate Smith-waterman alignments on low level become external.
@@ -371,6 +429,9 @@ Contents: prototypes for "public" BLAST functions (ones that other utilitiles
 #include <ncbi.h>
 #include <blastdef.h>
 
+/* AM: Support for query multiplexing. */
+#include "blastconcatdef.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -378,9 +439,10 @@ extern "C" {
 /*
 	Call this function to allocate the "options" structure.  The
 	fields will be filled in with the default values, which depend
-	on the program.
+	on the program. Defaults are #defined constants in blastdef.h
 */
 BLAST_OptionsBlkPtr LIBCALL BLASTOptionNew PROTO((CharPtr progname, Boolean gapped));
+BLAST_OptionsBlkPtr LIBCALL BLASTOptionNewEx PROTO((CharPtr progname, Boolean gapped, Boolean is_megablast));
 
 BLAST_OptionsBlkPtr LIBCALL BLASTOptionDelete PROTO((BLAST_OptionsBlkPtr));
 
@@ -390,6 +452,18 @@ Int2 LIBCALL BLASTOptionValidateEx PROTO((BLAST_OptionsBlkPtr options, CharPtr p
 
 Int2 LIBCALL BLASTOptionSetGapParams PROTO((BLAST_OptionsBlkPtr options, CharPtr matrix, Int4 open, Int4 extended));
 
+/********************* BLASTCalculateSearchSpace **************************
+Purpose: Calculate the effective search space for a gapped search with a
+         minimal set of options. Assumes the query is a protein sequence (ie:
+         no multiple contexts to consider).
+Parameters: options [in]: Blast options structure
+            nseq [in]: Number of sequences in the database
+            dblen [in]: Length of the database
+            qlen [in]: Length of the query sequence
+Returns: Effective search space
+**************************************************************************/
+FloatHi LIBCALL BLASTCalculateSearchSpace PROTO((BLAST_OptionsBlkPtr options, 
+            Int4 nseq, Int8 dblen, Int4 qlen)); 
 
 /* 
 	the setup functions, call before running blast.
@@ -401,7 +475,8 @@ BlastSearchBlkPtr LIBCALL BLASTSetUpSearchWithReadDbEx PROTO((BioseqPtr query_bs
 
 BlastSearchBlkPtr LIBCALL BLASTSetUpSearchByLocWithReadDb PROTO((SeqLocPtr slp, CharPtr prog_name, Int4 qlen, CharPtr dbname, BLAST_OptionsBlkPtr options, int (LIBCALLBACK *index_callback)PROTO((Int4 done, Int4 positives))));
 
-BlastSearchBlkPtr LIBCALL BLASTSetUpSearchByLocWithReadDbEx PROTO((SeqLocPtr slp, CharPtr prog_name, Int4 qlen, CharPtr dbname, BLAST_OptionsBlkPtr options, int (LIBCALLBACK *index_callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total));
+BlastSearchBlkPtr LIBCALL BLASTSetUpSearchByLocWithReadDbEx PROTO((SeqLocPtr slp, CharPtr prog_name, Int4 qlen, CharPtr dbname, BLAST_OptionsBlkPtr options, int (LIBCALLBACK *index_callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total, QueriesPtr mult_queries)); 
+/* --KM added mult_queries param: struct holding info about individual queries that got concatenated when -B option used */
 
 BlastSearchBlkPtr LIBCALL BLASTSetUpSearch PROTO((BioseqPtr query_bsp, CharPtr prog_name, Int4 qlen, Int8 dblen, BlastAllWordPtr all_words, BLAST_OptionsBlkPtr options, int (LIBCALLBACK *index_callback)PROTO((Int4 done, Int4 positives))));
 
@@ -440,7 +515,39 @@ SeqAlignPtr LIBCALL BlastTwoSequencesByLoc PROTO((SeqLocPtr slp1, SeqLocPtr slp2
 
 SeqAlignPtr LIBCALL BlastTwoSequencesByLocEx PROTO((SeqLocPtr slp1, SeqLocPtr slp2, CharPtr progname, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns));
 
-SeqAlignPtr LIBCALL BlastTwoSequencesByLocWithCallback PROTO((SeqLocPtr slp1, SeqLocPtr slp2, CharPtr progname, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *handle_results)PROTO((VoidPtr srch))));
+/* Notes for psi-blast2sequences (compare a PSSM with sequence slp2):  (CC)
+ * =============================
+ * 1) This functionality requires (at least) the residue frequencies
+ * (BLAST_MatrixPtr->posFreqs) to compute the PSSM. If the PSSM is 
+ * provided (BLAST_MatrixPtr->matrix), then this takes precedence and the 
+ * PSSM will not be recomputed. Please note that the PSSM and residue 
+ * frequencies are matrices with dimensions 26 by query_length+1. The last 
+ * row should be set to BLAST_SCORE_MIN (for PSSMs) or 0.0 (for residue 
+ * frequencies). 26 is the alphabet size (also defined as PRO_ALPHABET_SIZE in
+ * profiles.h).
+ *
+ * 2) The slp1 parameter is the master sequence for the PSSM (used to display 
+ * the alignment) and can only be shorter than the PSSM (the PSSM will be
+ * trimmed accordingly).
+ *
+ * 3) If the scalingFactor is set to 0.0 (default in the options structure),
+ * the PSSM will be calculated in the same way as psiblast (blastpgp) does it
+ * (that is, without scaling the PSSM). In order to use composition-based
+ * statistics (default in psiblast), please set the options->tweak_parameters
+ * option to TRUE. This is *not* the default in the options structure.
+ *
+ * 4) Also, if the scalingFactor is not 0.0 in the options parameter, this
+ * value will be used to scale the PSSM only if it is calculated by this
+ * function (if the PSSM is calculated outside this function it is assumed
+ * that the PSSM has been scaled already and that the same scalingFactor 
+ * that was used to create it is passed into this function).  This value is 
+ * also used to multiply various parameters such as gap costs, X dropoff 
+ * values, when performing the matrix rescaling, and to adjust the scores 
+ * and Lambda parameters when performing the traceback stage. 
+ * The matrix rescaling step will take place prior to the traceback stage. 
+ * This functionality resembles what rpsblast/impala do.
+ */
+SeqAlignPtr LIBCALL BlastTwoSequencesByLocWithCallback PROTO((SeqLocPtr slp1, SeqLocPtr slp2, CharPtr progname, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *handle_results)PROTO((VoidPtr srch)), BLAST_MatrixPtr matrix));
 
 SeqAlignPtr LIBCALL BlastTwoSequencesEx PROTO((BioseqPtr bsp1, BioseqPtr bsp2, CharPtr progname, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns));
 
@@ -457,8 +564,33 @@ BlastQuerySequenceSetUp PROTO((BioseqPtr bsp, CharPtr progname,
 BlastSearchBlkPtr LIBCALL
 BlastSequencesOnTheFlyEx PROTO((BlastSearchBlkPtr search, BioseqPtr subject_bsp));
    
+/*** PSIBLAST2Sequences API ***/
+Boolean LIBCALL 
+B2SPssmSetupSearch PROTO((BlastSearchBlkPtr search, SeqLocPtr pssm_slp, 
+BLAST_MatrixPtr matrix));
+
+Boolean LIBCALL 
+B2SPssmCleanUpSearch PROTO((BlastSearchBlkPtr search, BLAST_MatrixPtr matrix));
+
+SeqAlignPtr LIBCALL 
+B2SPssmOnTheFly PROTO((BlastSearchBlkPtr search, BioseqPtr subj_bsp));
+
+SeqAlignPtr LIBCALL 
+B2SPssmOnTheFlyByLoc PROTO((BlastSearchBlkPtr search, SeqLocPtr subj_slp));
+
+/* Compare pssm against all sequences in target_seqs. 
+   Returns an array of length ntargets with the corresponding alignments.
+   Caller is responsible for deallocating the return value */
+SeqAlignPtr * LIBCALL 
+B2SPssmMultipleQueries PROTO((SeqLocPtr pssm_slp, BLAST_MatrixPtr matrix, 
+SeqLocPtr *target_seqs, Int4 ntargets, BLAST_OptionsBlkPtr options));
+
+/*** END PSIBLAST2Sequences API ***/
+
+
 SeqAlignPtr LIBCALL SumBlastGetGappedAlignmentTraceback PROTO((BlastSearchBlkPtr search, Int4 hit_number, Boolean reverse, Boolean ordinal_number, Uint1Ptr subject, Int4 subject_length));
 
+Boolean LIBCALL SumBlastGetGappedAlignmentEx PROTO((BlastSearchBlkPtr search, Int4 hit_number, Boolean reverse, Boolean ordinal_number, Uint1Ptr subject, Int4 subject_length, Boolean do_traceback, SeqAlignPtr PNTR seqalignP, BlastHitRangePtr bhrp, Int2 query_number));
 
 /*
 	Performs a complete BLAST search and returns a SeqAnlign.
@@ -468,13 +600,17 @@ SeqAlignPtr LIBCALL BioseqBlastEngine PROTO((BioseqPtr bsp, CharPtr progname, Ch
 
 SeqAlignPtr LIBCALL BioseqBlastEngineWithCallback PROTO((BioseqPtr bsp, CharPtr progname, CharPtr database, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), int (LIBCALLBACK *handle_results)PROTO((VoidPtr srch))));
 
+SeqAlignPtr LIBCALL BioseqBlastEngineWithCallbackMult PROTO((BioseqPtr bsp, CharPtr progname, CharPtr database, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), int (LIBCALLBACK *handle_results)PROTO((VoidPtr srch)), QueriesPtr mult_queries)); /* AM: Added mult_queries param. */
+
 SeqAlignPtr LIBCALL BioseqBlastEngineEx PROTO((BioseqPtr bsp, CharPtr progname, CharPtr database, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total));
 
 SeqAlignPtr LIBCALL BioseqBlastEngineByLoc PROTO((SeqLocPtr slp, CharPtr progname, CharPtr database, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives))));
 
 SeqAlignPtr LIBCALL BioseqBlastEngineByLocEx PROTO((SeqLocPtr slp, CharPtr progname, CharPtr database, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total));
 
-SeqAlignPtr LIBCALL BioseqBlastEngineByLocWithCallback PROTO((SeqLocPtr slp, CharPtr progname, CharPtr database, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total, int (LIBCALLBACK *handle_results)PROTO((VoidPtr srch))));
+SeqAlignPtr LIBCALL BioseqBlastEngineByLocWithCallback PROTO((SeqLocPtr slp, CharPtr progname, CharPtr database, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total, int (LIBCALLBACK *handle_results)PROTO((VoidPtr srch)))); 
+
+SeqAlignPtr LIBCALL BioseqBlastEngineByLocWithCallbackMult PROTO((SeqLocPtr slp, CharPtr progname, CharPtr database, BLAST_OptionsBlkPtr options, ValNodePtr *other_returns, ValNodePtr *error_returns, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total, int (LIBCALLBACK *handle_results)PROTO((VoidPtr srch)), QueriesPtr mult_queries));  /* --KM added mult_queries param */
 /*
 	Prints error messages. 
 */
@@ -535,8 +671,8 @@ Boolean MegaBlastBuildLookupTable PROTO((BlastSearchBlkPtr search));
 Int2 DefineToFrame PROTO((Uint1 define));
 
 CharPtr BLASTGetDatabaseTitleFromSeqAnnot PROTO((SeqAnnotPtr seqannot));
-void BLASTAddBlastDBTitleToSeqAnnot PROTO((SeqAnnotPtr seqannot, 
-                                           CharPtr title));
+void BLASTAddBlastDBTitleToSeqAnnot PROTO((SeqAnnotPtr seqannot, CharPtr title));
+void BLASTAddBlastDBTitleToSeqAnnotEx PROTO((SeqAnnotPtr seqannot, CharPtr title, Boolean is_na));
 
 Int4 reverse_seq (Uint1 *seq, Uint1 *pos, Uint1 *target);
 
@@ -555,9 +691,69 @@ void PrintTabularOutputHeader PROTO((CharPtr blast_database, BioseqPtr query_bsp
 
 /* ------ Functions related to Smith-Waterman algorithm ------ */
 
-Nlm_FloatHi BLbasicSmithWatermanScoreOnly(Uint1 * matchSeq, Int4 matchSeqLength, Uint1 *query, Int4 queryLength, BLAST_Score **matrix, Int4 gapOpen, Int4 gapExtend,  Int4 *matchSeqEnd, Int4 *queryEnd, Int4 *score, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi effSearchSpace, Boolean positionSpecific);
+Nlm_FloatHi BLbasicSmithWatermanScoreOnly PROTO((Uint1 * matchSeq, Int4 matchSeqLength, Uint1 *query, Int4 queryLength, BLAST_Score **matrix, Int4 gapOpen, Int4 gapExtend,  Int4 *matchSeqEnd, Int4 *queryEnd, Int4 *score, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi effSearchSpace, Boolean positionSpecific));
 
-Int4 BLSmithWatermanFindStart(Uint1 * matchSeq, Int4 matchSeqLength, Uint1 *query, Int4 queryLength, BLAST_Score **matrix, Int4 gapOpen, Int4 gapExtend,  Int4 matchSeqEnd, Int4 queryEnd, Int4 score, Int4 *matchSeqStart, Int4 *queryStart, Boolean positionSpecific);
+Int4 BLSmithWatermanFindStart PROTO((Uint1 * matchSeq, Int4 matchSeqLength, Uint1 *query, Int4 queryLength, BLAST_Score **matrix, Int4 gapOpen, Int4 gapExtend,  Int4 matchSeqEnd, Int4 queryEnd, Int4 score, Int4 *matchSeqStart, Int4 *queryStart, Boolean positionSpecific));
+
+Boolean
+CheckStartForGappedAlignment PROTO((BlastSearchBlkPtr search, BLAST_HSPPtr hsp, Uint1Ptr query, Uint1Ptr subject, Int4Ptr PNTR matrix));
+
+Int4 GetStartForGappedAlignment PROTO((BlastSearchBlkPtr search, BLAST_HSPPtr hsp, Uint1Ptr query, Uint1Ptr subject, Int4Ptr PNTR matrix));
+
+BioseqPtr BlastMakeTempProteinBioseq PROTO((Uint1Ptr sequence, Int4 length, 
+                                            Uint1 alphabet));
+
+void HackSeqLocId PROTO((SeqLocPtr slp, SeqIdPtr id));
+
+/*  --------------------------------------------------------------------
+ *
+ *  BLAST_Wizard & related functions.
+ *
+ *  Use BLAST_WizardOptionsBlkInit to initialize a
+ *  BLAST_WizardOptionsBlk by setting every field to zero or FALSE.
+ *
+ *  Use BLAST_WizardOptionsBlkDone to free any memory owned by a
+ *  BLAST_WizardOptionsBlk, excluding the memory for the
+ *  BLAST_WizardOptionsBlk itself, which should be allocated on the
+ *  stack.
+ *
+ *  Use BLAST_WizardOptionsMaskInit to initialize a
+ *  BLAST_WizardOptionsMask by setting every field to FALSE, indicating
+ *  that the corresponding fields of some BLAST_WizardOptionsBlk aren't
+ *  set.
+ *
+ *  Use BLAST_Wizard to initialize a BLAST_WizardOptionsBlk according
+ *  to program, service, options, and mask.  Alignments, descriptions,
+ *  and errors are output parameters.  Alignments and descriptions are
+ *	optional; error is required.  BLAST_Wizard returns a null pointer
+ *	if and only if *error != 0 on exit.  The returned object must be
+ *  freed with BLASTOptionDelete; if *error != 0, then *error must be
+ *  freed with MemFree.
+ *
+ *  --------------------------------------------------------------------
+ */
+
+void
+BLAST_WizardOptionsBlkInit(
+    BLAST_WizardOptionsBlkPtr   options);
+
+void
+BLAST_WizardOptionsBlkDone(
+    BLAST_WizardOptionsBlkPtr   options);
+
+void
+BLAST_WizardOptionsMaskInit(
+    BLAST_WizardOptionsMaskPtr  mask);
+
+BLAST_OptionsBlkPtr
+BLAST_Wizard(
+    const char*                 program,
+    const char*                 service,
+    BLAST_WizardOptionsBlkPtr   options,
+    BLAST_WizardOptionsMaskPtr  mask,
+    int*                        alignments,
+    int*                        descriptions,
+    char**                      error);
 
 /* ----------------------------------------------------------- */
 
