@@ -30,8 +30,95 @@ Author: Tom Madden
 Contents: #defines and definitions for structures used by BLAST.
 
 ******************************************************************************/
-/* $Revision: 6.129 $ 
+/* $Revision: 6.157 $ 
 * $Log: blastdef.h,v $
+* Revision 6.157  2004/04/30 12:45:45  coulouri
+* bump version to 2.2.9
+*
+* Revision 6.156  2004/04/13 21:02:52  madden
+* Add ignore_gilist Boolean to Options for use in formatting
+*
+* Revision 6.155  2004/02/04 15:35:03  camacho
+* Rollback to fix problems in release 2.2.7
+*
+* Revision 6.154  2004/01/27 20:46:06  dondosha
+* Allow values 0, 1, 2 for no_traceback megablast option
+*
+* Revision 6.153  2004/01/05 22:09:26  madden
+* Put back dashes in date
+*
+* Revision 6.152  2004/01/02 13:44:32  coulouri
+* Revert to hardcoded BLAST_RELEASE_DATE
+*
+* Revision 6.151  2003/12/29 15:51:18  coulouri
+* Bump version, use __DATE__ instead of hardcoded date
+*
+* Revision 6.150  2003/11/06 19:52:13  dondosha
+* Added error MBTemplateType, so it can be returned when wordsize/template length combination is wrong
+*
+* Revision 6.149  2003/10/02 19:30:11  madden
+* add field seAlign to SWResults for use in kappa.c
+*
+* Revision 6.148  2003/04/09 14:18:34  madden
+* Update version and release-date
+*
+* Revision 6.147  2003/03/26 15:45:48  boemker
+* Documented relationships among BLAST_OptionsBlk, BLAST_WizardOptionsBlk,
+* and BLAST_WizardOptionsMask.
+*
+* Revision 6.146  2003/03/25 22:23:06  boemker
+* Replaced cutoff_s2, which isn't used, with cutoff_s, which is.
+* Added query_lcase_mask.
+*
+* Revision 6.145  2003/03/25 19:58:18  boemker
+* Moved code to initialize search options from blastcgicmd.cpp to here, as
+* BLAST_Wizard et al.
+*
+* Revision 6.144  2003/03/24 19:42:14  madden
+* Changes to support query concatenation for blastn and tblastn
+*
+* Revision 6.143  2002/11/22 23:28:43  dondosha
+* Use array of structures instead of array of pointers for initial offset pairs
+*
+* Revision 6.142  2002/11/16 17:12:55  madden
+* Change version and date
+*
+* Revision 6.141  2002/11/04 22:51:13  dondosha
+* Changed FloatHi pvalue to Int4 num_ident in HSP structures
+*
+* Revision 6.140  2002/09/13 19:11:02  camacho
+* Added rps_qlen field
+*
+* Revision 6.139  2002/09/11 21:15:23  camacho
+* Removed obsolete #define and comment about BlastSeqIdList structure
+*
+* Revision 6.138  2002/09/11 20:46:25  camacho
+* Removed deprecated BlastSeqIdListPtr code
+*
+* Revision 6.137  2002/08/26 15:49:51  madden
+* Change release date and version
+*
+* Revision 6.136  2002/08/09 19:39:20  camacho
+* Added constants for some blast search parameters
+*
+* Revision 6.135  2002/06/21 21:43:01  camacho
+* Removed obsolete BlastSeqIdList structure and functions
+*
+* Revision 6.134  2002/05/17 21:40:13  dondosha
+* Added 2 optimal Mega BLAST word templates for length 21
+*
+* Revision 6.133  2002/05/14 22:20:20  dondosha
+* Renamed maximal discontiguous template type into optimal
+*
+* Revision 6.132  2002/04/23 20:59:53  madden
+* Change version and date for release
+*
+* Revision 6.131  2002/04/09 18:16:43  dondosha
+* Added more options/parameters for megablast
+*
+* Revision 6.130  2002/03/28 18:53:18  madden
+* Add ValNodePtr mask1 to BlastSearch structure
+*
 * Revision 6.129  2001/12/28 20:38:39  dondosha
 * Moved Mega BLAST related parameters into a separate structure
 *
@@ -793,8 +880,8 @@ extern "C" {
 #endif
 
 /* the version of BLAST. */
-#define BLAST_ENGINE_VERSION "2.2.2"
-#define BLAST_RELEASE_DATE "Jan-08-2002"
+#define BLAST_ENGINE_VERSION "2.2.9"
+#define BLAST_RELEASE_DATE "May-01-2004"
 
 /* Defines for program numbers. (Translated in BlastGetProgramNumber). */
 #define blast_type_undefined 0
@@ -820,6 +907,56 @@ extern "C" {
 /* Specifies minimum search space size for an awak thread. */
 #define AWAKE_THR_MIN_SIZE 2000000000000.0
 
+#ifndef _BLASTCONCAT_
+#include "blastconcat.h"
+#endif
+/* --KM concat */
+
+/* Some default values (used when creating blast options block and for
+ * command-line program defaults. When changing these defaults, please
+ * remember to update the defaults in the command-line programs */
+#define WINDOW_SIZE_PROT 40
+#define WINDOW_SIZE_NUCL 0
+#define WINDOW_SIZE_MEGABLAST 0
+
+#define WORDSIZE_PROT 3
+#define WORDSIZE_NUCL 11
+#define WORDSIZE_MEGABLAST 28
+
+/* Protein gap costs are the defaults for the BLOSUM62 scoring matrix.
+ * More gap costs are listed in BLASTOptionSetGapParams */
+#define GAP_OPEN_PROT 11
+#define GAP_OPEN_NUCL 5
+#define GAP_OPEN_MEGABLAST 0
+
+#define GAP_EXTN_PROT 1
+#define GAP_EXTN_NUCL 2
+#define GAP_EXTN_MEGABLAST 0
+
+#define WORD_THRESHOLD_BLASTP 11
+#define WORD_THRESHOLD_BLASTN 0
+#define WORD_THRESHOLD_BLASTX 12
+#define WORD_THRESHOLD_TBLASTN 13
+#define WORD_THRESHOLD_TBLASTX 13
+#define WORD_THRESHOLD_MEGABLAST 0
+
+#define UNGAPPED_X_DROPOFF_PROT 7
+#define UNGAPPED_X_DROPOFF_NUCL 20
+#define UNGAPPED_X_DROPOFF_MEGABLAST 10
+
+#define GAP_X_DROPOFF_PROT 15
+#define GAP_X_DROPOFF_NUCL 30 
+#define GAP_X_DROPOFF_MEGABLAST 20 
+#define GAP_X_DROPOFF_TBLASTX 0
+
+#define GAP_X_DROPOFF_FINAL_PROT 25
+#define GAP_X_DROPOFF_FINAL_NUCL 50 
+#define GAP_X_DROPOFF_FINAL_TBLASTX 0
+
+/* reward and penalty only apply to blastn/megablast */
+#define PENALTY -3
+#define REWARD 1
+
 /********************************************************************
 *
 *	define for collecting BLAST stats.
@@ -895,10 +1032,19 @@ typedef struct _blast_prune_hits_from_sap {
 #define FILTER_DUST 1
 #define FILTER_SEG  2
 
+typedef enum {
+   MB_WORD_CODING = 0,
+   MB_WORD_OPTIMAL = 1,
+   MB_TWO_TEMPLATES = 2
+} MBDiscWordType;
+
 /**********************************************************************
 	Structure for the blast options (available to user/programmer).
 	This should be filled in by the "Main" program before blast
 	is called.
+
+    If changes are made to this structure, corresponding changes should
+    likely be made to BLAST_WizardOptionsBlk and BLAST_WizardOptionsMask.
 ***********************************************************************/
 
 typedef struct _blast_optionsblk {
@@ -982,7 +1128,7 @@ typedef struct _blast_optionsblk {
         Int4            max_num_patterns; /* Maximum number of patterns to be used in PHI-Blast search */
 	Boolean         no_check_score;
         Boolean         is_megablast_search; /* Is this a MegaBlast search? */
-        Boolean         no_traceback;    /* No traceback in MegaBLAST extension */
+        Uint1         no_traceback;    /* No traceback in MegaBLAST extension */
         Boolean         is_rps_blast;     /* If this RPS Blast ? */
         SeqLocPtr       query_lcase_mask; /* Masking of input DNA regions */
         Boolean         sort_gi_list;     /* Should the gi list be sorted? */
@@ -1008,10 +1154,137 @@ typedef struct _blast_optionsblk {
         Int2 mb_template_length;  /* Length of the discontiguous word */
         Boolean mb_use_dyn_prog;  /* Use dynamic programming gapped extension in
                                      megablast with affine gap scores */ 
+        MBDiscWordType mb_disc_type;
+	Int4 NumQueries;		/*--KM for query concatenation in [t]blastn */
+        Boolean ignore_gilist;    /* Used in traceback stage to not lookup gi's */
       } BLAST_OptionsBlk, PNTR BLAST_OptionsBlkPtr;
 
+
+/*  --------------------------------------------------------------------
+ *
+ *  BLAST_WizardOptionsBlk contains those fields of BLAST_OptionsBlk
+ *  that a user can set.
+ *
+ *  BLAST_WizardOptionsMask contains a Boolean for each field defined in
+ *  BLAST_WizardOptionsBlk, except those holding pointers.  TRUE means
+ *  that the corresponding field in BLAST_WizardOptionsBlk is set.
+ *
+ *  These structures are used only in conjunction with BLAST_Wizard.
+ *
+ *  --------------------------------------------------------------------
+ */
+
+struct _blast_wizardoptionsblk {
+    Int4            block_width;
+    Int4            cutoff_s;
+    Int4            db_genetic_code;
+    CharPtr         entrez_query;
+    Nlm_FloatHi     ethresh;
+    Nlm_FloatHi     expect_value;
+    CharPtr         filter_string;
+    Int4            first_db_seq;
+    Int4            final_db_seq;
+    Int4            gap_extend;
+    Int4            gap_open;
+    Boolean         gapped_calculation;
+    Int4            genetic_code;
+    ValNodePtr      gilist;
+    Int4            hitlist_size;
+    Int4            hsp_range_max;
+    Boolean         is_ooframe;
+    CharPtr         matrix;
+    MBDiscWordType  mb_disc_type;
+    Int2            mb_template_length;
+    Uint1           no_traceback;
+    Int2            penalty;
+    FloatLo         perc_identity;
+    Boolean         perform_culling;
+    CharPtr         phi_pattern;
+    Int4            pseudoCountConst;
+    SeqLocPtr       query_lcase_mask;
+    Int4            required_end;
+    Int4            required_start;
+    Int2            reward;
+    Nlm_FloatHi     searchsp_eff;
+    Boolean         smith_waterman;
+    Uint1           strand_option;
+    Int4            threshold_first;
+    Int4            threshold_second;
+    Boolean         tweak_parameters;
+    Boolean         use_best_align;
+    Boolean         use_real_db_size;
+    Int4            window_size;
+    Int2            wordsize;
+
+    Boolean         two_hits;
+    CharPtr         string_options;
+};
+
+typedef struct _blast_wizardoptionsblk
+            BLAST_WizardOptionsBlk,
+    PNTR    BLAST_WizardOptionsBlkPtr;
+
+struct _blast_wizardoptionsmask {
+    Boolean         block_width;
+    Boolean         cutoff_s;
+    Boolean         db_genetic_code;
+    Boolean         ethresh;
+    Boolean         expect_value;
+    Boolean         first_db_seq;
+    Boolean         final_db_seq;
+    Boolean         gap_extend;
+    Boolean         gap_open;
+    Boolean         gapped_calculation;
+    Boolean         genetic_code;
+    Boolean         hitlist_size;
+    Boolean         hsp_range_max;
+    Boolean         is_ooframe;
+    Boolean         mb_disc_type;
+    Boolean         mb_template_length;
+    Boolean         no_traceback;
+    Boolean         penalty;
+    Boolean         perc_identity;
+    Boolean         perform_culling;
+    Boolean         pseudoCountConst;
+    Boolean         required_end;
+    Boolean         required_start;
+    Boolean         reward;
+    Boolean         searchsp_eff;
+    Boolean         smith_waterman;
+    Boolean         strand_option;
+    Boolean         threshold_first;
+    Boolean         threshold_second;
+    Boolean         tweak_parameters;
+    Boolean         use_best_align;
+    Boolean         use_real_db_size;
+    Boolean         window_size;
+    Boolean         wordsize;
+
+    Boolean         two_hits;
+};
+
+typedef struct _blast_wizardoptionsmask
+            BLAST_WizardOptionsMask,
+    PNTR    BLAST_WizardOptionsMaskPtr;
+
+typedef enum {
+   TEMPL_11_16 = 0,
+   TEMPL_12_16 = 1,
+   TEMPL_11_18 = 2,
+   TEMPL_12_18 = 3,
+   TEMPL_11_21 = 4,
+   TEMPL_12_21 = 5,
+   TEMPL_11_16_OPT = 6,
+   TEMPL_12_16_OPT = 7,
+   TEMPL_11_18_OPT = 8,
+   TEMPL_12_18_OPT = 9,
+   TEMPL_11_21_OPT = 10,
+   TEMPL_12_21_OPT = 11,
+   TEMPL_ERROR = -1
+} MBTemplateType;
+
 typedef struct _mb_parameter_blk_ {
-   Boolean no_traceback;    /* No traceback in greedy extension */
+   Uint1 no_traceback;    /* No traceback in greedy extension */
    Boolean is_neighboring;  /* Is this a neighboring task? */ 
    Boolean full_seqids;     /* Print full seqids in tabular output? */
    FloatLo perc_identity;   /* Identity percentage cut-off */
@@ -1024,6 +1297,8 @@ typedef struct _mb_parameter_blk_ {
    Int2    template_length; /* Length of a discontiguous word template */
    Boolean use_dyn_prog;    /* Use dynamic programming extension for affine gap
                                scores */
+   MBTemplateType template_type; /* Type of a discontiguous template */
+   Boolean use_two_templates;
 } MegaBlastParameterBlk, PNTR MegaBlastParameterBlkPtr;
 
 /****************************************************************************
@@ -1228,7 +1503,7 @@ typedef struct _blast_hsp {
 		/* If TRUE this HSP starts a chain along the "link" pointer. */
 		Boolean 	start_of_chain;
 		BLAST_Score	score;
-		Nlm_FloatHi	pvalue;
+		Int4    	num_ident;
 		Nlm_FloatHi	evalue;
 		BLAST_Seg query,	/* query sequence info. */
 			subject;	/* subject sequence info. */
@@ -1278,7 +1553,7 @@ typedef struct _blast_hitlist {
         	/* added -cfj */
         LinkHelpStruct *lh_helper;
         Int4 lh_helper_size;
-        MegaBlastExactMatchPtr PNTR exact_match_array; /* Array to hold initial
+        MegaBlastExactMatchPtr exact_match_array; /* Array to hold initial
                                                           exact match hits */
         Int4 exact_match_max;
 	} BLAST_HitList, PNTR BLAST_HitListPtr;
@@ -1293,8 +1568,8 @@ typedef struct _blast_results_hsp {
 		Int4 		number;	/* number of HSP's used to calculate the p-value. */
 		BLAST_Score	score;	/* score of this HSP. */
 		Nlm_FloatHi	e_value,/* expect value of this set of HSP's. */
-				p_value,/* p-value of this set of HSP's. */
 				bit_score; /* above score * lambda/ln2 */
+		Int4		num_ident;/* number of identities in this HSP. */
 		Int2		context;	/* context number of query. */
 		Int2		query_frame, /* frame of query, non-zero if transl. */
 				subject_frame; /* frame of subject, non-zero if transl. */
@@ -1373,13 +1648,6 @@ typedef struct _blast_all_words {
 			specific;	/* specific (limited) words are to be indexed. */
 	} BlastAllWord, *BlastAllWordPtr;
 		
-typedef struct _blast_seqid_list {
-    SeqIdPtr 	seqid_list;	/* A list of SeqId's (may or may not be in the database) that should serve as subject sequences for BLAST'ing. */
-    Uint1 		mode;	/* Either BLAST_OWN, or BLAST_NOT_OWN.  Specifies whether they should be deleted. */
-    SeqIdPtr seqid_ptr;
-} BlastSeqIdList, *BlastSeqIdListPtr;
-		
-
 /*
 	Contains gi and ordinal number for use by random access BLAST.
 */
@@ -1452,6 +1720,7 @@ typedef struct SWResults {
     struct SWResults *next;
     Boolean isFirstAlignment;
     Int4 subject_index;  /* needed to break ties on rare occasions */
+    SeqAlignPtr seqAlign; /*needed when seqAlign is already computed*/
 } SWResults;
     
 /* Average sizes of protein and nucl. sequences. */
@@ -1472,12 +1741,6 @@ typedef struct _BlastThrInfo {
     TNlmMutex callback_mutex; /*lock for issuing update ticks on the screen*/
     /* Mutex for recalculation of ambiguities, in BlastReevaluateWithAmbiguities */
     TNlmMutex ambiguities_mutex;
-    /*
-      SeqId lists if only a certain number of the database sequences will be
-      used for the search.
-      */
-    BlastSeqIdListPtr blast_seqid_list;
-
 
     /*
       GI List to be used if database will be searched by GI.
@@ -1567,8 +1830,8 @@ typedef struct _blast_matrix_rescale {
 #define BLAST_SEARCH_ALLOC_TRANS_INFO 1024
 #define BLAST_SEARCH_ALLOC_ALL_WORDS 2048
 #define BLAST_SEARCH_ALLOC_QUERY_SLP 4096
-/* #define BLAST_SEARCH_ALLOC_GILIST 8192 */
-#define BLAST_SEARCH_ALLOC_THRINFO 8192 /* 16384 */
+#define BLAST_SEARCH_ALLOC_THRINFO 8192
+#define BLAST_SEARCH_ALLOC_MASK1 16384
 
 typedef struct blast_search_block {
     Int4 allocated; 
@@ -1591,7 +1854,7 @@ a field is allocated, then it's bit is non-zero.
 		translation_table_rc
 		all_words		BLAST_SEARCH_ALLOC_ALL_WORDS
 		query_slp		BLAST_SEARCH_ALLOC_QUERY_SLP
-		blast_gi_list		BLAST_SEARCH_ALLOC_GILIST
+		mask1			BLAST_SEARCH_ALLOC_MASK1
 */
 
 /*
@@ -1657,6 +1920,7 @@ a field is allocated, then it's bit is non-zero.
       ValNodePtr containing masking SeqLocPtr's
       */
     ValNodePtr mask;
+    ValNodePtr mask1;
     /*
       What genetic codes are we using to translate the query or database
       when needed.  Based upon NCBI genetic codes.
@@ -1674,6 +1938,12 @@ a field is allocated, then it's bit is non-zero.
     CharPtr original_seq;	/* Original (i.e.,  untransl.) sequence. */
     BlastSequenceBlkPtr	subject;/* subject sequence. */
     
+
+    /* KM-- info about individual queries from a concatenated query in
+       blastn or tblastn */
+    struct queries PNTR mult_queries;	/* struct defined in blastconcat.h */ 
+
+
     /*
       SeqLocPtr for the query, owned by the called and not by BLAST.
       */
@@ -1733,6 +2003,7 @@ a field is allocated, then it's bit is non-zero.
     Int4                    dbseq_num;      /* number of sequences in the database. */
     Int4                    length_adjustment; /* amount removed from end of query and db sequences. */
     Nlm_FloatHi		searchsp_eff;	/* Effective search space (used for statistics). */
+    Int4            rps_qlen; /* original query sequence length (RPS-BLAST only) */
     ReadDBFILEPtr		rdfp, /* I/O PTR for database files. */
         rdfp_list;	/* linked rdfp list of all databases. */
     /* The subject info (id and defline) is kept here for the current sequence
@@ -1740,11 +2011,6 @@ a field is allocated, then it's bit is non-zero.
        be used if rdfp is NULL.
        */
     BLASTSubjectInfoPtr subject_info;
-    /*
-      A list of SeqId's from the database to use for the BLAST run.
-      */
-    /* BlastSeqIdListPtr blast_seqid_list; */
-    /* BlastGiListPtr	blast_gi_list; */
     
     /* Data used in threads - previously global variables */
 
