@@ -32,8 +32,103 @@ Contents: definitions and prototypes used by blastkar.c to calculate BLAST
 
 ******************************************************************************/
 
-/* $Revision: 6.8 $ */
-/* $Log: blastkar.h,v $
+/* $Revision: 6.38 $ 
+* $Log: blastkar.h,v $
+* Revision 6.38  2004/04/28 14:36:34  madden
+* Changes from Mike Gertz:
+*  - Added a function prototype for BlastGapDecayDivisor
+*   - Removed the function prototypes for  BlastGapDecay and BlastGapDecayInverse
+*   - Modified function prototypes for BlastSmallGapSumE, BlastLargeGapSumE and
+*   BlastUnevenGapSumE
+*
+* Revision 6.37  2004/04/23 13:20:25  madden
+* Change: BLAST_KARLIN_K_SUMLIMIT_DEFAULT from 0.01 to 0.00001
+*
+* Revision 6.36  2004/03/31 18:39:24  kans
+* fixed typo in BlastBlastComputeLengthAdjustment
+*
+* Revision 6.35  2004/03/31 17:58:51  papadopo
+* Mike Gertz' changes for length adjustment calculations
+*
+* Revision 6.34  2003/12/12 16:00:46  madden
+* Add gap_decay_rate to BlastCutoffs, remove BlastCutoffs_simple, removal of defunct _real variables (all from Mike Gertz)
+*
+* Revision 6.33  2003/11/28 22:39:41  camacho
+* +static keyword to BlastKarlinLtoH
+*
+* Revision 6.32  2003/11/26 19:09:09  madden
+* Remove ref to BlastKarlinLambdaBis (no longer needed per Mike Gertz)
+*
+* Revision 6.31  2003/02/27 19:07:56  madden
+* Add functions PrintMatrixMessage and PrintAllowedValuesMessage
+*
+* Revision 6.30  2003/02/26 18:23:50  madden
+* Add functions BlastKarlinkGapBlkFill and BlastKarlinReportAllowedValues, call from BlastKarlinBlkGappedCalcEx
+*
+* Revision 6.29  2002/12/04 13:28:37  madden
+* Add effective length parameters
+*
+* Revision 6.28  2002/09/03 14:21:50  camacho
+* Changed type of karlinK from FloatHi to Nlm_FloatHi
+*
+* Revision 6.27  2002/08/29 13:58:08  camacho
+* Added field to store K parameter associated with posMatrix
+*
+* Revision 6.26  2002/03/18 21:31:56  madden
+* Added comments
+*
+* Revision 6.25  2000/12/28 16:23:24  madden
+* Function getAlphaBeta from AS
+*
+* Revision 6.24  2000/12/26 17:46:20  madden
+* Add function BlastKarlinGetMatrixValuesEx2 to return alpha and beta
+*
+* Revision 6.23  2000/11/24 22:07:33  shavirin
+* Added new function BlastResFreqFree().
+*
+* Revision 6.22  2000/09/27 21:27:15  dondosha
+* Added original_matrix member to BLAST_Matrix structure
+*
+* Revision 6.21  2000/09/12 16:03:51  dondosha
+* Added functions to create and destroy matrix used in txalign
+*
+* Revision 6.20  2000/08/31 15:45:07  dondosha
+* Added function BlastUnevenGapSumE for sum evalue computation with different gap size on two sequences
+*
+* Revision 6.19  2000/08/23 18:50:02  madden
+* Changes to support decline-to-align checking
+*
+* Revision 6.18  2000/08/04 15:49:26  sicotte
+* added BlastScoreBlkMatCreateEx(reward,penalty) and BlastKarlinGetDefaultMatrixValues as external functions
+*
+* Revision 6.17  2000/07/07 21:20:08  vakatov
+* Get all "#include" out of the 'extern "C" { }' scope!
+*
+* Revision 6.16  2000/05/26 17:29:55  shavirin
+* Added array of pos frequencies into BLAST_Matrix structure and it's
+* handling.
+*
+* Revision 6.15  2000/04/17 20:41:37  madden
+* Added BLAST_MatrixFetch
+*
+* Revision 6.14  1999/12/22 21:06:35  shavirin
+* Added new function BlastPSIMaxScoreGet().
+*
+* Revision 6.13  1999/09/16 17:38:42  madden
+* Add posFreqs for position-specific frequencies
+*
+* Revision 6.12  1999/03/17 16:49:11  madden
+* Removed comment within comment
+*
+* Revision 6.11  1998/12/31 18:17:05  madden
+* Added strand option
+*
+ * Revision 6.10  1998/09/11 19:02:07  madden
+ * Added paramC
+ *
+ * Revision 6.9  1998/07/17 15:39:58  madden
+ * Changes for Effective search space.
+ *
  * Revision 6.8  1998/04/24 19:27:55  madden
  * Added BlastKarlinBlkStandardCalcEx for ideal KarlinBlk
  *
@@ -171,13 +266,14 @@ Contents: definitions and prototypes used by blastkar.c to calculate BLAST
  * */
 #ifndef __BLASTKAR__
 #define __BLASTKAR__
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 #include <ncbi.h>
 #include <objalign.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
 	Defines for the matrix 'preferences' (as specified by S. Altschul).
 */
@@ -193,7 +289,7 @@ Note:  statistical significance is often not greatly affected by the value
 of K, so high accuracy is generally unwarranted.
 *****************************************************************************/
 /* K_SUMLIMIT_DEFAULT == sumlimit used in BlastKarlinLHtoK() */
-#define BLAST_KARLIN_K_SUMLIMIT_DEFAULT 0.01
+#define BLAST_KARLIN_K_SUMLIMIT_DEFAULT 0.0001
 
 /* LAMBDA_ACCURACY_DEFAULT == accuracy to which Lambda should be calc'd */
 #define BLAST_KARLIN_LAMBDA_ACCURACY_DEFAULT    (1.e-5)
@@ -219,6 +315,7 @@ of K, so high accuracy is generally unwarranted.
 #define BLASTNA_SIZE 16
 /* Identifies the blastna alphabet, for use in blast only. */
 #define BLASTNA_SEQ_CODE 99	
+#define PSI_ALPHABET_SIZE  26 /* For PSI Blast this is the only 26 */
 
 extern Uint1 ncbi4na_to_blastna[BLASTNA_SIZE];
 
@@ -236,9 +333,7 @@ typedef struct {
 		Nlm_FloatHi	Lambda; /* Lambda value used in statistics */
 		Nlm_FloatHi	K, logK; /* K value used in statistics */
 		Nlm_FloatHi	H; /* H value used in statistics */
-		/* "real" values are ones actually found, may be replaced by above */
-		Nlm_FloatHi	Lambda_real, K_real, logK_real, H_real;
-		Int4 q_frame, s_frame; /* reading frame for query and subject.*/
+		Nlm_FloatHi	paramC;	/* for use in seed. */
 	} BLAST_KarlinBlk, PNTR BLAST_KarlinBlkPtr;
 
 
@@ -296,6 +391,7 @@ protein alphabet (e.g., ncbistdaa etc.), FALSE for nt. alphabets. */
 	BLASTMatrixStructurePtr matrix_struct;	/* Holds info about matrix. */
 	BLAST_ScorePtr PNTR matrix;  /* Substitution matrix */
 	BLAST_ScorePtr PNTR posMatrix;  /* Sub matrix for position depend BLAST. */
+    Nlm_FloatHi karlinK; /* Karlin-Altschul parameter associated with posMatrix */
 	Int2		mat_dim1, mat_dim2;	/* dimensions of matrix. */
 	BLAST_ScorePtr	maxscore; /* Max. score for each letter */
 	BLAST_Score	loscore, hiscore; /* Min. & max. substitution scores */
@@ -303,6 +399,7 @@ protein alphabet (e.g., ncbistdaa etc.), FALSE for nt. alphabets. */
 	Boolean		read_in_matrix; /* If TRUE, matrix is read in, otherwise
 					produce one from penalty and reward above. */
 	BLAST_ScoreFreqPtr PNTR sfp;	/* score frequencies. */
+	Nlm_FloatHi **posFreqs; /*matrix of position specific frequencies*/
 	/* kbp & kbp_gap are ptrs that should be set to kbp_std, kbp_psi, etc. */
 	BLAST_KarlinBlkPtr 	PNTR kbp; 	/* Karlin-Altschul parameters. */
 	BLAST_KarlinBlkPtr 	PNTR kbp_gap; /* K-A parameters for gapped alignments. */
@@ -323,17 +420,24 @@ protein alphabet (e.g., ncbistdaa etc.), FALSE for nt. alphabets. */
 /**** Andy's modification ****/
 	Int4    	query_length;   /* the length of the query. */
 /**** end Andy's modification ****/
+	Int4	length_adjustment; /* length to trim query/db sequence by. */
+	Int4	effective_query_length; /* shortened query length. */
+	Int8	effective_db_length;	/* trimmed db length */
+	Int8	effective_search_sp;	/* product of above two */
 	} BLAST_ScoreBlk, PNTR BLAST_ScoreBlkPtr;
 
 /* Used for communicating between BLAST and other applications. */
 typedef struct _blast_matrix {
 		Boolean is_prot;	/* Matrix is for proteins */
 		CharPtr name;		/* Name of Matrix (i.e., BLOSUM62). */
-		/* Position-specific BLAST rows and columns are different. */
-		Int4	rows,		
-			columns;
+		/* Position-specific BLAST rows and columns are different, otherwise they are the
+		alphabet length. */
+		Int4	rows,		/* query length + 1 for PSSM. */
+			columns;	/* alphabet size in all cases (26). */
 		Int4Ptr PNTR matrix;
-		FloatHi karlinK;
+                Nlm_FloatHi ** posFreqs;
+		Nlm_FloatHi karlinK;
+                Int4Ptr PNTR original_matrix;
 } BLAST_Matrix, PNTR BLAST_MatrixPtr;
 
 typedef struct _blast_rescomp {
@@ -358,10 +462,13 @@ Int2 LIBCALL BlastScoreSetAmbigRes PROTO((BLAST_ScoreBlkPtr sbp, Char ambiguous_
 Int2 LIBCALL BlastScoreBlkFill PROTO((BLAST_ScoreBlkPtr sbp, CharPtr string, Int4 length, Int2 context_number));
  
 Int2 LIBCALL BlastScoreBlkMatFill PROTO((BLAST_ScoreBlkPtr sbp, CharPtr matrix));
+BLAST_ScorePtr PNTR LIBCALL BlastScoreBlkMatCreateEx(BLAST_ScorePtr PNTR matrix,BLAST_Score penalty, BLAST_Score reward);
  
 Int2 LIBCALL BlastScoreBlkMatRead PROTO((BLAST_ScoreBlkPtr sbp, FILE *fp));
  
 Int2 LIBCALL BlastScoreBlkMaxScoreSet PROTO((BLAST_ScoreBlkPtr sbp));
+BLAST_ScorePtr BlastPSIMaxScoreGet(BLAST_ScorePtr PNTR posMatrix, 
+                                   Int4 start, Int4 length);
 
 BLAST_ResCompPtr LIBCALL BlastResCompNew PROTO((BLAST_ScoreBlkPtr sbp));
 
@@ -372,7 +479,7 @@ Int2 LIBCALL BlastResCompStr PROTO((BLAST_ScoreBlkPtr sbp, BLAST_ResCompPtr rcp,
 /* 
 Produces a Karlin Block, and parameters, with standard protein frequencies.
 */
-Int2 LIBCALL BlastKarlinBlkStandardCalc PROTO((BLAST_ScoreBlkPtr sbp, Int2 context_total));
+Int2 LIBCALL BlastKarlinBlkStandardCalc PROTO((BLAST_ScoreBlkPtr sbp, Int2 context_start, Int2 context_end));
 BLAST_KarlinBlkPtr LIBCALL BlastKarlinBlkStandardCalcEx PROTO((BLAST_ScoreBlkPtr sbp));
 
 
@@ -389,26 +496,53 @@ Int2 LIBCALL BlastKarlinBlkCalc PROTO((BLAST_KarlinBlkPtr kbp, BLAST_ScoreFreqPt
 
 Int2 LIBCALL BlastKarlinBlkGappedCalc PROTO((BLAST_KarlinBlkPtr kbp, Int4 gap_open, Int4 gap_extend, CharPtr matrix_name, ValNodePtr PNTR error_return));
 
+Int2 LIBCALL BlastKarlinBlkGappedCalcEx PROTO((BLAST_KarlinBlkPtr kbp, Int4 gap_open, Int4 gap_extend, Int4 decline_align, CharPtr matrix_name, ValNodePtr PNTR error_return));
+
+
+/*
+        Attempts to fill KarlinBlk for given gap opening, extensions etc.
+        Will return non-zero status if that fails.
+
+        return values:  -1 if matrix_name is NULL;
+                        1 if matrix not found
+                        2 if matrix found, but open, extend etc. values not supported.
+*/
+Int2 LIBCALL BlastKarlinkGapBlkFill(BLAST_KarlinBlkPtr kbp, Int4 gap_open, Int4 gap_extend, Int4 decline_align, CharPtr matrix_name);
+
+/* Prints a messages about the allowed matrices, BlastKarlinkGapBlkFill should return 1 before this is called. */
+CharPtr PrintMatrixMessage(const Char *matrix);
+
+/* Prints a messages about the allowed open etc values for the given matrix, 
+BlastKarlinkGapBlkFill should return 2 before this is called. */
+CharPtr PrintAllowedValuesMessage(const Char *matrix, Int4 gap_open, Int4 gap_extend, Int4 decline_align);
+
+Int2 LIBCALL BlastKarlinReportAllowedValues(const Char *matrix_name, ValNodePtr PNTR error_return);
+
 
 Nlm_FloatHi BlastKarlinLHtoK PROTO((BLAST_ScoreFreqPtr sfp, Nlm_FloatHi lambda, Nlm_FloatHi H));
 
-Nlm_FloatHi BlastKarlinLambdaBis PROTO((BLAST_ScoreFreqPtr sfp));
-
 Nlm_FloatHi BlastKarlinLambdaNR PROTO((BLAST_ScoreFreqPtr sfp));
 
-Nlm_FloatHi LIBCALL BlastKarlinLtoH PROTO((BLAST_ScoreFreqPtr sfp, Nlm_FloatHi  lambda));
-
 BLAST_Score LIBCALL BlastKarlinEtoS PROTO((Nlm_FloatHi  E, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi  qlen, Nlm_FloatHi  dblen));
 
+BLAST_Score LIBCALL BlastKarlinEtoS_simple PROTO((Nlm_FloatHi  E, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi searchsp)); 
+
 Nlm_FloatHi LIBCALL BlastKarlinPtoE PROTO((Nlm_FloatHi p));
 
 Nlm_FloatHi LIBCALL BlastKarlinEtoP PROTO((Nlm_FloatHi x));
 
 Nlm_FloatHi LIBCALL BlastKarlinStoP PROTO((BLAST_Score S, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi  qlen, Nlm_FloatHi  dblen));
 
+Nlm_FloatHi LIBCALL BlastKarlinStoP_simple PROTO((BLAST_Score S, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi  searchsp));
+
 Nlm_FloatHi LIBCALL BlastKarlinStoE PROTO((BLAST_Score S, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi  qlen, Nlm_FloatHi  dblen));
 
-Int2 LIBCALL BlastCutoffs PROTO((BLAST_ScorePtr S, Nlm_FloatHi PNTR E, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi qlen, Nlm_FloatHi dblen, Nlm_Boolean dodecay));
+Nlm_FloatHi LIBCALL BlastKarlinStoE_simple PROTO((BLAST_Score S, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi  searchsp));
+
+Nlm_FloatHi LIBCALL BlastGapDecayDivisor PROTO((Nlm_FloatHi decayrate, unsigned nsegs ));
+
+Int2 LIBCALL BlastCutoffs PROTO((BLAST_ScorePtr S, Nlm_FloatHi PNTR E, BLAST_KarlinBlkPtr kbp, Nlm_FloatHi search_sp, Nlm_Boolean dodecay,
+  Nlm_FloatHi gap_decay_rate));
 
 Nlm_FloatHi LIBCALL BlastKarlinStoLen PROTO((BLAST_KarlinBlkPtr kbp, BLAST_Score S));
 
@@ -416,9 +550,11 @@ Nlm_FloatHi LIBCALL BlastKarlinStoLen PROTO((BLAST_KarlinBlkPtr kbp, BLAST_Score
 Nlm_FloatHi LIBCALL BlastSumP PROTO((Int4 r, Nlm_FloatHi s));
 
 /* Functions to calculate SumE (for large and small gaps). */
-Nlm_FloatHi LIBCALL BlastSmallGapSumE PROTO((BLAST_KarlinBlkPtr kbp, Int4 gap, Nlm_FloatHi gap_prob, Nlm_FloatHi gap_decay_rate, Int2 num, Int4 sum,  Nlm_FloatHi xsum, Int4 query_length, Int4 subject_length, Boolean min_length_one));
+Nlm_FloatHi LIBCALL BlastSmallGapSumE PROTO((BLAST_KarlinBlkPtr kbp, Int4 gap, Int2 num,  Nlm_FloatHi xsum, Int4 query_length, Int4 subject_length, Nlm_FloatHi weight_divisor));
 
-Nlm_FloatHi LIBCALL BlastLargeGapSumE PROTO((BLAST_KarlinBlkPtr kbp, Nlm_FloatHi gap_prob, Nlm_FloatHi gap_decay_rate, Int2 num, Int4 sum,  Nlm_FloatHi xsum, Int4 query_length, Int4 subject_length, Boolean old_stats));
+Nlm_FloatHi LIBCALL BlastUnevenGapSumE PROTO((BLAST_KarlinBlkPtr kbp, Int4 p_gap, Int4 n_gap, Int2 num, Nlm_FloatHi xsum, Int4 query_length, Int4 subject_length, Nlm_FloatHi weight_divisor));
+
+Nlm_FloatHi LIBCALL BlastLargeGapSumE PROTO((BLAST_KarlinBlkPtr kbp, Int2 num,  Nlm_FloatHi xsum, Int4 query_length, Int4 subject_length, Nlm_FloatHi weight_divisor));
 
 /* Used to produce random sequences. */
 CharPtr  LIBCALL BlastRepresentativeResidues PROTO((Int2 length));
@@ -426,6 +562,7 @@ CharPtr  LIBCALL BlastRepresentativeResidues PROTO((Int2 length));
 Int2 LIBCALL BlastResFreqNormalize PROTO((BLAST_ScoreBlkPtr sbp, BLAST_ResFreqPtr rfp, Nlm_FloatHi norm));
 
 BLAST_ResFreqPtr LIBCALL BlastResFreqNew PROTO((BLAST_ScoreBlkPtr sbp));
+void LIBCALL BlastResFreqFree PROTO((BLAST_ResFreqPtr rfp));
 
 BLAST_ResFreqPtr LIBCALL BlastResFreqDestruct PROTO((BLAST_ResFreqPtr rfp));
 
@@ -445,6 +582,9 @@ BLAST_MatrixPtr LIBCALL BLAST_MatrixDestruct PROTO((BLAST_MatrixPtr blast_matrix
 
 BLAST_MatrixPtr LIBCALL BLAST_MatrixFill PROTO((BLAST_ScoreBlkPtr sbp, Boolean positionBased));
 
+BLAST_MatrixPtr LIBCALL BLAST_MatrixFetch PROTO((CharPtr matrix_name));
+
+
 Int2 LIBCALL BlastGetStdAlphabet PROTO((Uint1 alphabet_code, Uint1Ptr residues, Int4 residues_size));
 /*
 Functions used to convert between Stephen's pseudo scores
@@ -470,6 +610,28 @@ arrays.
 
 Int2 LIBCALL BlastKarlinGetMatrixValues PROTO((CharPtr matrix, Int4Ptr PNTR open, Int4Ptr PNTR extension, FloatHiPtr PNTR lambda, FloatHiPtr PNTR K, FloatHiPtr PNTR H, Int4Ptr PNTR pref_flags));
 
+Int2 LIBCALL BlastKarlinGetMatrixValuesEx PROTO((CharPtr matrix, Int4Ptr PNTR open, Int4Ptr PNTR extension, Int4Ptr PNTR decline_align, FloatHiPtr PNTR lambda, FloatHiPtr PNTR K, FloatHiPtr PNTR H, Int4Ptr PNTR pref_flags));
+
+Int2 LIBCALL BlastKarlinGetMatrixValuesEx2 PROTO((CharPtr matrix, Int4Ptr PNTR open, Int4Ptr PNTR extension, Int4Ptr PNTR decline_align, FloatHiPtr PNTR lambda, FloatHiPtr PNTR K, FloatHiPtr PNTR H, FloatHiPtr PNTR alpha, FloatHiPtr PNTR beta, Int4Ptr PNTR pref_flags));
+
+void LIBCALL getAlphaBeta PROTO((CharPtr matrixName, Nlm_FloatHi *alpha,
+Nlm_FloatHi *beta, Boolean gapped, Int4 gap_open, Int4 gap_extend));
+
+Int2 LIBCALL BlastKarlinGetDefaultMatrixValues PROTO((CharPtr matrix, Int4Ptr open, Int4Ptr extension, FloatHiPtr lambda, FloatHiPtr K, FloatHiPtr H));
+
+Int4Ptr PNTR LIBCALL BlastMatrixToTxMatrix PROTO((BLAST_MatrixPtr matrix));
+Int4Ptr PNTR LIBCALL TxMatrixDestruct PROTO((Int4Ptr PNTR txmatrix)); 
+
+Int4
+BlastComputeLengthAdjustment(Nlm_FloatHi K,
+                                  Nlm_FloatHi logK,
+                                  Nlm_FloatHi alpha_d_lambda,
+                                  Nlm_FloatHi beta,
+                                  Int4 query_length,
+                                  Int8 db_length,
+                                  Int4 db_num_seqs,
+                                  Int4 * length_adjustment);
+
 #ifdef __cplusplus
 }
 #endif
