@@ -32,8 +32,78 @@ Contents: prototypes for "private" BLAST functions, these should not be called
 
 ******************************************************************************/
 
-/* $Revision: 6.90 $ 
+/* $Revision: 6.112 $ 
 * $Log: blastpri.h,v $
+* Revision 6.112  2004/03/31 17:58:51  papadopo
+* Mike Gertz' changes for length adjustment calculations
+*
+* Revision 6.111  2004/01/16 23:43:43  dondosha
+* No more need for special argument for partial search: it is set in options
+*
+* Revision 6.110  2004/01/06 22:36:24  dondosha
+* Added BLAST_HSPFree function that frees the edit block and the HSP
+*
+* Revision 6.109  2003/08/20 22:12:56  dondosha
+* Added BlastPrintTabularResults with an extra boolean parameter for OOF alignments
+*
+* Revision 6.108  2003/08/04 16:19:16  dondosha
+* Added effective HSP length (length adjustment) to other returns, so it can be reported in XML output
+*
+* Revision 6.107  2003/04/22 21:52:13  dondosha
+* Added function OOFBlastHSPGetNumIdentical
+*
+* Revision 6.106  2003/03/24 19:42:14  madden
+* Changes to support query concatenation for blastn and tblastn
+*
+* Revision 6.105  2002/11/04 22:47:02  dondosha
+* Added prototype for BlastHSPGetNumIdentical
+*
+* Revision 6.104  2002/08/30 15:50:06  dondosha
+* Several prototypes moved from mblast.h
+*
+* Revision 6.103  2002/08/01 20:45:35  dondosha
+* Changed prototype of the BLASTPostSearchLogic function to make it
+* more convenient
+*
+* Revision 6.102  2002/07/09 16:09:42  camacho
+* Changed interface to BlastCreateVirtualOIDList
+*
+* Revision 6.101  2002/06/26 00:56:29  camacho
+*
+* 1. Fixed bug when searching a mixture of real and mask databases.
+* 2. Clean up of code that calculates the number of sequences and database
+*    length.
+*
+* Revision 6.100  2002/06/25 19:46:07  camacho
+* Added PROTO around function declaration
+*
+* Revision 6.99  2002/06/25 19:39:38  camacho
+* Made BlastCreateVirtualOIDList public for use by neighboring software
+*
+* Revision 6.98  2002/06/21 21:43:01  camacho
+* Removed obsolete BlastSeqIdList structure and functions
+*
+* Revision 6.97  2002/06/19 22:50:33  dondosha
+* Added all queries information for tabular output with multiple queries
+*
+* Revision 6.96  2002/06/11 14:44:48  dondosha
+* Return status from some functions instead of search block pointer
+*
+* Revision 6.95  2002/04/18 16:18:20  dondosha
+* Added BlastPrintTabulatedResultsEx with extra argument to keep track of progress
+*
+* Revision 6.94  2002/04/15 20:42:05  jianye
+* Added getFastaStyleTitle(BioseqPtr bsp)
+*
+* Revision 6.93  2002/03/26 16:46:41  madden
+* Move calculation of effective lengths to BlastCalculateEffectiveLengths
+*
+* Revision 6.92  2002/03/06 18:34:32  dondosha
+* Pass the filtered locations back from the megablast engine to use in formatting
+*
+* Revision 6.91  2002/03/05 17:58:57  dondosha
+* Set same offsets for the traceback as for preliminary extension for megablast with non-greedy extensions
+*
 * Revision 6.90  2002/01/04 20:16:12  dondosha
 * Correction for single strand blastx with OOF gapping
 *
@@ -599,6 +669,7 @@ typedef struct _txdbinfo {
 #define TXPARAMETERS 14
 #define TXMATRIX 15
 #define EFF_SEARCH_SPACE 16
+#define EFF_HSP_LENGTH 17
 	
 /*
 	Allocates memory for TxDfDbInfoPtr.
@@ -662,9 +733,9 @@ Int2 LIBCALL BlastReapHitlistByEvalue PROTO ((BlastSearchBlkPtr search));
 
 Int4 LIBCALL BlastSaveCurrentHitlist PROTO((BlastSearchBlkPtr search));
 
-BlastSearchBlkPtr LIBCALL BLASTPerformSearchWithReadDb PROTO((BlastSearchBlkPtr search, Int4 sequence_number));
+Int2 LIBCALL BLASTPerformSearchWithReadDb PROTO((BlastSearchBlkPtr search, Int4 sequence_number));
 
-BlastSearchBlkPtr LIBCALL BLASTPerformSearch PROTO((BlastSearchBlkPtr search, Int4 subject_length, Uint1Ptr subject_seq));
+Int2 LIBCALL BLASTPerformSearch PROTO((BlastSearchBlkPtr search, Int4 subject_length, Uint1Ptr subject_seq));
 
 BLASTResultsStructPtr LIBCALL BLASTResultsStructDelete PROTO((BLASTResultsStructPtr result_struct));
 
@@ -683,6 +754,9 @@ BlastSearchBlkPtr LIBCALL BlastSearchBlkDestruct PROTO((BlastSearchBlkPtr search
 
 BlastSearchBlkPtr BLASTSetUpSearchWithReadDbInternal PROTO((SeqLocPtr query_slp, BioseqPtr query_bsp, CharPtr prog_name, Int4 qlen, CharPtr dbname, BLAST_OptionsBlkPtr options, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total, ReadDBFILEPtr rdfp));
 
+BlastSearchBlkPtr BLASTSetUpSearchWithReadDbInternalMult PROTO((SeqLocPtr query_slp, BioseqPtr query_bsp, CharPtr prog_name, Int4 qlen, CharPtr dbname, BLAST_OptionsBlkPtr options, int (LIBCALLBACK *callback)PROTO((Int4 done, Int4 positives)), SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, Int4 gi_list_total, ReadDBFILEPtr rdfp, QueriesPtr mult_queries));
+/* --KM added mult_queries param */
+
 BlastSearchBlkPtr LIBCALL
 BLASTSetUpSearchWithReadDbInternalEx PROTO((SeqLocPtr query_slp, BioseqPtr query_bsp,
 				      CharPtr prog_name, Int4 qlen, CharPtr
@@ -755,6 +829,11 @@ void BlastSaveCurrentHspGapped PROTO((BlastSearchBlkPtr search, BLAST_Score
 				      score, Int4 q_offset, Int4 s_offset, Int4
 				      q_length, Int4 s_length, Int2 context,
 				      GapXEditScriptPtr esp));
+void BlastNtSaveCurrentHspGapped PROTO((BlastSearchBlkPtr search, BLAST_Score
+				      score, Int4 q_offset, Int4 s_offset, Int4
+				      q_length, Int4 s_length, 
+                                      Int4 q_gapped_start, Int4 s_gapped_start,
+                                      Int2 context, GapXEditScriptPtr esp));
 
 void BlastNtSaveCurrentHsp PROTO((BlastSearchBlkPtr search, BLAST_Score score, Int4 q_offset, Int4 s_offset, Int4 length, Int2 context, Int4 query_gap_start, Int4 subject_gap_start));
 
@@ -762,7 +841,6 @@ CharPtr FormatBlastParameters PROTO((BlastSearchBlkPtr search));
 
 
 SeqAlignPtr LIBCALL BioseqBlastEngineCore PROTO((BlastSearchBlkPtr search, BLAST_OptionsBlkPtr options, Int4Ptr *pos_matrix));
-SeqAlignPtr LIBCALL BioseqBlastEngineCoreEx PROTO((BlastSearchBlkPtr search, BLAST_OptionsBlkPtr options, Int4Ptr *pos_matrix, Boolean partial));
 
 Uint1Ptr GetSequenceWithDenseSeg PROTO((DenseSegPtr dsp, Boolean query, Int4Ptr start, Int4Ptr length));
 
@@ -813,13 +891,18 @@ BlastAllWordPtr BlastPopulateAllWordArrays PROTO((Int4 wordsize, Int4 alphabet_s
 
 Uint1Ptr BlastGetSequenceFromBioseq PROTO((BioseqPtr bsp, Int4Ptr length));
 
+#if 0 /* deprecated */
 BlastSeqIdListPtr BlastSeqIdListNew PROTO((void));
 BlastSeqIdListPtr BlastSeqIdListDestruct PROTO((BlastSeqIdListPtr seqid_list));
+#endif
+
+/* Retrieve the virtual oidlist from rdfp_chain */
+OIDListPtr LIBCALL BlastGetVirtualOIDList PROTO((ReadDBFILEPtr rdfp_chain));
 
 Boolean BlastAdjustDbNumbers PROTO((ReadDBFILEPtr rdfp_list, Int8Ptr db_length, Int4Ptr db_number, SeqIdPtr seqid_list, BlastDoubleInt4Ptr gi_list, OIDListPtr oidlist, BlastDoubleInt4Ptr PNTR gi_list_pointers, Int4 gi_list_total));
 
 BlastGiListPtr BlastGiListDestruct PROTO((BlastGiListPtr blast_gi_list, Boolean contents));
-BlastGiListPtr BlastGiListNew PROTO((BlastDoubleInt4Ptr gi_list, BlastDoubleInt4Ptr PNTR gi_list_pointers, Int4 total));
+BlastGiListPtr BlastGiListNew PROTO((BlastDoubleInt4Ptr gi_list, Int4 total));
 
 
 ScorePtr LIBCALL GetScoreSetFromBlastResultHsp PROTO((BLASTResultHspPtr hsp, SeqIdPtr gi_list));
@@ -912,6 +995,14 @@ Int4
 BlastGetNumIdentical PROTO((Uint1Ptr query, Uint1Ptr subject, Int4 q_start, 
                          Int4 s_start, Int4 length, Boolean reverse));
 
+Int2
+BlastHSPGetNumIdentical PROTO((BlastSearchBlkPtr search, BLAST_HSPPtr hsp,
+   BLASTResultHspPtr result_hsp, Int4Ptr num_ident, Int4Ptr align_length));
+Int2
+OOFBlastHSPGetNumIdentical PROTO((Uint1Ptr query_seq, Uint1Ptr subject_seq,
+   BLAST_HSPPtr hsp, BLASTResultHspPtr result_hsp, Int4Ptr num_ident, 
+   Int4Ptr align_length));
+
 SeqIdPtr GetTheSeqAlignID PROTO((SeqIdPtr seq_id));
 StdSegPtr BLASTHspToStdSeg PROTO((BlastSearchBlkPtr search, Int4 subject_length, BLAST_HSPPtr hsp, SeqIdPtr sip, Boolean reverse, SeqIdPtr gi_list));
 
@@ -921,21 +1012,86 @@ StdSegPtr BLASTHspToStdSeg PROTO((BlastSearchBlkPtr search, Int4 subject_length,
 
 int LIBCALLBACK BlastPrintAlignInfo PROTO((VoidPtr srch));
 int LIBCALLBACK MegaBlastPrintAlignInfo PROTO((VoidPtr srch));
-void BlastPrintTabulatedResults PROTO((SeqAlignPtr seqalign, BioseqPtr query_bsp, SeqLocPtr query_slp, Int4 num_alignments, CharPtr blast_program, Boolean is_ungapped, Boolean believe_query, Int4 q_shift, Int4 s_shift, FILE *fp));
+void BlastPrintTabulatedResults PROTO((SeqAlignPtr seqalign, BioseqPtr query_bsp, SeqLocPtr query_slp, Int4 num_alignments, CharPtr blast_program, Boolean is_ungapped, Boolean believe_query, Int4 q_shift, Int4 s_shift, FILE *fp, Boolean print_query_info));
+void BlastPrintTabulatedResultsEx PROTO((SeqAlignPtr seqalign, BioseqPtr query_bsp, SeqLocPtr query_slp, Int4 num_alignments, CharPtr blast_program, Boolean is_ungapped, Boolean believe_query, Int4 q_shift, Int4 s_shift, FILE *fp, int *num_formatted, Boolean print_query_info));
+
+void BlastPrintTabularResults(SeqAlignPtr seqalign, BioseqPtr query_bsp,
+        SeqLocPtr query_slp, Int4 num_alignments, CharPtr blast_program, 
+        Boolean is_ungapped, Boolean is_ooframe, Boolean believe_query, 
+        Int4 q_shift, Int4 s_shift, FILE *fp, int *num_formatted, 
+        Boolean print_query_info);
+
 void
-BlastProcessGiLists PROTO((BlastSearchBlkPtr search, BLAST_OptionsBlkPtr options,
-                           BlastDoubleInt4Ptr gi_list, Int4Ptr gi_list_size));
+BlastProcessGiLists PROTO((BlastSearchBlkPtr search, 
+            BLAST_OptionsBlkPtr options, BlastDoubleInt4Ptr gi_list, 
+            Int4 gi_list_size));
+
+/* Purpose: Create the virtual oidlist that ultimately will limit the 
+ * blast search being set up. This will be an intersection of the bglp
+ * parameter as well as any oidlists attached to the rdfp_chain.
+ *
+ * Parameters: 
+ * bglp: The gi list to limit the search (this will be freed this by this
+ * function)
+ * rdfp_chain: The head of the rdfp list that will be searched.
+ * oidlist_for_all_rdfp: if true, the virtual oidlist will be attached to the
+ * beginning of the rdfp_chain, else it will be attached to the first rdfp in
+ * rdfp_chain that had an oidlist (and therefore covering all remaining
+ * rdfps).
+ * options: the blast options structure (see comments in blastool.c).
+ * 
+ * Return value:
+ * The intersection of the bglp parameter and any oidlists attached to the
+ * rdfp_chain. */
+BlastGiListPtr BlastCreateVirtualOIDList PROTO((BlastGiListPtr bglp, 
+                                         ReadDBFILEPtr rdfp_chain,
+                                         Boolean oidlist_forall_rdfp, 
+                                         BLAST_OptionsBlkPtr options));
 Boolean FastaCheckDna PROTO((CharPtr seq));
 BLASTHSPSegmentPtr BLASTHSPSegmentFromSeqAlign PROTO((SeqAlignPtr sap));
 SeqAlignPtr BlastClusterHitsFromSeqAlign PROTO((SeqAlignPtr seqalign, CharPtr prog_name, CharPtr database, BLAST_OptionsBlkPtr options, FloatHi length_thresh, FloatHi score_thresh, FloatHi overlap_thresh, Boolean two_sided));
 SeqAlignPtr RedoAlignmentCore PROTO((BlastSearchBlkPtr search,   
   BLAST_OptionsBlkPtr options, Int4 hitlist_count, Boolean adjustParameters,
   Boolean SmithWaterman));
+BLAST_HSPPtr BLAST_HSPFree PROTO((BLAST_HSPPtr hsp));
 void BLASTResultFreeHsp PROTO((BLASTResultHitlistPtr result));
-SeqAlignPtr BLASTPostSearchLogic PROTO((BlastSearchBlkPtr search,BLAST_OptionsBlkPtr options));
+void BLASTPostSearchLogic PROTO((BlastSearchBlkPtr search, BLAST_OptionsBlkPtr
+                                 options, SeqAlignPtr PNTR seqalignp,
+                                 Boolean single_chain));
 Boolean
 BlastNtWordUngappedExtend PROTO((BlastSearchBlkPtr search, Int4 q_off, 
                                  Int4 s_off, Int4 cutoff));
+SeqLocPtr blastMergeFilterLocs PROTO((SeqLocPtr filter_slp, 
+                                      SeqLocPtr lcmask, Boolean translate,
+                                      Int2 frame, Int4 length));
+
+/*
+        function to calculate effective query length and
+        effective db length.
+*/
+
+/*return query fasta style title(id+title). New memory was allocated for this title*/
+CharPtr getFastaStyleTitle(BioseqPtr bsp);
+
+CharPtr load_options_to_buffer(CharPtr instructions, CharPtr buffer);
+
+VoidPtr index_proc PROTO((VoidPtr dummy));
+
+CharPtr BlastConstructFilterString PROTO((Int4 filter_value));
+
+int LIBCALLBACK evalue_compare_hits PROTO((VoidPtr v1, VoidPtr v2));
+
+Uint1 FrameToDefine PROTO((Int2 frame));
+
+Uint1Ptr
+GetPrivatTranslationTable PROTO((CharPtr genetic_code, 
+                                 Boolean reverse_complement));
+Int2 blast_set_parameters PROTO((BlastSearchBlkPtr search, Int4
+                                 dropoff_number_of_bits_1st_pass, Int4
+                                 dropoff_number_of_bits_2nd_pass, Nlm_FloatHi
+                                 avglen, Nlm_FloatHi searchsp, Int4 window));
+Boolean 
+BlastGetFirstAndLastContext PROTO((CharPtr prog_name, SeqLocPtr query_slp, Int2Ptr first_context, Int2Ptr last_context, Uint1 strand_options));
 
 #ifdef __cplusplus
 }
