@@ -41,7 +41,7 @@ Contents: defines and prototypes used by readdb.c and formatdb.c.
 *
 * Version Creation Date:   3/21/95
 *
-* $Revision: 6.93 $
+* $Revision: 6.147 $
 *
 * File Description: 
 *       Functions to rapidly read databases from files produced by formatdb.
@@ -56,6 +56,180 @@ Contents: defines and prototypes used by readdb.c and formatdb.c.
 *
 * RCS Modification History:
 * $Log: readdb.h,v $
+* Revision 6.147  2004/04/16 18:14:50  camacho
+* Made division field in DI_Record larger
+*
+* Revision 6.146  2004/02/24 14:06:01  camacho
+* Added support for approximate sequence length calculation for nucleotide
+* sequences.
+*
+* Revision 6.145  2004/02/04 15:35:05  camacho
+* Rollback to fix problems in release 2.2.7
+*
+* Revision 6.143  2003/07/08 18:42:40  camacho
+* Elaborated fastacmd return values
+*
+* Revision 6.142  2003/06/13 19:56:48  dondosha
+* Removed unneeded argument in FastaToBlastDB
+*
+* Revision 6.141  2003/04/25 18:55:27  camacho
+* 1. Added readdb_merge_gifiles to deal with Microbial blast database issues.
+* 2. Minor fixes to Int4List functions.
+*
+* Revision 6.140  2003/04/22 21:30:14  camacho
+* Added Int4 list utilities
+*
+* Revision 6.139  2003/04/22 19:04:57  camacho
+* Moved GiList structure to generic list of 4-byte integers
+*
+* Revision 6.138  2003/04/16 15:39:37  coulouri
+* fix compiler warning
+*
+* Revision 6.137  2003/04/15 19:09:13  camacho
+* Completed implementation of PIG interface
+*
+* Revision 6.136  2003/04/10 15:11:37  camacho
+* Include PIG interface in __cplusplus
+*
+* Revision 6.135  2003/04/09 21:46:00  camacho
+* Added basic PIG interface
+*
+* Revision 6.134  2003/04/08 19:45:35  camacho
+* Defined invalid PIG
+*
+* Revision 6.133  2003/04/08 15:37:15  camacho
+* Extended FDBAddSequence2 to take pig
+*
+* Revision 6.132  2003/04/01 21:51:36  camacho
+* Made fastacmd functions & structure non-static
+*
+* Revision 6.131  2003/03/27 22:26:04  camacho
+* Add error messages and non-zero return value on error for fastacmd
+*
+* Revision 6.130  2003/03/26 19:11:22  camacho
+* Minor change to previous commit
+*
+* Revision 6.129  2003/03/26 18:50:07  camacho
+* Added eFDBCleanOpt to formatdb API
+*
+* Revision 6.128  2003/01/30 21:57:28  camacho
+* Added more detailed comment to readdb_new_ex2
+*
+* Revision 6.127  2003/01/22 19:41:21  camacho
+* Added function to build multi-volume db list for creating alias files
+*
+* Revision 6.126  2002/12/20 14:37:34  coulouri
+* Fix prototype for RDBTaxInfoInit()
+*
+* Revision 6.125  2002/12/17 20:33:25  camacho
+* Removed unnecessary function attribute
+*
+* Revision 6.124  2002/12/16 20:22:48  camacho
+* Removed unused options in formatdb options structure
+*
+* Revision 6.123  2002/12/16 05:01:55  camacho
+* Fixes to previous commit
+*
+* Revision 6.122  2002/12/13 13:43:25  camacho
+* Changes to set links and membership bits in formatdb API
+*
+* Revision 6.121  2002/11/25 17:23:28  camacho
+* 1) Changed file access to blast taxonomy databases: only 2 files are loaded
+*    for an entire chain of rdfp's.
+* 2) Fixed memory leak in FindBlastDBFile.
+* 3) Protect NlmOpenMFILE against NULL argument.
+*
+* Revision 6.120  2002/10/25 16:49:45  camacho
+* Added Michael Kimelman's FDBAddSequence2
+*
+* Revision 6.119  2002/10/03 14:13:44  camacho
+* Added support for gilist field in alias file in multivolume databases
+*
+* Revision 6.118  2002/09/26 02:14:42  camacho
+* Allow limiting the number of sequences per volume
+*
+* Revision 6.117  2002/09/25 20:14:20  camacho
+* Fix for multivolume databases with non-parseable seqids
+*
+* Revision 6.116  2002/07/30 15:28:50  camacho
+* Added fastacmd function to parse SeqLocs
+*
+* Revision 6.115  2002/07/29 15:45:19  camacho
+* Made readdb_get_taxnames a LIBCALL function
+*
+* Revision 6.114  2002/07/24 19:31:48  raytseli
+* much simpler and more efficient approach to using madvise()
+* .
+*
+* Revision 6.113  2002/07/22 13:06:42  raytseli
+* explicitly allow setting of the advice type for madvise()
+* .
+*
+* Revision 6.112  2002/07/18 17:39:54  raytseli
+* changed ifdef OS_UNIX_SUN to ifdef OS_UNIX_SOL for madvise()
+* .
+*
+* Revision 6.111  2002/07/18 15:54:26  raytseli
+* added function to explicitly set madvise() block size, and madvise() sync mode.
+*
+* Revision 6.110  2002/07/18 15:01:54  raytseli
+* correct problem with pointer format "%p" ErrPostEx() handling on linux.
+* Add extern func to allow explicit madvise() functionality activation.
+*
+* Revision 6.109  2002/07/17 17:15:06  raytseli
+* only allow madvise()-related stuff on SUN or Linux.
+* .
+*
+* Revision 6.108  2002/07/17 16:54:54  raytseli
+* additional #ifdefs to allow compilation.
+*
+* Revision 6.106  2002/07/17 14:36:54  raytseli
+* incorporated madvise into readdb
+* .
+*
+* Revision 6.105  2002/07/14 21:02:08  camacho
+* Added extra features to fastacmd
+*
+* Revision 6.104  2002/07/09 16:41:52  camacho
+* Made taxonomy databases multi-thread safe
+*
+* Revision 6.103  2002/06/26 00:45:37  camacho
+*
+* Added readdb_get_totals_ex2 to allow recalculation of database length as
+* well as total number of sequences after the virtual oidlist has been
+* created.
+*
+* Revision 6.102  2002/06/04 21:45:39  dondosha
+* Corrected the readdb_get_sequence_number function in case of multiple-volume databases
+*
+* Revision 6.101  2002/06/04 20:22:56  camacho
+* Fixed taxonomy databases to work w/o mmap
+*
+* Revision 6.100  2002/05/15 20:23:47  camacho
+* Added wgs_{mouse,anthrax} criteria functions
+*
+* Revision 6.99  2002/05/02 21:52:06  camacho
+* Support for genmask's new month/subset mask combinations
+*
+* Revision 6.98  2002/04/18 19:35:07  camacho
+* 1. Added fdfilter/genmask callbacks for wgs subsets
+* 2. Modified fdfilter/genmask refseq_protein callback function
+* 3. Fixed problem in readdb_read_alias_file to read multiple oidlists
+*
+* Revision 6.97  2002/03/08 16:58:50  camacho
+* Added accessions to dump info files *.[pn]di
+*
+* Revision 6.96  2002/01/25 17:06:57  camacho
+* Added new criteria to create new refseq databases
+*
+* Revision 6.95  2002/01/24 18:47:48  camacho
+* Moved RDBTaxNamesFree from readdb.[ch] to txalign.[ch]
+*
+* Revision 6.94  2002/01/11 19:22:26  camacho
+* 1. Added preferred_gi field to ReadDBFILE structure.
+* 2. Modified FDReadDeflineAsn to return the preferred gi as the
+*    first element of the list of BlastDefLine structures (if set).
+*
 * Revision 6.93  2001/12/18 13:01:51  camacho
 * Added new flag -D to dump blast database in FASTA format
 *
@@ -547,13 +721,18 @@ belong to the same sequence. */
 #define READDB_NO_SEQ_FILE         0x00000010
 #define READDB_KEEP_HDR_AND_SEQ    0x00000020
 
-/* Choices for how much to initialize on startup in readdb_new_internal. */
-#define READDB_NEW_DO_ALL (Uint1) 1 /* attempt to memory map all files. */
-#define READDB_NEW_DO_REPORT (Uint1) 2 /* Only open the nin or pin files for a database report. */
-#define READDB_NEW_DO_SEARCH (Uint1) 4 /* Only open the nin (or pin) and nsq 
-					  (or psq) files for a search. */
-#define READDB_NEW_INDEX (Uint1) 8 /* Open only index (nin or pin) files for
-				      memory mapping */
+/*** Choices for how much to initialize on startup in readdb_new_internal. ***/
+
+/* attempt to memory map all files. */
+#define READDB_NEW_DO_ALL               ((Uint1) (1<<0))
+/* Only open the nin or pin files for a database report. */
+#define READDB_NEW_DO_REPORT            ((Uint1) (1<<1))
+/* Only open the nin (or pin) and nsq (or psq) files for a search. */
+#define READDB_NEW_DO_SEARCH            ((Uint1) (1<<2))
+/* Open only index (nin or pin) files for memory mapping */
+#define READDB_NEW_INDEX                ((Uint1) (1<<3))
+/* Same as above and memory map blast taxonomy db files */
+#define READDB_NEW_DO_TAXDB             ((Uint1) (1<<4))
 
 /* The following variables are shared by formatdb and readdb. */
 /* version of formatdb.
@@ -569,7 +748,6 @@ belong to the same sequence. */
 #define FORMATDB_VER_TEXT 3
 #define FORMATDB_VER      4
 
-
 /* 'Magic' number at the beginning of a binary gi list that indicates it is binary. */
 #define READDB_MAGIC_NUMBER UINT4_MAX
 
@@ -588,6 +766,7 @@ typedef struct nlm_mfile {
 	Boolean   mfile_true;	/* If TRUE then mmap succeeded. */
 	Boolean   contents_allocated; /* If TRUE, the contents have been allocated
 					and are not merely a copy. */
+	Uint1Ptr mmp_madvise_end; /* madvise() file offset */
 } NlmMFILE, PNTR NlmMFILEPtr;
 
 /*
@@ -617,6 +796,59 @@ Int4 LIBCALL NlmSeekInMFILE PROTO((NlmMFILEPtr mfp, long offset, Int4 ptrname));
 */
 Int4 LIBCALL NlmTellMFILE PROTO((NlmMFILEPtr mfp));
 
+/* Generic 4-byte integer list */
+typedef struct _gilist {
+    Int4    count, allocated;
+    Int4Ptr i;
+} Int4List, *Int4ListPtr;
+
+/* Creates a new list of 4-byte integers */
+Int4ListPtr LIBCALL 
+Int4ListNew PROTO((void));
+
+/* Creates a new list of 4-byte integers of size s */
+Int4ListPtr LIBCALL 
+Int4ListNewEx PROTO((Int4 s));
+
+/* Deallocates the list of 4-byte integers */
+Int4ListPtr LIBCALL 
+Int4ListFree PROTO((Int4ListPtr lp));
+
+/* Reads a list of newline separated 4-byte integers.
+ * Caller is responsible for deallocating the return value */
+Int4ListPtr LIBCALL 
+Int4ListReadFromFile PROTO((CharPtr filename));
+
+/* Appends i to the end of the list, reallocating memory if necessary. Returns
+ * FALSE if it cannot allocate more memory */
+Boolean LIBCALL 
+Int4ListAdd PROTO((Int4ListPtr lp, Int4 i));
+
+/* Returns the concatenation of list1 and list2, freeing both parameters. It
+ * returns NULL if both lists are empty and if it cannot allocate more memory */
+Int4ListPtr LIBCALL
+Int4ListConcat PROTO((Int4ListPtr *list1, Int4ListPtr *list2));
+
+/* Attempts to reallocate new_size elements to the list. Returns NULL on
+ * incorrect arguments or if it cannot allocate more memory */
+Int4ListPtr LIBCALL
+Int4ListResize PROTO((Int4ListPtr listp, Int4 new_size));
+
+/* Performs a binary search for key on lp.
+   Returns the index into lp->i where key is located or -1 if key is not found
+ */
+Int4 LIBCALL
+Int4ListBSearch PROTO((Int4ListPtr lp, Int4 key));
+
+/* Ascendingly sorts the list and removes repeated entries */
+Int4ListPtr LIBCALL
+Int4ListMakeUnique PROTO((Int4ListPtr list));
+
+/* Returns the ascending sorted intersection of list1 and list2, freeing the
+ * both parameters */
+Int4ListPtr LIBCALL
+Int4ListIntersect PROTO((Int4ListPtr *list1, Int4ListPtr *list2));
+
 
 /*
 	Common index structures
@@ -677,6 +909,11 @@ typedef struct read_db_shared_info {
 /* -- Here is set of definitions used with taxonomy info database ----- */
 /* ---------------------------------------------------------------------*/
 
+/* The following #define allows for the creation of taxonomy databases along
+ * with the blast databases. Please note that the code to create the blast
+ * databases is NOT thread-safe! */
+/*#define FDB_TAXONOMYDB*/
+
 typedef	struct _RDBTaxId {
     Uint4 taxid;
     Uint4 offset;
@@ -687,7 +924,9 @@ typedef	struct _RDBTaxInfo {
     Int4        reserved[4];     /* reserved */
     NlmMFILEPtr taxfp;           /* Memory mapped index file */
     RDBTaxIdPtr taxdata;         /* Index tax_id/file offset */
-    FILE        *name_fd;        /* Pointer to the file with taxonomy names */
+    Boolean     taxdata_alloc;   /* true if taxdata was allocated */
+    NlmMFILEPtr name_fd;         /* Pointer to the file with taxonomy names */
+    Boolean     taxinfo_alloc;   /* Flag to determine structure ptr ownership */
 } RDBTaxInfo, *RDBTaxInfoPtr;
 
 typedef	struct _RDBTaxLookup {
@@ -706,9 +945,12 @@ typedef Boolean (*TaxCallbackFunc) (RDBTaxLookupPtr tax_lookup, Int4 tax_id);
       Taxonomy blast database 
       ----  */
 
+#define TAXDB_ON_FTP "ftp://ftp.ncbi.nih.gov/blast/db/taxdb.tar.gz"
+#define BLAST_TAXDB_FILENAME "taxdb"
+
 /* Initialize taxonomy lookup database. returns NULL if failure or
    this database do not exists */
-RDBTaxInfoPtr  RDBTaxInfoInit(CharPtr base_filename, Boolean is_prot);
+RDBTaxInfoPtr  RDBTaxInfoInit(void);
 
 /* Free memory, unmap files etc. related to the taxonomy database */
 void RDBTaxInfoClose(RDBTaxInfoPtr tip);
@@ -716,7 +958,6 @@ void RDBTaxInfoClose(RDBTaxInfoPtr tip);
 /* Main function to get taxonomy names for given tax_id from
    blast taxonomy database. Returns NULL if tax_id is not in the database */
 RDBTaxNamesPtr RDBGetTaxNames(RDBTaxInfoPtr tip, Int4 tax_id);
-void RDBTaxNamesFree(RDBTaxNamesPtr tnames);
 
 #define TAX_DB_MAGIC_NUMBER 0x8739
 
@@ -759,9 +1000,11 @@ sequence information. */
 /* Buffer and allocated amount of this buffer.  These should always be
 NULL (i.e., NOT USED) if mem-mapping is used; only used to store sequence
 if there is no mem-mapping or it failed. */
-        ISAMObjectPtr nisam_opt;  /* Object for numeric search */
-        ISAMObjectPtr sisam_opt;  /* Object for string search */
-        RDBTaxInfoPtr taxinfo;    /* This object if not NULL - pointer to
+
+    ISAMObjectPtr nisam_opt;  /* Object for numeric search */
+    ISAMObjectPtr sisam_opt;  /* Object for string search */
+    ISAMObjectPtr isam_pig;   /* Object for PIG search */
+    RDBTaxInfoPtr taxinfo;    /* This object if not NULL - pointer to
                                      the taxonomy names database */
 	Uint1Ptr buffer;
 	Int4 allocated_length;
@@ -769,13 +1012,18 @@ if there is no mem-mapping or it failed. */
 	Int2	            filebit;   /* bit corresponding to the DB file */
 	Int2		    aliasfilebit;/* bit corresponding to the DB alias file */
 	OIDListPtr	    oidlist;   /* structure containing a list of ordinal ID's. */
-    Int4            membership_bit; /* membership bit read from .[pn]al file for
-                                       structured asn deflines */
+    Int4            membership_bit; /* membership bit read from .[pn]al file for structured asn deflines */
 	Int4		    sparse_idx;/* Sparse indexes indicator */
         Char                full_filename[PATH_MAX]; /* Full path for the file */
         ReadDBSharedInfoPtr shared_info;
-	Int4 	            gi_target; /* only this gi should be retrieved if non-zero. */
-        CharPtr             gifile;    /* Path to a file with the gi list */
+	Int4 	            gi_target; /* only this gi should be retrieved */
+                                       /* if non-zero. */
+    CharPtr             gifile;   /* Path to a file with the gi list, should 
+									 always be NULL after readdb_new* calls */
+    Int4ListPtr     gilist; 	  /* storage for the above file in memory */
+    Int4		    preferred_gi; /* this gi should be listed first */
+                                  /* in the bioseq if non-zero */
+	Int4    last_preloaded; /* starting ordinal id of the last preloaded file block */
 } ReadDBFILE, PNTR ReadDBFILEPtr;
     
 /* Function prototypes */
@@ -795,7 +1043,8 @@ Int2	bit_engine_numofbits(Int4 word);
 Int2	ParseDBConfigFile(DataBaseIDPtr *dbidsp, CharPtr path);
 CharPtr	FindBlastDBFile (CharPtr filename);
 CharPtr	FindDBbyGI(CommonIndexHeadPtr cih, Int4 gi, Uint1 *is_prot);
-RDBTaxNamesPtr readdb_get_taxnames(ReadDBFILEPtr rdfp, Int4 tax_id);
+RDBTaxNamesPtr LIBCALL readdb_get_taxnames PROTO((
+            ReadDBFILEPtr rdfp, Int4 tax_id));
 
 /* mmap's */
  
@@ -819,9 +1068,18 @@ ReadDBFILEPtr LIBCALL readdb_new PROTO((CharPtr filename, Uint1 is_prot));
 */
 ReadDBFILEPtr LIBCALL readdb_new_ex PROTO((CharPtr filename, Uint1 is_prot, Boolean init_indices));
 
-/* allows a list of ordinalid's (or list of list of ordinal id's) to be specified.
+/* 
+ * Initializes the blast database specified in the argument list.
+ * filename: blast database to initialize
+ * is_prot: is this database protein ?
+ * init_state: bitwise-OR of the READDB_NEW_* values (selectively mmap certain
+ *             files)
+ * oidlist: Path to the ordinal id list to use (this is mmap'd)
+ * gilist: Path to the gi list to use (this is not resolved until the search
+ *         is conducted (see BlastProcessGiLists)
 */
-ReadDBFILEPtr LIBCALL readdb_new_ex2 PROTO((CharPtr filename, Uint1 is_prot, Uint1 init_state, CharPtr oidlist));
+ReadDBFILEPtr LIBCALL readdb_new_ex2 PROTO((CharPtr filename, Uint1 is_prot,
+            Uint1 init_state, CharPtr oidlist, CharPtr gilist));
 
 
 /* 
@@ -852,6 +1110,7 @@ Boolean LIBCALL readdb_copy PROTO((ReadDBFILEPtr rdfp));
 */
 Boolean LIBCALL readdb_compare PROTO((ReadDBFILEPtr rdfp1, ReadDBFILEPtr rdfp2));
 
+
 /*
         Get total length and number of sequences in multiple databases.
 */
@@ -867,7 +1126,37 @@ Boolean LIBCALL readdb_get_totals PROTO((ReadDBFILEPtr rdfp_list, Int8Ptr total_
 Boolean LIBCALL
 readdb_get_totals_ex PROTO((ReadDBFILEPtr rdfp_list, Int8Ptr total_len, Int4Ptr total_num, Boolean use_alias));
 
+/* retrieves the total number of sequences and database length in the
+ * rdfp_list. use_alias and use_virtual_oidlist are mutually exclusive
+ * options (both of them cannot be true at the same time). If 
+ * use_virtual_oidlist is TRUE, this function assumes that this rdfp_list has 
+ * been processed by BlastProcessGiLists */
+Boolean LIBCALL
+readdb_get_totals_ex2 PROTO ((ReadDBFILEPtr rdfp_list, Int8Ptr dblen, 
+        Int4Ptr nseq, Boolean use_alias, Boolean use_virtual_oidlist));
 
+/* Enumerated type to determine if the database length (number of
+ * bases/residues) should be approximated or calculated exactly by
+ * readdb_get_totals_ex3 */
+typedef enum {
+    eExact,
+    eApproximate
+} EAccountingMode;
+
+/* This function is identical to readdb_get_totals_ex2 but it uses its last
+ * argument to determine if in the case of nucleotide databases the exact
+ * database length is required. If eExact is used, the exact database size is
+ * calculated, if eApproximate is used, an approximation is returned. This is
+ * done to avoid having to touch every last byte of each sequence to determine
+ * the exact length of the database when it is restricted by a virtual oidlist.
+ * The EAccountingMode argument is irrelevant for protein databases, where this
+ * function always return the exact database length. Same assumption about 
+ * BlastProcessGiLists as in readdb_get_totals_ex2 applies.
+ */
+Boolean LIBCALL
+readdb_get_totals_ex3 PROTO ((ReadDBFILEPtr rdfp_list, Int8Ptr dblen, 
+        Int4Ptr nseq, Boolean use_alias, Boolean use_virtual_oidlist,
+        EAccountingMode acc_mode));
 
 /* 
 Get the sequence with sequence_number and put it in buffer.  No memory
@@ -922,6 +1211,14 @@ Gets a BioseqPtr containing the sequence in sequence_number.
 BioseqPtr LIBCALL readdb_get_bioseq PROTO((ReadDBFILEPtr rdfp, Int4 sequence_number));
 BioseqPtr LIBCALL readdb_get_bioseq_ex PROTO((ReadDBFILEPtr rdfp, Int4 sequence_number, Boolean use_objmgr, Boolean insert_ctrlA));
 
+/* 
+   Gets the exact sequence length for protein sequences, but for nucleotide
+   sequences it gets the length of the sequence +/- at most 3 bases (last byte
+   is not examined, therefore the return value is an approximation).
+ */
+Int4 LIBCALL readdb_get_sequence_length_approx PROTO((ReadDBFILEPtr rdfp,
+                                                       Int4 sequence_number));
+
 /*
 Get the length of the sequence.
 */
@@ -1064,13 +1361,76 @@ typedef struct FASTALookup {
     Int4    used;           /* Number of Uint4 used      */
 } FASTALookup, PNTR FASTALookupPtr;
 
+/* Structure that holds the link information as read from the file */
+typedef struct _linkinfo {
+    Int4 bit_number;      /* indicates the position in links bit array */
+    Int4ListPtr gi_list;  /* update links bit array for gis in this list */
+} LinkInfo, *LinkInfoPtr;
+
+/* Structure that holds the membership information */
+typedef Boolean (*GMCriteriaFunc) (VoidPtr direc);
+
+typedef struct _membinfo {
+    Int4 bit_number;    /* indicates the position in the membership bit array */
+    GMCriteriaFunc criteria; /* function pointer that is invoked to
+                                 determine wheather certain sequence
+                                 belongs to the membership represented by
+                                 this bit_number */
+} MembInfo, *MembInfoPtr;
+
+/* Options to clean up blast database files from a previous instance of the
+ * database with the same name as the one about to be created. This has been
+ * added to prevent the case in which an alias file might have precedence over
+ * a single-volume blast database. */
+typedef enum EFDBCleanOpt {
+    eCleanNever = 0,    /* don't remove older files of the db to be created,
+                           just overwrite them or ignore alias files */
+    eCleanAlways,       /* clean up all older files of the db to be created */
+    eCleanPrompt,       /* Assumes interactive program */
+    eCleanOptMax
+} EFDBCleanOpt;
+
+/*** PIG (Protein Identifier Group) interface ***/
+
+#define PIG_NONE        -1          /* No protein identifier group */
+
+/* PIG table structure
+ * From this information the formatdb API creates a pair of ISAM files to map 
+ * PIGs to ordinal ids */
+typedef struct FDBPigTable {
+    Int4Ptr     pop;                /* list of pig/ordinal id pairs */
+    Int4        count, allocated;   /* keep track of table size */
+} FDBPigTable, * FDBPigTablePtr;
+
+/* Allocate a PIG table structure */
+FDBPigTablePtr LIBCALL
+FDBPigTableNew PROTO((void));
+
+/* Deallocate a PIG table structure */
+FDBPigTablePtr LIBCALL
+FDBPigTableFree PROTO((FDBPigTablePtr fptp));
+
+/* Add a PIG to the PIG table structure, return FALSE on error */
+Boolean LIBCALL
+FDBAddPig PROTO((FDBPigTablePtr fptp, Int4 pig, Int4 oid));
+
+/* Retrieve the PIG for a given ordinal id */
+Int4 LIBCALL
+readdb_get_pig PROTO((ReadDBFILEPtr rdfp, Int4 oid));
+
+/* Retrieve the ordinal id corresponding to a given PIG (analogous to
+ * readdb_gi2seq) */
+Int4 LIBCALL
+readdb_pig2oid PROTO((ReadDBFILEPtr rdfp, Int4 pig, Int4Ptr start));
+
+/************************************************/
+
 typedef struct _FDB_options {
     Int4  version;   /* Version of the database created by formatdb program
-	    	 	currently supported are 3 - FORMATDV_VER_TEXT and
+	    	 	currently supported are 3 - FORMATDB_VER_TEXT and
 	    	 	4 - FORMATDB_VER - for ASN.1 structured deflines */
     CharPtr db_title;    /* Title for the database to be created */
     CharPtr db_file;     /* Name for input data file - 'IN' name */
-    CharPtr LogFileName; /* Used only in formatdb.c */
     Int4 is_protein;     /* Is this protein database ? */
     Int4 parse_mode;     /* Do we assume, that deflines are started from 
                              valid SeqIds ? */
@@ -1095,10 +1455,19 @@ typedef struct _FDB_options {
                                      Taxonomy server */
    Int8 bases_in_volume;  /* The maximal number of bases that can be stored in
                              one volume of the database */
+   Int4 sequences_in_volume; /* Maximum number of sequences to be stored in a
+                                volume */
    Int2 volume;      /* Largest volume */
-   CharPtr	alias_file, /* name of alias file to be generated. */
-		gi_file,	/* Gi file to be used in processing. */
-		gi_file_bin;	/* Gi file to be used in processing. */
+   Int4 total_num_of_seqs; /* total number of sequences for this database */
+   CharPtr	gi_file;        /* Gi file to be used in processing. */ 
+   CharPtr  gi_file_bin;	/* Gi file to be used in processing. */
+
+   ValNodePtr    linkbit_listp; /* list of gis and the bits to set */
+   ValNodePtr    memb_tblp;     /* Linked list of MembInfo structures */
+   VoidPtr       memb_argp;     /* Argument to criteria function in MembInfo
+                                   structure */
+   EFDBCleanOpt clean_opt;      /* clean up option */
+
 } FDB_options, PNTR FDB_optionsPtr;
     
 typedef struct formatdb 
@@ -1121,7 +1490,7 @@ typedef struct formatdb
     /* ASN.1 defline output if structured defline */
     AsnIoPtr aip_def;
     
-    Int4 num_of_seqs;  /* number of parsed sequences */
+    Int4 num_of_seqs;  /* number of parsed sequences in this volume */
     Int8 TotalLen;
     Int4 MaxSeqLen;
     
@@ -1134,6 +1503,9 @@ typedef struct formatdb
 
     FASTALookupPtr	lookup;
 
+    /* Table to map PIGs to ordinal ids */
+    FDBPigTablePtr   ptable;   
+
     /* General formatdb options */
 
     FDB_optionsPtr options;
@@ -1151,6 +1523,44 @@ typedef struct formatdb
 
 /* Function prototypes for formatdb library*/
 
+/* --------------------- FDBOptionsNew ----------------------------
+   Purpose: Creates formatdb options structure with parameters from
+            the argument list.
+   Returns: Pointer to initialized structure.
+   Notes: If alias_file_name is provided, the function FDB_MakeAlias
+          should be called after FDBClose. (FIXME)
+   ---------------------------------------------------------------- */
+FDB_optionsPtr FDBOptionsNew(
+        CharPtr input, /* [in] name of input file */
+        Boolean is_prot, /* [in] input contains protein sequences? */
+        CharPtr title,  /* [in] title to give this database */
+        Boolean is_asn, /* [in] true if input is in ASN.1 */
+        Boolean is_asn_bin, /* [in] true if ASN.1 input is binary */
+        Boolean is_seqentry, /* [in] true of input is a seqentry */
+        Boolean sparse_idx, /* [in] should sparce ISAM indices be used? */
+        Boolean test_non_unique, /* [in] test for repeated string identifiers
+                                    in database */
+        Boolean parse_deflines, /* [in] input contains parseable deflines? */
+        CharPtr basename, /* [in] name for the database to create */
+        CharPtr alias_file_name, /* [in] name for the alias file to create */
+        Int8 bases_per_volume, /* [in] max num of residues/bases per volume */
+        Int4 seqs_per_volume, /* [in] max num of sequences per volume */
+        Int4 version, /* [in] database version */
+        Boolean dump_info, /* [in] should basename.[pn]di be created? */
+        EFDBCleanOpt clean_opt);/* [in] should basename.* files be removed ? */
+
+/* --------------------- FDBOptionsFree ---------------------------
+   Purpose: Frees the memory allocated for the formatdb options structure.
+   Returns: NULL
+   ---------------------------------------------------------------- */
+FDB_optionsPtr FDBOptionsFree(FDB_optionsPtr options);
+
+/* The next 4 functions are for production database dump ({id,rs}dump_blast) */
+ValNodePtr FDBLoadLinksTable(void);
+ValNodePtr FDBDestroyLinksTable(ValNodePtr list);
+ValNodePtr FDBLoadMembershipsTable(void);
+ValNodePtr FDBDestroyMembershipsTable(ValNodePtr tbl);
+
 FormatDBPtr	FormatDBInit(FDB_optionsPtr options);
 
 Int2 FDBAddSequence (FormatDBPtr fdbp,  BlastDefLinePtr bdp, 
@@ -1158,11 +1568,34 @@ Int2 FDBAddSequence (FormatDBPtr fdbp,  BlastDefLinePtr bdp,
                      Int4 SequenceLen, 
                      CharPtr seq_id, CharPtr title, 
                      Int4 gi, Int4 tax_id, CharPtr div, Int4 owner, Int4 date);
-    
+
+Int2 FDBAddSequence2 (FormatDBPtr fdbp, BlastDefLinePtr bdp,
+                      Uint1 seq_data_type, ByteStorePtr *seq_data, 
+                      Int4 SequenceLen, 
+                      /* These 2 parameters are left for the backward
+                         compatibility. They are not used for ASN.1 structues
+                         deflines dump */
+                      
+                      CharPtr seq_id, CharPtr title,
+                      
+                      /* These parameters suppose, that this function adds
+                         sequence to the Blast database with single definition 
+                         line. Generally speaking, this is not the common case
+                         and if this function is used to add sequence item
+                         with many definition lines these parameters must not
+                         be used at all. */
+                      
+                      Int4 gi, Int4 tax_id, CharPtr div, Int4 owner, Int4 date,
+                      Uint4Ptr  AmbCharPtr,
+                      Int4 pig_id); /* protein identifier group */
+
 Int2 FDBAddBioseq(FormatDBPtr fdbp, BioseqPtr bsp, BlastDefLinePtr bdp);
 Int2 FormatDBClose(FormatDBPtr fdbp);
 
-void FDB_FreeCLOptions(FDB_optionsPtr options);
+
+Boolean FDBAddLinksInformation(BlastDefLinePtr bdp, ValNodePtr links_tblp);
+Boolean FDBAddMembershipInformation(BlastDefLinePtr bdp, ValNodePtr memb_tblp, 
+                                    VoidPtr criteria_arg);
 
 Int2 process_sep (SeqEntryPtr sep, FormatDBPtr fdbp);
 
@@ -1189,27 +1622,57 @@ typedef	struct di_record {
     Int4    gi;
     Int4    taxid;
     Int4    owner;
-    Char    div[3];
+    Char    div[4];
     Int4    len;
     Int4    hash;
     Int4    date;
-	Int4	gi_threshold; /* for 'month' subset */
+    CharPtr acc; /* accession should not exceed this size */
+	Int4	gi_threshold;   /* for 'month' subset */
 } DI_Record, *DI_RecordPtr;
 
-Boolean	ScanDIFile(CharPtr difilename, CharPtr subset,
+/******** genmask structures and functions *********/
+
+/* genmask scans the *.[pn]di files and sets membership bits according to the
+   criteria specified by the GMCriteria function (see typedef above). This is
+   one example of how to set the membership bits in the new database format.
+   Note that the MembInfo structure has a criteria function pointer that
+   returns a boolean value and takes a void ptr as an argument to allow
+   flexibility in specifying the criteria to belong to a particular
+   membership. */
+
+typedef struct {
+    Int4           count, allocated;
+    CharPtr        *subset_name;
+    GMCriteriaFunc *criteria;
+    Int4           *membership_bit;
+} GMSubsetData, * GMSubsetDataPtr;
+
+Boolean	ScanDIFile(CharPtr difilename, GMSubsetDataPtr gmsubsetdp,
 	Boolean(*callback)(DI_RecordPtr direc, VoidPtr data), VoidPtr data,
 	FILE *out, Int4 gi_threshold);
 
-/* These functions determine the criteria for the membership bits */
-Boolean is_EST_HUMAN(DI_Record direc);
-Boolean is_EST_MOUSE(DI_Record direc);
-Boolean is_EST_OTHERS(DI_Record direc);
-Boolean is_SWISSPROT(DI_Record direc);
-Boolean is_MONTH(DI_Record direc);
-Boolean is_PDB(DI_Record direc);
-Boolean is_REFSEQ(DI_Record direc);
-Boolean is_CONTIG(DI_Record direc);
+CharPtr FDFGetAccessionFromSeqIdChain(SeqIdPtr seqid_list);
 
+/* These functions determine the criteria for the membership bits for genmask.
+   Only protein sequences have memberships because they are in non-redundant
+   databases */
+Boolean is_EST_HUMAN(VoidPtr di_record);
+Boolean is_EST_MOUSE(VoidPtr di_record);
+Boolean is_EST_OTHERS(VoidPtr di_record);
+Boolean is_SWISSPROT(VoidPtr di_record);
+Boolean is_MONTH(VoidPtr di_record);
+Boolean is_PDB(VoidPtr di_record);
+Boolean is_REFSEQ_GENOMIC(VoidPtr di_record);
+Boolean is_REFSEQ_RNA(VoidPtr di_record);
+Boolean is_REFSEQ_PROTEIN(VoidPtr di_record);
+Boolean is_CONTIG(VoidPtr di_record);
+Boolean is_WGS_ANOPHELES(VoidPtr di_record);
+Boolean is_WGS_RICE(VoidPtr di_record);
+Boolean is_WGS_MOUSE(VoidPtr di_record);
+Boolean is_WGS_ANTHRAX(VoidPtr di_record);
+
+#if 0
+/* The common index is deprecated  - camacho 09/02/2002 */
 typedef	struct updateindex_struct {
     FILE	*cifile;/* CommonIndex file */
     Int2	shift;	/* database shift in mask */
@@ -1219,24 +1682,50 @@ typedef	struct updateindex_struct {
 
 Int4	UpdateCommonIndexFile (CharPtr dbfilename, Boolean proteins,
 		FILE *fout, CharPtr difile, Int4 gi_threshold);
+#endif
 
-/* Fastacmd API */
+/************************************************************************/
+/*        Fastacmd API                                           */
+/************************************************************************/
+
+typedef struct FCMDAccList {
+    CharPtr acc;
+    Int4 gi;
+    struct FCMDAccList *next;
+} FCMDAccList, PNTR FCMDAccListPtr;
+
+FCMDAccListPtr LIBCALL GetAccList(CharPtr file, Int4Ptr TotalItems);
+void LIBCALL FCMDAccListFree(FCMDAccListPtr falp);
+
+#define FASTACMD_DEFAULT_DB "nr"
+
+#define FASTACMD_SUCCESS 0
+#define FASTACMD_ERROR 1
+#define FASTACMD_DB_NOT_FOUND 2
+#define FASTACMD_FAILED_SEARCH 3
+#define FASTACMD_NO_TAXDB 4
+
+/* Fastacmd_Search and Fastacmd_Search_ex return non-zero on failure */
 Int2 Fastacmd_Search (CharPtr searchstr, CharPtr database,
 	CharPtr batchfile, Boolean dupl, Int4 linelen, FILE *out);
-Int2 Fastacmd_Search_ex (CharPtr searchstr, CharPtr database,
+Int2 Fastacmd_Search_ex (CharPtr searchstr, CharPtr database, Uint1 is_prot,
 	CharPtr batchfile, Boolean dupl, Int4 linelen, FILE *out, 
-	Boolean use_target, Boolean use_ctrlAs, Boolean dump_db);
+	Boolean use_target, Boolean use_ctrlAs, Boolean dump_db, 
+    CharPtr seqlocstr, Uint1 strand, Boolean taxonomy_info_only, 
+    Boolean dbinfo_only, Int4 pig);
+
 Int2 BlastDBToFasta(ReadDBFILEPtr rdfp, FILE *fp, Int4 line_length, 
 		    Boolean use_ctrlAs);
 
 Int4 LIBCALL readdb_MakeGiFileBinary PROTO((CharPtr input_file, CharPtr
 					    output_file));
 
-Int4 FastaToBlastDB PROTO((FDB_optionsPtr options, CharPtr basename, 
-			   Int4 Bases_In_Volume));
+Int4 FastaToBlastDB PROTO((FDB_optionsPtr options, Int4 Bases_In_Volume));
 
 BlastDefLinePtr FDReadDeflineAsn(ReadDBFILEPtr rdfp, Int4 sequence_number);
 
+CharPtr FD_ConstructMultivolumeDBList(CharPtr basename, Int4 vols);
+
 Boolean FD_CreateAliasFileEx PROTO((CharPtr title, CharPtr basename, 
             Int4 volumes, Boolean is_protein, CharPtr parent,
             Int4 first_oid, Int4 last_oid, Int8 total_length, Int4 number_seqs,
@@ -1248,7 +1737,39 @@ Boolean FD_CreateAliasFile PROTO((CharPtr title, CharPtr basename,
 /* simple function to make alias file give FDB_optionsPtr, alias file is only made if appropriate. */
 Boolean FD_MakeAliasFile PROTO((FDB_optionsPtr options));
 Int4 LIBCALL 
-readdb_get_sequence_number PROTO((ReadDBFILEPtr rdfp, Int4 first_seq, Int4 offset));
+readdb_get_sequence_number PROTO((ReadDBFILEPtr rdfp, Int4 first_seq, Int8 offset));
+
+#if defined(OS_UNIX_SOL) || defined(OS_UNIX_LINUX)
+#ifdef HAVE_MADVISE
+
+/* enable/disable madvise functionality, -- disabled by default */
+void LIBCALL
+readdb_madvise_enable PROTO((Boolean enable));
+
+/* set madvise type, -- default eMMA_Normal */
+void LIBCALL
+readdb_madvise_type PROTO((EMemMapAdvise advice));
+
+/* explicitly set madvise sync mode:
+ * default is sync on Solaris, async on Linux 
+ */
+void LIBCALL
+readdb_madvise_sync_mode PROTO((Boolean mode));
+
+/* explicitly set madvise block size, which is the
+ * number of sequences preloaded in a single madvise
+ * operation, default is 65536
+ */
+void LIBCALL
+readdb_madvise_block PROTO((Int4 nSeqs));
+
+/* call preload directly -- run madvise on a chunk of memory mapped file */
+void LIBCALL
+readdb_preload PROTO((ReadDBFILEPtr rdfp, Int4 first_db_seq,
+				Int4 final_db_seq, EMemMapAdvise advice, Boolean sync));
+
+#endif /* HAVE_MADVISE */
+#endif /* SOL || LINUX */
 
 #ifdef __cplusplus
 }
